/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.control;

import java.util.LinkedList;
import java.util.List;
import java.util.Random;

import org.apache.jmeter.control.InterleaveControl;

/**
 * 
 * @author Jacob
 */
public class RandomInterleaveController extends InterleaveControl {

	private static final long serialVersionUID = 1L;

	private static final Random RAND = new Random();
	
	private static final String WORK_LIST = "RandomInterleaveController.WorkList";

	/**
	 * @see org.apache.jmeter.control.GenericController#resetCurrent()
	 */
	@Override
	protected void resetCurrent() {
		random();
	}
	
	/**
	 * @see org.apache.jmeter.control.GenericController#incrementCurrent()
	 */
	@Override
	protected void incrementCurrent() {
		super.incrementCurrent();
		random();
	}

	@SuppressWarnings("unchecked")
	private void random() {
		if (getSubControllers().size() > 0) {
			List<Integer> controllersAndSamplers = (List<Integer>) getThreadContext().getVariables().getObject(WORK_LIST);
			if (controllersAndSamplers == null) {
				controllersAndSamplers = new LinkedList<Integer>();
				getThreadContext().getVariables().putObject(WORK_LIST, controllersAndSamplers);
			}
			if (controllersAndSamplers.isEmpty()) {
				for (int i = 0; i < getSubControllers().size(); i++) {
					controllersAndSamplers.add(i);
				}
			}
			int index = RAND.nextInt(controllersAndSamplers.size());
			current = controllersAndSamplers.get(index);
			controllersAndSamplers.remove(index);
		} else {
			current = 0;
		}
	}
}
