/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.nsp.assertion;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.xml.namespace.NamespaceContext;
import javax.xml.xpath.XPathFactory;

import org.apache.jmeter.assertions.Assertion;
import org.apache.jmeter.testelement.AbstractTestElement;

import dk.sosi.seal.model.Reply;

/**
 * 
 * @author Jacob
 */
public abstract class AbstractNSPAssertion extends AbstractTestElement implements Assertion {

	private static final long serialVersionUID = 1L;

	protected static final XPathFactory XPATH_FACTORY = XPathFactory.newInstance();

	protected Reply getReply() {
		return (Reply) getThreadContext().getVariables().getObject(Reply.class.getName());
	}

	/**
	 * Create a simple namespace context that maps prefixes (provided at index 0,2,4 and so on) to namespaces (provided
	 * at index 1,3,5 and so on)
	 * @throws ArrayIndexOutOfBoundsException if an uneven number of arguments is provided
	 */
	protected static NamespaceContext namespaceContext(String... prefixToNamespaces) throws ArrayIndexOutOfBoundsException {
		if (prefixToNamespaces.length % 2 != 0) {
			throw new ArrayIndexOutOfBoundsException("Must be called with an even number of arguments");
		}
		SimpleNamespaceContext ctx = new SimpleNamespaceContext();
		for (int i = 0; i + 1 < prefixToNamespaces.length; i += 2) {
			ctx.put(prefixToNamespaces[i], prefixToNamespaces[i + 1]);
		}
		return ctx;
	}

	private static final class SimpleNamespaceContext implements NamespaceContext {

		private final Map<String, String> namespaces = new HashMap<String, String>();

		private final Map<String, String> prefixes = new HashMap<String, String>();

		public void put(String prefix, String namespace) {
			String oldNamespace = namespaces.get(prefix);
			String oldPrefix = prefixes.get(namespace);
			namespaces.remove(oldPrefix);
			prefixes.remove(oldNamespace);
			namespaces.put(prefix, namespace);
			prefixes.put(namespace, prefix);
		}

		@Override
		@SuppressWarnings("rawtypes")
		public Iterator getPrefixes(String namespaceURI) {
			return Arrays.asList(prefixes.get(namespaceURI)).iterator();
		}

		@Override
		public String getPrefix(String namespaceURI) {
			return prefixes.get(namespaceURI);
		}

		@Override
		public String getNamespaceURI(String prefix) {
			return namespaces.get(prefix);
		}
	}

}
