/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.nsp.processor;

import java.util.HashSet;
import java.util.Set;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.jmeter.config.Argument;
import org.apache.jmeter.config.Arguments;
import org.apache.jmeter.testelement.TestStateListener;
import org.apache.jmeter.testelement.property.PropertyIterator;
import org.apache.jorphan.logging.LoggingManager;
import org.apache.log.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import dk.sosi.seal.SOSIFactory;
import dk.sosi.seal.model.IDCard;
import dk.sosi.seal.modelbuilders.IDCardModelBuilder;

/**
 * 
 * @author Jacob
 */
public class PullSharedVariablesPostProcessor extends AbstractNSPPreProcessor implements TestStateListener {

	private static final long serialVersionUID = 1L;

	private static final Logger log = LoggingManager.getLoggerForClass();

	public static final String VARIABLES = "PushSharedVariablesPostProcessor.Variables";

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void process() {
		Set<String> variables = new HashSet<String>();
		Arguments arguments = (Arguments) getProperty(VARIABLES).getObjectValue();
		if (arguments != null) {
			for (PropertyIterator iterator = arguments.iterator(); iterator.hasNext();) {
				Argument argument = (Argument) iterator.next().getObjectValue();
				variables.add(argument.getValue());
			}
		}

		/*
		 * IDCards are not thread safe when calling IDCard#serialize2DOMDocument(SOSIFactory, Document) So we make a
		 * copy for each thread.
		 */
		if (variables.contains(IDCard.class.getName())) {
			IDCard idCard = (IDCard) SharedObjects.get(IDCard.class.getName());
			SOSIFactory factory = (SOSIFactory) SharedObjects.get(SOSIFactory.class.getName());
			if (idCard != null && factory != null) {
				IDCard idCardCopy = copyIDCard(idCard, factory);
				getThreadContext().getVariables().putObject(IDCard.class.getName(), idCardCopy);
				variables.remove(IDCard.class.getName());
				if (log.isDebugEnabled()) {
					log.debug("Pulled thread safe copy of IDCard from shared objects");
				}
			} else {
				getThreadContext().getVariables().putObject(IDCard.class.getName(), null);
				log.warn("Pulled null value for IDCard from shared objects");
			}
		}

		for (String variable : variables) {
			if (getThreadContext().getVariables().getObject(variable) == null) {
				Object object = SharedObjects.get(variable);
				getThreadContext().getVariables().putObject(variable, object);
				if (object != null) {
					if (log.isDebugEnabled()) {
						log.debug("Pulled " + variable + " from shared objects");
					}
				} else {
					log.warn("Pulled null value for " + variable + " from shared objects");
				}
			} else {
				if (log.isDebugEnabled()) {
					log.debug("Pull of " + variable + " skipped, since another value already in thread context");
				}
			}
		}
	}

	private IDCard copyIDCard(IDCard idCard, SOSIFactory factory) {
		Document document;
		try {
			document = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
		} catch (ParserConfigurationException e) {
			throw new IllegalStateException(e);
		}
		IDCardModelBuilder builder = new IDCardModelBuilder();
		Element assertion = idCard.serialize2DOMDocument(factory, document);
		IDCard idCardCopy = builder.buildModel(assertion);
		return idCardCopy;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void testStarted() {
		testStarted("local");
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void testStarted(String host) {
		SharedObjects.clear();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void testEnded() {
		testEnded("local");
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void testEnded(String host) {
		SharedObjects.clear();
	}

}
