/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.nsp.processor;

import java.util.HashSet;
import java.util.Set;

import org.apache.jmeter.config.Argument;
import org.apache.jmeter.config.Arguments;
import org.apache.jmeter.testelement.TestStateListener;
import org.apache.jmeter.testelement.property.PropertyIterator;
import org.apache.jorphan.logging.LoggingManager;
import org.apache.log.Logger;

import dk.sosi.seal.SOSIFactory;
import dk.sosi.seal.model.IDCard;


/**
 *
 * @author Jacob
 */
public class PushSharedVariablesPostProcessor extends AbstractNSPPostProcessor implements TestStateListener {

	private static final long serialVersionUID = 1L;

	private static final Logger log = LoggingManager.getLoggerForClass();
	
	public static final String VARIABLES = "PushSharedVariablesPostProcessor.Variables";

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void process() {
		Set<String> variables = new HashSet<String>();
		Arguments arguments = (Arguments) getProperty(VARIABLES).getObjectValue();
		if (arguments != null) {
			for (PropertyIterator iterator = arguments.iterator(); iterator.hasNext();) {
				Argument argument = (Argument) iterator.next().getObjectValue();
				variables.add(argument.getValue());
			}
		}
		
		if (variables.contains(IDCard.class.getName())) {
			variables.add(SOSIFactory.class.getName()); // Needed when pulling the id card
		}
		
		for (String variable : variables) {
			Object object = getThreadContext().getVariables().getObject(variable);
			SharedObjects.put(variable, object);
			if (log.isDebugEnabled()) {
				if (object != null) {
					log.debug("Pushed " + variable + " to shared objects");
				} else {
					log.warn("Pushed null value for " + variable + " to shared objects");
				}
			}
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void testStarted() {
		testStarted("local");
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void testStarted(String host) {
		SharedObjects.clear();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void testEnded() {
		testEnded("local");
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void testEnded(String host) {
		SharedObjects.clear();
	}

}
