/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.nsp.processor.gui;

import java.awt.BorderLayout;
import java.awt.Component;

import javax.swing.BorderFactory;
import javax.swing.JPanel;

import org.apache.jmeter.config.Arguments;
import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;
import org.apache.jmeter.testelement.property.TestElementProperty;

import com.arosii.jmeter.protocol.nsp.processor.PullSharedVariablesPostProcessor;
import com.arosii.jmeter.resources.nsp.Messages;


/**
 *
 * @author Jacob
 */
public class PullSharedVariablesPostProcessorGui extends AbstractNSPPreProcessorGui {

	private static final long serialVersionUID = 1L;
	private VariablesPanel variables;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getLabelResource() {
		return "nsp_pull_shared_variables";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected Component createConfigurationPanel() {
		final VerticalPanel configurationPanel = new VerticalPanel();
		appendVariablesPanel(configurationPanel);
		return configurationPanel;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public TestElement createTestElement() {
		PullSharedVariablesPostProcessor extractor = new PullSharedVariablesPostProcessor();
		modifyTestElement(extractor);
		return extractor;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void configure(TestElement element) {
		super.configure(element);
		Arguments arguments = (Arguments) element.getProperty(PullSharedVariablesPostProcessor.VARIABLES).getObjectValue();
		if (arguments != null) {
			variables.configure(arguments);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void modifyTestElement(TestElement element) {
		super.modifyTestElement(element);
		Arguments arguments = (Arguments) variables.createTestElement();
		element.setProperty(new TestElementProperty(PullSharedVariablesPostProcessor.VARIABLES, arguments));
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void clearGui() {
		super.clearGui();
		variables.clearGui();
	}

	private void appendVariablesPanel(VerticalPanel configurationPanel) {
		JPanel variableSelectionPanel = new VerticalPanel();
		variableSelectionPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("nsp_variables")));
		variableSelectionPanel.add(getVariableNamesPanel());
		configurationPanel.add(variableSelectionPanel);
	}

	private JPanel getVariableNamesPanel() {
		variables = new VariablesPanel();
		
		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(variables, BorderLayout.CENTER);
		return panel;
	}
}
