/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.nsp.sampler;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Properties;

import org.apache.jmeter.protocol.http.sampler.HTTPSampleResult;
import org.apache.jmeter.protocol.http.sampler.SoapSampler;
import org.apache.jorphan.logging.LoggingManager;
import org.apache.log.Logger;
import org.w3c.dom.Document;

import dk.sosi.seal.OIOSAMLFactory;
import dk.sosi.seal.SOSIFactory;
import dk.sosi.seal.model.IDCard;
import dk.sosi.seal.model.IDCardToOIOSAMLAssertionResponse;
import dk.sosi.seal.model.OIOSAMLAssertionToIDCardResponse;
import dk.sosi.seal.model.Reply;
import dk.sosi.seal.model.SecurityTokenResponse;
import dk.sosi.seal.modelbuilders.ModelBuildException;
import dk.sosi.seal.vault.CredentialVault;
import dk.sosi.seal.xml.XmlUtil;

/**
 * 
 * @author Jacob
 */
public abstract class AbstractNSPSoapSampler extends SoapSampler {

	private static final long serialVersionUID = 1L;

	protected static final Logger log = LoggingManager.getLoggerForClass();

	/**
	 * {@inheritDoc}
	 */
	@Override
	public URL getUrl() throws MalformedURLException {
		return new URL(getProtocol(), getDomain(), getPort(), getPath());
	}

	protected void putSOSIFactory(SOSIFactory sosiFactory) {
		getThreadContext().getVariables().putObject(SOSIFactory.class.getName(), sosiFactory);
	}

	protected SOSIFactory getSOSIFactory() {
		SOSIFactory sosiFactory = (SOSIFactory) getThreadContext().getVariables().getObject(SOSIFactory.class.getName());
		return sosiFactory;
	}

	protected void putCredentialVault(CredentialVault credentialVault) {
		getThreadContext().getVariables().putObject(CredentialVault.class.getName(), credentialVault);
	}

	protected CredentialVault getCredentialVault() {
		return (CredentialVault) getThreadContext().getVariables().getObject(CredentialVault.class.getName());
	}

	protected void putIDCard(HTTPSampleResult sample) {
		if (sample.isSuccessful()) {
			try {
				SecurityTokenResponse securityTokenResponse = getSOSIFactory().deserializeSecurityTokenResponse(sample.getResponseDataAsString());
				getThreadContext().getVariables().putObject(SecurityTokenResponse.class.getName(), securityTokenResponse);
				if (!securityTokenResponse.isFault()) {
					IDCard idCard = securityTokenResponse.getIDCard();
					getThreadContext().getVariables().putObject(IDCard.class.getName(), idCard);
				} else {
					getThreadContext().getVariables().remove(IDCard.class.getName());
				}
			} catch (ModelBuildException e) {
				log.error("putIDCard failed", e);
				getThreadContext().getVariables().remove(Reply.class.getName());
				getThreadContext().getVariables().remove(IDCard.class.getName());
				sample.setSuccessful(false);
				sample.setResponseMessage("ModelBuildException:" + e.getMessage());
			}
		} else {
			getThreadContext().getVariables().remove(IDCard.class.getName());
			getThreadContext().getVariables().remove(SecurityTokenResponse.class.getName());
		}
	}

	protected IDCard getIDCard() {
		return (IDCard) getThreadContext().getVariables().getObject(IDCard.class.getName());
	}
	
	protected void clearIDCard() {
		getThreadContext().getVariables().remove(IDCard.class.getName());
	}

	protected void putIDCardToOIOSAMLAssertionResponse(HTTPSampleResult sample) {
		if (sample.isSuccessful()) {
			try {
				OIOSAMLFactory factory = new OIOSAMLFactory();
				Document document = XmlUtil.readXml(new Properties(), sample.getResponseDataAsString(), false);
				IDCardToOIOSAMLAssertionResponse response = factory.createIDCardToOIOSAMLAssertionResponseModelBuilder().build(document);
				getThreadContext().getVariables().putObject(IDCardToOIOSAMLAssertionResponse.class.getName(), response);
			} catch (ModelBuildException e) {
				log.error("putOIOSAML failed", e);
				getThreadContext().getVariables().remove(IDCardToOIOSAMLAssertionResponse.class.getName());
				sample.setSuccessful(false);
				sample.setResponseMessage("ModelBuildException:" + e.getMessage());
			}
		} else {
			getThreadContext().getVariables().remove(IDCardToOIOSAMLAssertionResponse.class.getName());
		}
	}

	protected void clearIDCardToOIOSAMLAssertionResponse() {
		getThreadContext().getVariables().remove(IDCardToOIOSAMLAssertionResponse.class.getName());
	}

	protected IDCardToOIOSAMLAssertionResponse getIDCardToOIOSAMLAssertionResponse() {
		return (IDCardToOIOSAMLAssertionResponse) getThreadContext().getVariables().getObject(IDCardToOIOSAMLAssertionResponse.class.getName());
	}

	protected void putOIOSAMLAssertionToIDCardResponse(HTTPSampleResult sample) {
		if (sample.isSuccessful()) {
			try {
				OIOSAMLFactory factory = new OIOSAMLFactory();
				Document document = XmlUtil.readXml(new Properties(), sample.getResponseDataAsString(), false);
				OIOSAMLAssertionToIDCardResponse response = factory.createOIOSAMLAssertionToIDCardResponseModelBuilder().build(document);
				getThreadContext().getVariables().putObject(OIOSAMLAssertionToIDCardResponse.class.getName(), response);
				if (!response.isFault()) {
					IDCard idCard = response.getIDCard();
					getThreadContext().getVariables().putObject(IDCard.class.getName(), idCard);
				} else {
					getThreadContext().getVariables().remove(IDCard.class.getName());
				}
			} catch (ModelBuildException e) {
				log.error("putIDCard failed", e);
				getThreadContext().getVariables().remove(IDCard.class.getName());
				getThreadContext().getVariables().remove(OIOSAMLAssertionToIDCardResponse.class.getName());
				sample.setSuccessful(false);
				sample.setResponseMessage("ModelBuildException:" + e.getMessage());
			}
		} else {
			getThreadContext().getVariables().remove(IDCard.class.getName());
			getThreadContext().getVariables().remove(OIOSAMLAssertionToIDCardResponse.class.getName());
		}
	}

	protected void clearOIOSAMLAssertionToIDCardResponse() {
		getThreadContext().getVariables().remove(IDCard.class.getName());
		getThreadContext().getVariables().remove(OIOSAMLAssertionToIDCardResponse.class.getName());
	}

	protected OIOSAMLAssertionToIDCardResponse getOIOSAMLAssertionToIDCardResponse() {
		return (OIOSAMLAssertionToIDCardResponse) getThreadContext().getVariables().getObject(OIOSAMLAssertionToIDCardResponse.class.getName());
	}

	protected void putReply(HTTPSampleResult sample) {
		if (sample.isSuccessful()) {
			try {
				Reply reply = getSOSIFactory().deserializeReply(sample.getResponseDataAsString());
				if (!reply.isFault()) {
					getThreadContext().getVariables().putObject(Reply.class.getName(), reply);
				} else {
					getThreadContext().getVariables().remove(Reply.class.getName());
				}
			} catch (ModelBuildException e) {
				log.error("putReply failed", e);
				getThreadContext().getVariables().remove(Reply.class.getName());
				sample.setSuccessful(false);
				sample.setResponseMessage("ModelBuildException:" + e.getMessage());
			}
		} else {
			getThreadContext().getVariables().remove(Reply.class.getName());
		}
	}

	protected void clearReply() {
		getThreadContext().getVariables().remove(Reply.class.getName());
	}

	protected Reply getReply() {
		return (Reply) getThreadContext().getVariables().getObject(Reply.class.getName());
	}
	
	/**
	 * @return The id of the node in this iteration. <code>0</code> means this is the master node
	 */
	protected int getNodeId() {
		String nodeId = getThreadContext().getVariables().get("NODE_ID");
		nodeId = nodeId != null && !nodeId.trim().isEmpty() ? nodeId : "0";
		return Integer.parseInt(nodeId);
	}

	/**
	 * @return The number of active nodes running this iteration.
	 */
	protected int getNodeCount() {
		String nodeCount = getThreadContext().getVariables().get("NODE_COUNT");
		nodeCount = nodeCount != null && !nodeCount.trim().isEmpty() ? nodeCount : "1";
		return Integer.parseInt(nodeCount);
	}
}
