/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.reporters;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.text.DecimalFormat;

import org.apache.jmeter.reporters.ResultCollector;
import org.apache.jmeter.samplers.SampleEvent;
import org.apache.jmeter.samplers.SampleResult;
import org.apache.jmeter.services.FileServer;
import org.apache.jmeter.util.Calculator;
import org.apache.jorphan.logging.LoggingManager;
import org.apache.log.Logger;

/**
 * 
 * @author Jacob
 */
public class ThroughputResultCollector extends ResultCollector {

	private static final long serialVersionUID = 1L;

	private static final Logger log = LoggingManager.getLoggerForClass();

	private static final Object LOCK = new Object();

	private transient volatile Calculator total;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void testStarted(String host) {
		super.testStarted(host);
		synchronized (LOCK) {
			String filename = getFilename();
			if (filename != null) {
				total = new Calculator();
			}
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void testEnded(String host) {
		super.testEnded(host);
		synchronized (LOCK) {
			String filename = getFilename();
			if (filename != null && filename.trim().length() > 0) {
				String errorPercentage = new DecimalFormat("#######.##").format(total.getErrorPercentage() * 100);
				String throughput = new DecimalFormat("#######.##").format(total.getRate());

				filename = FileServer.resolveBaseRelativeName(filename);
				PrintWriter out = null;
				try {
					out = new PrintWriter(new FileOutputStream(filename + ".throughput"));
					out.println(errorPercentage);
					out.println(throughput);
					out.flush();
				} catch (FileNotFoundException e) {
					log.error("Could not write throughput to file ");
				} finally {
					if (out != null) {
						out.close();
					}
				}
			}
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void sampleOccurred(SampleEvent event) {
		SampleResult sampleResult = event.getResult();
		if (sampleResult != null) {
			String sampleLabel = sampleResult.getSampleLabel();
			if (sampleLabel != null && sampleLabel.toUpperCase().contains("IGNORE")) {
				return;
			}
		}
		super.sampleOccurred(event);
		SampleResult result = event.getResult();

		if (isSampleWanted(result.isSuccessful())) {
			synchronized (total) {
				total.addSample(result);
			}
		}
	}

}
