/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.utils.nsp.sync;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketAddress;
import java.net.SocketTimeoutException;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * 
 * @author Jacob
 */
public class SyncMaster {

	private static final int ONE_MINUTE = 1000 * 60;

	private static final ConcurrentMap<SocketAddress, Slave> SLAVES = new ConcurrentHashMap<SocketAddress, Slave>();

	private static final AtomicBoolean ALL_CONNECTED = new AtomicBoolean(false);

	private static final AtomicInteger EXIT_STATUS = new AtomicInteger(0);

	public static void main(String[] args) throws Exception {
		ServerSocket serverSocket;
		int expectedSlaves = Integer.valueOf(args[0]);
		int connectedSlaves = 0;
		int timeout = Integer.valueOf(args[1]) * ONE_MINUTE;
		if (args.length > 2) {
			serverSocket = new ServerSocket(Integer.valueOf(args[2]));
		} else {
			serverSocket = new ServerSocket();
			serverSocket.bind(null);
		}
		serverSocket.setSoTimeout(timeout);
		System.out.println("Listening for " + expectedSlaves + " slaves on port " + serverSocket.getLocalPort());
		while (true) {
			if (connectedSlaves >= expectedSlaves) {
				signalSlaves();
				break;
			}
			Socket socket = null;
			try {
				socket = serverSocket.accept();
				Slave connectedSlave = new Slave(socket);
				connectedSlave.start(++connectedSlaves);
				
			} catch (SocketTimeoutException e) {
				System.err.println("Timeout waiting for slaves.");
				for (SocketAddress address : SLAVES.keySet()) {
					System.err.println(address + " connected.");
				}
				EXIT_STATUS.set(1);
				signalSlaves();
				joinSlaves();
				System.exit(1);
			}
		}
	}

	private static void signalSlaves() {
		ALL_CONNECTED.set(true);
		for (Slave slave : SLAVES.values()) {
			synchronized (slave) {
				slave.notify();
			}
		}
	}
	
	private static void joinSlaves() {
		for (Slave slave : SLAVES.values()) {
			try {
				slave.thread.join();
			} catch (InterruptedException ignore) {
			}
		}
	}

	private static class Slave implements Runnable {

		private final Socket socket;

		private Thread thread;

		private int slaveId;

		public Slave(Socket socket) {
			this.socket = socket;
			SLAVES.put(socket.getRemoteSocketAddress(), this);
		}

		public void start(int slaveId) {
			this.slaveId = slaveId;
			this.thread = new Thread(this);
			this.thread.start();
		}

		/**
		 * {@inheritDoc}
		 */
		@Override
		public void run() {
			try {
				BufferedReader in = new BufferedReader(new InputStreamReader(socket.getInputStream()));
				System.out.println("Slave " + slaveId + " connected from " + in.readLine());
			} catch (IOException e) {
				e.printStackTrace();
			}

			synchronized (this) {
				while (!ALL_CONNECTED.get()) {
					try {
						wait();
					} catch (InterruptedException ignore) {
					}
				}
			}

			try {
				OutputStream outputStream = socket.getOutputStream();
				outputStream.write(EXIT_STATUS.get());
				outputStream.flush();
			} catch (IOException e) {
				e.printStackTrace();
			}
			try {
				socket.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

}
