/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.gw.sampler;

import java.io.ByteArrayInputStream;
import java.io.StringWriter;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Random;
import java.util.UUID;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;

import oasis.names.tc.saml._2_0.assertion.AssertionType;
import oasis.names.tc.saml._2_0.assertion.Attribute;
import oasis.names.tc.saml._2_0.assertion.AttributeStatement;
import oasis.names.tc.saml._2_0.assertion.Conditions;
import oasis.names.tc.saml._2_0.assertion.NameIDType;
import oasis.names.tc.saml._2_0.assertion.Subject;

import org.apache.jmeter.protocol.http.sampler.HTTPSampleResult;
import org.oasis_open.docs.wss._2004._01.oasis_200401_wss_wssecurity_secext_1_0.Security;
import org.oasis_open.docs.wss._2004._01.oasis_200401_wss_wssecurity_utility_1_0.Timestamp;
import org.w3._2005._08.addressing.AttributedURIType;
import org.xmlsoap.schemas.soap.envelope.Envelope;
import org.xmlsoap.schemas.soap.envelope.Fault;
import org.xmlsoap.schemas.soap.envelope.Header;

import com.arosii.jmeter.protocol.nsp.sampler.AbstractNSPSoapSampler;
import com.arosii.jmeter.protocol.sts.sampler.SecurityTokenRequestSampler;

import dk.medcom.dgws._2006._04.dgws_1_0.Linking;
import dk.sosi.gw._2007_09.RequestIdCardDigestForSigningResponse;
import dk.sosi.gw._2007_09.SignIdCardRequestBody;
import dk.sosi.seal.model.IDCard;
import dk.sosi.seal.model.constants.IDValues;
import dk.sosi.seal.model.constants.MedcomAttributes;
import dk.sosi.seal.model.constants.SOSIAttributes;
import dk.sosi.seal.model.constants.SubjectIdentifierTypeValues;

/**
 * 
 * @author Jacob Qvortrup
 */
public abstract class AbstractGWSoapSampler extends AbstractNSPSoapSampler {

	private static final long serialVersionUID = 1L;

	private static final Random RANDOM = new Random();

	private static JAXBContext jaxbContext;
	
	private static DatatypeFactory datatypeFactory;

	public AbstractGWSoapSampler() {
		synchronized (AbstractGWSoapSampler.class) {
			if (jaxbContext == null) {
				try {
					jaxbContext = JAXBContext.newInstance(Envelope.class, Security.class, RequestIdCardDigestForSigningResponse.class, SignIdCardRequestBody.class, AttributedURIType.class, dk.medcom.dgws._2006._04.dgws_1_0.Header.class);
				} catch (JAXBException e) {
					throw new RuntimeException(e);
				}
			}
			if (datatypeFactory == null) {
				try {
					datatypeFactory = DatatypeFactory.newInstance();
				} catch (DatatypeConfigurationException e) {
					throw new RuntimeException(e);
				}
			}
		}
	}

	protected void putRequestIdCardDigestForSigningResponse(HTTPSampleResult sample) {
		if (sample.isSuccessful()) {
			try {
				Envelope RequestIdCardDigestForSigningEnvelope = (Envelope) jaxbContext.createUnmarshaller().unmarshal(new ByteArrayInputStream(sample.getResponseData()));
				List<Object> bodyElements = RequestIdCardDigestForSigningEnvelope.getBody().getAny();
				if (!bodyElements.isEmpty()) {
					Object bodyElement = bodyElements.get(0);
					if (bodyElement instanceof RequestIdCardDigestForSigningResponse) {
						RequestIdCardDigestForSigningResponse requestIdCardDigestForSigningResponse = (RequestIdCardDigestForSigningResponse) bodyElement;
						getThreadContext().getVariables().putObject(RequestIdCardDigestForSigningResponse.class.getName(), requestIdCardDigestForSigningResponse);
					} else if (bodyElement instanceof Fault) {
						getThreadContext().getVariables().remove(RequestIdCardDigestForSigningResponse.class.getName());
						sample.setSuccessful(false);
						sample.setResponseMessage("Soap fault");
					} else {
						getThreadContext().getVariables().remove(RequestIdCardDigestForSigningResponse.class.getName());
						sample.setSuccessful(false);
						sample.setResponseMessage("Unknown body element found");
					}
				} else {
					getThreadContext().getVariables().remove(RequestIdCardDigestForSigningResponse.class.getName());
					sample.setSuccessful(false);
					sample.setResponseMessage("No soap body elements found");
				}
			} catch (Exception e) {
				log.error("putRequestIdCardDigestForSigningResponse failed", e);
				getThreadContext().getVariables().remove(RequestIdCardDigestForSigningResponse.class.getName());
				sample.setSuccessful(false);
				sample.setResponseMessage(e.getClass().getSimpleName() + ":" + e.getMessage());
			}
		} else {
			getThreadContext().getVariables().remove(RequestIdCardDigestForSigningResponse.class.getName());
		}
	}

	protected RequestIdCardDigestForSigningResponse getRequestIdCardDigestForSigningResponse() {
		return (RequestIdCardDigestForSigningResponse) getThreadContext().getVariables().getObject(RequestIdCardDigestForSigningResponse.class.getName());
	}

	protected void clearRequestIdCardDigestForSigningResponse() {
		getThreadContext().getVariables().remove(RequestIdCardDigestForSigningResponse.class.getName());
	}

	protected void putNameId(String value) {
		NameIDType nameID = new NameIDType();
		nameID.setValue(value);
		nameID.setFormat(SubjectIdentifierTypeValues.OTHER);
		getThreadContext().getVariables().putObject(NameIDType.class.getName(), nameID);
	}

	protected NameIDType getNameID() {
		return (NameIDType) getThreadContext().getVariables().getObject(NameIDType.class.getName());
	}

	protected Header makeIdCardInSecurityHeader() {
		Security security = new Security();
		security.setTimestamp(makeTimestamp());
		security.setAssertion(makeAssertion());
		Header header = new Header();
		header.getAny().add(security);
		header.getAny().add(makeMedcomHeader());
		return header;
	}

	/**
	 * @return
	 */
	private dk.medcom.dgws._2006._04.dgws_1_0.Header makeMedcomHeader() {
		dk.medcom.dgws._2006._04.dgws_1_0.Header header = new dk.medcom.dgws._2006._04.dgws_1_0.Header();
		header.setSecurityLevel(4);
		Linking linking = new Linking();
		String messageId = "performance-" + getThreadName() + "-" + UUID.randomUUID().toString();
		linking.setMessageID(messageId);
		linking.setFlowID(messageId);
		header.setLinking(linking);
		header.setRequireNonRepudiationReceipt("no");
		return header;
	}

	private Timestamp makeTimestamp() {
		Timestamp timestamp = new Timestamp();
		timestamp.setCreated(datatypeFactory.newXMLGregorianCalendar(new GregorianCalendar()));
		return timestamp;
	}

	private AssertionType makeAssertion() {
		AssertionType assertion = new AssertionType();
		assertion.setIssuer(getThreadName());
		assertion.setId(IDValues.IDCARD);

		Subject subject = new Subject();
		subject.setNameID(getNameID());
		assertion.setSubject(subject);

		AttributeStatement stmt = new AttributeStatement();
		stmt.setId(IDValues.IDCARD_DATA);
		stmt.getAttribute().add(makeAttribute(SOSIAttributes.IDCARD_VERSION, null, "1.0.1"));
		stmt.getAttribute().add(makeAttribute(SOSIAttributes.IDCARD_TYPE, null, IDCard.IDCARDTYPE_USER));
		stmt.getAttribute().add(makeAttribute(SOSIAttributes.AUTHENTICATION_LEVEL, null, "1"));
		stmt.getAttribute().add(makeAttribute(SOSIAttributes.IDCARD_ID, null, "id-card-id"));
		assertion.getAttributeStatement().add(stmt);

		stmt = new AttributeStatement();
		stmt.setId(IDValues.USER_LOG);
		stmt.getAttribute().add(makeAttribute(MedcomAttributes.USER_CIVIL_REGISTRATION_NUMBER, null, getPropertyAsString(SecurityTokenRequestSampler.USER_CPR)));
		stmt.getAttribute().add(makeAttribute(MedcomAttributes.USER_GIVEN_NAME, null, getPropertyAsString(SecurityTokenRequestSampler.USER_GIVENNAME)));
		stmt.getAttribute().add(makeAttribute(MedcomAttributes.USER_SURNAME, null, getPropertyAsString(SecurityTokenRequestSampler.USER_SURNAME)));
		stmt.getAttribute().add(makeAttribute(MedcomAttributes.USER_EMAIL_ADDRESS, null, getPropertyAsString(SecurityTokenRequestSampler.USER_EMAIL)));
		stmt.getAttribute().add(makeAttribute(MedcomAttributes.USER_ROLE, null, getPropertyAsString(SecurityTokenRequestSampler.USER_ROLE)));
		stmt.getAttribute().add(makeAttribute(MedcomAttributes.USER_OCCUPATION, null, getPropertyAsString(SecurityTokenRequestSampler.USER_OCCUPATION)));
		stmt.getAttribute().add(makeAttribute(MedcomAttributes.USER_AUTHORIZATION_CODE, null, getPropertyAsString(SecurityTokenRequestSampler.USER_AUTHORIZATIONCODE)));
		assertion.getAttributeStatement().add(stmt);

		stmt = new AttributeStatement();
		stmt.setId(IDValues.SYSTEM_LOG);
		stmt.getAttribute().add(makeAttribute(MedcomAttributes.IT_SYSTEM_NAME, null, getThreadName()));
		stmt.getAttribute().add(makeAttribute(MedcomAttributes.CARE_PROVIDER_ID, getPropertyAsString(SecurityTokenRequestSampler.ORG_TYPE), getPropertyAsString(SecurityTokenRequestSampler.ORG_ID)));
		stmt.getAttribute().add(makeAttribute(MedcomAttributes.CARE_PROVIDER_NAME, null, getPropertyAsString(SecurityTokenRequestSampler.ORG_NAME)));
		assertion.getAttributeStatement().add(stmt);

		assertion.setConditions(new Conditions());

		Calendar notBefore = Calendar.getInstance();
		notBefore.add(Calendar.MINUTE, RANDOM.nextInt(10) - 15);

		Calendar notOnOrAfter = Calendar.getInstance();
		notOnOrAfter.add(Calendar.HOUR, RANDOM.nextInt(5) + 5);

		notBefore.add(Calendar.MILLISECOND, notBefore.getTimeZone().getOffset(notBefore.getTimeInMillis()));
		notOnOrAfter.add(Calendar.MILLISECOND, notOnOrAfter.getTimeZone().getOffset(notOnOrAfter.getTimeInMillis()));

		assertion.getConditions().setNotBefore(datatypeFactory.newXMLGregorianCalendar((GregorianCalendar) notBefore));
		assertion.getConditions().setNotOnOrAfter(datatypeFactory.newXMLGregorianCalendar((GregorianCalendar) notOnOrAfter));
		return assertion;
	}

	private Attribute makeAttribute(String name, String format, String value) {
		Attribute attribute = new Attribute();
		attribute.setName(name);
		if (format != null) {
			attribute.setNameFormat(format);
		}
		attribute.setAttributeValue(value);
		return attribute;
	}

	protected String marshall(Envelope envelope) throws JAXBException {
		StringWriter writer = new StringWriter();
		Marshaller m = jaxbContext.createMarshaller();
		m.marshal(envelope, writer);
		String request = writer.toString();
		return request;
	}

}
