/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.gw.sampler;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Arrays;
import java.util.Collection;

import javax.xml.bind.JAXBException;

import org.apache.jmeter.protocol.http.sampler.HTTPSampleResult;
import org.w3._2005._08.addressing.AttributedURIType;
import org.w3._2005._08.addressing.ObjectFactory;
import org.xmlsoap.schemas.soap.envelope.Body;
import org.xmlsoap.schemas.soap.envelope.Envelope;


/**
 *
 * @author Jacob Qvortrup
 */
public class GatewayProxyRequestSampler extends AbstractGWSoapSampler {

	private static final long serialVersionUID = 1L;

	public GatewayProxyRequestSampler() {
		setPath("/sosigw/proxy/soap-request");
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected HTTPSampleResult sample(URL url, String method, boolean areFollowingRedirect, int frameDepth) {
		setXmlData(createRequest(url));
		HTTPSampleResult sample = super.sample(url, method, areFollowingRedirect, frameDepth);
		return sample;
	}

	private String createRequest(URL gatewayUrl) {
		try {
			Envelope envelope = new Envelope();
			envelope.setHeader(makeIdCardInSecurityHeader());
			envelope.getHeader().getAny().addAll(makeWSAddressingHeaders(gatewayUrl));
			envelope.setBody(new Body());

			return marshall(envelope);
		} catch (JAXBException e) {
			throw new RuntimeException(e);
		} catch (MalformedURLException e) {
			throw new RuntimeException(e);
		}
	}

	@SuppressWarnings("unchecked")
	private Collection<? extends Object> makeWSAddressingHeaders(URL gatewayUrl) throws MalformedURLException {
		URL echoUrl = new URL(gatewayUrl.getProtocol(), gatewayUrl.getHost(), gatewayUrl.getPort(), "/echo");
		ObjectFactory objectFactory = new ObjectFactory();
		AttributedURIType to = new AttributedURIType();
		to.setValue(echoUrl.toExternalForm());
		AttributedURIType action = new AttributedURIType();
		action.setValue("Echo");
		return Arrays.asList(objectFactory.createTo(to), objectFactory.createAction(action));
	}
}
