/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.gw.sampler;

import java.net.URL;
import java.util.UUID;

import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.namespace.QName;

import org.apache.jmeter.protocol.http.sampler.HTTPSampleResult;
import org.xmlsoap.schemas.soap.envelope.Body;
import org.xmlsoap.schemas.soap.envelope.Envelope;

/**
 * 
 * @author Jacob Qvortrup
 */
public class GatewayRequestIdCardDigestForSigningRequestSampler extends AbstractGWSoapSampler {

	private static final long serialVersionUID = 1L;


	public GatewayRequestIdCardDigestForSigningRequestSampler() {
		setPath("/sosigw/service/sosigw");
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected HTTPSampleResult sample(URL url, String method, boolean areFollowingRedirect, int frameDepth) {
		putNameId("sosigwperformance-" + UUID.randomUUID());
		clearRequestIdCardDigestForSigningResponse();
		setXmlData(createRequest());
		HTTPSampleResult sample = super.sample(url, method, areFollowingRedirect, frameDepth);
		putRequestIdCardDigestForSigningResponse(sample);
		return sample;
	}

	private String createRequest() {
		try {
			Envelope envelope = new Envelope();
			envelope.setHeader(makeIdCardInSecurityHeader());
			envelope.setBody(makeRequestIdCardDigestForSigningRequestBody());
			
			return marshall(envelope);
		} catch (JAXBException e) {
			throw new RuntimeException(e);
		}
	}

	private Body makeRequestIdCardDigestForSigningRequestBody() {
		QName qName = new QName("http://sosi.dk/gw/2007.09.01", "requestIdCardDigestForSigningRequestBody");
		JAXBElement<String> requestIdCardDigestForSigningRequestBody = new JAXBElement<String>(qName, String.class, null, "");
		Body body = new Body();
		body.getAny().add(requestIdCardDigestForSigningRequestBody);
		return body;
	}

}
