import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.Arrays;
import java.util.stream.Collectors;

import javax.activation.DataHandler;
import javax.xml.namespace.QName;

import org.apache.cxf.binding.soap.SoapMessage;
import org.apache.cxf.binding.soap.interceptor.AbstractSoapInterceptor;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import org.apache.cxf.interceptor.Fault;
import org.apache.cxf.phase.Phase;
import org.openehealth.ipf.commons.ihe.ws.WsTransactionConfiguration;
import org.openehealth.ipf.commons.ihe.xds.core.XdsClientFactory;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.EbXMLAdhocQueryRequest;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.EbXMLQueryResponse;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.EbXMLRetrieveDocumentSetRequest;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.EbXMLRetrieveDocumentSetResponse;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.ebxml30.EbXMLFactory30;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.ebxml30.EbXMLQueryResponse30;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.ebxml30.EbXMLRetrieveDocumentSetResponse30;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.ebxml30.RetrieveDocumentSetRequestType;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.ebxml30.RetrieveDocumentSetResponseType;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.AssigningAuthority;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.AvailabilityStatus;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.DocumentEntry;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.DocumentEntryType;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.Identifiable;
import org.openehealth.ipf.commons.ihe.xds.core.requests.QueryRegistry;
import org.openehealth.ipf.commons.ihe.xds.core.requests.RetrieveDocument;
import org.openehealth.ipf.commons.ihe.xds.core.requests.RetrieveDocumentSet;
import org.openehealth.ipf.commons.ihe.xds.core.requests.query.FindDocumentsQuery;
import org.openehealth.ipf.commons.ihe.xds.core.requests.query.QueryReturnType;
import org.openehealth.ipf.commons.ihe.xds.core.responses.ErrorInfo;
import org.openehealth.ipf.commons.ihe.xds.core.responses.QueryResponse;
import org.openehealth.ipf.commons.ihe.xds.core.responses.RetrievedDocument;
import org.openehealth.ipf.commons.ihe.xds.core.responses.RetrievedDocumentSet;
import org.openehealth.ipf.commons.ihe.xds.core.responses.Status;
import org.openehealth.ipf.commons.ihe.xds.core.stub.ebrs30.query.AdhocQueryRequest;
import org.openehealth.ipf.commons.ihe.xds.core.stub.ebrs30.query.AdhocQueryResponse;
import org.openehealth.ipf.commons.ihe.xds.core.transform.requests.QueryRegistryTransformer;
import org.openehealth.ipf.commons.ihe.xds.core.transform.requests.RetrieveDocumentSetRequestTransformer;
import org.openehealth.ipf.commons.ihe.xds.core.transform.responses.QueryResponseTransformer;
import org.openehealth.ipf.commons.ihe.xds.core.transform.responses.RetrieveDocumentSetResponseTransformer;
import org.openehealth.ipf.commons.ihe.xds.iti18.Iti18PortType;
import org.openehealth.ipf.commons.ihe.xds.iti43.Iti43PortType;

public class Test {

	public static void main(String[] args) throws Exception {
		// "http://test1-cnsp.ekstern-test.nspop.dk:8080/registry/services/xds-iti18";

		Iti18PortType iti18Client = createRegistryClient("wsdl/iti18.wsdl", "http://test1-cnsp.ekstern-test.nspop.dk:8080/registry/services/xds-iti18", Iti18PortType.class);
		Iti43PortType iti43Client = createRepositoryClient("wsdl/iti43.wsdl", "http://test1-cnsp.ekstern-test.nspop.dk:8080/repository/services/xds-iti43", Iti43PortType.class);

		QueryResponse queryResponse = unmarshall(iti18Client.documentRegistryRegistryStoredQuery(marshall(buildQueryRegistry("2708599967"))));

		if (queryResponse.getStatus().equals(Status.SUCCESS)) {
			System.out.println(queryResponse.getStatus().getOpcode30());
		} else if (queryResponse.getStatus().equals(Status.PARTIAL_SUCCESS)) {
			System.err.println(queryResponse.getStatus().getOpcode30());
			for (ErrorInfo error : queryResponse.getErrors()) {
				System.err.println(error);
			}
		} else if (queryResponse.getStatus().equals(Status.FAILURE)) {
			System.err.println(queryResponse.getStatus().getOpcode30());
			for (ErrorInfo error : queryResponse.getErrors()) {
				System.err.println(error);
			}
		}
		System.out.println();

		for (DocumentEntry documentEntry : queryResponse.getDocumentEntries()) {
			System.out.println(documentEntry.getTitle().getValue() + ": " + documentEntry.getUniqueId());
			RetrievedDocumentSet retrievedDocumentSet = unmarshall(iti43Client.documentRepositoryRetrieveDocumentSet(marshall(buildRetrieveDocumentSet(documentEntry.getRepositoryUniqueId(), documentEntry.getUniqueId()))));

			if (retrievedDocumentSet.getStatus().equals(Status.SUCCESS)) {
				System.out.println(retrievedDocumentSet.getStatus().getOpcode30());
			} else if (retrievedDocumentSet.getStatus().equals(Status.PARTIAL_SUCCESS)) {
				System.err.println(retrievedDocumentSet.getStatus().getOpcode30());
				for (ErrorInfo error : retrievedDocumentSet.getErrors()) {
					System.err.println(error);
				}
			} else if (retrievedDocumentSet.getStatus().equals(Status.FAILURE)) {
				System.err.println(retrievedDocumentSet.getStatus().getOpcode30());
				for (ErrorInfo error : retrievedDocumentSet.getErrors()) {
					System.err.println(error);
				}
			}

			for (RetrievedDocument retrievedDocument : retrievedDocumentSet.getDocuments()) {
				DataHandler data = retrievedDocument.getDataHandler();
				System.out.println(data.getContentType());
				String documentString = new BufferedReader(new InputStreamReader(data.getInputStream())).lines().collect(Collectors.joining());
				System.out.println(documentString.length());
			}
			System.out.println();
		}
	}

	@SuppressWarnings("unchecked")
	private static <T> T createRegistryClient(String wsdl, String url, Class<T> clazz) {
		QName serviceName = new QName("urn:ihe:iti:xds-b:2007", "DocumentRegistry_Service", "ihe");
		QName bindingName = new QName("urn:ihe:iti:xds-b:2007", "DocumentRegistry_Binding_Soap12", "ihe");
		WsTransactionConfiguration configuration = new WsTransactionConfiguration(serviceName, clazz, bindingName, false, wsdl, true, false, false, false);
		Object clientObject = new XdsClientFactory(configuration, url, null, null, null).getClient();
		Client client = ClientProxy.getClient(clientObject);
		client.getOutInterceptors().add(new Interceptor());
		return (T) clientObject;
	}

	@SuppressWarnings("unchecked")
	private static <T> T createRepositoryClient(String wsdl, String url, Class<T> clazz) {
		QName serviceName = new QName("urn:ihe:iti:xds-b:2007", "DocumentRepository_Service", "ihe");
		QName bindingName = new QName("urn:ihe:iti:xds-b:2007", "DocumentRepository_Binding_Soap12", "ihe");
		WsTransactionConfiguration configuration = new WsTransactionConfiguration(serviceName, clazz, bindingName, true, wsdl, true, false, false, false);
		Object clientObject = new XdsClientFactory(configuration, url, null, null, null).getClient();
		Client client = ClientProxy.getClient(clientObject);
		client.getOutInterceptors().add(new Interceptor());
		return (T) clientObject;
	}

	private static QueryRegistry buildQueryRegistry(String citizenId) {
		FindDocumentsQuery findDocumentsQuery = new FindDocumentsQuery();
		// Patient ID
		findDocumentsQuery.setPatientId(new Identifiable(citizenId, new AssigningAuthority("1.2.208.176.1.2")));

		// Availability Status
		findDocumentsQuery.setStatus(Arrays.asList(AvailabilityStatus.APPROVED));

		// Søg efter både stable og on-demand
		findDocumentsQuery.setDocumentEntryTypes(Arrays.asList(DocumentEntryType.ON_DEMAND, DocumentEntryType.STABLE));

		// Fjern måske?
		// findDocumentsQuery.setTypeCodes(Arrays.asList(new Code("39289-4", new LocalizedString("Dato og tidspunkt for møde
		// mellem patient og sundhedsperson"), "2.16.840.1.113883.6.1")));

		QueryRegistry queryRegistry = new QueryRegistry(findDocumentsQuery);
		queryRegistry.setReturnType(QueryReturnType.LEAF_CLASS);

		return queryRegistry;
	}

	private static RetrieveDocumentSet buildRetrieveDocumentSet(String repositoryUniqueId, String... documentUniqueIds) {
		RetrieveDocumentSet retrieveDocumentSet = new RetrieveDocumentSet();
		for (String documentUniqueId : documentUniqueIds) {
			RetrieveDocument retrieveDocument = new RetrieveDocument();
			retrieveDocument.setRepositoryUniqueId(repositoryUniqueId);
			retrieveDocument.setDocumentUniqueId(documentUniqueId);
			retrieveDocumentSet.getDocuments().add(retrieveDocument);
		}
		return retrieveDocumentSet;
	}

	private static AdhocQueryRequest marshall(QueryRegistry queryRegistry) {
		QueryRegistryTransformer transformer = new QueryRegistryTransformer();
		EbXMLAdhocQueryRequest ebxmlRequest = transformer.toEbXML(queryRegistry);
		AdhocQueryRequest adhocQueryRequest = (AdhocQueryRequest) ebxmlRequest.getInternal();
		return adhocQueryRequest;
	}

	private static QueryResponse unmarshall(AdhocQueryResponse adhocQueryResponse) {
		QueryResponseTransformer transformer = new QueryResponseTransformer(new EbXMLFactory30());
		EbXMLQueryResponse ebXmlresponse = new EbXMLQueryResponse30(adhocQueryResponse);
		QueryResponse queryResponse = transformer.fromEbXML(ebXmlresponse);
		return queryResponse;
	}

	private static RetrieveDocumentSetRequestType marshall(RetrieveDocumentSet retrieveDocumentSet) {
		RetrieveDocumentSetRequestTransformer transformer = new RetrieveDocumentSetRequestTransformer(new EbXMLFactory30());
		EbXMLRetrieveDocumentSetRequest ebXMLRequest = transformer.toEbXML(retrieveDocumentSet);
		RetrieveDocumentSetRequestType retrieveDocumentSetRequestType = (RetrieveDocumentSetRequestType) ebXMLRequest.getInternal();
		return retrieveDocumentSetRequestType;
	}

	private static RetrievedDocumentSet unmarshall(RetrieveDocumentSetResponseType retrieveDocumentSetResponseType) {
		RetrieveDocumentSetResponseTransformer transformer = new RetrieveDocumentSetResponseTransformer(new EbXMLFactory30());
		EbXMLRetrieveDocumentSetResponse ebXmlResponse = new EbXMLRetrieveDocumentSetResponse30(retrieveDocumentSetResponseType);
		RetrievedDocumentSet retrievedDocumentSet = transformer.fromEbXML(ebXmlResponse);
		return retrievedDocumentSet;
	}

	private static class Interceptor extends AbstractSoapInterceptor {

		public Interceptor() {
			super(Phase.PRE_STREAM);
		}

		@Override
		public void handleMessage(SoapMessage message) throws Fault {
			// TODO Auto-generated method stub

		}

	}

}
