package com.arosii.jmeter.protocol.ihe.assertion;

import static org.openehealth.ipf.commons.ihe.xds.core.responses.Status.SUCCESS;

import org.apache.jmeter.assertions.AssertionResult;
import org.apache.jmeter.samplers.SampleResult;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.DocumentEntry;
import org.openehealth.ipf.commons.ihe.xds.core.responses.QueryResponse;

import com.arosii.jmeter.protocol.ihe.sampler.ITI18SampleResult;

public class ITI18Assertion extends AbstractIHEAssertion {

	public static final String MIN_NUMBER_OF_DOCUMENTS = "ITI18Assertion.min_number_of_documents";

	public static final String MAX_NUMBER_OF_DOCUMENTS = "ITI18Assertion.max_number_of_documents";

	@Override
	public AssertionResult getResult(SampleResult sample) {
		// no error as default
		AssertionResult assertion = new AssertionResult(getName());

		if (sample.getResponseData().length == 0) {
			return assertion.setResultForNull();
		}

		assertion.setFailure(false);

		if (sample.isSuccessful()) {
			if (sample instanceof ITI18SampleResult) {
				ITI18SampleResult iti18SampleResult = (ITI18SampleResult) sample;
				QueryResponse queryResponse = iti18SampleResult.getQueryResponse();
				if (queryResponse != null) {
					if (queryResponse.getStatus().equals(SUCCESS)) {
						int min = Integer.valueOf(getPropertyAsString(MIN_NUMBER_OF_DOCUMENTS, String.valueOf(0)));
						int max = Integer.valueOf(getPropertyAsString(MAX_NUMBER_OF_DOCUMENTS, String.valueOf(Integer.MAX_VALUE)));
						int size = queryResponse.getDocumentEntries().size();
						if (size < min) {
							assertion.setResultForFailure("Less than " + min + " document entries found: " + size);
						}
						if (size > max) {
							assertion.setResultForFailure("More than " + max + " document entries found: " + size);
						}
					} else {
						assertion.setResultForFailure("IHE call did not return SUCCESS");
					}
				} else {
					assertion.setResultForFailure("Sampler did not produce a QueryResponse");
				}
			} else {
				assertion.setResultForFailure("Sampler not is not ITI18");
			}
		} else {
			assertion.setResultForFailure("Sample not successful");
		}

		if (assertion.isFailure()) {
			/*
			 * Af grunde jeg ikke kan forstå så insistere JMeter på at køre en Extractor inden eventuelle Assertions. Det betyder at
			 * vi får trukket DocumentEntries ud af svaret, også selv om dette fejler i den efterfølgende assertion. Derfor så må vi
			 * lige cleare dem her.
			 */
			getThreadContext().getVariables().remove(DocumentEntry.class.getName());
		}

		return assertion;
	}

}
