package com.arosii.jmeter.protocol.ihe.assertion.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.FlowLayout;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;

import com.arosii.jmeter.protocol.ihe.assertion.ITI18Assertion;
import com.arosii.jmeter.resources.ihe.Messages;

public class ITI18AssertionGui extends AbstractIHEAssertionGui {

	private JTextField minNumberOfDocuments;

	private JTextField maxNumberOfDocuments;

	@Override
	public String getLabelResource() {
		return "ihe_iti18_assertion";
	}

	@Override
	public TestElement createTestElement() {
		ITI18Assertion assertion = new ITI18Assertion();
		modifyTestElement(assertion);
		return assertion;
	}

	@Override
	protected Component createConfigurationPanel() {
		final VerticalPanel configurationPanel = new VerticalPanel();
		appendNumberOfDocumentsPanel(configurationPanel);
		return configurationPanel;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void configure(TestElement element) {
		super.configure(element);
		minNumberOfDocuments.setText(element.getPropertyAsString(ITI18Assertion.MIN_NUMBER_OF_DOCUMENTS, ""));
		maxNumberOfDocuments.setText(element.getPropertyAsString(ITI18Assertion.MAX_NUMBER_OF_DOCUMENTS, ""));
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void modifyTestElement(TestElement element) {
		super.modifyTestElement(element);
		element.setProperty(ITI18Assertion.MIN_NUMBER_OF_DOCUMENTS, minNumberOfDocuments.getText());
		element.setProperty(ITI18Assertion.MAX_NUMBER_OF_DOCUMENTS, maxNumberOfDocuments.getText());
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void clearGui() {

	}

	private void appendNumberOfDocumentsPanel(VerticalPanel configurationPanel) {
		JPanel failurePanel = new VerticalPanel();
		failurePanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("ihe_iti18_assertion_number_of_documents")));
		failurePanel.add(getMinNumberOfDocumentsPanel());
		failurePanel.add(getMaxNumberOfDocumentsPanel());
		configurationPanel.add(failurePanel);
	}

	private JPanel getMinNumberOfDocumentsPanel() {
		minNumberOfDocuments = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("ihe_iti18_assertion_min_number_of_documents"));

		JPanel minNumberOfDocumentsPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		minNumberOfDocumentsPanel.add(minNumberOfDocuments);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(minNumberOfDocumentsPanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getMaxNumberOfDocumentsPanel() {
		maxNumberOfDocuments = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("ihe_iti18_assertion_max_number_of_documents"));

		JPanel maxNumberOfDocumentsPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		maxNumberOfDocumentsPanel.add(maxNumberOfDocuments);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(maxNumberOfDocumentsPanel, BorderLayout.CENTER);

		return panel;
	}
}
