package com.arosii.jmeter.protocol.ihe.control.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.FlowLayout;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.apache.jmeter.config.Arguments;
import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;
import org.apache.jmeter.testelement.property.TestElementProperty;

import com.arosii.jmeter.protocol.ihe.sampler.ITI18Sampler;
import com.arosii.jmeter.resources.ihe.Messages;

public class ITI18SamplerGui extends AbstractIHESamplerGui {

	private JCheckBox useCprFile;

	private JTextField cprFilename;

	private JCheckBox stable;

	private JCheckBox onDemand;

	private JTextField serviceStartTimeFrom;

	private JTextField serviceStartTimeTo;

	private ClassificationsPanel typeCodes;

	private ClassificationsPanel formatCodes;

	@Override
	public String getLabelResource() {
		return "ihe_iti18_request";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public TestElement createTestElement() {
		ITI18Sampler sampler = new ITI18Sampler();
		modifyTestElement(sampler);
		return sampler;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected Component createConfigurationPanel() {
		final VerticalPanel configurationPanel = new VerticalPanel();
		appendMessageHeadersPanel(configurationPanel);
		appendDocumentSearchParametersPanel(configurationPanel);
		appendClassificationsPanel(configurationPanel);
		return configurationPanel;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void configure(TestElement element) {
		super.configure(element);
		useCprFile.setSelected(element.getPropertyAsBoolean(ITI18Sampler.USE_CITIZEN_CPR_NUMBER_FILE, false));
		cprFilename.setText(element.getPropertyAsString(ITI18Sampler.CITIZEN_CPR_NUMBER_FILE_NAME, ""));
		stable.setSelected(element.getPropertyAsBoolean(ITI18Sampler.STABLE, true));
		onDemand.setSelected(element.getPropertyAsBoolean(ITI18Sampler.ON_DEMAND, false));
		serviceStartTimeFrom.setText(element.getPropertyAsString(ITI18Sampler.SERVICE_START_TIME_FROM, ""));
		serviceStartTimeTo.setText(element.getPropertyAsString(ITI18Sampler.SERVICE_START_TIME_TO, ""));
		Arguments typeCodeArguments = (Arguments) element.getProperty(ITI18Sampler.TYPE_CODES).getObjectValue();
		if (typeCodeArguments != null) {
			typeCodes.configure(typeCodeArguments);
		} else {
			typeCodes.clear();
		}
		Arguments formatCodeArguments = (Arguments) element.getProperty(ITI18Sampler.FORMAT_CODES).getObjectValue();
		if (formatCodeArguments != null) {
			formatCodes.configure(formatCodeArguments);
		} else {
			formatCodes.clear();
		}
		updateFields();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void modifyTestElement(TestElement element) {
		super.modifyTestElement(element);
		element.setProperty(ITI18Sampler.USE_CITIZEN_CPR_NUMBER_FILE, useCprFile.isSelected());
		element.setProperty(ITI18Sampler.CITIZEN_CPR_NUMBER_FILE_NAME, cprFilename.getText());
		element.setProperty(ITI18Sampler.STABLE, stable.isSelected());
		element.setProperty(ITI18Sampler.ON_DEMAND, onDemand.isSelected());
		element.setProperty(ITI18Sampler.SERVICE_START_TIME_FROM, getLocalDateTime(serviceStartTimeFrom));
		element.setProperty(ITI18Sampler.SERVICE_START_TIME_TO, getLocalDateTime(serviceStartTimeTo));
		element.setProperty(new TestElementProperty(ITI18Sampler.TYPE_CODES, (Arguments) typeCodes.createTestElement()));
		element.setProperty(new TestElementProperty(ITI18Sampler.FORMAT_CODES, (Arguments) formatCodes.createTestElement()));
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void clearGui() {
		super.clearGui();
		useCprFile.setSelected(false);
		cprFilename.setText("");
		typeCodes.clearGui();
		formatCodes.clearGui();
		stable.setSelected(true);
		onDemand.setSelected(false);
		updateFields();
	}

	private void appendDocumentSearchParametersPanel(VerticalPanel configurationPanel) {
		JPanel panel = new VerticalPanel();
		panel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("ihe_iti18_document_search_parameters")));
		panel.add(getStablePanel());
		panel.add(getOnDemandPanel());
		panel.add(getServiceStartTimeFromPanel());
		panel.add(getServiceStartTimeToPanel());
		configurationPanel.add(panel);
	}

	private void appendClassificationsPanel(VerticalPanel configurationPanel) {
		JPanel panel = new VerticalPanel();
		panel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("ihe_iti18_classifications_panel")));
		panel.add(getTypeCodePanel());
		panel.add(getFormatCodePanel());
		configurationPanel.add(panel);
	}

	@Override
	protected JPanel getCPRPanel() {
		cpr = new JTextField(20);
		useCprFile = new JCheckBox();
		cprFilename = new JTextField(40);
		useCprFile.addActionListener(e -> cpr.setEnabled(!useCprFile.isSelected()));
		useCprFile.addActionListener(e -> cprFilename.setEnabled(useCprFile.isSelected()));
		JLabel cprLabel = new JLabel(Messages.getResString("ihe_headers_cpr"));
		JLabel useCprFileLabel = new JLabel(Messages.getResString("ihe_headers_cpr_file"));

		JPanel cprPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		cprPanel.add(cpr);
		cprPanel.add(useCprFileLabel);
		cprPanel.add(useCprFile);
		cprPanel.add(cprFilename);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(cprLabel, BorderLayout.WEST);
		panel.add(cprPanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getStablePanel() {
		stable = new JCheckBox();
		JLabel label = new JLabel(Messages.getResString("ihe_iti18_document_search_parameters_stable"));

		JPanel stablePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		stablePanel.add(stable);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(stablePanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getOnDemandPanel() {
		onDemand = new JCheckBox();
		JLabel label = new JLabel(Messages.getResString("ihe_iti18_document_search_parameters_on_demand"));

		JPanel onDemandPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		onDemandPanel.add(onDemand);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(onDemandPanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getServiceStartTimeFromPanel() {
		serviceStartTimeFrom = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("ihe_iti18_document_search_parameters_service_start_time_from"));

		JPanel serviceStartTimeFromPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		serviceStartTimeFromPanel.add(serviceStartTimeFrom);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(serviceStartTimeFromPanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getServiceStartTimeToPanel() {
		serviceStartTimeTo = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("ihe_iti18_document_search_parameters_service_start_time_to"));

		JPanel serviceStartTimeToPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		serviceStartTimeToPanel.add(serviceStartTimeTo);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(serviceStartTimeToPanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getTypeCodePanel() {
		typeCodes = new ClassificationsPanel(Messages.getResString("ihe_iti18_classifications_type_codes"));

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(typeCodes, BorderLayout.CENTER);
		return panel;
	}

	private JPanel getFormatCodePanel() {
		formatCodes = new ClassificationsPanel(Messages.getResString("ihe_iti18_classifications_format_codes"));

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(formatCodes, BorderLayout.CENTER);
		return panel;
	}

	private void updateFields() {

	}

}
