package com.arosii.jmeter.protocol.ihe.control.gui;

import static com.arosii.jmeter.protocol.ihe.sampler.ITI41Sampler.CLASSIFICATION_CODES_;
import static com.arosii.jmeter.protocol.ihe.sampler.ITI41Sampler.CLASS_CODE;
import static com.arosii.jmeter.protocol.ihe.sampler.ITI41Sampler.CONFIDENTIALITY_CODE;
import static com.arosii.jmeter.protocol.ihe.sampler.ITI41Sampler.FILE;
import static com.arosii.jmeter.protocol.ihe.sampler.ITI41Sampler.FORMAT_CODE;
import static com.arosii.jmeter.protocol.ihe.sampler.ITI41Sampler.HEALTHCARE_FACILITY_TYPE_CODE;
import static com.arosii.jmeter.protocol.ihe.sampler.ITI41Sampler.PRACTICE_SETTING_CODE;
import static com.arosii.jmeter.protocol.ihe.sampler.ITI41Sampler.SERVICE_START_TIME;
import static com.arosii.jmeter.protocol.ihe.sampler.ITI41Sampler.SERVICE_STOP_TIME;
import static com.arosii.jmeter.protocol.ihe.sampler.ITI41Sampler.TITLE;
import static com.arosii.jmeter.protocol.ihe.sampler.ITI41Sampler.TYPE_CODE;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.FlowLayout;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;
import org.apache.jmeter.testelement.property.JMeterProperty;
import org.apache.jmeter.testelement.property.TestElementProperty;

import com.arosii.jmeter.protocol.ihe.sampler.ITI41Sampler;
import com.arosii.jmeter.resources.ihe.Messages;

public class ITI41SamplerGUI extends AbstractIHESamplerGui {

	protected static final List<String> codes = Arrays.asList(CLASS_CODE, TYPE_CODE, FORMAT_CODE, HEALTHCARE_FACILITY_TYPE_CODE, PRACTICE_SETTING_CODE, CONFIDENTIALITY_CODE);

	protected JTextField title;

	protected JTextField file;

	protected JTextField serviceStartTime;

	protected JTextField serviceStopTime;

	protected Map<String, JTextField[]> codeTextFields;

	@Override
	public String getLabelResource() {
		return "ihe_iti41_request";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public TestElement createTestElement() {
		ITI41Sampler sampler = new ITI41Sampler();
		modifyTestElement(sampler);
		return sampler;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void configure(TestElement element) {
		super.configure(element);
		title.setText(element.getPropertyAsString(TITLE, ""));
		file.setText(element.getPropertyAsString(FILE, ""));
		serviceStartTime.setText(element.getPropertyAsString(SERVICE_START_TIME, ""));
		serviceStopTime.setText(element.getPropertyAsString(SERVICE_STOP_TIME, ""));
		for (String code : codes) {
			JMeterProperty property = element.getProperty(CLASSIFICATION_CODES_ + code);
			if (property instanceof TestElementProperty) {
				TestElementProperty testElementProperty = (TestElementProperty) property;
				TestElement testElement = testElementProperty.getElement();
				if (testElement instanceof ClassificationCode) {
					ClassificationCode classificationCode = (ClassificationCode) testElement;
					codeTextFields.get(code)[0].setText(classificationCode.getValue());
					codeTextFields.get(code)[1].setText(classificationCode.getScheme());
					codeTextFields.get(code)[2].setText(classificationCode.getDisplayName());
				}
			}
		}
		updateFields();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void modifyTestElement(TestElement element) {
		super.modifyTestElement(element);
		element.setProperty(TITLE, title.getText());
		element.setProperty(FILE, file.getText());
		element.setProperty(SERVICE_START_TIME, getLocalDateTime(serviceStartTime));
		element.setProperty(SERVICE_STOP_TIME, getLocalDateTime(serviceStopTime));
		for (String code : codes) {
			ClassificationCode classificationCode = new ClassificationCode(codeTextFields.get(code)[0].getText(), codeTextFields.get(code)[1].getText(), codeTextFields.get(code)[2].getText());
			element.setProperty(new TestElementProperty(CLASSIFICATION_CODES_ + code, classificationCode));
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected Component createConfigurationPanel() {
		final VerticalPanel configurationPanel = new VerticalPanel();
		appendMessageHeadersPanel(configurationPanel);
		appendFilePanel(configurationPanel);
		appendServiceTimePanel(configurationPanel);
		appendCodesPanel(configurationPanel);
		return configurationPanel;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void clearGui() {
		super.clearGui();
		for (String code : codes) {
			codeTextFields.get(code)[0].setText("");
			codeTextFields.get(code)[1].setText("");
			codeTextFields.get(code)[2].setText("");
		}
		updateFields();
	}

	protected void appendFilePanel(VerticalPanel configurationPanel) {
		JPanel panel = new VerticalPanel();
		panel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("ihe_iti41_file_panel")));
		panel.add(getTitlePanel());
		panel.add(getFilePanel());
		configurationPanel.add(panel);
	}

	protected void appendServiceTimePanel(VerticalPanel configurationPanel) {
		JPanel panel = new VerticalPanel();
		panel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("ihe_iti41_service_time_panel")));
		panel.add(getServiceStartTimePanel());
		panel.add(getServiceStopTimePanel());
		configurationPanel.add(panel);
	}

	protected void appendCodesPanel(final VerticalPanel configurationPanel) {
		for (String code : codes) {
			appendCodePanel(configurationPanel, code);
		}
	}

	protected void appendCodePanel(VerticalPanel configurationPanel, String code) {
		JPanel panel = new VerticalPanel();
		panel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("ihe_iti41_code_panel_" + code)));
		panel.add(getCodePanel(code));
		panel.add(getSchemePanel(code));
		panel.add(getDislayNamePanel(code));
		configurationPanel.add(panel);
	}

	protected JPanel getTitlePanel() {
		title = new JTextField(40);
		JLabel label = new JLabel(Messages.getResString("ihe_iti41_title"));

		JPanel titlePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		titlePanel.add(title);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(titlePanel, BorderLayout.CENTER);

		return panel;
	}

	protected JPanel getFilePanel() {
		file = new JTextField(40);
		JLabel label = new JLabel(Messages.getResString("ihe_iti41_file"));

		JPanel filePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		filePanel.add(file);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(filePanel, BorderLayout.CENTER);

		return panel;
	}

	protected JPanel getServiceStartTimePanel() {
		serviceStartTime = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("ihe_iti41_service_start_time"));

		JPanel serviceStartTimePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		serviceStartTimePanel.add(serviceStartTime);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(serviceStartTimePanel, BorderLayout.CENTER);

		return panel;
	}

	protected JPanel getServiceStopTimePanel() {
		serviceStopTime = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("ihe_iti41_service_stop_time"));

		JPanel serviceStopTimePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		serviceStopTimePanel.add(serviceStopTime);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(serviceStopTimePanel, BorderLayout.CENTER);

		return panel;
	}

	protected JPanel getCodePanel(String name) {
		JTextField code = new JTextField(20);
		put(name, code, 0);
		JLabel label = new JLabel(Messages.getResString("ihe_iti41_code"));

		JPanel codePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		codePanel.add(code);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(codePanel, BorderLayout.CENTER);

		return panel;
	}

	protected JPanel getSchemePanel(String name) {
		JTextField scheme = new JTextField(30);
		put(name, scheme, 1);
		JLabel label = new JLabel(Messages.getResString("ihe_iti41_scheme"));

		JPanel schemePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		schemePanel.add(scheme);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(schemePanel, BorderLayout.CENTER);

		return panel;
	}

	protected JPanel getDislayNamePanel(String name) {
		JTextField displayName = new JTextField(40);
		put(name, displayName, 2);
		JLabel label = new JLabel(Messages.getResString("ihe_iti41_display_name"));

		JPanel displayNamePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		displayNamePanel.add(displayName);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(displayNamePanel, BorderLayout.CENTER);

		return panel;
	}

	protected void put(String name, JTextField textField, int index) {
		if (codeTextFields == null) {
			codeTextFields = new HashMap<>();
		}
		JTextField[] textFields = codeTextFields.get(name);
		if (textFields == null) {
			textFields = new JTextField[3];
			codeTextFields.put(name, textFields);
		}
		textFields[index] = textField;
	}

	protected void updateFields() {
	}
}
