package com.arosii.jmeter.protocol.ihe.cxf;

import javax.xml.namespace.QName;

import org.apache.cxf.binding.soap.Soap11;
import org.apache.cxf.binding.soap.SoapMessage;
import org.apache.cxf.binding.soap.interceptor.AbstractSoapInterceptor;
import org.apache.cxf.headers.Header;
import org.apache.cxf.interceptor.Fault;
import org.apache.cxf.phase.Phase;
import org.apache.jmeter.threads.JMeterContext;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import dk.sosi.seal.SOSIFactory;
import dk.sosi.seal.model.IDCard;
import dk.sosi.seal.model.Request;

public class DGWSInterceptor extends AbstractSoapInterceptor {

	private final JMeterContext jMeterContext;

	public DGWSInterceptor(JMeterContext jMeterContext) {
		super(Phase.PRE_STREAM);
		this.jMeterContext = jMeterContext;
	}

	@Override
	public void handleMessage(SoapMessage message) throws Fault {
		// DGWS is SOAP11
		message.setVersion(Soap11.getInstance());

		// Add the DGWS headers
		SOSIFactory sosiFactory = (SOSIFactory) jMeterContext.getVariables().getObject(SOSIFactory.class.getName());
		IDCard idCard = (IDCard) jMeterContext.getVariables().getObject(IDCard.class.getName());
		Request request = sosiFactory.createNewRequest(false, null);
		request.setIDCard(idCard);
		Document sosi = request.serialize2DOMDocument();
		NodeList children = sosi.getDocumentElement().getFirstChild().getChildNodes();
		for (int i = 0; i < children.getLength(); i++) {
			Node element = children.item(i);
			QName qname = new QName(element.getNamespaceURI(), element.getLocalName());
			Header dgwsHeader = new Header(qname, element);
			message.getHeaders().add(dgwsHeader);
		}
	}

}
