package com.arosii.jmeter.protocol.ihe.extractor;

import java.io.Serializable;
import java.util.LinkedList;

import org.apache.jmeter.processor.PostProcessor;
import org.apache.jmeter.samplers.SampleResult;
import org.apache.jmeter.testelement.AbstractScopedTestElement;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.DocumentEntry;
import org.openehealth.ipf.commons.ihe.xds.core.responses.QueryResponse;

import com.arosii.jmeter.protocol.ihe.sampler.ITI18SampleResult;

public class ITI18DocumentEntryExtractor extends AbstractScopedTestElement implements PostProcessor, Serializable {

	public static final String ACCUMULATE = "ITI18DocumentEntryExtractor.accumulate";

	public static final String MAX_ACCUMULATED = "ITI18DocumentEntryExtractor.max_accumulated";

	@Override
	@SuppressWarnings("unchecked")
	public void process() {
		SampleResult previousResult = getThreadContext().getPreviousResult();
		if (previousResult.isSuccessful()) {
			if (previousResult != null && previousResult instanceof ITI18SampleResult) {
				ITI18SampleResult sampleResult = (ITI18SampleResult) previousResult;
				QueryResponse queryResponse = sampleResult.getQueryResponse();
				if (queryResponse != null) {
					LinkedList<DocumentEntry> documentEntries;
					if (getPropertyAsBoolean(ACCUMULATE, false)) {
						documentEntries = (LinkedList<DocumentEntry>) getThreadContext().getVariables().getObject(DocumentEntry.class.getName());
						if (documentEntries == null) {
							documentEntries = new LinkedList<DocumentEntry>();
							getThreadContext().getVariables().putObject(DocumentEntry.class.getName(), documentEntries);
						}
					} else {
						documentEntries = new LinkedList<DocumentEntry>();
						getThreadContext().getVariables().putObject(DocumentEntry.class.getName(), documentEntries);
					}
					documentEntries.addAll(queryResponse.getDocumentEntries());
					if (getPropertyAsBoolean(ACCUMULATE, false)) {
						while (documentEntries.size() > getPropertyAsInt(MAX_ACCUMULATED, 25)) {
							documentEntries.removeFirst();
						}
					}
				}
			}
		}
	}
}
