package com.arosii.jmeter.protocol.ihe.extractor.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.FlowLayout;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;

import com.arosii.jmeter.protocol.ihe.extractor.ITI18DocumentEntryExtractor;
import com.arosii.jmeter.resources.ihe.Messages;

public class ITI18DocumentEntryExtractorGUI extends AbstractIHEPostProcessorGui {
	
	JCheckBox accumulate;
	
	JTextField maxAccumulated;

	@Override
	public String getLabelResource() {
		return "ihe_iti18_document_entry_extractor";
	}

	@Override
	public TestElement createTestElement() {
		ITI18DocumentEntryExtractor extractor = new ITI18DocumentEntryExtractor();
		modifyTestElement(extractor);
		return extractor;
	}
	
	@Override
	public void configure(TestElement element) {
		super.configure(element);
		accumulate.setSelected(element.getPropertyAsBoolean(ITI18DocumentEntryExtractor.ACCUMULATE, false));
		if (accumulate.isSelected()) {
			maxAccumulated.setText(String.valueOf(element.getPropertyAsInt(ITI18DocumentEntryExtractor.MAX_ACCUMULATED, 25)));
		} else {
			maxAccumulated.setText("");
		}
		updateFields();
	}
	
	@Override
	public void modifyTestElement(TestElement element) {
		super.modifyTestElement(element);
		element.setProperty(ITI18DocumentEntryExtractor.ACCUMULATE, accumulate.isSelected());
		if (accumulate.isSelected()) {
			element.setProperty(ITI18DocumentEntryExtractor.MAX_ACCUMULATED, Integer.valueOf(maxAccumulated.getText()));
		} else {
			element.removeProperty(ITI18DocumentEntryExtractor.MAX_ACCUMULATED);
		}
	}
	
	@Override
	public void clearGui() {
		super.clearGui();
		accumulate.setSelected(false);
		updateFields();
	}

	@Override
	protected Component createConfigurationPanel() {
		final VerticalPanel configurationPanel = new VerticalPanel();
		appendAccumulatePanel(configurationPanel);
		return configurationPanel;
	}

	private void appendAccumulatePanel(VerticalPanel configurationPanel) {
		JPanel cprPanel = new VerticalPanel();
		cprPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("ihe_accumulate")));
		cprPanel.add(getAccumulatePanel());
		cprPanel.add(getMaxAccumulatedPanel());
		configurationPanel.add(cprPanel);
	}

	private JPanel getAccumulatePanel() {
		accumulate = new JCheckBox();
		accumulate.addActionListener((e -> updateFields()));
		JLabel label = new JLabel(Messages.getResString("ihe_accumulate_document_entries"));

		JPanel accumulatePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		accumulatePanel.add(accumulate);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(accumulatePanel, BorderLayout.CENTER);

		return panel;
	}

	private Component getMaxAccumulatedPanel() {
		maxAccumulated = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("ihe_max_accumulated_document_entries"));

		JPanel maxAccumulatedPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		maxAccumulatedPanel.add(maxAccumulated);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(maxAccumulatedPanel, BorderLayout.CENTER);

		return panel;
	}

	private void updateFields() {
		if (accumulate.isSelected()) {
			maxAccumulated.setEnabled(true);
		} else {
			maxAccumulated.setEnabled(false);
			maxAccumulated.setText("");
		}
	}

}
