package com.arosii.jmeter.protocol.ihe.sampler;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringWriter;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.UUID;

import javax.activation.DataHandler;
import javax.xml.bind.annotation.XmlEnumValue;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.apache.jmeter.samplers.SampleResult;
import org.apache.jmeter.testelement.TestElement;
import org.apache.jmeter.testelement.property.JMeterProperty;
import org.apache.jmeter.testelement.property.TestElementProperty;
import org.joda.time.DateTime;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.EbXMLProvideAndRegisterDocumentSetRequest;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.EbXMLRegistryResponse;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.ebxml30.EbXMLFactory30;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.ebxml30.EbXMLRegistryResponse30;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.ebxml30.ProvideAndRegisterDocumentSetRequestType;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.AssigningAuthority;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.Association;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.AssociationLabel;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.AssociationType;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.Author;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.AvailabilityStatus;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.Code;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.Document;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.DocumentEntry;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.DocumentEntryType;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.Identifiable;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.LocalizedString;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.Organization;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.SubmissionSet;
import org.openehealth.ipf.commons.ihe.xds.core.requests.ProvideAndRegisterDocumentSet;
import org.openehealth.ipf.commons.ihe.xds.core.responses.Response;
import org.openehealth.ipf.commons.ihe.xds.core.stub.ebrs30.rs.RegistryResponseType;
import org.openehealth.ipf.commons.ihe.xds.core.transform.requests.ProvideAndRegisterDocumentSetTransformer;
import org.openehealth.ipf.commons.ihe.xds.core.transform.responses.ResponseTransformer;
import org.openehealth.ipf.commons.ihe.xds.iti41.Iti41PortType;

import com.arosii.jmeter.protocol.ihe.control.gui.ClassificationCode;
import com.sun.istack.ByteArrayDataSource;

import dk.nsi.hsuid._2016._08.hsuid_1_1.SubjectIdentifierType;

public class ITI41Sampler extends AbstractIHEDocumentRepositorySampler<Iti41PortType> {

	public static final String CLASSIFICATION_CODES_ = "ITI41Sampler.codes.";

	public static final String CLASS_CODE = "class_code";

	public static final String TYPE_CODE = "type_code";

	public static final String FORMAT_CODE = "format_code";

	public static final String HEALTHCARE_FACILITY_TYPE_CODE = "healthcare_facility_type_code";

	public static final String PRACTICE_SETTING_CODE = "practice_setting_code";

	public static final String CONFIDENTIALITY_CODE = "confidentiality_code";

	public static final String SERVICE_START_TIME = "ITI41Sampler.service_start_time";

	public static final String SERVICE_STOP_TIME = "ITI41Sampler.service_stop_time";

	public static final String TITLE = "ITI41Sampler.title";

	public static final String FILE = "ITI41Sampler.file";

	protected static DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();

	protected static TransformerFactory transformerFactory = TransformerFactory.newInstance();

	protected static EbXMLFactory30 ebXMLFactory30 = new EbXMLFactory30();

	protected static ProvideAndRegisterDocumentSetTransformer provideAndRegisterDocumentSetTransformer = new ProvideAndRegisterDocumentSetTransformer(ebXMLFactory30);

	protected static ResponseTransformer responseTransformer = new ResponseTransformer(ebXMLFactory30);

	@Override
	protected String getPath(boolean usingDGWS) {
		if (usingDGWS) {
			return "/drs/proxy";
		} else {
			return "/repository/services/xds-iti41";
		}
	}

	@Override
	protected SampleResult sample() throws Exception {
		ITI41SampleResult res = new ITI41SampleResult();
		res.setHTTPMethod("ITI18");
		Iti41PortType iti41Client = getClient(Iti41PortType.class);
		byte[] documentData = getDocumentData();
		ProvideAndRegisterDocumentSet provideAndRegisterDocumentSet = buildProvideAndRegisterDocumentSet(documentData);
		res.setProvideAndRegisterDocumentSet(provideAndRegisterDocumentSet, documentData);

		ProvideAndRegisterDocumentSetRequestType provideAndRegisterDocumentSetRequestType = marshall(provideAndRegisterDocumentSet);
		res.sampleStart();
		RegistryResponseType registryResponseType = iti41Client.documentRepositoryProvideAndRegisterDocumentSetB(provideAndRegisterDocumentSetRequestType);
		res.sampleEnd();
		Response response = unmarshall(registryResponseType);
		res.setResponse(response);

		updateSampleResult(res, response.getStatus());
		return res;
	}

	protected ProvideAndRegisterDocumentSet buildProvideAndRegisterDocumentSet(byte[] documentData) {
		ProvideAndRegisterDocumentSet provideAndRegisterDocumentSet = new ProvideAndRegisterDocumentSet();

		DocumentEntry documentEntry = new DocumentEntry();
		String documentEntryUUID = getDocumentEntryUUID();
		String documentEntryUniqueId = getDocumentEntryUniqueId();
		documentEntry.setEntryUuid(documentEntryUUID);
		documentEntry.setUniqueId(documentEntryUniqueId);
		documentEntry.setLogicalUuid(documentEntryUUID);

		Identifiable patientIdentifiable = new Identifiable(getCitizenCPRNumber(), getCitizenAssigningAuthority());
		documentEntry.setPatientId(patientIdentifiable);
		documentEntry.setSourcePatientId(patientIdentifiable);

		Author author = new Author();
		author.getAuthorInstitution().add(new Organization(getOrganisationName(), getOrganisationIdentifier(), getOrganisationAssigningAuthority()));
		documentEntry.setAuthor(author);

		documentEntry.setAvailabilityStatus(getAvailabilityStatus());
		documentEntry.setType(getDocumentEntryType());

		DateTime creationTime = getCreationTime();
		DateTime serviceStartTime = getServiceStartTime();
		DateTime serviceStopTime = getServiceStopTime();
		documentEntry.setCreationTime(creationTime);
		documentEntry.setServiceStartTime(serviceStartTime);
		documentEntry.setServiceStopTime(serviceStopTime);

		Code classCode = getClassCode();
		Code typeCode = getTypeCode();
		Code formatCode = getFormatCode();
		Code healthcareFacilityTypeCode = getHealthcareFacilityTypeCode();
		Code practiceSettingCode = getPracticeSettingCode();
		documentEntry.setClassCode(classCode);
		documentEntry.setTypeCode(typeCode);
		documentEntry.setFormatCode(formatCode);
		documentEntry.setHealthcareFacilityTypeCode(healthcareFacilityTypeCode);
		documentEntry.setPracticeSettingCode(practiceSettingCode);

		for (Code confidentialityCode : getConfidentialityCodes()) {
			documentEntry.getConfidentialityCodes().add(confidentialityCode);
		}
		for (Code eventCode : getEventCodes()) {
			documentEntry.getEventCodeList().add(eventCode);
		}

		String title = getTitle();
		String languageCode = getLanguageCode();
		String mimeType = getMimeType();
		documentEntry.setTitle(new LocalizedString(title));
		documentEntry.setLanguageCode(languageCode);
		documentEntry.setMimeType(mimeType);

		Document document = new Document(documentEntry, new DataHandler(new ByteArrayDataSource(documentData, mimeType)));
		provideAndRegisterDocumentSet.getDocuments().add(document);

		String submissionSetUniqueId = getSubmissionSetUniqueId();
		SubmissionSet submissionSet = new SubmissionSet();
		submissionSet.setUniqueId(submissionSetUniqueId);
		submissionSet.setSourceId(submissionSetUniqueId);
		submissionSet.setEntryUuid(submissionSetUniqueId);
		submissionSet.setPatientId(patientIdentifiable);
		submissionSet.setContentTypeCode(typeCode);
		submissionSet.setAvailabilityStatus(getAvailabilityStatus());
		submissionSet.setSubmissionTime(getCreationTime());
		provideAndRegisterDocumentSet.setSubmissionSet(submissionSet);

		String associationEntryUUID = getAssociationEntryUUID();
		Association association = new Association();
		association.setAssociationType(getAssociationType());
		association.setEntryUuid(associationEntryUUID);
		association.setSourceUuid(submissionSet.getEntryUuid());
		association.setTargetUuid(documentEntry.getEntryUuid());
		association.setAvailabilityStatus(getAvailabilityStatus());
		association.setLabel(getAssociationLabel());
		provideAndRegisterDocumentSet.getAssociations().add(association);

		return provideAndRegisterDocumentSet;
	}

	protected byte[] getDocumentData() {
		InputStream in;
		String filename = getFilename();
		File inputFile = new File(filename);
		if (inputFile.exists() && inputFile.canRead()) {
			try {
				in = new FileInputStream(inputFile);
			} catch (IOException e) {
				throw new IllegalArgumentException(e.getMessage() + ": " + filename, e);
			}
		} else {
			in = getClass().getClassLoader().getResourceAsStream(filename);
			if (in == null) {
				throw new IllegalArgumentException(filename + " not found in file system or in classpath");
			}
		}
		try {
			in = new BufferedInputStream(in);
			DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
			Transformer transformer = transformerFactory.newTransformer();
			transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
			StringWriter writer = new StringWriter();
			transformer.transform(new DOMSource(documentBuilder.parse(in)), new StreamResult(writer));
			return writer.getBuffer().toString().getBytes();
		} catch (Exception e) {
			throw new IllegalArgumentException(e.getMessage() + ": " + filename, e);
		} finally {
			try {
				in.close();
			} catch (IOException e) {
				throw new IllegalArgumentException(e.getMessage() + ": " + filename, e);
			}
		}
	}

	protected String getFilename() {
		return getPropertyAsString(FILE);
	}

	protected String getAssociationEntryUUID() {
		return generateUUID();
	}

	protected String getSubmissionSetUniqueId() {
		return generateUUID();
	}

	protected String getDocumentEntryUniqueId() {
		return generateUUID();
	}

	protected String getDocumentEntryUUID() {
		return generateUUID();
	}

	protected AssociationLabel getAssociationLabel() {
		return AssociationLabel.ORIGINAL;
	}

	protected AssociationType getAssociationType() {
		return AssociationType.HAS_MEMBER;
	}

	protected DateTime getCreationTime() {
		return DateTime.now();
	}

	/*
	 * CPR
	 */
	protected AssigningAuthority getCitizenAssigningAuthority() {
		return new AssigningAuthority("1.2.208.176.1.2");
	}

	protected DocumentEntryType getDocumentEntryType() {
		return DocumentEntryType.STABLE;
	}

	protected AvailabilityStatus getAvailabilityStatus() {
		return AvailabilityStatus.APPROVED;
	}

	/*
	 * Sættes ikke i eksempler, så ved ikke om det er rigtigt.
	 */
	protected String getMimeType() {
		return "text/xml";
	}

	/*
	 * Sættes ikke i eksempler, så ved ikke om det er rigtigt.
	 */
	protected String getLanguageCode() {
		return "da-DK";
	}

	protected String getTitle() {
		return getPropertyAsString(TITLE, "(Ingen titel)");
	}

	/*
	 * Bruges ikke pt. Se https://www.ihe.net/uploadedFiles/Documents/ITI/IHE_ITI_TF_Vol3.pdf afsnit 4.2.3.2.8 for en
	 * forklaring
	 */
	protected List<Code> getEventCodes() {
		return Collections.emptyList();
	}

	protected List<Code> getConfidentialityCodes() {
		return Arrays.asList(getClassificationCode(CONFIDENTIALITY_CODE));
	}

	protected Code getPracticeSettingCode() {
		return getClassificationCode(PRACTICE_SETTING_CODE);
	}

	protected Code getHealthcareFacilityTypeCode() {
		return getClassificationCode(HEALTHCARE_FACILITY_TYPE_CODE);
	}

	protected Code getFormatCode() {
		return getClassificationCode(FORMAT_CODE);
	}

	protected Code getTypeCode() {
		return getClassificationCode(TYPE_CODE);
	}

	protected Code getClassCode() {
		return getClassificationCode(CLASS_CODE);
	}

	protected DateTime getServiceStopTime() {
		return getDateTime(SERVICE_STOP_TIME);
	}

	protected DateTime getServiceStartTime() {
		return getDateTime(SERVICE_START_TIME);
	}

	protected AssigningAuthority getOrganisationAssigningAuthority() {
		try {
			if (SubjectIdentifierType.class.getField(SubjectIdentifierType.NSI_SORCODE.name()).getAnnotation(XmlEnumValue.class).value().equals(getOrganisationIdentifierFormat())) {
				return new AssigningAuthority("1.2.208.176.1.1"); // SOR
			} else if (SubjectIdentifierType.class.getField(SubjectIdentifierType.NSI_YNUMBER.name()).getAnnotation(XmlEnumValue.class).value().equals(getOrganisationIdentifierFormat())) {
				return new AssigningAuthority("1.2.208.176.1.4"); // Yder
			} else if (SubjectIdentifierType.class.getField(SubjectIdentifierType.NSI_SKSCODE.name()).getAnnotation(XmlEnumValue.class).value().equals(getOrganisationIdentifierFormat())) {
				return new AssigningAuthority("1.2.208.176.2.4"); // SKS
			} else if (SubjectIdentifierType.class.getField(SubjectIdentifierType.NSI_CVR.name()).getAnnotation(XmlEnumValue.class).value().equals(getOrganisationIdentifierFormat())) {
				return new AssigningAuthority("2.16.840.1.113883.2.24.1.1"); // CVR
			} else {
				throw new IllegalArgumentException(getOrganisationIdentifierFormat() + " not supported");
			}
		} catch (NoSuchFieldException e) {
			throw new IllegalStateException(e);
		} catch (SecurityException e) {
			throw new IllegalStateException(e);
		}
	}

	protected String generateUUID() {
		UUID uuid = UUID.randomUUID();
		return Math.abs(uuid.getLeastSignificantBits()) + "." + Math.abs(uuid.getMostSignificantBits()) + "." + System.currentTimeMillis();
	}

	protected Code getClassificationCode(String code) {
		JMeterProperty property = getProperty(CLASSIFICATION_CODES_ + code);
		if (property instanceof TestElementProperty) {
			TestElementProperty testElementProperty = (TestElementProperty) property;
			TestElement testElement = testElementProperty.getElement();
			if (testElement instanceof ClassificationCode) {
				ClassificationCode classificationCode = (ClassificationCode) testElement;
				String value = classificationCode.getValue();
				String scheme = classificationCode.getScheme();
				String displayName = classificationCode.getDisplayName();
				if (value != null && !value.trim().isEmpty() && scheme != null && !scheme.trim().isEmpty() && displayName != null && !displayName.trim().isEmpty()) {
					return new Code(value, new LocalizedString(displayName, getLanguageCode(), "utf-8"), scheme);
				}
			}
		}
		return null;
	}

	protected ProvideAndRegisterDocumentSetRequestType marshall(ProvideAndRegisterDocumentSet provideAndRegisterDocumentSet) {
		EbXMLProvideAndRegisterDocumentSetRequest ebXMLRequest = provideAndRegisterDocumentSetTransformer.toEbXML(provideAndRegisterDocumentSet);
		ProvideAndRegisterDocumentSetRequestType provideAndRegisterDocumentSetRequestType = (ProvideAndRegisterDocumentSetRequestType) ebXMLRequest.getInternal();
		return provideAndRegisterDocumentSetRequestType;
	}

	protected Response unmarshall(RegistryResponseType registryResponseType) {
		EbXMLRegistryResponse ebXMLResponse = new EbXMLRegistryResponse30(registryResponseType);
		Response response = responseTransformer.fromEbXML(ebXMLResponse);
		return response;
	}
}
