package com.arosii.jmeter.protocol.ihe.sampler;

import java.util.Collection;

import org.apache.jmeter.samplers.SampleResult;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.EbXMLRetrieveDocumentSetRequest;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.EbXMLRetrieveDocumentSetResponse;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.ebxml30.EbXMLFactory30;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.ebxml30.EbXMLRetrieveDocumentSetResponse30;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.ebxml30.RetrieveDocumentSetRequestType;
import org.openehealth.ipf.commons.ihe.xds.core.ebxml.ebxml30.RetrieveDocumentSetResponseType;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.DocumentEntry;
import org.openehealth.ipf.commons.ihe.xds.core.requests.RetrieveDocument;
import org.openehealth.ipf.commons.ihe.xds.core.requests.RetrieveDocumentSet;
import org.openehealth.ipf.commons.ihe.xds.core.responses.RetrievedDocumentSet;
import org.openehealth.ipf.commons.ihe.xds.core.transform.requests.RetrieveDocumentSetRequestTransformer;
import org.openehealth.ipf.commons.ihe.xds.core.transform.responses.RetrieveDocumentSetResponseTransformer;
import org.openehealth.ipf.commons.ihe.xds.iti43.Iti43PortType;

public class ITI43Sampler extends AbstractIHEDocumentRepositorySampler<Iti43PortType> {

	public static final String USE_CITIZEN_CPR_NUMBER_FROM_OTHER_IHE_SAMPLER = "ITI43Sampler.cpr_from_ihe_sampler";

	public static final String FROM_EXTRACTOR = "ITI43Sampler.from_extractor";

	public static final String REPOSITORY_UNIQUE_ID = "ITI43Sampler.repository_unique_id";

	public static final String DOCUMENT_UNIQUE_ID = "ITI43Sampler.document_unique_id";

	protected static EbXMLFactory30 ebXMLFactory30 = new EbXMLFactory30();

	protected static RetrieveDocumentSetRequestTransformer retrieveDocumentSetRequestTransformer = new RetrieveDocumentSetRequestTransformer(ebXMLFactory30);

	protected static RetrieveDocumentSetResponseTransformer retrieveDocumentSetResponseTransformer = new RetrieveDocumentSetResponseTransformer(ebXMLFactory30);

	@Override
	protected String getPath(boolean usingDGWS) {
		if (usingDGWS) {
			return "/ddsrepository/service";
		} else {
			return "/repository/services/xds-iti43";
		}
	}

	@Override
	public String getCitizenCPRNumber() {
		if (getPropertyAsBoolean(USE_CITIZEN_CPR_NUMBER_FROM_OTHER_IHE_SAMPLER, false)) {
			return getThreadContext().getVariables().get(ITI18Sampler.class.getName() + ".cpr");
		} else {
			return super.getCitizenCPRNumber();
		}
	}

	@Override
	protected SampleResult sample() throws Exception {
		ITI43SampleResult res = new ITI43SampleResult();
		res.setHTTPMethod("ITI18");
		Iti43PortType iti43Client = getClient(Iti43PortType.class);
		RetrieveDocumentSet retrieveDocumentSet = buildRetrieveDocumentSet();
		res.setRetrieveDocumentSet(retrieveDocumentSet);

		RetrieveDocumentSetRequestType retrieveDocumentSetRequestType = marshall(retrieveDocumentSet);
		res.sampleStart();
		RetrieveDocumentSetResponseType retrieveDocumentSetResponseType = iti43Client.documentRepositoryRetrieveDocumentSet(retrieveDocumentSetRequestType);
		res.sampleEnd();
		RetrievedDocumentSet retrievedDocumentSet = unmarshall(retrieveDocumentSetResponseType);
		res.setRetrievedDocumentSet(retrievedDocumentSet);

		updateSampleResult(res, retrievedDocumentSet.getStatus());
		return res;
	}

	@SuppressWarnings("unchecked")
	private RetrieveDocumentSet buildRetrieveDocumentSet() {
		RetrieveDocumentSet retrieveDocumentSet = new RetrieveDocumentSet();
		if (getPropertyAsBoolean(FROM_EXTRACTOR, false)) {
			Collection<DocumentEntry> documentEntries = (Collection<DocumentEntry>) getThreadContext().getVariables().getObject(DocumentEntry.class.getName());
			for (DocumentEntry documentEntry : documentEntries) {
				RetrieveDocument retrieveDocument = new RetrieveDocument();
				retrieveDocument.setRepositoryUniqueId(documentEntry.getRepositoryUniqueId());
				retrieveDocument.setDocumentUniqueId(documentEntry.getUniqueId());
				retrieveDocumentSet.getDocuments().add(retrieveDocument);
			}
		} else {
			RetrieveDocument retrieveDocument = new RetrieveDocument();
			retrieveDocument.setRepositoryUniqueId(getPropertyAsString(REPOSITORY_UNIQUE_ID));
			retrieveDocument.setDocumentUniqueId(getPropertyAsString(DOCUMENT_UNIQUE_ID));
			retrieveDocumentSet.getDocuments().add(retrieveDocument);
		}
		return retrieveDocumentSet;
	}

	private static RetrieveDocumentSetRequestType marshall(RetrieveDocumentSet retrieveDocumentSet) {
		EbXMLRetrieveDocumentSetRequest ebXMLRequest = retrieveDocumentSetRequestTransformer.toEbXML(retrieveDocumentSet);
		RetrieveDocumentSetRequestType retrieveDocumentSetRequestType = (RetrieveDocumentSetRequestType) ebXMLRequest.getInternal();
		return retrieveDocumentSetRequestType;
	}

	private static RetrievedDocumentSet unmarshall(RetrieveDocumentSetResponseType retrieveDocumentSetResponseType) {
		EbXMLRetrieveDocumentSetResponse ebXmlResponse = new EbXMLRetrieveDocumentSetResponse30(retrieveDocumentSetResponseType);
		RetrievedDocumentSet retrievedDocumentSet = retrieveDocumentSetResponseTransformer.fromEbXML(ebXmlResponse);
		return retrievedDocumentSet;
	}
}
