package com.arosii.jmeter.protocol.ihe.visualizers;

import static com.arosii.jmeter.protocol.ihe.visualizers.RenderAsIHE.format;

import java.awt.BorderLayout;

import javax.swing.JEditorPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import org.apache.jmeter.samplers.SampleResult;
import org.apache.jmeter.visualizers.RequestView;
import org.apache.jorphan.gui.GuiUtils;
import org.joda.time.format.DateTimeFormat;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.AvailabilityStatus;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.Code;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.Document;
import org.openehealth.ipf.commons.ihe.xds.core.metadata.DocumentEntryType;
import org.openehealth.ipf.commons.ihe.xds.core.requests.ProvideAndRegisterDocumentSet;
import org.openehealth.ipf.commons.ihe.xds.core.requests.QueryRegistry;
import org.openehealth.ipf.commons.ihe.xds.core.requests.RetrieveDocument;
import org.openehealth.ipf.commons.ihe.xds.core.requests.RetrieveDocumentSet;
import org.openehealth.ipf.commons.ihe.xds.core.requests.query.FindDocumentsQuery;
import org.openehealth.ipf.commons.ihe.xds.core.requests.query.Query;

import com.arosii.jmeter.protocol.ihe.sampler.ITI18SampleResult;
import com.arosii.jmeter.protocol.ihe.sampler.ITI41DocumentSampleResult;
import com.arosii.jmeter.protocol.ihe.sampler.ITI41SampleResult;
import com.arosii.jmeter.protocol.ihe.sampler.ITI43SampleResult;
import com.arosii.jmeter.resources.ihe.Messages;

public class RequestViewIHE implements RequestView {

	private JPanel panel;

	private JEditorPane requests;

	private JScrollPane resultsScrollPane;

	@Override
	public void init() {
		panel = new JPanel(new BorderLayout(0, 5));
		requests = new JEditorPane();
		requests.setEditable(false);
		resultsScrollPane = GuiUtils.makeScrollPane(requests);
		panel.add(resultsScrollPane, BorderLayout.CENTER);
	}

	@Override
	public void clearData() {
		requests.setText("");
	}

	@Override
	public void setSamplerResult(Object userObject) {
		if (userObject instanceof SampleResult) {
			SampleResult sampleResult = (SampleResult) userObject;
			StringBuilder iheText = new StringBuilder();
			String contentType = "text/plain";
			if (userObject instanceof ITI18SampleResult) {
				contentType = "text/html";
				iheText.append("<html><body>");
				ITI18SampleResult iti18SampleResult = (ITI18SampleResult) userObject;
				iheText.append("<h3>").append("ITI18").append("</h3>");
				QueryRegistry queryRegistry = iti18SampleResult.getQueryRegistry();
				if (queryRegistry != null) {
					Query query = queryRegistry.getQuery();
					if (query instanceof FindDocumentsQuery) {
						FindDocumentsQuery findDocumentsQuery = (FindDocumentsQuery) query;
						iheText.append("<p>").append(Messages.getResString("view_results_table_request_tab_ihe_cpr")).append(" : ").append(findDocumentsQuery.getPatientId().getId()).append("</p>");
						if (findDocumentsQuery.getStatus() != null && !findDocumentsQuery.getStatus().isEmpty()) {
							iheText.append("<p>");
							for (AvailabilityStatus availabilityStatus : findDocumentsQuery.getStatus()) {
								iheText.append(Messages.getResString("view_results_table_request_tab_ihe_availability_status")).append(" : ").append(availabilityStatus.toString()).append("<br/>");
							}
							iheText.append("</p>");
						}
						if (findDocumentsQuery.getDocumentEntryTypes() != null && !findDocumentsQuery.getDocumentEntryTypes().isEmpty()) {
							iheText.append("<p>");
							for (DocumentEntryType documentEntryType : findDocumentsQuery.getDocumentEntryTypes()) {
								iheText.append(Messages.getResString("view_results_table_request_tab_ihe_document_entry_type")).append(" : ").append(documentEntryType.toString()).append("<br/>");
							}
							iheText.append("</p>");
						}
						if (findDocumentsQuery.getServiceStartTime() != null && (findDocumentsQuery.getServiceStartTime().getFrom() != null || findDocumentsQuery.getServiceStartTime().getTo() != null)) {
							iheText.append("<p>");
							if (findDocumentsQuery.getServiceStartTime().getFrom() != null) {
								iheText.append(Messages.getResString("view_results_table_request_tab_ihe_service_start_time_from")).append(" : ").append(DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss").print(findDocumentsQuery.getServiceStartTime().getFrom())).append("<br/>");
							}
							if (findDocumentsQuery.getServiceStartTime().getTo() != null) {
								iheText.append(Messages.getResString("view_results_table_request_tab_ihe_service_start_time_to")).append(" : ").append(DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss").print(findDocumentsQuery.getServiceStartTime().getTo())).append("<br/>");
							}
							iheText.append("</p>");
						}
						if (findDocumentsQuery.getTypeCodes() != null && !findDocumentsQuery.getTypeCodes().isEmpty()) {
							iheText.append("<p>");
							for (Code code : findDocumentsQuery.getTypeCodes()) {
								iheText.append(Messages.getResString("view_results_table_request_tab_ihe_type_codes")).append(" : ").append(format(code)).append("<br/>");
							}
							iheText.append("</p>");
						}
						if (findDocumentsQuery.getFormatCodes() != null) {
							iheText.append("<p>");
							for (Code code : findDocumentsQuery.getFormatCodes()) {
								iheText.append(Messages.getResString("view_results_table_request_tab_ihe_format_codes")).append(" : ").append(format(code)).append("<br/>");
							}
							iheText.append("</p>");
						}
					}
				} else {
					iheText.append("NULL");
				}
				iheText.append("</body></html>");
			} else if (userObject instanceof ITI43SampleResult) {
				contentType = "text/html";
				iheText.append("<html><body>");
				ITI43SampleResult iti43SampleResult = (ITI43SampleResult) userObject;
				iheText.append("<h3>").append("ITI43").append("</h3>");
				RetrieveDocumentSet retrieveDocumentSet = iti43SampleResult.getRetrieveDocumentSet();
				if (retrieveDocumentSet != null && retrieveDocumentSet.getDocuments() != null && !retrieveDocumentSet.getDocuments().isEmpty()) {
					for (RetrieveDocument retrieveDocument : retrieveDocumentSet.getDocuments()) {
						iheText.append("<p>");
						iheText.append(Messages.getResString("view_results_table_request_tab_ihe_repository_unique_id")).append(" : ").append(retrieveDocument.getRepositoryUniqueId()).append("<br/>");
						iheText.append(Messages.getResString("view_results_table_request_tab_ihe_document_unique_id")).append(" : ").append(retrieveDocument.getDocumentUniqueId()).append("<br/>");
						iheText.append("</p>");
					}
				} else {
					iheText.append("NULL");
				}
				iheText.append("</body></html>");
			} else if (userObject instanceof ITI41SampleResult) {
				contentType = "text/html";
				iheText.append("<html><body>");
				ITI41SampleResult iti41SampleResult = (ITI41SampleResult) userObject;
				iheText.append("<h3>").append("ITI41").append("</h3>");
				ProvideAndRegisterDocumentSet provideAndRegisterDocumentSet = iti41SampleResult.getProvideAndRegisterDocumentSet();
				
				if (provideAndRegisterDocumentSet != null && provideAndRegisterDocumentSet.getDocuments() != null && !provideAndRegisterDocumentSet.getDocuments().isEmpty()) {
					for (Document document : provideAndRegisterDocumentSet.getDocuments()) {
						iheText.append("<p>");
						iheText.append(Messages.getResString("view_results_table_request_tab_ihe_document_title")).append(" : ").append(document.getDocumentEntry().getTitle().getValue()).append("<br/>");
						iheText.append(Messages.getResString("view_results_table_request_tab_ihe_document_unique_id")).append(" : ").append(document.getDocumentEntry().getUniqueId()).append("<br/>");
						iheText.append("</p>");
					}
				} else {
					iheText.append("NULL");
				}
				iheText.append("</body></html>");
			} else if (userObject instanceof ITI41DocumentSampleResult) {
				contentType = "text/plain";
				ITI41DocumentSampleResult iti41DocumentSampleResult = (ITI41DocumentSampleResult) sampleResult;
				iheText.append(iti41DocumentSampleResult.getDocument());
			}

			if (iheText.length() > 0) {
				showTextResponse(iheText.toString(), contentType);
			} else {
				showTextResponse(sampleResult.getSamplerData(), "text/plain");
			}
		} else {
			showTextResponse(userObject != null ? userObject.getClass().getName() : "null", "text/plain");
		}
	}

	private void showTextResponse(String response, String contentType) {
		requests.setContentType(contentType); // $NON-NLS-1$
		requests.setText(response == null ? "" : response); // $NON-NLS-1$
		requests.setCaretPosition(0);
		resultsScrollPane.setViewportView(requests);
	}

	@Override
	public JPanel getPanel() {
		return panel;
	}

	@Override
	public String getLabel() {
		return Messages.getResString("view_results_table_request_tab_ihe");
	}

}
