/**
* The MIT License
*   
* Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
*   
* Copyright (C) 2018 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
*   
*   
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*   
*   
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*   
*   
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*/ 

package dk.nsp.jmeter.protocol.lar.control.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.FlowLayout;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;

import dk.nsp.jmeter.protocol.lar.sampler.ListAllergyRequestSampler;
import dk.nsp.jmeter.resources.lar.Messages;

/**
 * 
 * @author
 */
public class ListAllergyRequestSamplerGui extends AbstractLarserviceSamplerGui {

	private static final long serialVersionUID = 1L;

	private JTextField cpr;
	
	private JTextField cprSystem;

	private JTextField cprFile;
	
	private JTextField orgCode;
	
	private JTextField orgType;

	private JCheckBox consentOverride;
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getLabelResource() {
		return "larservice_list_allergy_request";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public TestElement createTestElement() {
		ListAllergyRequestSampler sampler = new ListAllergyRequestSampler();
		modifyTestElement(sampler);
		return sampler;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected Component createConfigurationPanel() {
		final VerticalPanel configurationPanel = new VerticalPanel();
		appendCPRPanel(configurationPanel);
		appendOrganizationPanel(configurationPanel);
		appendSearchOptionsPanel(configurationPanel);
		return configurationPanel;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void configure(TestElement element) {
		super.configure(element);
		cpr.setText(element.getPropertyAsString(ListAllergyRequestSampler.CPR, ""));
		cprSystem.setText(element.getPropertyAsString(ListAllergyRequestSampler.CPR_SYSTEM, ""));
		cprFile.setText(element.getPropertyAsString(ListAllergyRequestSampler.CPR_FILE, ""));
		orgCode.setText(element.getPropertyAsString(ListAllergyRequestSampler.ORG_CODE, ""));
		orgType.setText(element.getPropertyAsString(ListAllergyRequestSampler.ORG_TYPE, ""));
		consentOverride.setSelected(element.getPropertyAsBoolean(ListAllergyRequestSampler.CONSENT_OVERRIDE, false));
		updateFields();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void modifyTestElement(TestElement element) {
		super.modifyTestElement(element);
		element.setProperty(ListAllergyRequestSampler.CPR, cpr.getText());
		element.setProperty(ListAllergyRequestSampler.CPR_SYSTEM, cprSystem.getText());
		element.setProperty(ListAllergyRequestSampler.CPR_FILE, cprFile.getText());
		element.setProperty(ListAllergyRequestSampler.ORG_CODE, orgCode.getText());
		element.setProperty(ListAllergyRequestSampler.ORG_TYPE, orgType.getText());
		element.setProperty(ListAllergyRequestSampler.CONSENT_OVERRIDE, consentOverride.isSelected());
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void clearGui() {
		super.clearGui();
		cpr.setText("");
		cprSystem.setText("");
		cprFile.setText("");
		orgCode.setText("");
		orgType.setText("");
		consentOverride.setSelected(false);
		updateFields();
	}

	private void appendCPRPanel(VerticalPanel configurationPanel) {
		JPanel cprPanel = new VerticalPanel();
		cprPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("larservice_list_allergy_request_cpr")));
		cprPanel.add(getCPRPanel());
		cprPanel.add(getCPRSystemPanel());
		cprPanel.add(getCPRFilePanel());
		configurationPanel.add(cprPanel);
	}
	
	private void appendOrganizationPanel(VerticalPanel configurationPanel) {
		JPanel organizationPanel = new VerticalPanel();
		organizationPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("larservice_list_allergy_request_requester_organization")));
		organizationPanel.add(getOrgCodePanel());
		organizationPanel.add(getOrgTypePanel());
		configurationPanel.add(organizationPanel);
	}
	
	private void appendSearchOptionsPanel(VerticalPanel configurationPanel) {
		JPanel cprPanel = new VerticalPanel();
		cprPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("larservice_list_allergy_request_search")));
		cprPanel.add(getConsentOverridePanel());
		configurationPanel.add(cprPanel);
	}

	private JPanel getCPRPanel() {
		cpr = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("larservice_cpr"));

		cpr.getDocument().addDocumentListener(new DisableCPRFileKeyListener());

		JPanel cprPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		cprPanel.add(cpr);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(cprPanel, BorderLayout.CENTER);

		return panel;
	}
	
	private JPanel getCPRSystemPanel() {
		cprSystem = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("larservice_cpr_system"));

		cprSystem.getDocument().addDocumentListener(new DisableCPRFileKeyListener());

		JPanel cprSystemPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		cprSystemPanel.add(cprSystem);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(cprSystemPanel, BorderLayout.CENTER);

		return panel;
	}


	private JPanel getCPRFilePanel() {
		cprFile = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("larservice_cpr_file"));

		cprFile.getDocument().addDocumentListener(new DisableCPRFileKeyListener());

		JPanel cprFilePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		cprFilePanel.add(cprFile);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(cprFilePanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getOrgCodePanel() {
		orgCode = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("larservice_org_code"));

		orgCode.getDocument().addDocumentListener(new DisableCPRFileKeyListener());

		JPanel orgCodePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		orgCodePanel.add(orgCode);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(orgCodePanel, BorderLayout.CENTER);

		return panel;
	}
	
	private JPanel getOrgTypePanel() {
		orgType = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("larservice_org_type"));
		JLabel labelValues = new JLabel ("shak, sor, ydernummer");

		orgType.getDocument().addDocumentListener(new DisableCPRFileKeyListener());

		JPanel orgTypePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		orgTypePanel.add(orgType);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(orgTypePanel, BorderLayout.CENTER);
		panel.add(labelValues, BorderLayout.EAST);

		return panel;
	}
	
	private JPanel getConsentOverridePanel() {
		consentOverride = new JCheckBox();
		JLabel label = new JLabel(Messages.getResString("larservice_consent_override"));

		JPanel consentOverridePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		consentOverridePanel.add(consentOverride);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(consentOverridePanel, BorderLayout.CENTER);

		return panel;
	}

	private void updateFields() {
		cprFile.setEnabled(cpr.getText().isEmpty());
		cpr.setEnabled(cprFile.getText().isEmpty());
	}

	private class DisableCPRFileKeyListener implements DocumentListener {

		@Override
		public void insertUpdate(DocumentEvent e) {
			updateFields();
		}

		@Override
		public void removeUpdate(DocumentEvent e) {
			updateFields();
		}

		@Override
		public void changedUpdate(DocumentEvent e) {
			updateFields();
		}
	}
}
