/**
* The MIT License
*   
* Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
*   
* Copyright (C) 2018 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
*   
*   
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*   
*   
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*   
*   
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*/ 

package dk.nsp.jmeter.resources.lar;

import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.apache.jmeter.util.JMeterUtils;
import org.apache.jmeter.util.LocaleChangeEvent;
import org.apache.jmeter.util.LocaleChangeListener;
import org.apache.jorphan.logging.LoggingManager;
import org.apache.log.Logger;

/**
 * 
 */
public class Messages implements LocaleChangeListener {

	private static final Logger log = LoggingManager.getLoggerForClass();

	private static boolean initialized = false;

	private static Messages messages = new Messages();

	private static ResourceBundle resources;

	private synchronized static void init() {
		if (!initialized) {
			JMeterUtils.addLocaleChangeListener(messages);
			messages.localeChanged(new LocaleChangeEvent(Messages.class, Locale.getDefault()));
			initialized = true;
		}
	}

	private Messages() {

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void localeChanged(LocaleChangeEvent event) {
		String baseName = getClass().getPackage().getName() + ".messages";
		resources = ResourceBundle.getBundle(baseName, event.getLocale());
		if (resources == null) {
			log.error("ERROR! Resource bundle '" + baseName + "'not found for locale " + event.getLocale());
		}
	}

	/**
	 * Gets the resource string for this key.
	 * 
	 * If the resource is not found, a warning is logged
	 * 
	 * @param key
	 *            the key in the resource file
	 * @return the resource string if the key is found; otherwise, return "[res_key="+key+"]"
	 */
	public static String getResString(String key) {
		init();
		if (key == null) {
			return null;
		}
		// Resource keys cannot contain spaces, and are forced to lower case
		String resKey = key.replace(' ', '_');
		resKey = resKey.toLowerCase(java.util.Locale.ENGLISH);
		String resString = null;
		try {
			resString = resources.getString(resKey);
		} catch (MissingResourceException mre) {
			return com.arosii.jmeter.resources.nsp.Messages.getResString(key);
		}
		return resString;
	}

}
