/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.minlog.control.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.FlowLayout;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;

import com.arosii.jmeter.protocol.minlog.sampler.ListLogStatementsRequestSampler;
import com.arosii.jmeter.resources.minlog.Messages;

/**
 * 
 * @author Jacob
 */
public class ListLogStatementsRequestSamplerGui extends AbstractMinlogSamplerGui {

	private static final long serialVersionUID = 1L;

	private JTextField cpr;

	private JTextField cprFile;

	private JCheckBox onBehalfOf;

	private JTextField pageStartAfter;

	private JTextField pageSize;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getLabelResource() {
		return "minlog_list_log_statements_request";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public TestElement createTestElement() {
		ListLogStatementsRequestSampler sampler = new ListLogStatementsRequestSampler();
		modifyTestElement(sampler);
		return sampler;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected Component createConfigurationPanel() {
		final VerticalPanel configurationPanel = new VerticalPanel();
		appendCPRPanel(configurationPanel);
		appendSearchOptionsPanel(configurationPanel);
		return configurationPanel;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void configure(TestElement element) {
		super.configure(element);
		cpr.setText(element.getPropertyAsString(ListLogStatementsRequestSampler.CPR, ""));
		cprFile.setText(element.getPropertyAsString(ListLogStatementsRequestSampler.CPR_FILE, ""));
		onBehalfOf.setSelected(element.getPropertyAsBoolean(ListLogStatementsRequestSampler.ON_BEHALF_OF, false));
		pageSize.setText(element.getPropertyAsString(ListLogStatementsRequestSampler.PAGE_SIZE, ""));
		pageStartAfter.setText(element.getPropertyAsString(ListLogStatementsRequestSampler.PAGE_START_AFTER, ""));
		updateFields();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void modifyTestElement(TestElement element) {
		super.modifyTestElement(element);
		element.setProperty(ListLogStatementsRequestSampler.CPR, cpr.getText());
		element.setProperty(ListLogStatementsRequestSampler.CPR_FILE, cprFile.getText());
		element.setProperty(ListLogStatementsRequestSampler.ON_BEHALF_OF, onBehalfOf.isSelected());
		element.setProperty(ListLogStatementsRequestSampler.PAGE_SIZE, pageSize.getText());
		element.setProperty(ListLogStatementsRequestSampler.PAGE_START_AFTER, pageStartAfter.getText());
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void clearGui() {
		super.clearGui();
		cpr.setText("");
		cprFile.setText("");
		onBehalfOf.setSelected(false);
		pageSize.setText("");
		pageStartAfter.setText("");
		updateFields();
	}

	private void appendCPRPanel(VerticalPanel configurationPanel) {
		JPanel cprPanel = new VerticalPanel();
		cprPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("minlog_list_log_statements_request_cpr")));
		cprPanel.add(getCPRPanel());
		cprPanel.add(getCPRFilePanel());
		configurationPanel.add(cprPanel);
	}

	private void appendSearchOptionsPanel(VerticalPanel configurationPanel) {
		JPanel cprPanel = new VerticalPanel();
		cprPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("minlog_list_log_statements_request_search")));
		cprPanel.add(getOnBehalfOfPanel());
		cprPanel.add(getPageSizePanel());
		cprPanel.add(getPageStartAfterPanel());
		configurationPanel.add(cprPanel);
	}

	private JPanel getCPRPanel() {
		cpr = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("minlog_cpr"));

		cpr.getDocument().addDocumentListener(new DisableCPRFileKeyListener());

		JPanel cprPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		cprPanel.add(cpr);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(cprPanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getCPRFilePanel() {
		cprFile = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("minlog_cpr_file"));

		cprFile.getDocument().addDocumentListener(new DisableCPRFileKeyListener());

		JPanel cprFilePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		cprFilePanel.add(cprFile);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(cprFilePanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getOnBehalfOfPanel() {
		onBehalfOf = new JCheckBox();
		JLabel label = new JLabel(Messages.getResString("minlog_on_behalf_of"));

		JPanel onBehalfOfPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		onBehalfOfPanel.add(onBehalfOf);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(onBehalfOfPanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getPageSizePanel() {
		pageSize = new JTextField(30);
		JLabel label = new JLabel(Messages.getResString("minlog_page_size"));

		JPanel pageSizePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		pageSizePanel.add(pageSize);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(pageSizePanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getPageStartAfterPanel() {
		pageStartAfter = new JTextField(30);
		JLabel label = new JLabel(Messages.getResString("minlog_page_start_after"));

		JPanel pageStartAfterPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		pageStartAfterPanel.add(pageStartAfter);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(pageStartAfterPanel, BorderLayout.CENTER);

		return panel;
	}

	private void updateFields() {
		cprFile.setEnabled(cpr.getText().isEmpty());
		cpr.setEnabled(cprFile.getText().isEmpty());
	}

	private class DisableCPRFileKeyListener implements DocumentListener {

		@Override
		public void insertUpdate(DocumentEvent e) {
			updateFields();
		}

		@Override
		public void removeUpdate(DocumentEvent e) {
			updateFields();
		}

		@Override
		public void changedUpdate(DocumentEvent e) {
			updateFields();
		}
	}
}
