/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.minlog.extractor;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.Serializable;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.apache.jmeter.processor.PostProcessor;
import org.apache.jmeter.samplers.SampleResult;
import org.apache.jmeter.testelement.AbstractScopedTestElement;
import org.apache.jmeter.threads.JMeterVariables;
import org.apache.jorphan.logging.LoggingManager;
import org.apache.log.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;

/**
 * 
 * @author Jacob
 */
public class ListLogStatementsRequestExtractor extends AbstractScopedTestElement implements PostProcessor, Serializable {

	private static final long serialVersionUID = 1L;

	private static final Logger log = LoggingManager.getLoggerForClass();

	private static final SAXParserFactory SAX_PARSER_FACTORY = SAXParserFactory.newInstance();

	public static final String PAGE_END_VARIABLE = "ListLogStatementsRequestExtractor.PageEndVariable";

	public static final String HAS_NEXT_PAGE_VARIABLE = "ListLogStatementsRequestExtractor.HasNextPageVariable";

	static {
		SAX_PARSER_FACTORY.setNamespaceAware(true);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void process() {
		SampleResult previousResult = getThreadContext().getPreviousResult();
		if (previousResult == null) {
			return;
		}

		try {
			SAXParser parser = SAX_PARSER_FACTORY.newSAXParser();
			String pageEndVariable = getPropertyAsString(PAGE_END_VARIABLE, "");
			String hasNextPageVariable = getPropertyAsString(HAS_NEXT_PAGE_VARIABLE, "");
			ListLogStatementsHandler handler = new ListLogStatementsHandler(pageEndVariable, hasNextPageVariable);
			XMLReader xmlReader = parser.getXMLReader();
			xmlReader.setContentHandler(handler);
			xmlReader.parse(new InputSource(new ByteArrayInputStream(previousResult.getResponseData())));
			handler.saveVariables(getThreadContext().getVariables());
		} catch (ParserConfigurationException e) {
			log.error(e.getMessage(), e);
		} catch (SAXException e) {
			log.error(e.getMessage(), e);
		} catch (IOException e) {
			log.error(e.getMessage(), e);
		}
	}

	private static class ListLogStatementsHandler extends DefaultHandler implements ContentHandler {

		private final String pageEndVariable;
		
		private final String hasNextPageVariable;

		private boolean hasNextPage;
		
		private String pageEnd;

		public ListLogStatementsHandler(String pageEndVariable, String hasNextPageVariable) {
			this.pageEndVariable = pageEndVariable;
			this.hasNextPageVariable = hasNextPageVariable;

		}

		public void saveVariables(JMeterVariables variables) {
			if (hasNextPageVariable != null && !hasNextPageVariable.trim().isEmpty()) {
				variables.put(hasNextPageVariable, String.valueOf(hasNextPage));
			}
			if (pageEndVariable != null && !pageEndVariable.trim().isEmpty()) {
				variables.put(pageEndVariable, pageEnd);
			}
		}

		@Override
		public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
			if (localName.equals("ListLogStatementsOnBehalfOfResponse") || localName.equals("ListLogStatementsResponse")) {
				hasNextPage = Boolean.valueOf(attributes.getValue("hasNextPage")).booleanValue();
				pageEnd = attributes.getValue("pageEnd");
			}
		}
	}
}
