/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.minlog.extractor.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.FlowLayout;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;

import com.arosii.jmeter.protocol.minlog.extractor.ListLogStatementsRequestExtractor;
import com.arosii.jmeter.resources.minlog.Messages;

/**
 * 
 * @author Jacob
 */
public class ListLogStatementsRequestExtractorGui extends AbstractMinlogPostProcesorGui {

	private static final long serialVersionUID = 1L;

	private JTextField pageEndVariable;

	private JTextField hasNextPageVariable;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getLabelResource() {
		return "minlog_list_log_statements_request_extractor";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected Component createConfigurationPanel() {
		final VerticalPanel configurationPanel = new VerticalPanel();
		appendScopePanel(configurationPanel);
		appendVariablesPanel(configurationPanel);
		return configurationPanel;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public TestElement createTestElement() {
		ListLogStatementsRequestExtractor extractor = new ListLogStatementsRequestExtractor();
		modifyTestElement(extractor);
		return extractor;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void configure(TestElement element) {
		super.configure(element);
		pageEndVariable.setText(element.getPropertyAsString(ListLogStatementsRequestExtractor.PAGE_END_VARIABLE, ""));
		hasNextPageVariable.setText(element.getPropertyAsString(ListLogStatementsRequestExtractor.HAS_NEXT_PAGE_VARIABLE, ""));
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void modifyTestElement(TestElement element) {
		super.modifyTestElement(element);
		element.setProperty(ListLogStatementsRequestExtractor.PAGE_END_VARIABLE, pageEndVariable.getText());
		element.setProperty(ListLogStatementsRequestExtractor.HAS_NEXT_PAGE_VARIABLE, hasNextPageVariable.getText());
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void clearGui() {
		super.clearGui();
		pageEndVariable.setText("");
		hasNextPageVariable.setText("");
	}

	private void appendVariablesPanel(VerticalPanel configurationPanel) {
		JPanel variablesPanel = new VerticalPanel();
		variablesPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("minlog_list_log_statements_request_extractor_title")));
		variablesPanel.add(getPageEndVariablePanel());
		variablesPanel.add(getHasNextPageVariablePanel());
		configurationPanel.add(variablesPanel);
	}

	private JPanel getPageEndVariablePanel() {
		pageEndVariable = new JTextField(40);
		JLabel label = new JLabel(Messages.getResString("minlog_list_log_statements_request_extractor_page_end_variable") + ":");

		JPanel pageEndVariablePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		pageEndVariablePanel.add(pageEndVariable);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(pageEndVariablePanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getHasNextPageVariablePanel() {
		hasNextPageVariable = new JTextField(40);
		JLabel label = new JLabel(Messages.getResString("minlog_list_log_statements_request_extractor_has_next_page_variable") + ":");

		JPanel hasNextPageVariablePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		hasNextPageVariablePanel.add(hasNextPageVariable);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(hasNextPageVariablePanel, BorderLayout.CENTER);

		return panel;
	}
}
