package dk.nsp.jmeter.protocol.nas.backgroundjob;

import java.io.File;
import java.time.Duration;
import java.util.Random;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.stream.IntStream;

import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.StringRequestEntity;

/**
 * Afvikles således:
 * <pre>
 * mvn exec:java -Dexec.mainClass="dk.nsp.jmeter.protocol.nas.backgroundjob.NotificationBrokerBackgroundJob" -Dexec.args="http://localhost:8084/notificationbroker TESTNAS-TOPIC-PERFORMANCE-01 id-type-0001 20 1000 3000"
 * </pre>
 * Herefter kører det indtil filen <code>STOP</code> oprettes.
 */
public class NotificationBrokerBackgroundJob {

	private static final Random RANDOM = new Random();

	private static ScheduledExecutorService executor;

	private static long start;

	private static String url;

	private static String topic;

	private static String idType;

	private static int numberOfPullpoints;

	private static int numberOfPullpointsInDatabase;

	private static int numberOfIdsPerPullpoint;

	private static int delay;

	private int id;

	private HttpClient client;

	private NotificationBrokerBackgroundJob(int id) {
		this.id = id;
		this.client = new HttpClient();
	}

	public static void main(String[] args) throws Exception {
		int argIndex = 0;
		url = args[argIndex++];
		topic = args[argIndex++];
		idType = args[argIndex++];
		numberOfPullpoints = Integer.parseInt(args[argIndex++]);
		numberOfPullpointsInDatabase = Integer.parseInt(args[argIndex++]);
		numberOfIdsPerPullpoint = Integer.parseInt(args[argIndex++]);
		delay = Integer.parseInt(args[argIndex++]);

		File stopFile = new File("STOP");
		if (stopFile.exists()) {
			stopFile.delete();
		}

		executor = Executors.newScheduledThreadPool(numberOfPullpoints);
		start = System.nanoTime();
		IntStream.range(0, numberOfPullpoints).boxed().map(NotificationBrokerBackgroundJob::new).forEach(NotificationBrokerBackgroundJob::schedule);

		while (!stopFile.exists()) {
			Thread.sleep(1000);
		}

		executor.shutdown();
		executor.awaitTermination(numberOfPullpoints / 2, TimeUnit.SECONDS);
		
		stopFile.delete();
	}

	private static void schedule(NotificationBrokerBackgroundJob job) {
		int initialDelay = 10000 + (job.id * (delay / numberOfPullpoints));
		executor.scheduleAtFixedRate(job::run, initialDelay, delay, TimeUnit.MILLISECONDS);
	}

	private void run() {
		String pullpointFormat = "%0" + Integer.toString(numberOfPullpointsInDatabase).length() + "d";
		String pullpoint = String.format("pullPoint" + pullpointFormat, this.id + 1);

		String idFormat = "%0" + Integer.toString(numberOfIdsPerPullpoint).length() + "d";
		String id = String.format("pullPoint" + pullpointFormat + idFormat, this.id + 1, RANDOM.nextInt(numberOfIdsPerPullpoint) + 1);

		try {
			String request = template(topic, idType, id);
			PostMethod post = new PostMethod(url);
			post.setRequestEntity(new StringRequestEntity(request, "text/xml; charset=utf-8", "utf-8"));
			post.setRequestHeader("SOAPAction", "\"http://docs.oasis-open.org/wsn/bw-2/NotificationConsumer/Notify\"");
			int statusCode = client.executeMethod(post);
			System.out.println(Duration.ofNanos(System.nanoTime() - start).toMillis() + "\t" + pullpoint + "\t" + id + "\t" + statusCode);
		} catch (Throwable e) {
			System.out.println(Duration.ofNanos(System.nanoTime() - start).toMillis() + "\t" + pullpoint + "\t" + id + "\t" + e.getClass().getName() + ": " + e.getMessage());
			e.printStackTrace(System.out);
		}
	}

	private static String template(String topic, String idtype, String id) {
		StringBuilder sb = new StringBuilder();
		sb.append("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>").append('\n');
		sb.append("<ns2:Envelope ").append('\n');
		sb.append("xmlns:ns2=\"http://schemas.xmlsoap.org/soap/envelope/\" ").append('\n');
		sb.append("xmlns:ns3=\"http://www.medcom.dk/dgws/2006/04/dgws-1.0.xsd\" ").append('\n');
		sb.append("xmlns:ns4=\"http://www.w3.org/2005/08/addressing\" ").append('\n');
		sb.append("xmlns:ns5=\"http://docs.oasis-open.org/wsn/b-2\" ").append('\n');
		sb.append("xmlns:ns6=\"http://docs.oasis-open.org/wsn/t-1\" ").append('\n');
		sb.append("xmlns:ns7=\"http://docs.oasis-open.org/wsrf/bf-2\" ").append('\n');
		sb.append("xmlns:ns8=\"http://docs.oasis-open.org/wsrf/rp-2\" ").append('\n');
		sb.append("xmlns:ns9=\"http://nsi.dk/advis/v10\">").append('\n');
		sb.append("  <ns2:Header/>").append('\n');
		sb.append("  <ns2:Body>").append('\n');
		sb.append("    <ns5:Notify>").append('\n');
		sb.append("      <ns5:NotificationMessage>").append('\n');
		sb.append("        <ns5:Topic Dialect=\"http://docs.oasis-open.org/wsn/t-1/TopicExpression/Simple\">").append(topic).append("</ns5:Topic>").append('\n');
		sb.append("        <ns5:Message>").append('\n');
		sb.append("          <ns9:NotifyContent id=\"").append(id).append("\" idType=\"").append(idtype).append("\">").append('\n');
		sb.append("            <test>Dette er en test</test>").append('\n');
		sb.append("          </ns9:NotifyContent>").append('\n');
		sb.append("        </ns5:Message>").append('\n');
		sb.append("      </ns5:NotificationMessage>").append('\n');
		sb.append("    </ns5:Notify>").append('\n');
		sb.append("  </ns2:Body>").append('\n');
		sb.append("</ns2:Envelope>");
		return sb.toString();
	}

}
