/**
* The MIT License
*   
* Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
*   
* Copyright (C) 2018 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
*   
*   
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*   
*   
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*   
*   
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*/ 

// This test is not in use at the moment.

package dk.nsp.jmeter.protocol.nas.control.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.FlowLayout;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;

import dk.nsp.jmeter.protocol.nas.sampler.CreateIDListRequestSampler;
import dk.nsp.jmeter.resources.nas.Messages;

/**
 * 
 * @author
 */
public class CreateIDListRequestSamplerGui extends AbstractNasSamplerGui {

	private static final long serialVersionUID = 1L;

	private JTextField idListName;
	
	private JTextField idType;
	
	private JTextField description;
	
	private JTextField content1;
	private JTextField content2;

	private JTextField createIDListFile; //hvad skal køre som fil?
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getLabelResource() {
		return "nas_create_idlist_request";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public TestElement createTestElement() {
		CreateIDListRequestSampler sampler = new CreateIDListRequestSampler();
		modifyTestElement(sampler);
		return sampler;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected Component createConfigurationPanel() {
		final VerticalPanel configurationPanel = new VerticalPanel();
		appendCreateIDListPanel(configurationPanel);
		appendContentPanel(configurationPanel);
		return configurationPanel;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void configure(TestElement element) {
		super.configure(element);
		idListName.setText(element.getPropertyAsString(CreateIDListRequestSampler.ID_LIST_NAME, ""));
		idType.setText(element.getPropertyAsString(CreateIDListRequestSampler.ID_TYPE, ""));
		description.setText(element.getPropertyAsString(CreateIDListRequestSampler.DESCRIPTION, ""));
		content1.setText(element.getPropertyAsString(CreateIDListRequestSampler.CONTENT1, ""));
		content2.setText(element.getPropertyAsString(CreateIDListRequestSampler.CONTENT2, ""));
		createIDListFile.setText(element.getPropertyAsString(CreateIDListRequestSampler.CREATE_ID_LIST_FILE, ""));
		updateFields();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void modifyTestElement(TestElement element) {
		super.modifyTestElement(element);
		element.setProperty(CreateIDListRequestSampler.ID_LIST_NAME, idListName.getText());
		element.setProperty(CreateIDListRequestSampler.ID_TYPE, idType.getText());
		element.setProperty(CreateIDListRequestSampler.DESCRIPTION, description.getText());
		element.setProperty(CreateIDListRequestSampler.CONTENT1, content1.getText());
		element.setProperty(CreateIDListRequestSampler.CONTENT2, content2.getText());
		element.setProperty(CreateIDListRequestSampler.CREATE_ID_LIST_FILE, createIDListFile.getText());
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void clearGui() {
		super.clearGui();
		idListName.setText("");
		idType.setText("");
		description.setText("");
		content1.setText("");
		content2.setText("");
		createIDListFile.setText("");
		updateFields();
	}

	private void appendCreateIDListPanel(VerticalPanel configurationPanel) {
		JPanel ppPanel = new VerticalPanel();
		ppPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("nas_create_idlist_request_input"))); 
		ppPanel.add(getIdListNamePanel());
		ppPanel.add(getIdTypePanel());
		ppPanel.add(getDescriptionPanel());
		ppPanel.add(getCreateIDListFilePanel());
		configurationPanel.add(ppPanel);
	}
	
	private void appendContentPanel(VerticalPanel configurationPanel) {
		JPanel contentPanel = new VerticalPanel();
		contentPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("nas_create_idlist_request_content")));
		contentPanel.add(getContent1Panel());
		contentPanel.add(getContent2Panel());
		configurationPanel.add(contentPanel);
	}

	private JPanel getIdListNamePanel() {
		idListName = new JTextField(255);
		JLabel label = new JLabel(Messages.getResString("nas_create_idlist_name_value"));

		idListName.getDocument().addDocumentListener(new DisableCreateIDListFileKeyListener());

		JPanel idListNamePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		idListNamePanel.add(idListName);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(idListNamePanel, BorderLayout.CENTER);

		return panel;
	}
	
	private JPanel getIdTypePanel() {
		idType = new JTextField(255);
		JLabel label = new JLabel(Messages.getResString("nas_create_idlist_name_id_type"));

//		idType.getDocument().addDocumentListener(new DisableCreateIDListFileKeyListener());

		JPanel idTypePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		idTypePanel.add(idType);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(idTypePanel, BorderLayout.CENTER);

		return panel;
	}
	
	private JPanel getDescriptionPanel() {
		description = new JTextField(255);
		JLabel label = new JLabel(Messages.getResString("nas_create_idlist_name_description"));

//		idType.getDocument().addDocumentListener(new DisableCreateIDListFileKeyListener());

		JPanel descriptionPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		descriptionPanel.add(description);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(descriptionPanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getContent1Panel() {
		content1 = new JTextField(255);
		JLabel label = new JLabel(Messages.getResString("nas_create_idlist_name_content1"));

//		idType.getDocument().addDocumentListener(new DisableCreateIDListFileKeyListener());

		JPanel content1Panel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		content1Panel.add(content1);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(content1Panel, BorderLayout.CENTER);

		return panel;
	}
	private JPanel getContent2Panel() {
		content2 = new JTextField(255);
		JLabel label = new JLabel(Messages.getResString("nas_create_idlist_name_content2"));

//		idType.getDocument().addDocumentListener(new DisableCreateIDListFileKeyListener());

		JPanel content2Panel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		content2Panel.add(content2);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(content2Panel, BorderLayout.CENTER);

		return panel;
	}

	
	
	private JPanel getCreateIDListFilePanel() {
		createIDListFile = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("nas_create_idlist_file"));

		createIDListFile.getDocument().addDocumentListener(new DisableCreateIDListFileKeyListener());

		JPanel createIDListFilePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		createIDListFilePanel.add(createIDListFile);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(createIDListFilePanel, BorderLayout.CENTER);

		return panel;
	}

	private void updateFields() {
		createIDListFile.setEnabled(idListName.getText().isEmpty());
		idListName.setEnabled(createIDListFile.getText().isEmpty());
	}

	private class DisableCreateIDListFileKeyListener implements DocumentListener {

		@Override
		public void insertUpdate(DocumentEvent e) {
			updateFields();
		}

		@Override
		public void removeUpdate(DocumentEvent e) {
			updateFields();
		}

		@Override
		public void changedUpdate(DocumentEvent e) {
			updateFields();
		}
	}
}