/**
* The MIT License
*   
* Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
*   
* Copyright (C) 2018 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
*   
*   
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*   
*   
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*   
*   
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*/ 

package dk.nsp.jmeter.protocol.nas.control.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.FlowLayout;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;

import dk.nsp.jmeter.protocol.nas.sampler.NotificationBrokerRequestSampler;
import dk.nsp.jmeter.resources.nas.Messages;

/**
 * 
 * @author
 */
public class NotificationBrokerRequestSamplerGui extends AbstractNasSamplerGui {

	private static final long serialVersionUID = 1L;

	private JTextField idValue;
	
	private JTextField idValueFile;
	
	private JTextField idType;
	
	private JTextField topic;
	
	private JTextArea notificationContent;
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getLabelResource() {
		return "nas_notification_broker_request";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public TestElement createTestElement() {
		NotificationBrokerRequestSampler sampler = new NotificationBrokerRequestSampler();
		modifyTestElement(sampler);
		return sampler;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected Component createConfigurationPanel() {
		final VerticalPanel configurationPanel = new VerticalPanel();
		appendIDPanel(configurationPanel);
		appendOtherPanel(configurationPanel);
		appendContentPanel(configurationPanel);
		return configurationPanel;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void configure(TestElement element) {
		super.configure(element);
		idValue.setText(element.getPropertyAsString(NotificationBrokerRequestSampler.ID_VALUE, ""));
		idValueFile.setText(element.getPropertyAsString(NotificationBrokerRequestSampler.ID_VALUE_FILE, ""));
		idType.setText(element.getPropertyAsString(NotificationBrokerRequestSampler.ID_TYPE, ""));
		topic.setText(element.getPropertyAsString(NotificationBrokerRequestSampler.TOPIC, ""));
		notificationContent.setText(element.getPropertyAsString(NotificationBrokerRequestSampler.NOTIFICATION_CONTENT, ""));
		updateFields();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void modifyTestElement(TestElement element) {
		super.modifyTestElement(element);
		element.setProperty(NotificationBrokerRequestSampler.ID_VALUE, idValue.getText());
		element.setProperty(NotificationBrokerRequestSampler.ID_VALUE_FILE, idValueFile.getText());
		element.setProperty(NotificationBrokerRequestSampler.ID_TYPE, idType.getText());
		element.setProperty(NotificationBrokerRequestSampler.TOPIC, topic.getText());
		element.setProperty(NotificationBrokerRequestSampler.NOTIFICATION_CONTENT, notificationContent.getText());
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void clearGui() {
		super.clearGui();
		idValue.setText("");
		idValueFile.setText("");
		idType.setText("");
		topic.setText("");
		notificationContent.setText("");
		updateFields();
	}

	private void appendIDPanel(VerticalPanel configurationPanel) {
		JPanel idPanel = new VerticalPanel();
		idPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("nas_notification_broker_request_id")));
		idPanel.add(getIdValuePanel());
		idPanel.add(getIdValueFilePanel());
		configurationPanel.add(idPanel);
	}
	
	private void appendOtherPanel(VerticalPanel configurationPanel) {
		JPanel otherPanel = new VerticalPanel();
		otherPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("nas_notification_broker_request_other")));
		otherPanel.add(getIdTypePanel());
		otherPanel.add(getTopicPanel());
		configurationPanel.add(otherPanel);
	}
	
	private void appendContentPanel(VerticalPanel configurationPanel) {
		JPanel contentPanel = new VerticalPanel();
		contentPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("nas_notification_broker_request_content")));
		contentPanel.add(getContentPanel());
		configurationPanel.add(contentPanel);
	}

	private JPanel getIdValuePanel() {
		idValue = new JTextField(255);
		JLabel label = new JLabel(Messages.getResString("nas_notification_id_value"));

		idValue.getDocument().addDocumentListener(new DisableIdValueFileKeyListener());

		JPanel idValuePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		idValuePanel.add(idValue);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(idValuePanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getIdValueFilePanel() {
		idValueFile = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("nas_notification_id_value_file"));

		idValueFile.getDocument().addDocumentListener(new DisableIdValueFileKeyListener());

		JPanel idValueFilePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		idValueFilePanel.add(idValueFile);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(idValueFilePanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getIdTypePanel() {
		idType = new JTextField(128);
		JLabel label = new JLabel(Messages.getResString("nas_notification_id_type"));

		idType.getDocument().addDocumentListener(new DisableIdValueFileKeyListener());

		JPanel idTypePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		idTypePanel.add(idType);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(idTypePanel, BorderLayout.CENTER);

		return panel;
	}
	
	private JPanel getTopicPanel() {
		topic = new JTextField(128);
		JLabel label = new JLabel(Messages.getResString("nas_notification_topic"));

		topic.getDocument().addDocumentListener(new DisableIdValueFileKeyListener());

		JPanel topicPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		topicPanel.add(topic);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(topicPanel, BorderLayout.CENTER);

		return panel;
	}
	
	private JPanel getContentPanel() {
		notificationContent = new JTextArea(16, 128);  
		notificationContent.setLineWrap(true);
		
		notificationContent.getDocument().addDocumentListener(new DisableIdValueFileKeyListener());

		JPanel contentPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		contentPanel.add(notificationContent);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(contentPanel, BorderLayout.CENTER);

		return panel;
	}

	private void updateFields() {
		idValueFile.setEnabled(idValue.getText().isEmpty());
		idValue.setEnabled(idValueFile.getText().isEmpty());
	}

	private class DisableIdValueFileKeyListener implements DocumentListener {

		@Override
		public void insertUpdate(DocumentEvent e) {
			updateFields();
		}

		@Override
		public void removeUpdate(DocumentEvent e) {
			updateFields();
		}

		@Override
		public void changedUpdate(DocumentEvent e) {
			updateFields();
		}
	}
}
