/**
* The MIT License
*   
* Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
*   
* Copyright (C) 2018 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
*   
*   
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*   
*   
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*   
*   
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*/ 

package dk.nsp.jmeter.protocol.nas.control.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;

import dk.nsp.jmeter.protocol.nas.sampler.PullPointRequestSampler;
import dk.nsp.jmeter.resources.nas.Messages;

/**
 * 
 * @author
 */
public class PullPointRequestSamplerGui extends AbstractNasSamplerGui {

	private static final long serialVersionUID = 1L;

	private JTextField pullPoint;
	
	private JTextField maximum;

	private JTextField pullPointFile;
	
	private JCheckBox autoGenerate;
	
	private JTextField prefixPullPoints;
	
	private JTextField noOfPullPoints;
	
	private JTextField noOfPullPointsInDatabase;
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getLabelResource() {
		return "nas_pullpoint_request";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public TestElement createTestElement() {
		PullPointRequestSampler sampler = new PullPointRequestSampler();
		modifyTestElement(sampler);
		return sampler;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected Component createConfigurationPanel() {
		final VerticalPanel configurationPanel = new VerticalPanel();
		appendPullPointPanel(configurationPanel);
		appendOtherPanel(configurationPanel);
		return configurationPanel;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void configure(TestElement element) {
		super.configure(element);
		pullPoint.setText(element.getPropertyAsString(PullPointRequestSampler.PULL_POINT, ""));
		autoGenerate.setSelected(element.getPropertyAsBoolean(PullPointRequestSampler.PULL_POINT_AUTO, false));
		maximum.setText(element.getPropertyAsString(PullPointRequestSampler.MAXIMUM, ""));
		pullPointFile.setText(element.getPropertyAsString(PullPointRequestSampler.PULL_POINT_FILE, ""));
		noOfPullPoints.setText(element.getPropertyAsString(PullPointRequestSampler.NO_OF_PULL_POINTS, ""));
		noOfPullPointsInDatabase.setText(element.getPropertyAsString(PullPointRequestSampler.NO_OF_PULL_POINTS_DB, ""));
		prefixPullPoints.setText(element.getPropertyAsString(PullPointRequestSampler.PREFIX_PULL_POINTS, ""));
		updateFields();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void modifyTestElement(TestElement element) {
		super.modifyTestElement(element);
		element.setProperty(PullPointRequestSampler.PULL_POINT, pullPoint.getText());
		element.setProperty(PullPointRequestSampler.PULL_POINT_AUTO, autoGenerate.isSelected());
		element.setProperty(PullPointRequestSampler.MAXIMUM, maximum.getText());
		element.setProperty(PullPointRequestSampler.PULL_POINT_FILE, pullPointFile.getText());
		element.setProperty(PullPointRequestSampler.NO_OF_PULL_POINTS, noOfPullPoints.getText());
		element.setProperty(PullPointRequestSampler.NO_OF_PULL_POINTS_DB, noOfPullPointsInDatabase.getText());
		element.setProperty(PullPointRequestSampler.PREFIX_PULL_POINTS, prefixPullPoints.getText());
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void clearGui() {
		super.clearGui();
		pullPoint.setText("");
		autoGenerate.setSelected(false);
		maximum.setText("");
		pullPointFile.setText("");
		noOfPullPoints.setText("");
		noOfPullPointsInDatabase.setText("");
		prefixPullPoints.setText("");
		updateFields();
	}

	private void appendPullPointPanel(VerticalPanel configurationPanel) {
		JPanel ppPanel = new VerticalPanel();
		ppPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("nas_pullpoint_request_pullpoint")));
		JLabel selectOnelabel = new JLabel(Messages.getResString("nas_pullpoint_select_one"));
		ppPanel.add(selectOnelabel);
		ppPanel.add(getPullPointPanel());
		ppPanel.add(getAutoGeneratePanel());
		ppPanel.add(getPullPointFilePanel());
		configurationPanel.add(ppPanel);
	}
	
	private void appendOtherPanel(VerticalPanel configurationPanel) {
		JPanel otherPanel = new VerticalPanel();
		otherPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("nas_pullpoint_request_other")));
		otherPanel.add(getMaxPanel());
		configurationPanel.add(otherPanel);
	}

	private JPanel getPullPointPanel() {
		pullPoint = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("nas_pullpoint_pullpoint"));

		pullPoint.getDocument().addDocumentListener(new DisablePullPointFileKeyListener());

		JPanel pullPointPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		pullPointPanel.add(pullPoint);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(pullPointPanel, BorderLayout.CENTER);

		return panel;
	}
	
	private JPanel getAutoGeneratePanel() {
		autoGenerate = new JCheckBox();
		JLabel label = new JLabel(Messages.getResString("nas_pullpoint_auto_generate_no"));
		autoGenerate.setText(Messages.getResString("nas_pullpoint_auto_generate"));
		
		autoGenerate.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent e) {
				pullPointFile.setEnabled((pullPoint.getText().isEmpty() && !autoGenerate.isSelected()));
				pullPoint.setEnabled((pullPointFile.getText().isEmpty() && !autoGenerate.isSelected()));
				autoGenerate.setEnabled((pullPointFile.getText().isEmpty() && pullPoint.getText().isEmpty()));
			}

	    });

		JLabel labelNoofPullPoint = new JLabel(Messages.getResString("nas_pullpoint_auto_pullpoint_numbers"));
		JLabel labelNoofPullPointInDatabase = new JLabel(Messages.getResString("nas_pullpoint_auto_pullpoint_numbers_db"));
		JLabel labelPrefixPullPoint = new JLabel(Messages.getResString("nas_pullpoint_auto_prefix"));
		
		noOfPullPoints = new JTextField(10);
		noOfPullPointsInDatabase = new JTextField(10);
		prefixPullPoints = new JTextField(20);

		JPanel noOfPullPointsPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		
//		JPanel noOfIdListPerPullPointPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		JPanel noOfPullPointsInDatabasePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		JPanel prefixPullPointsPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		
		JPanel noOfPanel = new JPanel(new FlowLayout(5));
		noOfPanel.add(labelPrefixPullPoint);
		noOfPanel.add(prefixPullPointsPanel);
		noOfPanel.add(labelNoofPullPoint);
		noOfPanel.add(noOfPullPointsPanel);
		noOfPanel.add(labelNoofPullPointInDatabase);
		noOfPanel.add(noOfPullPointsInDatabasePanel);
//		noOfPanel.add(noOfIdListPerPullPointPanel);

		
		noOfPullPointsPanel.add(noOfPullPoints);
		noOfPullPointsInDatabasePanel.add(noOfPullPointsInDatabase);
		prefixPullPointsPanel.add(prefixPullPoints);
		
		JPanel autoGeneratePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		autoGeneratePanel.add(autoGenerate);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(autoGeneratePanel, BorderLayout.CENTER);
		panel.add(noOfPanel, BorderLayout.EAST);
		
		return panel;
	}

	
	private JPanel getMaxPanel() {
		maximum = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("nas_pullpoint_max"));

		JPanel maxPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		maxPanel.add(maximum);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(maxPanel, BorderLayout.CENTER);

		return panel;
	}


	private JPanel getPullPointFilePanel() {
		pullPointFile = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("nas_pullpoint_file"));

		pullPointFile.getDocument().addDocumentListener(new DisablePullPointFileKeyListener());

		JPanel pullPointFilePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		pullPointFilePanel.add(pullPointFile);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(pullPointFilePanel, BorderLayout.CENTER);

		return panel;
	}

	private void updateFields() {
		pullPointFile.setEnabled((pullPoint.getText().isEmpty() && !autoGenerate.isSelected()));
		pullPoint.setEnabled((pullPointFile.getText().isEmpty() && !autoGenerate.isSelected()));
		autoGenerate.setEnabled((pullPointFile.getText().isEmpty() && pullPoint.getText().isEmpty()));
	}

	private class DisablePullPointFileKeyListener implements DocumentListener {

		@Override
		public void insertUpdate(DocumentEvent e) {
			updateFields();
		}

		@Override
		public void removeUpdate(DocumentEvent e) {
			updateFields();
		}

		@Override
		public void changedUpdate(DocumentEvent e) {
			updateFields();
		}
	}
	
	
}