/**
* The MIT License
*   
* Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
*   
* Copyright (C) 2018 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
*   
*   
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*   
*   
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*   
*   
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*/ 

// This test is not in use at the moment.

package dk.nsp.jmeter.protocol.nas.sampler;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Random;
import java.util.Set;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.jmeter.protocol.http.sampler.HTTPSampleResult;
import org.apache.jorphan.logging.LoggingManager;
import org.apache.log.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.arosii.jmeter.protocol.nsp.sampler.AbstractNSPSoapSampler;

import dk.sosi.seal.SOSIFactory;
import dk.sosi.seal.model.IDCard;
import dk.sosi.seal.model.Request;
import dk.sosi.seal.xml.XmlUtil;

/**
 * 
 * @author
 */
public class CreateIDListRequestSampler extends AbstractNSPSoapSampler {

	private static final long serialVersionUID = 1L;
	
	private static final Logger log = LoggingManager.getLoggerForClass();
	public static final String ID_LIST_NAME = "CreateIDListRequestSampler.IdListName";
	
	public static final String ID_TYPE = "CreateIDListRequestSampler.IdType";
	
	public static final String DESCRIPTION = "CreateIDListRequestSampler.Description";
	
	public static final String CONTENT1 = "CreateIDListRequestSampler.Content1";
	public static final String CONTENT2 = "CreateIDListRequestSampler.Content2";

	public static final String CREATE_ID_LIST_FILE = "CreateIDListRequestSampler.CreateIDListFile";

	public static final Random RANDOM = new Random();

	private static List<String> createIdListCache;

	public CreateIDListRequestSampler() {
		setPath("/idlist/");
		setSOAPAction("http://www.nsi.dk/nas/idlist#CreateIDList");
	}
	
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	protected HTTPSampleResult sample(URL url, String method, boolean areFollowingRedirect, int frameDepth) {
		clearReply();
		setXmlData(createRequest());
		HTTPSampleResult sample = super.sample(url, method, areFollowingRedirect, frameDepth);
		putReply(sample);
		return sample;
	}

	private String createRequest() {
	
		SOSIFactory sosiFactory = getSOSIFactory();
		IDCard idCard = getIDCard();

		// Create a new request and add the id card to it.
		Request request = sosiFactory.createNewRequest(false, null);
		request.setIDCard(idCard);
		
		Document bodyDocument = createIdListRequestBody();

		request.setBody(bodyDocument.getDocumentElement());
		
		Document doc = request.serialize2DOMDocument();

		return XmlUtil.node2String(doc, false, true);
	}
	
	private Document createIdListRequestBody() {
		Document bodyDocument;
		try {
			bodyDocument = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
		} catch (ParserConfigurationException e) {
			throw new IllegalStateException(e);
		}
		String requestMethod = "CreateIDListRequest";
		Element getMethodRequest = bodyDocument.createElementNS("http://www.nsi.dk/2012/12/nas/idlist", requestMethod);
		getMethodRequest.setPrefix("ns8");
		bodyDocument.appendChild(getMethodRequest);
		
		String idListNameProp = getPropertyAsString(ID_LIST_NAME);
		String descriptionProp = getPropertyAsString(DESCRIPTION);
		String idTypeProp = getPropertyAsString(ID_TYPE);
		String id1Prop = getPropertyAsString(CONTENT1);
		String id2Prop = getPropertyAsString(CONTENT2);
		
		Element name = bodyDocument.createElement("ns8:Name");
		name.setTextContent(idListNameProp);
		getMethodRequest.appendChild(name);
		
		Element description = bodyDocument.createElement("ns8:Description");
		description.setTextContent(descriptionProp);
		getMethodRequest.appendChild(description);
		
		Element idType = bodyDocument.createElement("ns8:IDType");
		idType.setTextContent(idTypeProp);
		getMethodRequest.appendChild(idType);
		
		Element id1 = bodyDocument.createElement("ns8:Id");
		id1.setTextContent(id1Prop);
		getMethodRequest.appendChild(id1);
		
		Element id2 = bodyDocument.createElement("ns8:Id");
		id2.setTextContent(id2Prop);
		getMethodRequest.appendChild(id2);
		
		return bodyDocument;
	
	}

}