/**
* The MIT License
*   
* Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
*   
* Copyright (C) 2018 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
*   
*   
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*   
*   
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*   
*   
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*/ 

package dk.nsp.jmeter.protocol.nas.sampler;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.List;
import java.util.Random;

import org.apache.jmeter.protocol.http.sampler.HTTPSampleResult;
import org.apache.jmeter.protocol.http.sampler.SoapSampler;
import org.apache.jorphan.logging.LoggingManager;
import org.apache.log.Logger;

/**
 * 
 * @author
 */
public class NotificationBrokerRequestSampler extends SoapSampler {
	
	private static final long serialVersionUID = 1L;
	
	public static final String ID_VALUE = "NotificationBrokerRequestSampler.IdValue";
	public static final String ID_VALUE_FILE = "NotificationBrokerRequestSampler.IdValueFile";
	public static final String ID_TYPE = "NotificationBrokerSampler.IdType";
	public static final String TOPIC = "NotificationBrokerRequestSampler.Topic";
	public static final String NOTIFICATION_CONTENT = "NotificationBrokerRequestSampler.NotificationContent";
	public static final Random RANDOM = new Random();

	private static List<String> idValueCache;
	
	private static final String SOAP_ACTION = "http://docs.oasis-open.org/wsn/bw-2/NotificationConsumer/Notify";
	private static final String PATH = "/nas2/notificationbroker";

	private String idValue;
	private String idType;
	private String topic;
	private String content;


	public NotificationBrokerRequestSampler() {
		setPath(PATH); 
		setSOAPAction(SOAP_ACTION);
	}
	

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected HTTPSampleResult sample(URL url, String method, boolean areFollowingRedirect, int frameDepth) {
//		clearReply();
		setXmlData(createRequest());
		HTTPSampleResult sample = super.sample(url, method, areFollowingRedirect, frameDepth);
//		putReply(sample);
		return sample;
	}

	@Override
	public URL getUrl() throws MalformedURLException {
		return new URL(getProtocol(), getDomain(), getPort(), getPath());
	}
	
	private String createRequest() {
		if (this.idValue == null) {
			this.idValue = getPropertyAsString(ID_VALUE);
			this.idType = getPropertyAsString(ID_TYPE);
			this.topic = getPropertyAsString(TOPIC);
			this.content = getPropertyAsString(NOTIFICATION_CONTENT);
//			this.content = "<test>&lt;noget&gt;xml&lt;/noget&gt;</test>";	
		}
		
			String aString = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>\n" + 
					"<ns2:Envelope \n" + 
					"xmlns:ns6=\"http://docs.oasis-open.org/wsn/t-1\" \n" + 
					"xmlns:ns5=\"http://docs.oasis-open.org/wsn/b-2\" \n" + 
					"xmlns:ns8=\"http://docs.oasis-open.org/wsrf/rp-2\" \n" + 
					"xmlns:ns7=\"http://docs.oasis-open.org/wsrf/bf-2\" \n" + 
					"xmlns:ns9=\"http://nsi.dk/advis/v10\" \n" + 
					"xmlns:ns2=\"http://schemas.xmlsoap.org/soap/envelope/\" \n" + 
					"xmlns:ns4=\"http://www.w3.org/2005/08/addressing\" \n" + 
					"xmlns:ns3=\"http://www.medcom.dk/dgws/2006/04/dgws-1.0.xsd\">\n" + 
					"  <ns2:Header/>\n" + 
					"  <ns2:Body>\n" + 
					"  \n" + 
					"    <ns5:Notify>\n" + 
					"      <ns5:NotificationMessage>\n" + 
					"        <ns5:Topic Dialect=\"http://docs.oasis-open.org/wsn/t-1/TopicExpression/Simple\">" + topic + "</ns5:Topic>\n" + 
					"        <ns5:Message>\n" + 
//					"          <ns9:NotifyContent id=\"" + idValue + "\" idType=\"" + idType +"\">\n" + 
//					content + "\n" +
					"          <ns9:NotifyContent id=\"" + idValue + "\" idType=\"" + idType +"\">" + 
					content +  "</ns9:NotifyContent>\n" + 
					"        </ns5:Message>\n" + 
					"      </ns5:NotificationMessage>\n" + 
					"    </ns5:Notify>\n" + 
					"  </ns2:Body>\n" + 
					"</ns2:Envelope>";
			
			return aString;
	}

}
