package dk.nsp.jmeter.protocol.nas.testdata;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.apache.kafka.common.utils.Utils;

public class GenerateTestdataSql {

	private FileOutputStream outputStream;
	private String lineSeparator;

	private String prefixPullPoint;
	private int noOfPullPoints;
	private int noOfIdListContentPerIdList;
	private String idType;
	private String topic;
	private String ownerId;

	private String pathName = "src/test/resources/";

	/*
	 * java -cp jmeter-nas.jar dk.nsp.jmeter.protocol.nas.testdata.GenerateTestdataSql pp 5 20 idtype TOPIC 30808460SOSITEST
	 */
	public static void main(String[] args) throws FileNotFoundException, IOException {
		System.out.println("Starting...");
		GenerateTestdataSql testdata = new GenerateTestdataSql(args);
		testdata.doSqlFile();
		System.out.println("Ending...");
	}

	public GenerateTestdataSql(String[] args) {
		// set parameters to use in case none is given
		prefixPullPoint = "pullPoint";
		noOfPullPoints = 1000;
		noOfIdListContentPerIdList = 3000;
		idType = "idtype01";
		topic = "TESTNAS-TOPIC-PERFORMANCE-01";
		ownerId = "30808460SOSITEST";
		pathName = ".";

		if (args.length > 0) {
			prefixPullPoint = args[0];
		}
		if (args.length > 1) {
			String noOfPullPointsInput = args[1];
			noOfPullPoints = Integer.parseInt(noOfPullPointsInput);
		}
		if (args.length > 2) {
			String noOfIdListContentPerIdListInput = args[2];
			noOfIdListContentPerIdList = Integer.parseInt(noOfIdListContentPerIdListInput);
		}
		if (args.length > 3) {
			idType = args[3];
		}
		if (args.length > 4) {
			topic = args[4];
		}
		if (args.length > 5) {
			ownerId = args[5];
		}
		if (args.length > 6) {
			pathName = args[6];
		}

	}

	private void doSqlFile() throws IOException {
		// prepare
		prepareSqlFile();
		String contentText = "prefix=" + prefixPullPoint + ", noOfPullPoints=" + noOfPullPoints + ", noOfIdListContentPerIdList=" + noOfIdListContentPerIdList + ", idType=" + idType + ", topic=" + topic + ", ownerId=" + ownerId;
		writeSqlFile("-- " + contentText);
		System.out.println(contentText);

		int noOfZeroesInFormatPullPoint = Integer.toString(noOfPullPoints).length();
		String pullPointFormat = "%0" + Integer.toString(noOfZeroesInFormatPullPoint) + "d";

		int noOfZeroesInFormatIdListContent = Integer.toString(noOfIdListContentPerIdList).length();
		String idListContentFormat = "%0" + Integer.toString(noOfZeroesInFormatIdListContent) + "d";

		// generate and write sqls to file
		writeSqlFile(getTopicSql(topic));

		for (int pullPointCount = 1; pullPointCount <= noOfPullPoints; pullPointCount++) {
			String name = prefixPullPoint + String.format(pullPointFormat, pullPointCount);

			int row_id = 10000 + pullPointCount;

			writeSqlFile(getIdListSql(row_id, name, ownerId, idType));

			writeSqlFile(getIdListContentSql1());
			for (int IdListContentCount = 1; IdListContentCount <= noOfIdListContentPerIdList; IdListContentCount++) {
				if (IdListContentCount % 800 == 0) {
					writeSqlFile(getIdListContentSql1());
				}
				// writeSqlFile(getIdListContentSql(pullPointCount, name, IdListContentCount, idListContentFormat, idType));
				String idListContentSql2 = "  " + getIdListContentSql2(row_id, name, IdListContentCount, idListContentFormat, idType);
				if (IdListContentCount + 1 > noOfIdListContentPerIdList || (IdListContentCount + 1) % 800 == 0) {
					idListContentSql2 += ";";
				} else {
					idListContentSql2 += ", ";
				}
				writeSqlFile(idListContentSql2);
			}
			writeSqlFile(getPullPointSql(row_id, name, ownerId));
			writeSqlFile(getSubscriptionSql(row_id, row_id, row_id, name, topic, ownerId));
			writeSqlFile(getNasConsumerSql(row_id, name));
		}

		// end
		terminateSqlFile();
	}

	private String getTopicSql(String topic) {
		StringBuilder sb = new StringBuilder();
		sb.append("insert into topic_mapping (topic, internal_topic) values (");
		sb.append("'").append(topic).append("', ");
		sb.append("'").append(topic).append("'");
		sb.append(");");
		return sb.toString();
	}

	private String getIdListSql(int id, String name, String owner, String idType) {
		StringBuilder sb = new StringBuilder();
		sb.append("insert into id_list (id, name, id_type, owner_id, auto_delete, description) values (");
		sb.append(id).append(", ");
		sb.append("'").append(name).append("', ");
		sb.append("'").append(idType).append("', ");
		sb.append("'").append(owner).append("', ");
		sb.append("0").append(", ");
		sb.append("'").append("Performance test: ").append(name).append("'");
		sb.append(");");
		return sb.toString();
	}

	private String getIdListContentSql1() {
		StringBuilder sb = new StringBuilder();
		sb.append("insert into id_list_content (idlist_id, id_value, hash) values");
		return sb.toString();
	}

	private String getIdListContentSql2(int idlist_id, String name, int itemCount, String idListContentFormat, String idType) {
		String idValue = name + String.format(idListContentFormat, itemCount);
		long hashValue = hash(idType, idValue);

		StringBuilder sb = new StringBuilder();
		sb.append("(");
		sb.append(idlist_id).append(", ");
		sb.append("'").append(idValue).append("', ");
		sb.append(hashValue);
		sb.append(")");
		return sb.toString();
	}

	private String getPullPointSql(int id, String name, String owner) {
		StringBuilder sb = new StringBuilder();
		sb.append("insert into pull_point (id, pull_point_id, owner_id) values (");
		sb.append(id).append(", ");
		sb.append("'").append(name).append("', ");
		sb.append("'").append(owner).append("'");
		sb.append(");");
		return sb.toString();
	}

	private String getSubscriptionSql(int id, int pullpointId, int idlistId, String name, String topic, String owner) {
		StringBuilder sb = new StringBuilder();
		sb.append("insert into subscription (id, subscription_id, pull_point, topic, owner_id, idlist_id) values (");
		sb.append(id).append(", ");
		sb.append("'").append(name).append("', ");
		sb.append(pullpointId).append(", ");
		sb.append("'").append(topic).append("', ");
		sb.append("'").append(owner).append("', ");
		sb.append(idlistId);
		sb.append(");");
		return sb.toString();
	}

	private String getNasConsumerSql(int subscriptionId, String name) {
		StringBuilder sb = new StringBuilder();
		sb.append("insert into nas_consumer (subscription, message_id, message_count, previous_id, offsets) values (");
		sb.append(subscriptionId).append(", ");
		sb.append("null, ");
		sb.append("0, ");
		sb.append("null, ");
		sb.append("'0'");
		sb.append(");");
		return sb.toString();
	}

	private void prepareSqlFile() throws FileNotFoundException {
		String filePath;

		Date now = new java.util.Date();
		SimpleDateFormat DATE_FORMAT = new SimpleDateFormat("yyyy-MM-dd_HH-mm-SS");
		String nowFormated = DATE_FORMAT.format(now);

		String fileName = "/sqlPerformancetest_" + nowFormated + ".sql";

		lineSeparator = System.getProperty("line.separator");

		File sqlFile = new File(pathName + fileName);
		outputStream = new FileOutputStream(sqlFile);
		filePath = sqlFile.getAbsolutePath();

		System.out.println("File location: " + filePath);
	}

	private void terminateSqlFile() throws IOException {
		outputStream.close();
	}

	private void writeSqlFile(String sqlLine) throws IOException {
		byte[] strToBytes = sqlLine.getBytes();
		outputStream.write(strToBytes);
		outputStream.write(lineSeparator.getBytes());
	}

	public static long hash(String idType, String id) {
		// TODO 0: fix hash bibliotek dependency i jmeter-nas pom filen, når det er endelig på plads i nas projektet.
		int upper = Utils.murmur2(idType.getBytes());
		int lower = Utils.murmur2(id.getBytes());
		return (((long) upper) << 32) | (lower & 0xffffffffL);
	}

}
