/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sdm.assertion;

import javax.xml.namespace.NamespaceContext;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;

import org.apache.jmeter.assertions.AssertionResult;
import org.apache.jmeter.samplers.SampleResult;
import org.w3c.dom.NodeList;

import com.arosii.jmeter.protocol.nsp.assertion.AbstractNSPAssertion;
import com.arosii.jmeter.protocol.sdm.sampler.DetGodeCPROpslagRequestSampler.Version;

import dk.sosi.seal.model.Reply;

/**
 * 
 * @author Jacob
 */
public class DetGodeCPROpslagRequestAssertion extends AbstractNSPAssertion {

	private static final long serialVersionUID = 1L;

	private static final NamespaceContext NAMESPACE_CONTEXT = namespaceContext("ns100", "http://rep.oio.dk/medcom.sundcom.dk/xml/schemas/2007/02/01/", "ns102", "urn:oio:medcom:cprservice:1.0.2");

	public static final String VERSION = "DetGodeCPROpslagRequestAssertion.Version";

	public static final String WITH_HEALTHCARE_INFORMATION = "DetGodeCPROpslagRequestAssertion.WithHealthCareInformation";

	/**
	 * {@inheritDoc}
	 */
	@Override
	public AssertionResult getResult(SampleResult sample) {

		// no error as default
		AssertionResult assertion = new AssertionResult(getName());

		if (sample.getResponseData().length == 0) {
			return assertion.setResultForNull();
		}

		assertion.setFailure(false);

		if (sample.isSuccessful()) {
			Reply reply = getReply();
			if (reply.isFault()) {
				assertion.setResultForFailure("Fault not expected: Code='" + reply.getFaultCode() + "', String='" + reply.getFaultString() + "'");
			} else {
				try {
					XPath personsXPath = XPATH_FACTORY.newXPath();
					personsXPath.setNamespaceContext(NAMESPACE_CONTEXT);
					Version version = Version.valueOf(getPropertyAsString(VERSION, Version.V_1_0_0.name()));
					boolean withHealthCareInformation = getPropertyAsBoolean(WITH_HEALTHCARE_INFORMATION, false);
					String namespacePrefix;
					switch (version) {
					case V_1_0_0:
						namespacePrefix = "ns100";
						break;
					case V_1_0_2:
						namespacePrefix = "ns102";
						break;
					default:
						throw new IllegalArgumentException(String.valueOf(version));
					}
					NodeList personInformationStructure = (NodeList) personsXPath.evaluate("//" + namespacePrefix + ":PersonInformationStructure", reply.getBody(), XPathConstants.NODESET);
					if (personInformationStructure.getLength() != 1) {
						assertion.setResultForFailure("Expected one person information structure but " + personInformationStructure.getLength() + " was returned.");
					} else if (withHealthCareInformation) {
						NodeList personHealthCareInformationStructure = (NodeList) personsXPath.evaluate("//" + namespacePrefix + ":PersonHealthCareInformationStructure", reply.getBody(), XPathConstants.NODESET);
						if (personHealthCareInformationStructure.getLength() != 1) {
							assertion.setResultForFailure("Expected one healthcare information structure but " + personHealthCareInformationStructure.getLength() + " was returned.");
						}
					}
				} catch (NumberFormatException e) {
					assertion.setResultForFailure(e.getMessage());
				} catch (XPathExpressionException e) {
					assertion.setResultForFailure(e.getMessage());
				}
			}
		} else {
			assertion.setResultForFailure("Sample not successful");
		}

		return assertion;
	}

}
