/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sdm.assertion;

import javax.xml.namespace.NamespaceContext;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;

import org.apache.jmeter.assertions.AssertionResult;
import org.apache.jmeter.samplers.SampleResult;
import org.w3c.dom.NodeList;

import com.arosii.jmeter.protocol.nsp.assertion.AbstractNSPAssertion;

import dk.sosi.seal.model.Reply;

/**
 * 
 * @author Jacob
 */
public class PersonLookupRequestAssertion extends AbstractNSPAssertion {

	private static final long serialVersionUID = 1L;

	private static final NamespaceContext NAMESPACE_CONTEXT = namespaceContext("ns17", "http://rep.oio.dk/medcom.sundcom.dk/xml/schemas/2007/02/01/");

	public static final String PERSON_COUNT = "PersonLookupRequestAssertion.PersonCount";

	/**
	 * {@inheritDoc}
	 */
	@Override
	public AssertionResult getResult(SampleResult sample) {

		// no error as default
		AssertionResult assertion = new AssertionResult(getName());

		if (sample.getResponseData().length == 0) {
			return assertion.setResultForNull();
		}

		assertion.setFailure(false);

		if (sample.isSuccessful()) {
			Reply reply = getReply();
			if (reply.isFault()) {
				assertion.setResultForFailure("Fault not expected: Code='" + reply.getFaultCode() + "', String='" + reply.getFaultString() + "'");
			} else {
				String personCount = getPropertyAsString(PERSON_COUNT, "");
				if (personCount.trim().length() > 0) {
					try {
						XPath personsXPath = XPATH_FACTORY.newXPath();
						personsXPath.setNamespaceContext(NAMESPACE_CONTEXT);
						NodeList persons = (NodeList) personsXPath.evaluate("//ns17:PersonInformationStructure", reply.getBody(), XPathConstants.NODESET);
						if (persons.getLength() != Integer.valueOf(personCount)) {
							assertion.setResultForFailure("Expected " + personCount + " persons but " + persons.getLength() + " was returned.");
						}
					} catch (NumberFormatException e) {
						assertion.setResultForFailure(e.getMessage());
					} catch (XPathExpressionException e) {
						assertion.setResultForFailure(e.getMessage());
					}
				}
			}
		} else {
			assertion.setResultForFailure("Sample not successful");
		}

		return assertion;
	}

}
