/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sdm.assertion.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.FlowLayout;
import java.util.EnumSet;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;

import com.arosii.jmeter.protocol.sdm.assertion.DetGodeCPROpslagRequestAssertion;
import com.arosii.jmeter.protocol.sdm.control.gui.DetGodeCPROpslagRequestSamplerGui.VersionRenderer;
import com.arosii.jmeter.protocol.sdm.sampler.DetGodeCPROpslagRequestSampler;
import com.arosii.jmeter.protocol.sdm.sampler.DetGodeCPROpslagRequestSampler.Version;
import com.arosii.jmeter.resources.sdm.Messages;

/**
 * 
 * @author Jacob
 */
public class DetGodeCPROpslagRequestAssertionGui extends AbstractSDMAssertionGui {

	private static final long serialVersionUID = 1L;

	private JComboBox version;
	
	private JCheckBox withHealthCareInformation;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getLabelResource() {
		return "sdm_assert_det_gode_cpr_opslag_request";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public TestElement createTestElement() {
		DetGodeCPROpslagRequestAssertion assertion = new DetGodeCPROpslagRequestAssertion();
		modifyTestElement(assertion);
		return assertion;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected Component createConfigurationPanel() {
		final VerticalPanel configurationPanel = new VerticalPanel();
		appendQueryPanel(configurationPanel);
		clearGui();
		return configurationPanel;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void configure(TestElement element) {
		super.configure(element);
		version.setSelectedItem(Version.valueOf(element.getPropertyAsString(DetGodeCPROpslagRequestAssertion.VERSION, Version.V_1_0_0.name())));
		withHealthCareInformation.setSelected(element.getPropertyAsBoolean(DetGodeCPROpslagRequestAssertion.WITH_HEALTHCARE_INFORMATION, false));
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void modifyTestElement(TestElement element) {
		super.modifyTestElement(element);
		element.setProperty(DetGodeCPROpslagRequestAssertion.VERSION, ((Version)version.getSelectedItem()).name());
		element.setProperty(DetGodeCPROpslagRequestAssertion.WITH_HEALTHCARE_INFORMATION, withHealthCareInformation.isSelected());
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void clearGui() {
		super.clearGui();
		version.setSelectedIndex(0);
		withHealthCareInformation.setSelected(false);
	}

	private void appendQueryPanel(VerticalPanel configurationPanel) {
		JPanel cprPanel = new VerticalPanel();
		cprPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("sdm_person_lookup_request_search")));
		cprPanel.add(getVersionPanel());
		cprPanel.add(getWithHealthCareInformationPanel());
		configurationPanel.add(cprPanel);
	}

	private JPanel getVersionPanel() {
		version = new JComboBox(EnumSet.allOf(DetGodeCPROpslagRequestSampler.Version.class).toArray());
		version.setRenderer(new VersionRenderer());
		JLabel versionLabel = new JLabel(Messages.getResString("sdm_det_gode_cpr_opslag_version"));

		JPanel versionPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		versionPanel.add(versionLabel);
		versionPanel.add(version);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(versionPanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getWithHealthCareInformationPanel() {
		withHealthCareInformation = new JCheckBox();
		JLabel withLabel = new JLabel(Messages.getResString("sdm_det_gode_cpr_opslag_with_healthcare_information"));

		JPanel withPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		withPanel.add(withLabel);
		withPanel.add(withHealthCareInformation);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(withPanel, BorderLayout.CENTER);

		return panel;
	}

}
