/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sdm.assertion.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.FlowLayout;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;

import com.arosii.jmeter.protocol.sdm.assertion.PersonLookupRequestAssertion;
import com.arosii.jmeter.resources.sdm.Messages;

/**
 * 
 * @author Jacob
 */
public class PersonLookupRequestAssertionGui extends AbstractSDMAssertionGui {

	private static final long serialVersionUID = 1L;

	private JTextField personCount;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getLabelResource() {
		return "sdm_assert_person_lookup_request";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public TestElement createTestElement() {
		PersonLookupRequestAssertion assertion = new PersonLookupRequestAssertion();
		modifyTestElement(assertion);
		return assertion;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected Component createConfigurationPanel() {
		final VerticalPanel configurationPanel = new VerticalPanel();
		appendResponsePanel(configurationPanel);
		return configurationPanel;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void configure(TestElement element) {
		super.configure(element);
		personCount.setText(element.getPropertyAsString(PersonLookupRequestAssertion.PERSON_COUNT, ""));
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void modifyTestElement(TestElement element) {
		super.modifyTestElement(element);
		element.setProperty(PersonLookupRequestAssertion.PERSON_COUNT, personCount.getText());
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void clearGui() {
		super.clearGui();
		personCount.setText("");
	}

	private void appendResponsePanel(VerticalPanel configurationPanel) {
		JPanel responsePanel = new VerticalPanel();
		responsePanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("sdm_person_lookup_request_response")));
		responsePanel.add(getPersonCountPanel());
		configurationPanel.add(responsePanel);
	}

	private Component getPersonCountPanel() {
		personCount = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("sdm_person_count"));

		JPanel personCountPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		personCountPanel.add(personCount);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(personCountPanel, BorderLayout.CENTER);

		return panel;
	}

}
