/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sdm.control;

import java.awt.Component;
import java.util.EnumSet;
import java.util.HashSet;
import java.util.Set;

import javax.swing.DefaultListCellRenderer;
import javax.swing.JList;

import com.arosii.jmeter.resources.sdm.Messages;

/**
 * 
 * @author Jacob
 */
public enum SDMRegister {

	NOT_FILLED("", "", null),

	CPR_PERSON_V1("cpr", "person", 1),
	CPR_CHILD_RELATION_V1("cpr", "barnrelation", 1),
	CPR_MARITAL_STATUS_V1("cpr", "civilstand", 1),
	CPR_BIRTH_PLACE_V1("cpr", "foedselsregistreringsoplysninger", 1),
	CPR_CHURCH_MEMBERSHIP_V1("cpr", "folkekirkeoplysninger", 1),
	CPR_CUSTODY_RELATION_V1("cpr", "foraeldremyndighedrelation", 1),
	CPR_INCIDENT_V1("cpr", "haendelse", 1),
	CPR_MUNICIPAL_RELATIONS_V1("cpr", "kommunaleforhold", 1),
	CPR_CITIZENSHIP_V1("cpr", "statsborgerskab", 1),
	CPR_EXIT_DATA_V1("cpr", "udrejseoplysninger", 1),
	CPR_GUARDIAN_RELATION_V1("cpr", "umyndiggoerlesevaergerelation", 1),
	CPR_ELECTION_DETAILS_V1("cpr", "valgoplysninger", 1),
	AUTHORIZATION_V1("autorisationsregisteret", "autorisation", 1),
	SECURED_SECURED_V1("sikrede", "sikrede", 1),
	SECURED_RELATION_V1("sikrede", "sikredeyderrelation", 1),
	SECURED_CARD_V1("sikrede", "saerligsundhedskort", 1),
	DOSAGE_VERSION_V1("doseringsforslag", "version", 1),
	DOSAGE_UNIT_V1("doseringsforslag", "dosageunit", 1),
	DOSAGE_STRUCTURE_V1("doseringsforslag", "dosagestructure", 1),
	DOSAGE_DRUG_V1("doseringsforslag", "drug", 1),
	DOSAGE_STRUCTURE_RELTATION_V1("doseringsforslag", "drugdosagestructurerelation", 1),

	;

	private static final Set<String> registerNames = new HashSet<String>();

	static {
		for (SDMRegister identity : EnumSet.allOf(SDMRegister.class)) {
			registerNames.add(registerName(identity));
		}
	}

	private final String name;

	private final String datatype;

	private final Integer version;

	private SDMRegister(String name, String datatype, Integer version) {
		this.name = name;
		this.datatype = datatype;
		this.version = version;
	}

	public String getName() {
		return name;
	}

	public String getDatatype() {
		return datatype;
	}

	public Integer getVersion() {
		return version;
	}

	public static SDMRegister[] all(boolean includeNothingSelected) {
		EnumSet<SDMRegister> all = EnumSet.allOf(SDMRegister.class);
		if (!includeNothingSelected) {
			all.remove(NOT_FILLED);
		}
		return (SDMRegister[]) all.toArray(new SDMRegister[all.size()]);
	}

	public static String registerName(SDMRegister register) {
		return Messages.getResString(SDMRegister.class.getSimpleName().toLowerCase() + '_' + register.name().toLowerCase());
	}

	public static boolean isRegisterName(String name) {
		return registerNames.contains(name);
	}

	public static class SDMRegisterRenderer extends DefaultListCellRenderer {

		private static final long serialVersionUID = 1L;

		private final String notFilled;

		public SDMRegisterRenderer(String notFilled) {
			this.notFilled = notFilled;
		}

		/**
		 * {@inheritDoc}
		 */
		@Override
		public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
			super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
			SDMRegister register = (SDMRegister) value;
			if (NOT_FILLED.equals(register)) {
				setText(notFilled);
			} else {
				setText(SDMRegister.registerName(register));
			}
			return this;
		}
	}
}
