/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sdm.control.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;

import com.arosii.jmeter.control.gui.ButtonClicker;
import com.arosii.jmeter.protocol.sdm.sampler.AuthorizationRequestSampler;
import com.arosii.jmeter.resources.sdm.Messages;

/**
 * 
 * @author Jacob
 */
public class AuthorizationRequestSamplerGui extends AbstractSDMSamplerGui {

	private static final long serialVersionUID = 1L;

	private JTextField cpr;

	private JCheckBox useExtractedData;

	private JTextField variable;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getLabelResource() {
		return "sdm_authorization_request";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public TestElement createTestElement() {
		AuthorizationRequestSampler sampler = new AuthorizationRequestSampler();
		modifyTestElement(sampler);
		return sampler;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected Component createConfigurationPanel() {
		final VerticalPanel configurationPanel = new VerticalPanel();
		appendCPRPanel(configurationPanel);
		appendExtractedPanel(configurationPanel);
		return configurationPanel;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void configure(TestElement element) {
		super.configure(element);
		cpr.setText(element.getPropertyAsString(AuthorizationRequestSampler.CPR, ""));
		useExtractedData.setSelected(element.getPropertyAsBoolean(AuthorizationRequestSampler.USE_EXTRACTED_DATA, false));
		variable.setText(element.getPropertyAsString(AuthorizationRequestSampler.EXTRACTION_VARIABLE, ""));
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void modifyTestElement(TestElement element) {
		super.modifyTestElement(element);
		element.setProperty(AuthorizationRequestSampler.CPR, cpr.getText());
		element.setProperty(AuthorizationRequestSampler.USE_EXTRACTED_DATA, useExtractedData.isSelected());
		element.setProperty(AuthorizationRequestSampler.EXTRACTION_VARIABLE, variable.getText());
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void clearGui() {
		super.clearGui();
		useExtractedData.setSelected(false);
		variable.setText("");
		cpr.setText("");
	}

	private void appendCPRPanel(VerticalPanel configurationPanel) {
		JPanel cprPanel = new VerticalPanel();
		cprPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("sdm_authorization_request_search")));
		cprPanel.add(getCPRPanel());
		configurationPanel.add(cprPanel);
	}

	private void appendExtractedPanel(VerticalPanel configurationPanel) {
		JPanel cprPanel = new VerticalPanel();
		cprPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("sdm_person_lookup_request_extracted")));
		cprPanel.add(getUseExtractedDataPanel());
		configurationPanel.add(cprPanel);
	}

	private JPanel getCPRPanel() {
		cpr = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("sdm_cpr"));

		JPanel cprPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		cprPanel.add(cpr);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(cprPanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getUseExtractedDataPanel() {
		useExtractedData = new JCheckBox();
		useExtractedData.addActionListener(new SearchSelectionListener());
		variable = new JTextField(20);
		JLabel useExtractedDataLabel = new JLabel(Messages.getResString("sdm_use_extracted_data"));
		JLabel variableLabel = new JLabel(Messages.getResString("nsp_variable"));
		
		variable.addMouseListener(new ButtonClicker(useExtractedData, variable));
		useExtractedDataLabel.addMouseListener(new ButtonClicker(useExtractedData, variable));
		variableLabel.addMouseListener(new ButtonClicker(useExtractedData, variable));

		JPanel extractedPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		extractedPanel.add(useExtractedData);
		extractedPanel.add(variableLabel);
		extractedPanel.add(variable);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(useExtractedDataLabel, BorderLayout.WEST);
		panel.add(extractedPanel, BorderLayout.CENTER);

		return panel;
	}

	private void updateFields() {
		cpr.setEnabled(!useExtractedData.isSelected());
		cpr.setText(!useExtractedData.isSelected() ? cpr.getText() : "");
		variable.setEnabled(useExtractedData.isSelected());
		variable.setText(useExtractedData.isSelected() ? variable.getText() : "");
	}

	private class SearchSelectionListener implements ActionListener {

		/**
		 * {@inheritDoc}
		 */
		@Override
		public void actionPerformed(ActionEvent e) {
			updateFields();
		}
	}
}
