/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sdm.control.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.EnumSet;

import javax.swing.BorderFactory;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;

import com.arosii.jmeter.control.gui.ButtonClicker;
import com.arosii.jmeter.protocol.sdm.sampler.DetGodeCPROpslagRequestSampler;
import com.arosii.jmeter.protocol.sdm.sampler.DetGodeCPROpslagRequestSampler.Version;
import com.arosii.jmeter.resources.sdm.Messages;

/**
 * 
 * @author Jacob
 */
public class DetGodeCPROpslagRequestSamplerGui extends AbstractSDMSamplerGui {

	private static final long serialVersionUID = 1L;

	private JComboBox version;
	
	private JCheckBox withHealthCareInformation;

	private JTextField cpr;

	private JCheckBox useExtractedData;

	private JTextField variable;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getLabelResource() {
		return "sdm_det_gode_cpr_opslag_request";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public TestElement createTestElement() {
		DetGodeCPROpslagRequestSampler sampler = new DetGodeCPROpslagRequestSampler();
		modifyTestElement(sampler);
		return sampler;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected Component createConfigurationPanel() {
		final VerticalPanel configurationPanel = new VerticalPanel();
		appendQueryPanel(configurationPanel);
		appendExtractedPanel(configurationPanel);
		clearGui();
		updateFields();
		return configurationPanel;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void configure(TestElement element) {
		super.configure(element);
		version.setSelectedItem(Version.valueOf(element.getPropertyAsString(DetGodeCPROpslagRequestSampler.VERSION, Version.V_1_0_0.name())));
		withHealthCareInformation.setSelected(element.getPropertyAsBoolean(DetGodeCPROpslagRequestSampler.WITH_HEALTHCARE_INFORMATION, false));
		cpr.setText(element.getPropertyAsString(DetGodeCPROpslagRequestSampler.CPR, ""));
		useExtractedData.setSelected(element.getPropertyAsBoolean(DetGodeCPROpslagRequestSampler.USE_EXTRACTED_DATA, false));
		variable.setText(element.getPropertyAsString(DetGodeCPROpslagRequestSampler.EXTRACTION_VARIABLE, ""));
		updateFields();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void modifyTestElement(TestElement element) {
		super.modifyTestElement(element);
		element.setProperty(DetGodeCPROpslagRequestSampler.VERSION, ((Version)version.getSelectedItem()).name());
		element.setProperty(DetGodeCPROpslagRequestSampler.WITH_HEALTHCARE_INFORMATION, withHealthCareInformation.isSelected());
		element.setProperty(DetGodeCPROpslagRequestSampler.CPR, cpr.getText());
		element.setProperty(DetGodeCPROpslagRequestSampler.USE_EXTRACTED_DATA, useExtractedData.isSelected());
		element.setProperty(DetGodeCPROpslagRequestSampler.EXTRACTION_VARIABLE, variable.getText());
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void clearGui() {
		super.clearGui();
		version.setSelectedIndex(0);
		withHealthCareInformation.setSelected(false);
		cpr.setText("");
		useExtractedData.setSelected(false);
		variable.setText("");
	}

	private void appendQueryPanel(VerticalPanel configurationPanel) {
		JPanel cprPanel = new VerticalPanel();
		cprPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("sdm_person_lookup_request_search")));
		cprPanel.add(getVersionPanel());
		cprPanel.add(getWithHealthCareInformationPanel());
		cprPanel.add(getCPRPanel());
		configurationPanel.add(cprPanel);
	}

	private void appendExtractedPanel(VerticalPanel configurationPanel) {
		JPanel cprPanel = new VerticalPanel();
		cprPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("sdm_person_lookup_request_extracted")));
		cprPanel.add(getUseExtractedDataPanel());
		configurationPanel.add(cprPanel);
	}

	private JPanel getVersionPanel() {
		version = new JComboBox(EnumSet.allOf(DetGodeCPROpslagRequestSampler.Version.class).toArray());
		version.setRenderer(new VersionRenderer());
		JLabel versionLabel = new JLabel(Messages.getResString("sdm_det_gode_cpr_opslag_version"));

		JPanel versionPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		versionPanel.add(versionLabel);
		versionPanel.add(version);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(versionPanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getWithHealthCareInformationPanel() {
		withHealthCareInformation = new JCheckBox();
		JLabel withLabel = new JLabel(Messages.getResString("sdm_det_gode_cpr_opslag_with_healthcare_information"));

		JPanel withPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		withPanel.add(withLabel);
		withPanel.add(withHealthCareInformation);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(withPanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getCPRPanel() {
		cpr = new JTextField(20);
		JLabel cprLabel = new JLabel(Messages.getResString("sdm_cpr"));

		JPanel cprPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		cprPanel.add(cprLabel);
		cprPanel.add(cpr);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(cprPanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getUseExtractedDataPanel() {
		useExtractedData = new JCheckBox();
		useExtractedData.addActionListener(new SearchSelectionListener());
		variable = new JTextField(20);
		JLabel useExtractedDataLabel = new JLabel(Messages.getResString("sdm_use_extracted_data"));
		JLabel variableLabel = new JLabel(Messages.getResString("nsp_variable"));

		variable.addMouseListener(new ButtonClicker(useExtractedData, variable));
		useExtractedDataLabel.addMouseListener(new ButtonClicker(useExtractedData, variable));
		variableLabel.addMouseListener(new ButtonClicker(useExtractedData, variable));

		JPanel extractedPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		extractedPanel.add(useExtractedData);
		extractedPanel.add(variableLabel);
		extractedPanel.add(variable);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(useExtractedDataLabel, BorderLayout.WEST);
		panel.add(extractedPanel, BorderLayout.CENTER);

		return panel;
	}

	private void updateFields() {
		variable.setEnabled(useExtractedData.isSelected());
		variable.setText(useExtractedData.isSelected() ? variable.getText() : "");
	}

	private class SearchSelectionListener implements ActionListener {

		/**
		 * {@inheritDoc}
		 */
		@Override
		public void actionPerformed(ActionEvent e) {
			updateFields();
		}
	}

	public static class VersionRenderer extends DefaultListCellRenderer {

		private static final long serialVersionUID = 1L;

		/**
		 * {@inheritDoc}
		 */
		@Override
		public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
			super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
			Version version = (Version) value;
			switch (version) {
			case V_1_0_0:
				setText("1.0.0");
				break;
			case V_1_0_2:
				setText("1.0.2");
				break;
			}
			return this;
		}
	}
}
