/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sdm.control.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;

import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;

import com.arosii.jmeter.control.gui.ButtonClicker;
import com.arosii.jmeter.protocol.sdm.sampler.PersonLookupRequestSampler;
import com.arosii.jmeter.resources.sdm.Messages;

/**
 * 
 * @author Jacob
 */
public class PersonLookupRequestSamplerGui extends AbstractSDMSamplerGui {

	private static final long serialVersionUID = 1L;

	private ButtonGroup searchGroup;

	private JRadioButton cprSearch;

	private JTextField cpr;

	private JRadioButton nameSearch;

	private JTextField firstName;

	private JTextField middleName;

	private JTextField lastName;

	private JRadioButton birthdaySearch;

	private JTextField birthday;

	private JCheckBox useExtractedData;

	private JTextField variable;

	private JTextField extractedCount;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getLabelResource() {
		return "sdm_person_lookup_request";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public TestElement createTestElement() {
		PersonLookupRequestSampler sampler = new PersonLookupRequestSampler();
		modifyTestElement(sampler);
		return sampler;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected Component createConfigurationPanel() {
		final VerticalPanel configurationPanel = new VerticalPanel();
		appendQueryPanel(configurationPanel);
		appendExtractedPanel(configurationPanel);
		clearGui();
		updateFields();
		return configurationPanel;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void configure(TestElement element) {
		super.configure(element);
		cpr.setText(element.getPropertyAsString(PersonLookupRequestSampler.CPR, ""));
		firstName.setText(element.getPropertyAsString(PersonLookupRequestSampler.FIRST_NAME, ""));
		middleName.setText(element.getPropertyAsString(PersonLookupRequestSampler.MIDDLE_NAME, ""));
		lastName.setText(element.getPropertyAsString(PersonLookupRequestSampler.LAST_NAME, ""));
		birthday.setText(element.getPropertyAsString(PersonLookupRequestSampler.BIRTHDAY, ""));
		String queryMethod = element.getPropertyAsString(PersonLookupRequestSampler.QUERY_METHOD, PersonLookupRequestSampler.CPR);
		if (queryMethod.equals(PersonLookupRequestSampler.CPR)) {
			cprSearch.setSelected(true);
		} else if (queryMethod.equals(PersonLookupRequestSampler.FIRST_NAME)) {
			nameSearch.setSelected(true);
		} else if (queryMethod.equals(PersonLookupRequestSampler.BIRTHDAY)) {
			birthdaySearch.setSelected(true);
		}
		useExtractedData.setSelected(element.getPropertyAsBoolean(PersonLookupRequestSampler.USE_EXTRACTED_DATA, false));
		variable.setText(element.getPropertyAsString(PersonLookupRequestSampler.EXTRACTION_VARIABLE, ""));
		extractedCount.setText(element.getPropertyAsString(PersonLookupRequestSampler.EXTRACTED_COUNT, ""));
		updateFields();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void modifyTestElement(TestElement element) {
		super.modifyTestElement(element);
		element.setProperty(PersonLookupRequestSampler.CPR, cpr.getText());
		element.setProperty(PersonLookupRequestSampler.FIRST_NAME, firstName.getText());
		element.setProperty(PersonLookupRequestSampler.MIDDLE_NAME, middleName.getText());
		element.setProperty(PersonLookupRequestSampler.LAST_NAME, lastName.getText());
		element.setProperty(PersonLookupRequestSampler.BIRTHDAY, birthday.getText());
		if (cprSearch.isSelected()) {
			element.setProperty(PersonLookupRequestSampler.QUERY_METHOD, PersonLookupRequestSampler.CPR);
		} else if (nameSearch.isSelected()) {
			element.setProperty(PersonLookupRequestSampler.QUERY_METHOD, PersonLookupRequestSampler.FIRST_NAME);
		} else if (birthdaySearch.isSelected()) {
			element.setProperty(PersonLookupRequestSampler.QUERY_METHOD, PersonLookupRequestSampler.BIRTHDAY);
		}
		element.setProperty(PersonLookupRequestSampler.USE_EXTRACTED_DATA, useExtractedData.isSelected());
		element.setProperty(PersonLookupRequestSampler.EXTRACTION_VARIABLE, variable.getText());
		element.setProperty(PersonLookupRequestSampler.EXTRACTED_COUNT, extractedCount.getText());
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void clearGui() {
		super.clearGui();
		cpr.setText("");
		firstName.setText("");
		middleName.setText("");
		lastName.setText("");
		birthday.setText("");
		cprSearch.setSelected(true);
		useExtractedData.setSelected(false);
		variable.setText("");
		extractedCount.setText("");
	}

	private void appendQueryPanel(VerticalPanel configurationPanel) {
		JPanel cprPanel = new VerticalPanel();
		cprPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("sdm_person_lookup_request_search")));
		cprPanel.add(getCPRPanel());
		cprPanel.add(getNameDetailsPanel());
		cprPanel.add(getBirthdayPanel());
		configurationPanel.add(cprPanel);
	}

	private void appendExtractedPanel(VerticalPanel configurationPanel) {
		JPanel cprPanel = new VerticalPanel();
		cprPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("sdm_person_lookup_request_extracted")));
		cprPanel.add(getUseExtractedDataPanel());
		configurationPanel.add(cprPanel);
	}

	private JPanel getCPRPanel() {
		searchGroup = new ButtonGroup();
		cprSearch = new JRadioButton();
		cprSearch.addActionListener(new SearchSelectionListener());
		searchGroup.add(cprSearch);
		cpr = new JTextField(20);
		JLabel cprLabel = new JLabel(Messages.getResString("sdm_cpr"));
		
		cpr.addMouseListener(new ButtonClicker(cprSearch, cpr));
		cprLabel.addMouseListener(new ButtonClicker(cprSearch, cpr));

		JPanel cprPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		cprPanel.add(cprLabel);
		cprPanel.add(cpr);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(cprSearch, BorderLayout.WEST);
		panel.add(cprPanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getNameDetailsPanel() {
		nameSearch = new JRadioButton();
		nameSearch.addActionListener(new SearchSelectionListener());
		searchGroup.add(nameSearch);
		firstName = new JTextField(20);
		middleName = new JTextField(20);
		lastName = new JTextField(20);
		JLabel firstNameLabel = new JLabel(Messages.getResString("sdm_first_name"));
		JLabel middleNameLabel = new JLabel(Messages.getResString("sdm_middle_name"));
		JLabel lastNameLabel = new JLabel(Messages.getResString("sdm_last_name"));
		
		firstName.addMouseListener(new ButtonClicker(nameSearch, firstName));
		middleName.addMouseListener(new ButtonClicker(nameSearch, middleName));
		lastName.addMouseListener(new ButtonClicker(nameSearch, lastName));
		firstNameLabel.addMouseListener(new ButtonClicker(nameSearch, firstName));
		middleNameLabel.addMouseListener(new ButtonClicker(nameSearch, middleName));
		lastNameLabel.addMouseListener(new ButtonClicker(nameSearch, lastName));

		JPanel namePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		namePanel.add(firstNameLabel);
		namePanel.add(firstName);
		namePanel.add(middleNameLabel);
		namePanel.add(middleName);
		namePanel.add(lastNameLabel);
		namePanel.add(lastName);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(nameSearch, BorderLayout.WEST);
		panel.add(namePanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getBirthdayPanel() {
		birthdaySearch = new JRadioButton();
		birthdaySearch.addActionListener(new SearchSelectionListener());
		searchGroup.add(birthdaySearch);
		birthday = new JTextField(20);
		JLabel birthdayLabel = new JLabel(Messages.getResString("sdm_birthday"));
		
		birthday.addMouseListener(new ButtonClicker(birthdaySearch, birthday));
		birthdayLabel.addMouseListener(new ButtonClicker(birthdaySearch, birthday));

		JPanel birthdayPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		birthdayPanel.add(birthdayLabel);
		birthdayPanel.add(birthday);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(birthdaySearch, BorderLayout.WEST);
		panel.add(birthdayPanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getUseExtractedDataPanel() {
		useExtractedData = new JCheckBox();
		useExtractedData.addActionListener(new SearchSelectionListener());
		variable = new JTextField(20);
		extractedCount = new JTextField(10);
		JLabel useExtractedDataLabel = new JLabel(Messages.getResString("sdm_use_extracted_data"));
		JLabel variableLabel = new JLabel(Messages.getResString("nsp_variable"));
		JLabel extractedCountLabel = new JLabel(Messages.getResString("sdm_extracted_count"));
		
		variable.addMouseListener(new ButtonClicker(useExtractedData, variable));
		extractedCount.addMouseListener(new ButtonClicker(useExtractedData, extractedCount));
		useExtractedDataLabel.addMouseListener(new ButtonClicker(useExtractedData, variable));
		variableLabel.addMouseListener(new ButtonClicker(useExtractedData, variable));
		extractedCountLabel.addMouseListener(new ButtonClicker(useExtractedData, extractedCount));

		JPanel extractedPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		extractedPanel.add(useExtractedData);
		extractedPanel.add(variableLabel);
		extractedPanel.add(variable);
		extractedPanel.add(extractedCountLabel);
		extractedPanel.add(extractedCount);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(useExtractedDataLabel, BorderLayout.WEST);
		panel.add(extractedPanel, BorderLayout.CENTER);

		return panel;
	}

	private void updateFields() {
		cpr.setEnabled(cprSearch.isSelected() && !useExtractedData.isSelected());
		cpr.setText(cprSearch.isSelected() && !useExtractedData.isSelected() ? cpr.getText() : "");
		firstName.setEnabled(nameSearch.isSelected() && !useExtractedData.isSelected());
		firstName.setText(nameSearch.isSelected() && !useExtractedData.isSelected() ? firstName.getText() : "");
		middleName.setEnabled(nameSearch.isSelected() && !useExtractedData.isSelected());
		middleName.setText(nameSearch.isSelected() && !useExtractedData.isSelected() ? middleName.getText() : "");
		lastName.setEnabled(nameSearch.isSelected() && !useExtractedData.isSelected());
		lastName.setText(nameSearch.isSelected() && !useExtractedData.isSelected() ? lastName.getText() : "");
		birthday.setEnabled(birthdaySearch.isSelected() && !useExtractedData.isSelected());
		birthday.setText(birthdaySearch.isSelected() && !useExtractedData.isSelected() ? birthday.getText() : "");
		variable.setEnabled(useExtractedData.isSelected());
		variable.setText(useExtractedData.isSelected() ? variable.getText() : "");
		extractedCount.setEnabled(useExtractedData.isSelected());
		extractedCount.setText(useExtractedData.isSelected() ? extractedCount.getText() : "");
	}

	private class SearchSelectionListener implements ActionListener {

		/**
		 * {@inheritDoc}
		 */
		@Override
		public void actionPerformed(ActionEvent e) {
			updateFields();
		}
	}

}
