/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sdm.control.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.BorderFactory;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;

import com.arosii.jmeter.protocol.sdm.control.SDMRegister;
import com.arosii.jmeter.protocol.sdm.control.SDMRegister.SDMRegisterRenderer;
import com.arosii.jmeter.protocol.sdm.sampler.ReplicationRequestSampler;
import com.arosii.jmeter.resources.sdm.Messages;

/**
 * 
 * @author Jacob
 */
public class ReplicationRequestSamplerGui extends AbstractSDMSamplerGui {

	private static final long serialVersionUID = 1L;

	private static final String PREDEFINED_IDENTITY = "ReplicationRequestSamplerGui.Predefined.Register";

	private JComboBox predefinedRegisters;

	private JTextField registerName;

	private JTextField registerDatatype;

	private JTextField registerVersion;

	private JTextField registerOffset;

	private JTextField registerMaxRowCount;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getLabelResource() {
		return "sdm_replication_request";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public TestElement createTestElement() {
		ReplicationRequestSampler sampler = new ReplicationRequestSampler();
		modifyTestElement(sampler);
		return sampler;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected Component createConfigurationPanel() {
		final VerticalPanel configurationPanel = new VerticalPanel();
		appendRegisterSelectionPanel(configurationPanel);
		appendRegisterOptionsPanel(configurationPanel);
		return configurationPanel;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void configure(TestElement element) {
		super.configure(element);
		String register = element.getPropertyAsString(PREDEFINED_IDENTITY, "");
		if (!register.equals("")) {
			predefinedRegisters.setSelectedItem(SDMRegister.valueOf(register));
		} else {
			predefinedRegisters.setSelectedItem(SDMRegister.NOT_FILLED);
		}

		registerName.setText(element.getPropertyAsString(ReplicationRequestSampler.REGISTER, ""));
		registerDatatype.setText(element.getPropertyAsString(ReplicationRequestSampler.DATATYPE, ""));
		registerVersion.setText(element.getPropertyAsString(ReplicationRequestSampler.VERSION, ""));
		registerOffset.setText(element.getPropertyAsString(ReplicationRequestSampler.OFFSET, ""));
		registerMaxRowCount.setText(element.getPropertyAsString(ReplicationRequestSampler.MAX_ROW_COUNT, ""));
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void modifyTestElement(TestElement element) {
		super.modifyTestElement(element);
		SDMRegister register = (SDMRegister) predefinedRegisters.getSelectedItem();
		if (register != null && register != SDMRegister.NOT_FILLED) {
			element.setProperty(PREDEFINED_IDENTITY, register.name());
		} else {
			element.setProperty(PREDEFINED_IDENTITY, "");
		}
		element.setProperty(ReplicationRequestSampler.REGISTER, registerName.getText());
		element.setProperty(ReplicationRequestSampler.DATATYPE, registerDatatype.getText());
		element.setProperty(ReplicationRequestSampler.VERSION, registerVersion.getText());
		element.setProperty(ReplicationRequestSampler.OFFSET, registerOffset.getText());
		element.setProperty(ReplicationRequestSampler.MAX_ROW_COUNT, registerMaxRowCount.getText());
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void clearGui() {
		super.clearGui();
		predefinedRegisters.setSelectedItem(SDMRegister.NOT_FILLED);
		registerName.setText("");
		registerDatatype.setText("");
		registerVersion.setText("");
		registerOffset.setText("");
		registerMaxRowCount.setText("");
	}

	private void appendRegisterSelectionPanel(VerticalPanel configurationPanel) {
		JPanel registerPanel = new VerticalPanel();
		registerPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("sdm_replication_request_register")));
		registerPanel.add(getPredefinedRegistersPanel());
		registerPanel.add(getRegisterDetailsPanel());
		configurationPanel.add(registerPanel);
	}

	private void appendRegisterOptionsPanel(VerticalPanel configurationPanel) {
		JPanel optionsPanel = new VerticalPanel();
		optionsPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("sdm_replication_request_options")));
		optionsPanel.add(getRegisterOffsetOptionPanel());
		optionsPanel.add(getRegisterMaximumOptionPanel());
		configurationPanel.add(optionsPanel);
	}

	private JPanel getPredefinedRegistersPanel() {
		predefinedRegisters = new JComboBox(SDMRegister.all(true));
		predefinedRegisters.setRenderer(new SDMRegisterRenderer(Messages.getResString("sdm_register_manually_edit")));
		predefinedRegisters.addItemListener(new SDMRegisterItemListener());
		JLabel label = new JLabel(Messages.getResString("sdm_predefined_registers"));

		JPanel predefinedIdentitiesPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		predefinedIdentitiesPanel.add(predefinedRegisters);

		JPanel identityPanel = new JPanel(new BorderLayout(5, 0));
		identityPanel.add(label, BorderLayout.WEST);
		identityPanel.add(predefinedIdentitiesPanel, BorderLayout.CENTER);

		return identityPanel;
	}

	private JPanel getRegisterDetailsPanel() {
		registerName = new JTextField(20);
		registerDatatype = new JTextField(20);
		registerVersion = new JTextField(10);
		JLabel label = new JLabel(Messages.getResString("sdm_register_name"));
		JLabel datatypeLabel = new JLabel(Messages.getResString("sdm_register_type"));
		JLabel versionLabel = new JLabel(Messages.getResString("sdm_register_version"));

		JPanel credentialVaultPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		credentialVaultPanel.add(registerName);
		credentialVaultPanel.add(datatypeLabel);
		credentialVaultPanel.add(registerDatatype);
		credentialVaultPanel.add(versionLabel);
		credentialVaultPanel.add(registerVersion);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(credentialVaultPanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getRegisterOffsetOptionPanel() {
		registerOffset = new JTextField(40);
		JLabel label = new JLabel(Messages.getResString("sdm_register_offset"));

		JPanel offsetPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		offsetPanel.add(registerOffset);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(offsetPanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getRegisterMaximumOptionPanel() {
		registerMaxRowCount = new JTextField(40);
		JLabel label = new JLabel(Messages.getResString("sdm_register_maxrowcount"));

		JPanel maximumPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		maximumPanel.add(registerMaxRowCount);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(maximumPanel, BorderLayout.CENTER);

		return panel;
	}

	private class SDMRegisterItemListener implements ItemListener {

		/**
		 * {@inheritDoc}
		 */
		@Override
		public void itemStateChanged(ItemEvent e) {
			if (e.getStateChange() == ItemEvent.SELECTED) {
				if (e.getItem() != SDMRegister.NOT_FILLED && e.getItem() != null) {
					SDMRegister register = (SDMRegister) e.getItem();
					if (getName().equals(getStaticLabel()) || SDMRegister.isRegisterName(getName()) || getName().trim().equals("")) {
						namePanel.setName(SDMRegister.registerName(register));
					}
					registerName.setText(register.getName());
					registerDatatype.setText(register.getDatatype());
					registerVersion.setText(register.getVersion() != null ? String.valueOf(register.getVersion()) : "");
					registerName.setEnabled(false);
					registerDatatype.setEnabled(false);
					registerVersion.setEnabled(false);
				} else {
					registerName.setEnabled(true);
					registerDatatype.setEnabled(true);
					registerVersion.setEnabled(true);
				}
			}
		}
	}
}
