/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sdm.extractor;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.Serializable;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.apache.jmeter.processor.PostProcessor;
import org.apache.jmeter.samplers.SampleResult;
import org.apache.jmeter.testelement.AbstractScopedTestElement;
import org.apache.jorphan.logging.LoggingManager;
import org.apache.log.Logger;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

import com.arosii.jmeter.protocol.sdm.control.SDMRegister;
import com.arosii.jmeter.protocol.sdm.extractor.registers.AuthorizationRegister;
import com.arosii.jmeter.protocol.sdm.extractor.registers.CprRegister;
import com.arosii.jmeter.protocol.sdm.extractor.registers.Extractor;
import com.arosii.jmeter.protocol.sdm.sampler.ReplicationRequestSampler;

/**
 * 
 * @author Jacob
 */
public class ReplicationRequestExtractor extends AbstractScopedTestElement implements PostProcessor, Serializable {

	private static final long serialVersionUID = 1L;

	private static final Logger log = LoggingManager.getLoggerForClass();

	private static final SAXParserFactory SAX_PARSER_FACTORY = SAXParserFactory.newInstance();

	public static final String REGISTER = "ReplicationRequestExtractor.Register";

	public static final String EXTRACTED_PERCENTAGE = "ReplicationRequestExtractor.Percentage";

	public static final String MAX_EXTRACTED = "ReplicationRequestExtractor.Max";

	public static final String MAX_READ = "ReplicationRequestExtractor.Max.Read";

	public static final String REGISTER_VARIABLE = "ReplicationRequestExtractor.Variable";

	public static final String REVISION_NUMBER_VARIABLE = "ReplicationRequestExtractor.RevisionNumberVariable";

	public static final String EXTRACTION_NOT_COMPLETE_VARIABLE = "ReplicationRequestExtractor.ExtractionNotCompleteVariable";

	static {
		SAX_PARSER_FACTORY.setNamespaceAware(true);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void process() {
		SampleResult previousResult = getThreadContext().getPreviousResult();
		if (previousResult == null) {
			return;
		}
		String maxRowCount = getThreadContext().getVariables().get(ReplicationRequestSampler.MAX_ROW_COUNT);
		if (maxRowCount != null && !maxRowCount.trim().isEmpty()) {
			setProperty(ReplicationRequestSampler.MAX_ROW_COUNT, maxRowCount);
		}

		String registerName = getPropertyAsString(REGISTER, "");

		Extractor<?> extractor;
		SDMRegister register = SDMRegister.valueOf(registerName);
		switch (register) {
		case NOT_FILLED:
			return;
		case CPR_PERSON_V1:
			extractor = CprRegister.Person.createParser(this);
			break;
		case AUTHORIZATION_V1:
			extractor = AuthorizationRegister.Authorization.createParser(this);
			break;
		default:
			throw new IllegalArgumentException(SDMRegister.registerName(register));
		}

		if (extractor != null) {
			try {
				SAXParser parser = SAX_PARSER_FACTORY.newSAXParser();
				previousResult.getResponseData();
				XMLReader xmlReader = parser.getXMLReader();
				xmlReader.setContentHandler(extractor);
				xmlReader.parse(new InputSource(new ByteArrayInputStream(previousResult.getResponseData())));
				extractor.saveVariables(getThreadContext().getVariables());
			} catch (ParserConfigurationException e) {
				log.error(e.getMessage(), e);
			} catch (SAXException e) {
				log.error(e.getMessage(), e);
			} catch (IOException e) {
				log.error(e.getMessage(), e);
			}
		}
	}
}
