/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sdm.extractor.gui;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.Collection;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;

import com.arosii.jmeter.control.gui.ButtonClicker;
import com.arosii.jmeter.protocol.sdm.control.SDMRegister;
import com.arosii.jmeter.protocol.sdm.control.SDMRegister.SDMRegisterRenderer;
import com.arosii.jmeter.protocol.sdm.extractor.ReplicationRequestExtractor;
import com.arosii.jmeter.protocol.sdm.extractor.registers.AuthorizationRegister;
import com.arosii.jmeter.protocol.sdm.extractor.registers.CprRegister;
import com.arosii.jmeter.resources.sdm.Messages;

/**
 * 
 * @author Jacob
 */
public class ReplicationRequestExtractorGui extends AbstractSDMPostProcesorGui {

	private static final long serialVersionUID = 1L;

	private Collection<JCheckBox> extractionOptions;

	private JComboBox predefinedRegisters;

	private JPanel registerFields;

	private CardLayout registerFieldsCardLayout;

	private JCheckBox cprPersonExtractCPR;

	private JCheckBox cprPersonExtractName;

	private JCheckBox cprPersonExtractBirthday;

	private JCheckBox authorizationExtractCPR;

	private JCheckBox authorizationExtractName;

	private JTextField extractionPercentage;

	private JTextField extractionMax;

	private JTextField readMax;

	private JTextField registerVariable;

	private JTextField revisionNumberVariable;

	private JTextField extractionNotCompleteVariable;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getLabelResource() {
		return "sdm_replication_request_extractor";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected Component createConfigurationPanel() {
		final VerticalPanel configurationPanel = new VerticalPanel();
		appendScopePanel(configurationPanel);
		appendRegisterPanel(configurationPanel);
		return configurationPanel;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public TestElement createTestElement() {
		ReplicationRequestExtractor extractor = new ReplicationRequestExtractor();
		modifyTestElement(extractor);
		return extractor;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void configure(TestElement element) {
		super.configure(element);
		predefinedRegisters.setSelectedItem(SDMRegister.valueOf(element.getPropertyAsString(ReplicationRequestExtractor.REGISTER, SDMRegister.CPR_PERSON_V1.name())));
		extractionPercentage.setText(element.getPropertyAsString(ReplicationRequestExtractor.EXTRACTED_PERCENTAGE, ""));
		extractionMax.setText(element.getPropertyAsString(ReplicationRequestExtractor.MAX_EXTRACTED, ""));
		readMax.setText(element.getPropertyAsString(ReplicationRequestExtractor.MAX_READ, ""));
		registerVariable.setText(element.getPropertyAsString(ReplicationRequestExtractor.REGISTER_VARIABLE, ""));
		extractionNotCompleteVariable.setText(element.getPropertyAsString(ReplicationRequestExtractor.EXTRACTION_NOT_COMPLETE_VARIABLE, ""));
		switch ((SDMRegister) predefinedRegisters.getSelectedItem()) {
		case CPR_PERSON_V1:
			cprPersonExtractCPR.setSelected(element.getPropertyAsBoolean(CprRegister.Person.Field.CPR.property(), false));
			cprPersonExtractName.setSelected(element.getPropertyAsBoolean(CprRegister.Person.Field.FIRST_NAME.property(), false));
			cprPersonExtractName.setSelected(element.getPropertyAsBoolean(CprRegister.Person.Field.MIDDLE_NAME.property(), false));
			cprPersonExtractName.setSelected(element.getPropertyAsBoolean(CprRegister.Person.Field.LAST_NAME.property(), false));
			cprPersonExtractBirthday.setSelected(element.getPropertyAsBoolean(CprRegister.Person.Field.BIRTHDAY.property(), false));
			break;
		case AUTHORIZATION_V1:
			authorizationExtractCPR.setSelected(element.getPropertyAsBoolean(AuthorizationRegister.Authorization.Field.CPR.property(), false));
			authorizationExtractName.setSelected(element.getPropertyAsBoolean(AuthorizationRegister.Authorization.Field.FIRST_NAME.property(), false));
			authorizationExtractName.setSelected(element.getPropertyAsBoolean(AuthorizationRegister.Authorization.Field.LAST_NAME.property(), false));
			break;
		default:
			break;
		}
		revisionNumberVariable.setText(element.getPropertyAsString(ReplicationRequestExtractor.REVISION_NUMBER_VARIABLE, ""));
		updateFields();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void modifyTestElement(TestElement element) {
		super.modifyTestElement(element);
		element.setProperty(ReplicationRequestExtractor.REGISTER, ((SDMRegister) predefinedRegisters.getSelectedItem()).name());
		element.setProperty(ReplicationRequestExtractor.EXTRACTED_PERCENTAGE, extractionPercentage.getText());
		element.setProperty(ReplicationRequestExtractor.MAX_EXTRACTED, extractionMax.getText());
		element.setProperty(ReplicationRequestExtractor.MAX_READ, readMax.getText());
		element.setProperty(ReplicationRequestExtractor.REGISTER_VARIABLE, registerVariable.getText());
		element.setProperty(ReplicationRequestExtractor.REVISION_NUMBER_VARIABLE, revisionNumberVariable.getText());
		element.setProperty(ReplicationRequestExtractor.EXTRACTION_NOT_COMPLETE_VARIABLE, extractionNotCompleteVariable.getText());
		switch ((SDMRegister) predefinedRegisters.getSelectedItem()) {
		case CPR_PERSON_V1:
			element.setProperty(CprRegister.Person.Field.CPR.property(), cprPersonExtractCPR.isSelected());
			element.setProperty(CprRegister.Person.Field.FIRST_NAME.property(), cprPersonExtractName.isSelected());
			element.setProperty(CprRegister.Person.Field.MIDDLE_NAME.property(), cprPersonExtractName.isSelected());
			element.setProperty(CprRegister.Person.Field.LAST_NAME.property(), cprPersonExtractName.isSelected());
			element.setProperty(CprRegister.Person.Field.BIRTHDAY.property(), cprPersonExtractBirthday.isSelected());
			break;
		case AUTHORIZATION_V1:
			element.setProperty(AuthorizationRegister.Authorization.Field.CPR.property(), authorizationExtractCPR.isSelected());
			element.setProperty(AuthorizationRegister.Authorization.Field.FIRST_NAME.property(), authorizationExtractName.isSelected());
			element.setProperty(AuthorizationRegister.Authorization.Field.LAST_NAME.property(), authorizationExtractName.isSelected());
			break;
		default:
			break;
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void clearGui() {
		super.clearGui();
		predefinedRegisters.setSelectedItem(SDMRegister.NOT_FILLED);
		cprPersonExtractCPR.setSelected(false);
		cprPersonExtractName.setSelected(false);
		cprPersonExtractBirthday.setSelected(false);
		authorizationExtractCPR.setSelected(false);
		authorizationExtractName.setSelected(false);
		extractionPercentage.setText("");
		extractionMax.setText("");
		readMax.setText("");
		registerVariable.setText("");
		revisionNumberVariable.setText("");
		extractionNotCompleteVariable.setText("");
		updateFields();
	}

	private void appendRegisterPanel(VerticalPanel configurationPanel) {
		JPanel sourcePanel = new VerticalPanel();
		sourcePanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("sdm_replication_request_extractor_source")));
		sourcePanel.add(getPredefinedRegistersPanel());
		sourcePanel.add(getRegisterFieldsPanel());
		sourcePanel.add(getExtractionPercentagePanel());
		sourcePanel.add(getRegisterVariablePanel());
		sourcePanel.add(getRevisionNumberVariablePanel());
		sourcePanel.add(getExtractionNotCompleteVariablePanel());
		configurationPanel.add(sourcePanel);
	}

	private JPanel getPredefinedRegistersPanel() {
		predefinedRegisters = new JComboBox(new SDMRegister[] { SDMRegister.NOT_FILLED, SDMRegister.CPR_PERSON_V1, SDMRegister.AUTHORIZATION_V1 });
		predefinedRegisters.setRenderer(new SDMRegisterRenderer(Messages.getResString("sdm_no_register")));
		predefinedRegisters.addItemListener(new SDMRegisterItemListener());
		JLabel label = new JLabel(Messages.getResString("sdm_register_name"));

		JPanel predefinedIdentitiesPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		predefinedIdentitiesPanel.add(predefinedRegisters);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(predefinedIdentitiesPanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getRegisterFieldsPanel() {
		extractionOptions = new ArrayList<JCheckBox>();
		registerFieldsCardLayout = new CardLayout();
		registerFields = new JPanel(registerFieldsCardLayout);

		registerFields.add(new JPanel(), SDMRegister.NOT_FILLED.name());
		registerFields.add(getCprPersonFieldsPanel(), SDMRegister.CPR_PERSON_V1.name());
		registerFields.add(getAuthorizationFieldsPanel(), SDMRegister.AUTHORIZATION_V1.name());

		return registerFields;
	}

	private JPanel getCprPersonFieldsPanel() {
		cprPersonExtractCPR = new JCheckBox();
		cprPersonExtractName = new JCheckBox();
		cprPersonExtractBirthday = new JCheckBox();
		JLabel cprLabel = new JLabel(Messages.getResString("sdm_cpr"));
		JLabel nameLabel = new JLabel(Messages.getResString("sdm_name"));
		JLabel birthdayLabel = new JLabel(Messages.getResString("sdm_birthday"));

		registerExtractionOption(cprPersonExtractCPR, cprLabel);
		registerExtractionOption(cprPersonExtractName, nameLabel);
		registerExtractionOption(cprPersonExtractBirthday, birthdayLabel);

		JPanel cprPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		JPanel namePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		JPanel birthdayPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		cprPanel.add(cprPersonExtractCPR);
		cprPanel.add(cprLabel);
		namePanel.add(cprPersonExtractName);
		namePanel.add(nameLabel);
		birthdayPanel.add(cprPersonExtractBirthday);
		birthdayPanel.add(birthdayLabel);

		JPanel cprPersonFieldsPanel = new VerticalPanel();
		cprPersonFieldsPanel.add(cprPanel);
		cprPersonFieldsPanel.add(namePanel);
		cprPersonFieldsPanel.add(birthdayPanel);

		JPanel panel = new JPanel(new BorderLayout(15, 0));
		panel.add(cprPersonFieldsPanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getAuthorizationFieldsPanel() {
		authorizationExtractCPR = new JCheckBox();
		authorizationExtractName = new JCheckBox();
		JLabel cprLabel = new JLabel(Messages.getResString("sdm_cpr"));
		JLabel nameLabel = new JLabel(Messages.getResString("sdm_name"));

		registerExtractionOption(authorizationExtractCPR, cprLabel);
		registerExtractionOption(authorizationExtractName, nameLabel);

		JPanel cprPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		JPanel namePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		cprPanel.add(authorizationExtractCPR);
		cprPanel.add(cprLabel);
		namePanel.add(authorizationExtractName);
		namePanel.add(nameLabel);

		JPanel authorizationFieldsPanel = new VerticalPanel();
		authorizationFieldsPanel.add(cprPanel);
		authorizationFieldsPanel.add(namePanel);

		JPanel panel = new JPanel(new BorderLayout(15, 0));
		panel.add(authorizationFieldsPanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getExtractionPercentagePanel() {
		extractionPercentage = new JTextField(3);
		extractionMax = new JTextField(5);
		readMax = new JTextField(6);
		JLabel label = new JLabel(Messages.getResString("sdm_extracted_percentage"));
		JLabel extractionMaxLabel = new JLabel(Messages.getResString("sdm_extracted_max"));
		JLabel readMaxLabel = new JLabel(Messages.getResString("sdm_read_max"));

		JPanel percentagePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		percentagePanel.add(extractionPercentage);
		percentagePanel.add(extractionMaxLabel);
		percentagePanel.add(extractionMax);
		percentagePanel.add(readMaxLabel);
		percentagePanel.add(readMax);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(percentagePanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getRegisterVariablePanel() {
		registerVariable = new JTextField(40);
		JLabel label = new JLabel(Messages.getResString("sdm_register_variable") + ":");

		JPanel registerVariablePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		registerVariablePanel.add(registerVariable);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(registerVariablePanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getRevisionNumberVariablePanel() {
		revisionNumberVariable = new JTextField(40);
		JLabel label = new JLabel(Messages.getResString("sdm_extraction_revision_number") + ":");

		JPanel revisionNumberVariablePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		revisionNumberVariablePanel.add(revisionNumberVariable);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(revisionNumberVariablePanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getExtractionNotCompleteVariablePanel() {
		extractionNotCompleteVariable = new JTextField(40);
		JLabel label = new JLabel(Messages.getResString("sdm_extraction_not_complete") + ":");

		JPanel extractionNotCompleteVariablePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		extractionNotCompleteVariablePanel.add(extractionNotCompleteVariable);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(extractionNotCompleteVariablePanel, BorderLayout.CENTER);

		return panel;
	}

	private void registerExtractionOption(JCheckBox option, JLabel label) {
		extractionOptions.add(option);
		label.addMouseListener(new ButtonClicker(option, null));
		option.addActionListener(new SDMRegisterFieldListener());
	}

	private void updateFields() {
		boolean registerSelected = predefinedRegisters.getSelectedIndex() != 0;
		boolean fieldSelected = false;
		for (JCheckBox checkBox : extractionOptions) {
			fieldSelected |= checkBox.isSelected();
		}
		extractionPercentage.setEnabled(registerSelected && fieldSelected);
		extractionPercentage.setText(registerSelected && fieldSelected ? extractionPercentage.getText() : "");
		extractionMax.setEnabled(registerSelected && fieldSelected);
		extractionMax.setText(registerSelected && fieldSelected ? extractionMax.getText() : "");
		readMax.setEnabled(registerSelected);
		readMax.setText(registerSelected ? readMax.getText() : "");
		registerVariable.setEnabled(registerSelected && fieldSelected);
		registerVariable.setText(registerSelected && fieldSelected ? registerVariable.getText() : "");
		revisionNumberVariable.setEnabled(registerSelected);
		revisionNumberVariable.setText(registerSelected ? revisionNumberVariable.getText() : "");
		extractionNotCompleteVariable.setEnabled(registerSelected);
		extractionNotCompleteVariable.setText(registerSelected ? extractionNotCompleteVariable.getText() : "");
	}

	private class SDMRegisterItemListener implements ItemListener {

		/**
		 * {@inheritDoc}
		 */
		@Override
		public void itemStateChanged(ItemEvent e) {
			if (e.getStateChange() == ItemEvent.SELECTED) {
				SDMRegister register = (SDMRegister) e.getItem();
				if (register != SDMRegister.NOT_FILLED) {
					if (getName().equals(getStaticLabel()) || SDMRegister.isRegisterName(getName()) || getName().trim().equals("")) {
						namePanel.setName(SDMRegister.registerName(register));
					}
				}
				for (JCheckBox checkBox : extractionOptions) {
					checkBox.setSelected(false);
				}
				updateFields();
				registerFieldsCardLayout.show(registerFields, register.name());
			}
		}
	}

	private class SDMRegisterFieldListener implements ActionListener {

		/**
		 * {@inheritDoc}
		 */
		@Override
		public void actionPerformed(ActionEvent e) {
			updateFields();
		}

	}

}
