/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sdm.sampler;

import java.net.URL;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.jmeter.protocol.http.sampler.HTTPSampleResult;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.arosii.jmeter.protocol.nsp.sampler.AbstractNSPSoapSampler;
import com.arosii.jmeter.protocol.sdm.extractor.registers.CprRegister.Person;

import dk.sosi.seal.SOSIFactory;
import dk.sosi.seal.model.IDCard;
import dk.sosi.seal.model.Request;
import dk.sosi.seal.xml.XmlUtil;

/**
 * 
 * @author Jacob
 */
public class AuthorizationRequestSampler extends AbstractNSPSoapSampler {

	private static final long serialVersionUID = 1L;

	public static final String CPR = "AuthorizationRequestSampler.CPR";

	public static final String USE_EXTRACTED_DATA = "AuthorizationRequestSampler.UseExtractedData";

	public static final String EXTRACTION_VARIABLE = "AuthorizationRequestSampler.ExtractionVariable";

	public AuthorizationRequestSampler() {
		setPath("/stamdata-authorization-lookup-ws/service/AuthorizationService");
		setSOAPAction("http://nsi.dk/sdm/Gateway");
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected HTTPSampleResult sample(URL url, String method, boolean areFollowingRedirect, int frameDepth) {
		clearReply();
		setXmlData(createRequest());
		HTTPSampleResult sample = super.sample(url, method, areFollowingRedirect, frameDepth);
		putReply(sample);
		return sample;
	}

	private String createRequest() {
		SOSIFactory sosiFactory = getSOSIFactory();
		IDCard idCard = getIDCard();

		// Create a new request and add the id card to it.
		Request request = sosiFactory.createNewRequest(false, null);
		request.setIDCard(idCard);

		Document bodyDocument = createPersonLookupRequestBody();

		request.setBody(bodyDocument.getDocumentElement());

		Document doc = request.serialize2DOMDocument();

		return XmlUtil.node2String(doc, false, true);
	}

	private Document createPersonLookupRequestBody() {
		Document bodyDocument;
		try {
			bodyDocument = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
		} catch (ParserConfigurationException e) {
			throw new IllegalStateException(e);
		}
		Element getPersonInformationIn = bodyDocument.createElementNS("http://nsi.dk/-/stamdata/3.0", "AuthorizationRequestStructure");
		getPersonInformationIn.setPrefix("sd");
		bodyDocument.appendChild(getPersonInformationIn);

		Element civilRegistrationNumberPersonQuery = bodyDocument.createElementNS("http://nsi.dk/-/stamdata/3.0", "cpr");
		civilRegistrationNumberPersonQuery.setTextContent(getPerson().getCpr());
		getPersonInformationIn.appendChild(civilRegistrationNumberPersonQuery);

		return bodyDocument;
	}

	private Person getPerson() {
		boolean useExtractedData = getPropertyAsBoolean(USE_EXTRACTED_DATA);

		Person cprPerson;
		if (useExtractedData) {
			String variable = getPropertyAsString(EXTRACTION_VARIABLE, "").trim();
			cprPerson = Person.getRandomCprPerson(getThreadContext(), variable);
		} else {
			String cpr = getPropertyAsString(CPR, "").trim();
			cprPerson = new Person(cpr, "", "", "", "");
		}
		return cprPerson;
	}
}
