/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sdm.sampler;

import java.net.URL;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.jmeter.protocol.http.sampler.HTTPSampleResult;
import org.apache.jmeter.samplers.SampleResult;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.arosii.jmeter.protocol.nsp.sampler.AbstractNSPSoapSampler;
import com.arosii.jmeter.protocol.sdm.extractor.registers.CprRegister.Person;

import dk.sosi.seal.SOSIFactory;
import dk.sosi.seal.model.IDCard;
import dk.sosi.seal.model.Request;
import dk.sosi.seal.xml.XmlUtil;

/**
 * 
 * @author Jacob
 */
public class DetGodeCPROpslagRequestSampler extends AbstractNSPSoapSampler {

	private static final long serialVersionUID = 1L;

	public static final String VERSION = "DetGodeCPROpslagRequestSampler.Version";

	public static final String WITH_HEALTHCARE_INFORMATION = "DetGodeCPROpslagRequestSampler.WithHealthCareInformation";

	public static final String CPR = "DetGodeCPROpslagRequestSampler.CPR";

	public static final String USE_EXTRACTED_DATA = "DetGodeCPROpslagRequestSampler.UseExtractedData";

	public static final String EXTRACTION_VARIABLE = "DetGodeCPROpslagRequestSampler.ExtractionVariable";

	public static enum Version {
		V_1_0_0, V_1_0_2;
	}

	public DetGodeCPROpslagRequestSampler() {
		setPath("/stamdata-cpr-ws/service/DetGodeCPROpslag");
		setSOAPAction("http://rep.oio.dk/medcom.sundcom.dk/xml/wsdl/2007/06/28/getPersonInformation");
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public SampleResult sample() {
		Version version = Version.valueOf(getPropertyAsString(VERSION, Version.V_1_0_0.name()));
		switch (version) {
		case V_1_0_0:
			setPath("/stamdata-cpr-ws/service/DetGodeCPROpslag");
			break;
		case V_1_0_2:
			setPath("/stamdata-cpr-ws/service/DetGodeCPROpslag-1.0.2");
			break;
		}

		boolean withHealthCareInformation = getPropertyAsBoolean(WITH_HEALTHCARE_INFORMATION, false);
		if (withHealthCareInformation) {
			setSOAPAction("http://rep.oio.dk/medcom.sundcom.dk/xml/wsdl/2007/06/28/getPersonInformation");
		} else {
			setSOAPAction("http://rep.oio.dk/medcom.sundcom.dk/xml/wsdl/2007/06/28/getPersonWithHealthCareInformation");
		}

		return super.sample();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected HTTPSampleResult sample(URL url, String method, boolean areFollowingRedirect, int frameDepth) {
		clearReply();
		setXmlData(createRequest());
		HTTPSampleResult sample = super.sample(url, method, areFollowingRedirect, frameDepth);
		putReply(sample);
		return sample;
	}

	private String createRequest() {
		SOSIFactory sosiFactory = getSOSIFactory();
		IDCard idCard = getIDCard();

		// Create a new request and add the id card to it.
		Request request = sosiFactory.createNewRequest(false, null);
		request.setIDCard(idCard);

		Document bodyDocument = createDetGodeCPROpslagRequestBody();

		request.setBody(bodyDocument.getDocumentElement());

		Document doc = request.serialize2DOMDocument();

		return XmlUtil.node2String(doc, false, true);
	}

	private Document createDetGodeCPROpslagRequestBody() {
		Document bodyDocument;
		try {
			bodyDocument = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
		} catch (ParserConfigurationException e) {
			throw new IllegalStateException(e);
		}
		Version version = Version.valueOf(getPropertyAsString(DetGodeCPROpslagRequestSampler.VERSION, Version.V_1_0_0.name()));
		boolean withHealthCareInformation = getPropertyAsBoolean(WITH_HEALTHCARE_INFORMATION, false);

		Element getPersonInformationIn;
		switch (version) {
		case V_1_0_0:
			if (withHealthCareInformation) {
				getPersonInformationIn = bodyDocument.createElementNS("http://rep.oio.dk/medcom.sundcom.dk/xml/wsdl/2007/06/28/", "getPersonWithHealthCareInformationIn");
			} else {
				getPersonInformationIn = bodyDocument.createElementNS("http://rep.oio.dk/medcom.sundcom.dk/xml/wsdl/2007/06/28/", "getPersonInformationIn");
			}
			break;
		case V_1_0_2:
			if (withHealthCareInformation) {
				getPersonInformationIn = bodyDocument.createElementNS("urn:oio:medcom:cprservice:1.0.2", "getPersonWithHealthCareInformationIn");
			} else {
				getPersonInformationIn = bodyDocument.createElementNS("urn:oio:medcom:cprservice:1.0.2", "getPersonInformationIn");
			}
			break;
		default:
			throw new IllegalArgumentException(String.valueOf(version));
		}
		bodyDocument.appendChild(getPersonInformationIn);

		Element personCivilRegistrationIdentifier = bodyDocument.createElementNS("http://rep.oio.dk/cpr.dk/xml/schemas/core/2005/03/18/", "PersonCivilRegistrationIdentifier");
		personCivilRegistrationIdentifier.setTextContent(getPersonCivilRegistrationIdentifier());
		getPersonInformationIn.appendChild(personCivilRegistrationIdentifier);

		return bodyDocument;
	}

	private String getPersonCivilRegistrationIdentifier() {
		if (getPropertyAsBoolean(USE_EXTRACTED_DATA)) {
			return Person.getRandomCprPerson(getThreadContext(), getPropertyAsString(EXTRACTION_VARIABLE, "").trim()).getCpr();
		} else {
			return getPropertyAsString(CPR, "").trim();
		}
	}
}
