/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sdm.sampler;

import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.jmeter.protocol.http.sampler.HTTPSampleResult;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.arosii.jmeter.protocol.nsp.sampler.AbstractNSPSoapSampler;
import com.arosii.jmeter.protocol.sdm.extractor.registers.CprRegister.Person;

import dk.sosi.seal.SOSIFactory;
import dk.sosi.seal.model.IDCard;
import dk.sosi.seal.model.Request;
import dk.sosi.seal.xml.XmlUtil;

/**
 * 
 * @author Jacob
 */
public class PersonLookupRequestSampler extends AbstractNSPSoapSampler {

	private static final long serialVersionUID = 1L;

	public static final String QUERY_METHOD = "PersonLookupRequestSampler.QueryMethod";

	public static final String CPR = "PersonLookupRequestSampler.CPR";

	public static final String FIRST_NAME = "PersonLookupRequestSampler.FirstName";

	public static final String MIDDLE_NAME = "PersonLookupRequestSampler.MiddleName";

	public static final String LAST_NAME = "PersonLookupRequestSampler.LastName";

	public static final String BIRTHDAY = "PersonLookupRequestSampler.Birthday";

	public static final String USE_EXTRACTED_DATA = "PersonLookupRequestSampler.UseExtractedData";

	public static final String EXTRACTED_COUNT = "PersonLookupRequestSampler.ExtractedCount";

	public static final String EXTRACTION_VARIABLE = "PersonLookupRequestSampler.ExtractionVariable";

	public PersonLookupRequestSampler() {
		setPath("/stamdata-cpr-ws/service/StamdataPersonLookup");
		setSOAPAction("http://nsi.dk/2011/09/23/StamdataCpr/getPersonDetails");
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected HTTPSampleResult sample(URL url, String method, boolean areFollowingRedirect, int frameDepth) {
		clearReply();
		setXmlData(createRequest());
		HTTPSampleResult sample = super.sample(url, method, areFollowingRedirect, frameDepth);
		putReply(sample);
		return sample;
	}

	private String createRequest() {
		SOSIFactory sosiFactory = getSOSIFactory();
		IDCard idCard = getIDCard();

		// Create a new request and add the id card to it.
		Request request = sosiFactory.createNewRequest(false, null);
		request.setIDCard(idCard);

		Document bodyDocument = createPersonLookupRequestBody();

		request.setBody(bodyDocument.getDocumentElement());

		Document doc = request.serialize2DOMDocument();

		return XmlUtil.node2String(doc, false, true);
	}

	private Document createPersonLookupRequestBody() {
		Document bodyDocument;
		try {
			bodyDocument = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
		} catch (ParserConfigurationException e) {
			throw new IllegalStateException(e);
		}
		Element getPersonInformationIn = bodyDocument.createElementNS("http://nsi.dk/2011/09/23/StamdataCpr/", "PersonLookupRequest");
		getPersonInformationIn.setPrefix("sd");
		bodyDocument.appendChild(getPersonInformationIn);

		String queryMethod = getPropertyAsString(QUERY_METHOD);

		List<Person> persons = getPersons();

		if (queryMethod.equals(CPR)) {
			if (persons.size() > 1) {
				Element civilRegistrationNumberListPersonQuery = bodyDocument.createElement("CivilRegistrationNumberListPersonQuery");
				for (Person person : persons) {
					Element civilRegistrationNumber = bodyDocument.createElement("CivilRegistrationNumber");
					civilRegistrationNumber.setTextContent(person.getCpr().trim());
					civilRegistrationNumberListPersonQuery.appendChild(civilRegistrationNumber);
				}
				getPersonInformationIn.appendChild(civilRegistrationNumberListPersonQuery);
			} else {
				Person person = persons.get(0);
				Element civilRegistrationNumberPersonQuery = bodyDocument.createElement("CivilRegistrationNumberPersonQuery");
				civilRegistrationNumberPersonQuery.setTextContent(person.getCpr().trim());
				getPersonInformationIn.appendChild(civilRegistrationNumberPersonQuery);
			}
		} else if (queryMethod.equals(FIRST_NAME)) {
			Person person = persons.get(0);
			Element namePersonQuery = bodyDocument.createElement("NamePersonQuery");
			Element personGivenName = bodyDocument.createElementNS("http://rep.oio.dk/ebxml/xml/schemas/dkcc/2003/02/13/", "PersonGivenName");
			personGivenName.setTextContent(person.getFirstName().trim());
			namePersonQuery.appendChild(personGivenName);
			if (person.getMiddleName() != null && person.getMiddleName().trim().length() > 0) {
				Element personMiddleName = bodyDocument.createElementNS("http://rep.oio.dk/ebxml/xml/schemas/dkcc/2003/02/13/", "PersonMiddleName");
				personMiddleName.setTextContent(person.getMiddleName().trim());
				namePersonQuery.appendChild(personMiddleName);
			}
			Element personSurnameName = bodyDocument.createElementNS("http://rep.oio.dk/ebxml/xml/schemas/dkcc/2003/02/13/", "PersonSurnameName");
			personSurnameName.setTextContent(person.getLastName().trim());
			namePersonQuery.appendChild(personSurnameName);
			getPersonInformationIn.appendChild(namePersonQuery);
		} else if (queryMethod.equals(BIRTHDAY)) {
			Person person = persons.get(0);
			Element birthDatePersonQuery = bodyDocument.createElement("BirthDatePersonQuery");
			birthDatePersonQuery.setTextContent(person.getBirthday().trim());
			getPersonInformationIn.appendChild(birthDatePersonQuery);
		}

		return bodyDocument;
	}

	private List<Person> getPersons() {
		boolean useExtractedData = getPropertyAsBoolean(USE_EXTRACTED_DATA);

		List<Person> personCprs = new ArrayList<Person>();
		if (useExtractedData) {
			String extractedCount = getPropertyAsString(EXTRACTED_COUNT, "").trim();
			String variable = getPropertyAsString(EXTRACTION_VARIABLE, "").trim();
			int personCprCount;
			if (extractedCount.length() == 0) {
				personCprCount = 1;
			} else {
				personCprCount = Integer.valueOf(extractedCount);
			}

			for (int i = 0; i < personCprCount; i++) {
				personCprs.add(Person.getRandomCprPerson(getThreadContext(), variable));
			}
		} else {
			String cpr = getPropertyAsString(CPR, "").trim();
			String firstName = getPropertyAsString(FIRST_NAME, "").trim();
			String middleName = getPropertyAsString(MIDDLE_NAME, "").trim();
			String lastName = getPropertyAsString(LAST_NAME, "").trim();
			String birthday = getPropertyAsString(BIRTHDAY, "").trim();

			if (cpr.length() > 0) {
				String[] cprs = cpr.split("[\\,\\;\\:]");
				if (cprs.length > 1) {
					for (String singleCpr : cprs) {
						personCprs.add(new Person(singleCpr, firstName, middleName, lastName, birthday));
					}
				}
			}
			if (personCprs.isEmpty()) {
				personCprs.add(new Person(cpr, firstName, middleName, lastName, birthday));
			}
		}
		return personCprs;
	}
}
