/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sdm.sampler;

import java.net.URL;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.jmeter.protocol.http.sampler.HTTPSampleResult;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.arosii.jmeter.protocol.nsp.sampler.AbstractNSPSoapSampler;

import dk.sosi.seal.SOSIFactory;
import dk.sosi.seal.model.IDCard;
import dk.sosi.seal.model.Request;
import dk.sosi.seal.xml.XmlUtil;

/**
 * 
 * @author Jacob
 */
public class ReplicationRequestSampler extends AbstractNSPSoapSampler {

	private static final long serialVersionUID = 1L;

	public static final String REGISTER = "ReplicationRequestSampler.Register";

	public static final String DATATYPE = "ReplicationRequestSampler.Datatype";

	public static final String VERSION = "ReplicationRequestSampler.Version";

	public static final String OFFSET = "ReplicationRequestSampler.Offset";

	public static final String MAX_ROW_COUNT = "ReplicationRequestSampler.MaxRowCount";

	public ReplicationRequestSampler() {
		setPath("/stamdata-batch-copy-ws/service/StamdataReplication");
		setSOAPAction("http://nsi.dk/2011/10/21/StamdataKrs/replicate");
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	protected HTTPSampleResult sample(URL url, String method, boolean areFollowingRedirect, int frameDepth) {
		clearReply();
		setXmlData(createRequest());
		HTTPSampleResult sample = super.sample(url, method, areFollowingRedirect, frameDepth);
		putReply(sample);
		return sample;
	}

	private String createRequest() {
		SOSIFactory sosiFactory = getSOSIFactory();
		IDCard idCard = getIDCard();

		// Create a new request and add the id card to it.
		Request request = sosiFactory.createNewRequest(false, null);
		request.setIDCard(idCard);

		Document bodyDocument = createReplicationRequestBody();

		request.setBody(bodyDocument.getDocumentElement());

		Document doc = request.serialize2DOMDocument();

		return XmlUtil.node2String(doc, false, true);
	}

	private Document createReplicationRequestBody() {
		Document bodyDocument;
		try {
			bodyDocument = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
		} catch (ParserConfigurationException e) {
			throw new IllegalStateException(e);
		}
		Element replicationRequest = bodyDocument.createElementNS("http://nsi.dk/2011/10/21/StamdataKrs/", "ReplicationRequest");
		replicationRequest.setPrefix("sd");
		bodyDocument.appendChild(replicationRequest);

		Element registerElement = bodyDocument.createElement("register");
		registerElement.setTextContent(getPropertyAsString(REGISTER));
		replicationRequest.appendChild(registerElement);

		Element datatypeElement = bodyDocument.createElement("datatype");
		datatypeElement.setTextContent(getPropertyAsString(DATATYPE));
		replicationRequest.appendChild(datatypeElement);

		Element versionElement = bodyDocument.createElement("version");
		versionElement.setTextContent(getPropertyAsString(VERSION));
		replicationRequest.appendChild(versionElement);

		String offset = getPropertyAsString(OFFSET);
		if (offset == null || offset.trim().equals("")) {
			offset = "0";
		}
		Element offsetElement = bodyDocument.createElement("offset");
		offsetElement.setTextContent(offset);
		replicationRequest.appendChild(offsetElement);

		String maxRowCount = getPropertyAsString(MAX_ROW_COUNT);
		if (maxRowCount != null && !maxRowCount.trim().equals("")) {
			Element maxRowCountElement = bodyDocument.createElement("maxRecords");
			maxRowCountElement.setTextContent(maxRowCount);
			replicationRequest.appendChild(maxRowCountElement);
		}
		
		getThreadContext().getVariables().put(MAX_ROW_COUNT, maxRowCount);

		return bodyDocument;
	}

}
