package dk.nsp.jmeter.protocol.skr.sampler;

import java.net.URL;

import org.apache.jmeter.protocol.http.sampler.HTTPSampleResult;
import org.w3c.dom.Document;

import com.arosii.jmeter.protocol.nsp.sampler.AbstractNSPSoapSampler;

import dk.sosi.seal.SOSIFactory;
import dk.sosi.seal.model.IDCard;
import dk.sosi.seal.model.Reply;
import dk.sosi.seal.model.Request;
import dk.sosi.seal.xml.XmlUtil;

public abstract class AbstractSKRSampler extends AbstractNSPSoapSampler {

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected HTTPSampleResult sample(URL url, String method, boolean areFollowingRedirect, int frameDepth) {
		clearReply();
		setXmlData(createRequest());
		HTTPSampleResult sample = super.sample(url, method, areFollowingRedirect, frameDepth);
		putReply(sample);
		return sample;
	}

	protected String createRequest() {
		SOSIFactory sosiFactory = getSOSIFactory();
		IDCard idCard = getIDCard();

		// Create a new request and add the id card to it.
		Request request = sosiFactory.createNewRequest(false, null);
		request.setIDCard(idCard);

		Document bodyDocument = createRequestBody();

		request.setBody(bodyDocument.getDocumentElement());

		Document doc = request.serialize2DOMDocument();

		return XmlUtil.node2String(doc, false, true);
	}

	protected void validateResult(HTTPSampleResult sample) {
		Reply reply = getSOSIFactory().deserializeReply(sample.getResponseDataAsString());
		if (reply.isFault()) {
			sample.setSuccessful(false);
			sample.setResponseMessage(reply.getFaultCode() + " / " + reply.getFaultString());
		} else {
			validateReply(sample, reply);
		}
	}

	/**
	 * @return the request body
	 */
	protected abstract Document createRequestBody();

	/**
	 * Validate any error codes or other HTTP 200 responses that may represent an unsuccessful reply.
	 * 
	 * @param sample The JMeter sample to update using {@link HTTPSampleResult#setSuccessful(boolean)} and
	 *               {@link HTTPSampleResult#setResponseMessage(String)}
	 * @param reply  The reply from calling the service
	 */
	protected abstract void validateReply(HTTPSampleResult sample, Reply reply);
}
