package dk.nsp.jmeter.resources.skr;

import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.apache.jmeter.util.JMeterUtils;
import org.apache.jmeter.util.LocaleChangeEvent;
import org.apache.jmeter.util.LocaleChangeListener;
import org.apache.jorphan.logging.LoggingManager;
import org.apache.log.Logger;

public class Messages implements LocaleChangeListener {

	private static final Logger log = LoggingManager.getLoggerForClass();

	private static boolean initialized = false;

	private static Messages messages = new Messages();

	private static ResourceBundle resources;

	private synchronized static void init() {
		if (!initialized) {
			JMeterUtils.addLocaleChangeListener(messages);
			messages.localeChanged(new LocaleChangeEvent(Messages.class, Locale.getDefault()));
			initialized = true;
		}
	}

	private Messages() {

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void localeChanged(LocaleChangeEvent event) {
		String baseName = getClass().getPackage().getName() + ".messages";
		resources = ResourceBundle.getBundle(baseName, event.getLocale());
		if (resources == null) {
			log.error("ERROR! Resource bundle '" + baseName + "'not found for locale " + event.getLocale());
		}
	}

	/**
	 * Gets the resource string for this key.
	 * 
	 * If the resource is not found, a warning is logged
	 * 
	 * @param key
	 *            the key in the resource file
	 * @return the resource string if the key is found; otherwise, return "[res_key="+key+"]"
	 */
	public static String getResString(String key) {
		init();
		if (key == null) {
			return null;
		}
		// Resource keys cannot contain spaces, and are forced to lower case
		String resKey = key.replace(' ', '_');
		resKey = resKey.toLowerCase(java.util.Locale.ENGLISH);
		String resString = null;
		try {
			resString = resources.getString(resKey);
		} catch (MissingResourceException mre) {
			return com.arosii.jmeter.resources.nsp.Messages.getResString(key);
		}
		return resString;
	}

}
