/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sor.control.gui;

import com.arosii.jmeter.protocol.sor.sampler.SORLookupGetSorEntityRequestSampler;
import com.arosii.jmeter.resources.sor.Messages;
import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;

import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.DateFormatter;
import javax.swing.text.MaskFormatter;
import javax.swing.text.NumberFormatter;

import java.awt.*;
import java.math.BigInteger;
import java.text.DateFormat;
import java.text.NumberFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;

/**
 *
 * @author Gtitan
 */
public class SORLookupGetSorEntityRequestSamplerGui extends AbstractSORSamplerGui {

	private static final long serialVersionUID = 1L;

	private JFormattedTextField sorIdentifier;

	private JFormattedTextField snapshotDate;

	private JTextField getSorEntityFile;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getLabelResource() {
		return "sor_lookup_get_sor_entity_request";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public TestElement createTestElement() {
		SORLookupGetSorEntityRequestSampler sampler = new SORLookupGetSorEntityRequestSampler();
		modifyTestElement(sampler);
		return sampler;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected Component createConfigurationPanel() {
		final VerticalPanel configurationPanel = new VerticalPanel();
		appendGetSorEntityPanel(configurationPanel);
		return configurationPanel;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void configure(TestElement element) {
		super.configure(element);
		sorIdentifier.setText(element.getPropertyAsString(SORLookupGetSorEntityRequestSampler.SOR_IDENTIFIER, ""));
		snapshotDate.setText(element.getPropertyAsString(SORLookupGetSorEntityRequestSampler.SNAPSHOT_DATE, ""));
		getSorEntityFile.setText(element.getPropertyAsString(SORLookupGetSorEntityRequestSampler.GET_SOR_ENTITY_FILE, ""));
		updateFields();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void modifyTestElement(TestElement element) {
		super.modifyTestElement(element);
		element.setProperty(SORLookupGetSorEntityRequestSampler.SOR_IDENTIFIER, sorIdentifier.getText());
		element.setProperty(SORLookupGetSorEntityRequestSampler.SNAPSHOT_DATE, snapshotDate.getText());
		element.setProperty(SORLookupGetSorEntityRequestSampler.GET_SOR_ENTITY_FILE, getSorEntityFile.getText());
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void clearGui() {
		super.clearGui();
		sorIdentifier.setText("Required");
		snapshotDate.setText("");
		getSorEntityFile.setText("");
		updateFields();
	}

	private void appendGetSorEntityPanel(VerticalPanel configurationPanel) {
		JPanel getSorEntityPanel = new VerticalPanel();
		getSorEntityPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("sor_lookup_get_sor_entity")));
		getSorEntityPanel.add(getSorIdentifierPanel());
		getSorEntityPanel.add(getSnapshotDatePanel());
		getSorEntityPanel.add(getGetSorEntityFilePanel());
		configurationPanel.add(getSorEntityPanel);
	}

	private JPanel getSorIdentifierPanel() {

		NumberFormatter maskFormatter = new NumberFormatter();
		maskFormatter.setMinimum(BigInteger.valueOf(100000000000L));
		maskFormatter.setMaximum(BigInteger.valueOf(999999999999999L));
		sorIdentifier = new JFormattedTextField(maskFormatter);
		sorIdentifier.setColumns(20);
		JLabel label = new JLabel(Messages.getResString("sor_lookup_get_sor_entity_sor_identifier") +
				                       Messages.getResString("sor_lookup_get_sor_entity_sor_identifier_format"));

		sorIdentifier.getDocument().addDocumentListener(new DisableFileKeyListener());

		JPanel sorIdentifierPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		sorIdentifierPanel.add(sorIdentifier);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(sorIdentifierPanel, BorderLayout.CENTER);
		return panel;
	}

	private JPanel getSnapshotDatePanel() {

		DateFormat format = new SimpleDateFormat("yyyy-MM-dd");
		DateFormatter dateFormatter = new DateFormatter(format);
		snapshotDate = new JFormattedTextField(dateFormatter);
		snapshotDate.setColumns(15);
		JLabel label = new JLabel(Messages.getResString("sor_lookup_get_sor_entity_snapshot_date") +
				                       Messages.getResString("sor_lookup_get_sor_entity_snapshot_date_format")   );

		JPanel snapshotDatePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		snapshotDatePanel.add(snapshotDate);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(snapshotDatePanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getGetSorEntityFilePanel() {
		getSorEntityFile = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("sor_lookup_get_sor_entity_file"));

		getSorEntityFile.getDocument().addDocumentListener(new DisableFileKeyListener());

		JPanel getSorEntityFilePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		getSorEntityFilePanel.add(getSorEntityFile);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(getSorEntityFilePanel, BorderLayout.CENTER);

		return panel;
	}

	private void updateFields() {
		getSorEntityFile.setEnabled(sorIdentifier.getText().isEmpty());
		sorIdentifier.setEnabled(getSorEntityFile.getText().isEmpty());
		snapshotDate.setEnabled(getSorEntityFile.getText().isEmpty() && !sorIdentifier.getText().isEmpty());
	}

	private class DisableFileKeyListener implements DocumentListener {

		@Override
		public void insertUpdate(DocumentEvent e) {
			updateFields();
		}

		@Override
		public void removeUpdate(DocumentEvent e) {
			updateFields();
		}

		@Override
		public void changedUpdate(DocumentEvent e) {
			updateFields();
		}
	}
}
