package com.arosii.jmeter.protocol.sor.control.gui;

import com.arosii.jmeter.protocol.sor.sampler.SORLookupSearchRequestSampler;
import com.arosii.jmeter.resources.sor.Messages;
import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;

import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.NumberFormatter;
import java.awt.*;
import java.math.BigInteger;

/**
 *
 * @author Gtitan
 */
public class SORLookupSearchRequestSamplerGui extends AbstractSORSamplerGui {

    private static final long serialVersionUID = 1L;

    private JFormattedTextField searchScopeSorIdentifier;

    private JFormattedTextField sorIdentifier;

    private JFormattedTextField eanLocationCode;

    private JFormattedTextField eanLocationCodeSystemSupplierId;

    private JFormattedTextField geographicalLocalisationId;

    private JTextField searchScopeSorType;

    private JTextField entityName;

    private JTextField providerIdentifier;

    private JTextField pharmacyIdentifier;

    private JTextField localCode;

    private JTextField cvrNumberIdentifier;

    private JTextField searchFile;

    /**
     * {@inheritDoc}
     */
    @Override
    public String getLabelResource() {
        return "sor_lookup_search_request";
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public TestElement createTestElement() {
        SORLookupSearchRequestSampler sampler = new SORLookupSearchRequestSampler();
        modifyTestElement(sampler);
        return sampler;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected Component createConfigurationPanel() {
        final VerticalPanel configurationPanel = new VerticalPanel();
        appendSearchPanel(configurationPanel);
        return configurationPanel;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void configure(TestElement element) {
        super.configure(element);
        searchScopeSorIdentifier.setText(element.getPropertyAsString(SORLookupSearchRequestSampler.SEARCH_SCOPE_IDENTIFIER, ""));
        searchScopeSorType.setText(element.getPropertyAsString(SORLookupSearchRequestSampler.SEARCH_SCOPE_TYPE, ""));
        sorIdentifier.setText(element.getPropertyAsString(SORLookupSearchRequestSampler.SOR_IDENTIFIER, ""));
        eanLocationCode.setText(element.getPropertyAsString(SORLookupSearchRequestSampler.EANLOCATION_CODE, ""));
        eanLocationCodeSystemSupplierId.setText(element.getPropertyAsString(SORLookupSearchRequestSampler.EANLOCATION_CODE_SYS_SUPL_ID, ""));
        entityName.setText(element.getPropertyAsString(SORLookupSearchRequestSampler.ENTITY_NAME, ""));
        providerIdentifier.setText(element.getPropertyAsString(SORLookupSearchRequestSampler.PROVIDER_IDENTIFIER, ""));
        pharmacyIdentifier.setText(element.getPropertyAsString(SORLookupSearchRequestSampler.PHARMACY_IDENTIFIER, ""));
        localCode.setText(element.getPropertyAsString(SORLookupSearchRequestSampler.LOCAL_CODE, ""));
        cvrNumberIdentifier.setText(element.getPropertyAsString(SORLookupSearchRequestSampler.CVR_NUMBER_IDENTIFIER, ""));
        geographicalLocalisationId.setText(element.getPropertyAsString(SORLookupSearchRequestSampler.GEOGRAPHICAL_LOCALISATION_ID, ""));
        searchFile.setText(element.getPropertyAsString(SORLookupSearchRequestSampler.SEARCH_FILE, ""));
        updateFields();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void modifyTestElement(TestElement element) {
        super.modifyTestElement(element);
        element.setProperty(SORLookupSearchRequestSampler.SEARCH_SCOPE_IDENTIFIER, searchScopeSorIdentifier.getText());
        element.setProperty(SORLookupSearchRequestSampler.SEARCH_SCOPE_TYPE, searchScopeSorType.getText());
        element.setProperty(SORLookupSearchRequestSampler.SOR_IDENTIFIER, sorIdentifier.getText());
        element.setProperty(SORLookupSearchRequestSampler.EANLOCATION_CODE, eanLocationCode.getText());
        element.setProperty(SORLookupSearchRequestSampler.EANLOCATION_CODE_SYS_SUPL_ID, eanLocationCodeSystemSupplierId.getText());
        element.setProperty(SORLookupSearchRequestSampler.ENTITY_NAME, entityName.getText());
        element.setProperty(SORLookupSearchRequestSampler.PROVIDER_IDENTIFIER, providerIdentifier.getText());
        element.setProperty(SORLookupSearchRequestSampler.PHARMACY_IDENTIFIER, pharmacyIdentifier.getText());
        element.setProperty(SORLookupSearchRequestSampler.LOCAL_CODE, localCode.getText());
        element.setProperty(SORLookupSearchRequestSampler.CVR_NUMBER_IDENTIFIER, cvrNumberIdentifier.getText());
        element.setProperty(SORLookupSearchRequestSampler.GEOGRAPHICAL_LOCALISATION_ID, geographicalLocalisationId.getText());
        element.setProperty(SORLookupSearchRequestSampler.SEARCH_FILE, searchFile.getText());
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void clearGui() {
        super.clearGui();
        searchScopeSorIdentifier.setText("");
        searchScopeSorType.setText("");
        sorIdentifier.setText("");
        eanLocationCode.setText("");
        eanLocationCodeSystemSupplierId.setText("");
        entityName.setText("");
        providerIdentifier.setText("");
        pharmacyIdentifier.setText("");
        localCode.setText("");
        cvrNumberIdentifier.setText("");
        geographicalLocalisationId.setText("");
        searchFile.setText("");
        updateFields();
    }

    private void appendSearchPanel(VerticalPanel configurationPanel) {
        JPanel searchPanel = new VerticalPanel();
        searchPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("sor_lookup_search")));

        searchPanel.add(getSearchScopeSorIdentifierPanel());
        searchPanel.add(getSearchScopeSorTypePanel());
        searchPanel.add(getSorIdentifierPanel());
        searchPanel.add(getEanLocationCodePanel());
        searchPanel.add(getEanLocationCodeSystemSupplierIdPanel());
        searchPanel.add(getEntityNamePanel());
        searchPanel.add(getProviderIdentifierPanel());
        searchPanel.add(getPharmacyIdentifierPanel());
        searchPanel.add(getLocalCodePanel());
        searchPanel.add(getCvrNumberIdentifierPanel());
        searchPanel.add(getGeographicalLocalisationIdPanel());
        searchPanel.add(getSearchFilePanel());

        configurationPanel.add(searchPanel);
    }

    private JPanel getSearchScopeSorIdentifierPanel() {

        searchScopeSorIdentifier = createNumberFormattedTextField();
        searchScopeSorIdentifier.setColumns(20);
        JLabel label = new JLabel(Messages.getResString("sor_lookup_search_sor_scope_id") +
                Messages.getResString("sor_lookup_search_sor_scope_id_format"));

        searchScopeSorIdentifier.getDocument().addDocumentListener(new SORLookupSearchRequestSamplerGui.DisableFileKeyListener());

        JPanel searchScopeSorIdPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        searchScopeSorIdPanel.add(searchScopeSorIdentifier);

        JPanel panel = new JPanel(new BorderLayout(5, 0));
        panel.add(label, BorderLayout.WEST);
        panel.add(searchScopeSorIdPanel, BorderLayout.CENTER);
        return panel;
    }

    private JPanel getSorIdentifierPanel() {

        sorIdentifier = createNumberFormattedTextField();
        sorIdentifier.setColumns(20);
        JLabel label = new JLabel(Messages.getResString("sor_lookup_search_sor_identifier") +
                Messages.getResString("sor_lookup_search_sor_identifier_format"));

        sorIdentifier.getDocument().addDocumentListener(new SORLookupSearchRequestSamplerGui.DisableFileKeyListener());

        JPanel sorIdentifierPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        sorIdentifierPanel.add(sorIdentifier);

        JPanel panel = new JPanel(new BorderLayout(5, 0));
        panel.add(label, BorderLayout.WEST);
        panel.add(sorIdentifierPanel, BorderLayout.CENTER);
        return panel;
    }

    private JPanel getEanLocationCodePanel() {

        eanLocationCode = createNumberFormattedTextField();
        eanLocationCode.setColumns(20);
        JLabel label = new JLabel(Messages.getResString("sor_lookup_search_ean_location_code") +
                Messages.getResString("sor_lookup_search_ean_location_code_format"));

        eanLocationCode.getDocument().addDocumentListener(new SORLookupSearchRequestSamplerGui.DisableFileKeyListener());

        JPanel eanLocationCodePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        eanLocationCodePanel.add(eanLocationCode);

        JPanel panel = new JPanel(new BorderLayout(5, 0));
        panel.add(label, BorderLayout.WEST);
        panel.add(eanLocationCodePanel, BorderLayout.CENTER);
        return panel;
    }

    private JPanel getEanLocationCodeSystemSupplierIdPanel() {

        eanLocationCodeSystemSupplierId = createNumberFormattedTextField();
        eanLocationCodeSystemSupplierId.setColumns(20);
        JLabel label = new JLabel(Messages.getResString("sor_lookup_search_ean_location_code_system_supplier_id") +
                Messages.getResString("sor_lookup_search_ean_location_code_system_supplier_id_format"));

        eanLocationCodeSystemSupplierId.getDocument().addDocumentListener(new SORLookupSearchRequestSamplerGui.DisableFileKeyListener());

        JPanel eanLocationCodeSystemSupplierIdPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        eanLocationCodeSystemSupplierIdPanel.add(eanLocationCodeSystemSupplierId);

        JPanel panel = new JPanel(new BorderLayout(5, 0));
        panel.add(label, BorderLayout.WEST);
        panel.add(eanLocationCodeSystemSupplierIdPanel, BorderLayout.CENTER);
        return panel;
    }

    private JPanel getGeographicalLocalisationIdPanel() {

        geographicalLocalisationId = createNumberFormattedTextField();
        geographicalLocalisationId.setColumns(20);
        JLabel label = new JLabel(Messages.getResString("sor_lookup_search_geographical_localisation_id") +
                Messages.getResString("sor_lookup_search_geographical_localisation_id_format"));

        geographicalLocalisationId.getDocument().addDocumentListener(new SORLookupSearchRequestSamplerGui.DisableFileKeyListener());

        JPanel geographicalLocalisationIdPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        geographicalLocalisationIdPanel.add(geographicalLocalisationId);

        JPanel panel = new JPanel(new BorderLayout(5, 0));
        panel.add(label, BorderLayout.WEST);
        panel.add(geographicalLocalisationIdPanel, BorderLayout.CENTER);
        return panel;
    }

    private JPanel getSearchScopeSorTypePanel() {
        searchScopeSorType = new JTextField(20);
        JLabel label = new JLabel(Messages.getResString("sor_lookup_search_sor_sor_scope_type"));

        searchScopeSorType.getDocument().addDocumentListener(new SORLookupSearchRequestSamplerGui.DisableFileKeyListener());

        JPanel searchScopeSorTypePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        searchScopeSorTypePanel.add(searchScopeSorType);

        JPanel panel = new JPanel(new BorderLayout(5, 0));
        panel.add(label, BorderLayout.WEST);
        panel.add(searchScopeSorTypePanel, BorderLayout.CENTER);

        return panel;
    }

    private JPanel getEntityNamePanel() {
        entityName = new JTextField(20);
        JLabel label = new JLabel(Messages.getResString("sor_lookup_search_entity_name"));

        entityName.getDocument().addDocumentListener(new SORLookupSearchRequestSamplerGui.DisableFileKeyListener());

        JPanel entityNamePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        entityNamePanel.add(entityName);

        JPanel panel = new JPanel(new BorderLayout(5, 0));
        panel.add(label, BorderLayout.WEST);
        panel.add(entityNamePanel, BorderLayout.CENTER);

        return panel;
    }

    private JPanel getProviderIdentifierPanel() {
        providerIdentifier = new JTextField(20);
        JLabel label = new JLabel(Messages.getResString("sor_lookup_search_provider_identifier"));

        providerIdentifier.getDocument().addDocumentListener(new SORLookupSearchRequestSamplerGui.DisableFileKeyListener());

        JPanel providerIdentifierPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        providerIdentifierPanel.add(providerIdentifier);

        JPanel panel = new JPanel(new BorderLayout(5, 0));
        panel.add(label, BorderLayout.WEST);
        panel.add(providerIdentifierPanel, BorderLayout.CENTER);

        return panel;
    }

    private JPanel getPharmacyIdentifierPanel() {
        pharmacyIdentifier = new JTextField(20);
        JLabel label = new JLabel(Messages.getResString("sor_lookup_search_pharmacy_identifier"));

        pharmacyIdentifier.getDocument().addDocumentListener(new SORLookupSearchRequestSamplerGui.DisableFileKeyListener());

        JPanel pharmacyIdentifierPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        pharmacyIdentifierPanel.add(pharmacyIdentifier);

        JPanel panel = new JPanel(new BorderLayout(5, 0));
        panel.add(label, BorderLayout.WEST);
        panel.add(pharmacyIdentifierPanel, BorderLayout.CENTER);

        return panel;
    }

    private JPanel getLocalCodePanel() {
        localCode = new JTextField(20);
        JLabel label = new JLabel(Messages.getResString("sor_lookup_search_local_code"));

        localCode.getDocument().addDocumentListener(new SORLookupSearchRequestSamplerGui.DisableFileKeyListener());

        JPanel localCodePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        localCodePanel.add(localCode);

        JPanel panel = new JPanel(new BorderLayout(5, 0));
        panel.add(label, BorderLayout.WEST);
        panel.add(localCodePanel, BorderLayout.CENTER);

        return panel;
    }

    private JPanel getCvrNumberIdentifierPanel() {
        cvrNumberIdentifier = new JTextField(20);
        JLabel label = new JLabel(Messages.getResString("sor_lookup_search_cvr_number_identifier"));

        cvrNumberIdentifier.getDocument().addDocumentListener(new SORLookupSearchRequestSamplerGui.DisableFileKeyListener());

        JPanel cvrNumberIdentifierPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        cvrNumberIdentifierPanel.add(cvrNumberIdentifier);

        JPanel panel = new JPanel(new BorderLayout(5, 0));
        panel.add(label, BorderLayout.WEST);
        panel.add(cvrNumberIdentifierPanel, BorderLayout.CENTER);

        return panel;
    }

    private JPanel getSearchFilePanel() {
        searchFile = new JTextField(20);
        JLabel label = new JLabel(Messages.getResString("sor_lookup_search_file"));

        searchFile.getDocument().addDocumentListener(new SORLookupSearchRequestSamplerGui.DisableFileKeyListener());

        JPanel searchFilePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        searchFilePanel.add(searchFile);

        JPanel panel = new JPanel(new BorderLayout(5, 0));
        panel.add(label, BorderLayout.WEST);
        panel.add(searchFilePanel, BorderLayout.CENTER);

        return panel;
    }

    private void updateFields() {
        searchScopeSorIdentifier.setEnabled(searchFile.getText().isEmpty());
        searchScopeSorType.setEnabled(searchFile.getText().isEmpty());
        sorIdentifier.setEnabled(searchFile.getText().isEmpty());
        eanLocationCode.setEnabled(searchFile.getText().isEmpty());
        eanLocationCodeSystemSupplierId.setEnabled(searchFile.getText().isEmpty());
        entityName.setEnabled(searchFile.getText().isEmpty());
        providerIdentifier.setEnabled(searchFile.getText().isEmpty());
        pharmacyIdentifier.setEnabled(searchFile.getText().isEmpty());
        localCode.setEnabled(searchFile.getText().isEmpty());
        cvrNumberIdentifier.setEnabled(searchFile.getText().isEmpty());
        geographicalLocalisationId.setEnabled(searchFile.getText().isEmpty());
        searchFile.setEnabled(sorIdentifier.getText().isEmpty());
    }

    private class DisableFileKeyListener implements DocumentListener {

        @Override
        public void insertUpdate(DocumentEvent e) {
            updateFields();
        }

        @Override
        public void removeUpdate(DocumentEvent e) {
            updateFields();
        }

        @Override
        public void changedUpdate(DocumentEvent e) {
            updateFields();
        }
    }

    private JFormattedTextField createNumberFormattedTextField() {

        NumberFormatter numberFormatter = new NumberFormatter();
        numberFormatter.setMinimum(BigInteger.valueOf(100000000000L));
        numberFormatter.setMaximum(BigInteger.valueOf(999999999999999L));
        return new JFormattedTextField(numberFormatter);
    }
}
