/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sor.control.gui;

import com.arosii.jmeter.protocol.sor.sampler.SORLookupShakSorMapRequestSampler;
import com.arosii.jmeter.resources.sor.Messages;
import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;

import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.DateFormatter;

import java.awt.*;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

/**
 *
 * @author Gtitan
 */
public class SORLookupShakSorMapRequestSamplerGui extends AbstractSORSamplerGui {

    private static final long serialVersionUID = 1L;

    private JTextField shakIdentifier;

    private JFormattedTextField date;

    private JTextField shakSorMapFile;

    /**
     * {@inheritDoc}
     */
    @Override
    public String getLabelResource() {
        return "sor_lookup_shak_sor_map_request";
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public TestElement createTestElement() {
        SORLookupShakSorMapRequestSampler sampler = new SORLookupShakSorMapRequestSampler();
        modifyTestElement(sampler);
        return sampler;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected Component createConfigurationPanel() {
        final VerticalPanel configurationPanel = new VerticalPanel();
        appendShakSorMapPanel(configurationPanel);
        return configurationPanel;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void configure(TestElement element) {
        super.configure(element);
        shakIdentifier.setText(element.getPropertyAsString(SORLookupShakSorMapRequestSampler.SHAK_IDENTIFIER, ""));
        date.setText(element.getPropertyAsString(SORLookupShakSorMapRequestSampler.DATE, ""));
        shakSorMapFile.setText(element.getPropertyAsString(SORLookupShakSorMapRequestSampler.SHAK_SOR_MAP_FILE, ""));
        updateFields();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void modifyTestElement(TestElement element) {
        super.modifyTestElement(element);
        element.setProperty(SORLookupShakSorMapRequestSampler.SHAK_IDENTIFIER, shakIdentifier.getText());
        element.setProperty(SORLookupShakSorMapRequestSampler.DATE, date.getText());
        element.setProperty(SORLookupShakSorMapRequestSampler.SHAK_SOR_MAP_FILE, shakSorMapFile.getText());
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void clearGui() {
        super.clearGui();
        shakIdentifier.setText("Required");
        date.setText("");
        shakSorMapFile.setText("");
        updateFields();
    }

    private void appendShakSorMapPanel(VerticalPanel configurationPanel) {
        JPanel getShakSorMapPanel = new VerticalPanel();
        getShakSorMapPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("sor_lookup_shak_sor_map")));
        getShakSorMapPanel.add(getShakIdPanel());
        getShakSorMapPanel.add(getDatePanel());
        getShakSorMapPanel.add(getShakSorMapFilePanel());
        configurationPanel.add(getShakSorMapPanel);
    }

    private JPanel getShakIdPanel() {

        shakIdentifier = new JTextField(20);
        JLabel label = new JLabel(Messages.getResString("sor_lookup_shak_sor_map_shak_identifier"));

        shakIdentifier.getDocument().addDocumentListener(new DisableFileKeyListener());

        JPanel shakIdentifierPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        shakIdentifierPanel.add(shakIdentifier);

        JPanel panel = new JPanel(new BorderLayout(5, 0));
        panel.add(label, BorderLayout.WEST);
        panel.add(shakIdentifierPanel, BorderLayout.CENTER);
        return panel;
    }

    private JPanel getDatePanel() {

        JPanel panel;
        DateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        DateFormatter dateFormatter = new DateFormatter(format);
        date = new JFormattedTextField(dateFormatter);
        date.setColumns(15);
        JLabel label = new JLabel(Messages.getResString("sor_lookup_shak_sor_map_date") +
                                       Messages.getResString("sor_lookup_shak_sor_map_date_format")   );

        JPanel datePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        datePanel.add(date);

        panel = new JPanel(new BorderLayout(5, 0));
        panel.add(label, BorderLayout.WEST);
        panel.add(datePanel, BorderLayout.CENTER);

        return panel;
    }

    private JPanel getShakSorMapFilePanel() {
        shakSorMapFile = new JTextField(20);
        JLabel label = new JLabel(Messages.getResString("sor_lookup_shak_sor_map_file"));

        shakSorMapFile.getDocument().addDocumentListener(new DisableFileKeyListener());

        JPanel shakSorMapPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        shakSorMapPanel.add(shakSorMapFile);

        JPanel panel = new JPanel(new BorderLayout(5, 0));
        panel.add(label, BorderLayout.WEST);
        panel.add(shakSorMapPanel, BorderLayout.CENTER);

        return panel;
    }

    private void updateFields() {
        shakSorMapFile.setEnabled(shakIdentifier.getText().isEmpty());
        shakIdentifier.setEnabled(shakSorMapFile.getText().isEmpty());
        date.setEnabled(shakSorMapFile.getText().isEmpty() && !shakIdentifier.getText().isEmpty());
    }

    private class DisableFileKeyListener implements DocumentListener {

        @Override
        public void insertUpdate(DocumentEvent e) {
            updateFields();
        }

        @Override
        public void removeUpdate(DocumentEvent e) {
            updateFields();
        }

        @Override
        public void changedUpdate(DocumentEvent e) {
            updateFields();
        }
    }
}
