package com.arosii.jmeter.protocol.sor.control.gui;

import com.arosii.jmeter.protocol.sor.sampler.SORLookupShakSorMapRequestSampler;
import com.arosii.jmeter.protocol.sor.sampler.SORLookupSorShakMapRequestSampler;
import com.arosii.jmeter.resources.sor.Messages;
import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;

import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.DateFormatter;
import javax.swing.text.NumberFormatter;
import java.awt.*;
import java.math.BigInteger;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

/**
 *
 * @author Gtitan
 */
public class SORLookupSorShakMapRequestSamplerGui extends AbstractSORSamplerGui {

    private static final long serialVersionUID = 1L;

    private JFormattedTextField sorIdentifier;

    private JFormattedTextField date;

    private JTextField sorShakMapFile;

    /**
     * {@inheritDoc}
     */
    @Override
    public String getLabelResource() {
        return "sor_lookup_sor_shak_map_request";
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public TestElement createTestElement() {
        SORLookupSorShakMapRequestSampler sampler = new SORLookupSorShakMapRequestSampler();
        modifyTestElement(sampler);
        return sampler;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected Component createConfigurationPanel() {
        final VerticalPanel configurationPanel = new VerticalPanel();
        appendSorShakMapPanel(configurationPanel);
        return configurationPanel;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void configure(TestElement element) {
        super.configure(element);
        sorIdentifier.setText(element.getPropertyAsString(SORLookupSorShakMapRequestSampler.SOR_IDENTIFIER, ""));
        date.setText(element.getPropertyAsString(SORLookupSorShakMapRequestSampler.DATE, ""));
        sorShakMapFile.setText(element.getPropertyAsString(SORLookupSorShakMapRequestSampler.SOR_SHAK_MAP_FILE, ""));
        updateFields();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void modifyTestElement(TestElement element) {
        super.modifyTestElement(element);
        element.setProperty(SORLookupSorShakMapRequestSampler.SOR_IDENTIFIER, sorIdentifier.getText());
        element.setProperty(SORLookupSorShakMapRequestSampler.DATE, date.getText());
        element.setProperty(SORLookupSorShakMapRequestSampler.SOR_SHAK_MAP_FILE, sorShakMapFile.getText());
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void clearGui() {
        super.clearGui();
        sorIdentifier.setText("Required");
        date.setText("");
        sorShakMapFile.setText("");
        updateFields();
    }

    private void appendSorShakMapPanel(VerticalPanel configurationPanel) {
        JPanel getSorShakMapPanel = new VerticalPanel();
        getSorShakMapPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(),
                                                           Messages.getResString("sor_lookup_sor_shak_map")));
        getSorShakMapPanel.add(getSorIdentifierPanel());
        getSorShakMapPanel.add(getDatePanel());
        getSorShakMapPanel.add(getSorShakMapFilePanel());
        configurationPanel.add(getSorShakMapPanel);
    }

    private JPanel getSorIdentifierPanel() {

        NumberFormatter maskFormatter = new NumberFormatter();
        maskFormatter.setMinimum(BigInteger.valueOf(100000000000L));
        maskFormatter.setMaximum(BigInteger.valueOf(999999999999999L));
        sorIdentifier = new JFormattedTextField(maskFormatter);
        sorIdentifier.setColumns(20);
        JLabel label = new JLabel(Messages.getResString("sor_lookup_sor_shak_map_sor_identifier") +
                                       Messages.getResString("sor_lookup_sor_shak_map_sor_identifier_format"));

        sorIdentifier.getDocument().addDocumentListener(new SORLookupSorShakMapRequestSamplerGui.DisableFileKeyListener());

        JPanel sorIdentifierPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        sorIdentifierPanel.add(sorIdentifier);

        JPanel panel = new JPanel(new BorderLayout(5, 0));
        panel.add(label, BorderLayout.WEST);
        panel.add(sorIdentifierPanel, BorderLayout.CENTER);
        return panel;
    }

    private JPanel getDatePanel() {

        JPanel panel;
        DateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        DateFormatter dateFormatter = new DateFormatter(format);
        date = new JFormattedTextField(dateFormatter);
        date.setColumns(15);
        JLabel label = new JLabel(Messages.getResString("sor_lookup_sor_shak_map_date") +
                Messages.getResString("sor_lookup_sor_shak_map_date_format")   );

        JPanel datePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        datePanel.add(date);

        panel = new JPanel(new BorderLayout(5, 0));
        panel.add(label, BorderLayout.WEST);
        panel.add(datePanel, BorderLayout.CENTER);

        return panel;
    }

    private JPanel getSorShakMapFilePanel() {
        sorShakMapFile = new JTextField(20);
        JLabel label = new JLabel(Messages.getResString("sor_lookup_sor_shak_map_file"));

        sorShakMapFile.getDocument().addDocumentListener(new SORLookupSorShakMapRequestSamplerGui.DisableFileKeyListener());

        JPanel sorShakMapPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        sorShakMapPanel.add(sorShakMapFile);

        JPanel panel = new JPanel(new BorderLayout(5, 0));
        panel.add(label, BorderLayout.WEST);
        panel.add(sorShakMapPanel, BorderLayout.CENTER);

        return panel;
    }

    private void updateFields() {
        sorShakMapFile.setEnabled(sorIdentifier.getText().isEmpty());
        sorIdentifier.setEnabled(sorShakMapFile.getText().isEmpty());
        date.setEnabled(sorShakMapFile.getText().isEmpty() && !sorIdentifier.getText().isEmpty());
    }

    private class DisableFileKeyListener implements DocumentListener {

        @Override
        public void insertUpdate(DocumentEvent e) {
            updateFields();
        }

        @Override
        public void removeUpdate(DocumentEvent e) {
            updateFields();
        }

        @Override
        public void changedUpdate(DocumentEvent e) {
            updateFields();
        }
    }
}
