/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sor.control.gui;

import com.arosii.jmeter.protocol.sor.sampler.SORUpdateCloseSorEntityRequestSampler;
import com.arosii.jmeter.resources.sor.Messages;
import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;

import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import java.awt.*;

/**
 * 
 * @author Jacob
 */
public class SORUpdateCloseSorEntityRequestSamplerGui extends AbstractSORSamplerGui {

	private static final long serialVersionUID = 1L;

    private JTextField sorIdentifier;

    private JTextField fromDate;

	private JTextField closeSorEntityFile;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getLabelResource() {
		return "sor_update_close_sor_entity_request";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public TestElement createTestElement() {
		SORUpdateCloseSorEntityRequestSampler sampler = new SORUpdateCloseSorEntityRequestSampler();
		modifyTestElement(sampler);
		return sampler;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected Component createConfigurationPanel() {
		final VerticalPanel configurationPanel = new VerticalPanel();
		appendCloseSorEntityPanel(configurationPanel);
		return configurationPanel;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void configure(TestElement element) {
		super.configure(element);
		sorIdentifier.setText(element.getPropertyAsString(SORUpdateCloseSorEntityRequestSampler.SOR_IDENTIFIER, ""));
		fromDate.setText(element.getPropertyAsString(SORUpdateCloseSorEntityRequestSampler.FROM_DATE, ""));
		closeSorEntityFile.setText(element.getPropertyAsString(SORUpdateCloseSorEntityRequestSampler.CLOSE_SOR_ENTITY_FILE, ""));
		updateFields();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void modifyTestElement(TestElement element) {
		super.modifyTestElement(element);
        element.setProperty(SORUpdateCloseSorEntityRequestSampler.SOR_IDENTIFIER, sorIdentifier.getText());
        element.setProperty(SORUpdateCloseSorEntityRequestSampler.FROM_DATE, fromDate.getText());
		element.setProperty(SORUpdateCloseSorEntityRequestSampler.CLOSE_SOR_ENTITY_FILE, closeSorEntityFile.getText());
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void clearGui() {
		super.clearGui();
		sorIdentifier.setText("");
		fromDate.setText("");
		closeSorEntityFile.setText("");
		updateFields();
	}

	private void appendCloseSorEntityPanel(VerticalPanel configurationPanel) {
		JPanel closeSorEntityPanel = new VerticalPanel();
		closeSorEntityPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("sor_update_close_sor_entity")));
        closeSorEntityPanel.add(getSorIdentifierPanel());
        closeSorEntityPanel.add(getFromDatePanel());
		closeSorEntityPanel.add(getCloseSorEntityFilePanel());
		configurationPanel.add(closeSorEntityPanel);
	}

    private JPanel getSorIdentifierPanel() {
		sorIdentifier = new JTextField(20);
        JLabel label = new JLabel(Messages.getResString("sor_update_close_sor_entity_sor_identifier"));

		sorIdentifier.getDocument().addDocumentListener(new DisableFileKeyListener());

        JPanel sorIdentifierPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        sorIdentifierPanel.add(sorIdentifier);

        JPanel panel = new JPanel(new BorderLayout(5, 0));
        panel.add(label, BorderLayout.WEST);
        panel.add(sorIdentifierPanel, BorderLayout.CENTER);

        return panel;
    }

    private JPanel getFromDatePanel() {
		fromDate = new JTextField(20);
        JLabel label = new JLabel(Messages.getResString("sor_update_close_sor_entity_from_date"));

        JPanel fromDatePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        fromDatePanel.add(fromDate);

        JPanel panel = new JPanel(new BorderLayout(5, 0));
        panel.add(label, BorderLayout.WEST);
        panel.add(fromDatePanel, BorderLayout.CENTER);

        return panel;
    }

	private JPanel getCloseSorEntityFilePanel() {
		closeSorEntityFile = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("sor_update_close_sor_entity_file"));

		closeSorEntityFile.getDocument().addDocumentListener(new DisableFileKeyListener());

		JPanel closeSorEntityFilePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		closeSorEntityFilePanel.add(closeSorEntityFile);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(closeSorEntityFilePanel, BorderLayout.CENTER);

		return panel;
	}

	private void updateFields() {
		closeSorEntityFile.setEnabled(sorIdentifier.getText().isEmpty());
		sorIdentifier.setEnabled(closeSorEntityFile.getText().isEmpty());
        fromDate.setEnabled(closeSorEntityFile.getText().isEmpty() && !sorIdentifier.getText().isEmpty());
	}

	private class DisableFileKeyListener implements DocumentListener {

		@Override
		public void insertUpdate(DocumentEvent e) {
			updateFields();
		}

		@Override
		public void removeUpdate(DocumentEvent e) {
			updateFields();
		}

		@Override
		public void changedUpdate(DocumentEvent e) {
			updateFields();
		}
	}
}
