/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sor.control.gui;

import com.arosii.jmeter.protocol.sor.sampler.SORUpdateEditEanLocationCodeCommunicationSupplierRequestSampler;
import com.arosii.jmeter.resources.sor.Messages;
import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;

import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import java.awt.*;

/**
 * 
 * @author Jacob
 */
public class SORUpdateEditEanLocationCodeCommunicationSupplierRequestSamplerGui extends AbstractSORSamplerGui {

	private static final long serialVersionUID = 1L;

    private JTextField communicationSupplierIdentifier;

    private JTextField eanLocationCodeEntities;

	private JTextField editEanLocationCodeCommunicationSupplierFile;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getLabelResource() {
		return "sor_update_edit_ean_location_code_communication_supplier_request";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public TestElement createTestElement() {
		SORUpdateEditEanLocationCodeCommunicationSupplierRequestSampler sampler = new SORUpdateEditEanLocationCodeCommunicationSupplierRequestSampler();
		modifyTestElement(sampler);
		return sampler;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected Component createConfigurationPanel() {
		final VerticalPanel configurationPanel = new VerticalPanel();
		appendEditEanLocationCodeCommunicationSupplierPanel(configurationPanel);
		return configurationPanel;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void configure(TestElement element) {
		super.configure(element);
		communicationSupplierIdentifier.setText(element.getPropertyAsString(SORUpdateEditEanLocationCodeCommunicationSupplierRequestSampler.COMMUNICATION_SUPPLIER_IDENTIFIER, ""));
		eanLocationCodeEntities.setText(element.getPropertyAsString(SORUpdateEditEanLocationCodeCommunicationSupplierRequestSampler.EAN_LOCATION_CODE_ENTRIES, ""));
		editEanLocationCodeCommunicationSupplierFile.setText(element.getPropertyAsString(SORUpdateEditEanLocationCodeCommunicationSupplierRequestSampler.EDIT_EAN_LOCATION_CODE_COMMUNICATION_SUPPLIER_FILE, ""));
		updateFields();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void modifyTestElement(TestElement element) {
		super.modifyTestElement(element);
        element.setProperty(SORUpdateEditEanLocationCodeCommunicationSupplierRequestSampler.COMMUNICATION_SUPPLIER_IDENTIFIER, communicationSupplierIdentifier.getText());
        element.setProperty(SORUpdateEditEanLocationCodeCommunicationSupplierRequestSampler.EAN_LOCATION_CODE_ENTRIES, eanLocationCodeEntities.getText());
		element.setProperty(SORUpdateEditEanLocationCodeCommunicationSupplierRequestSampler.EDIT_EAN_LOCATION_CODE_COMMUNICATION_SUPPLIER_FILE, editEanLocationCodeCommunicationSupplierFile.getText());
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void clearGui() {
		super.clearGui();
		communicationSupplierIdentifier.setText("");
		eanLocationCodeEntities.setText("");
		editEanLocationCodeCommunicationSupplierFile.setText("");
		updateFields();
	}

	private void appendEditEanLocationCodeCommunicationSupplierPanel(VerticalPanel configurationPanel) {
		JPanel editEanLocationCodeCommunicationSupplierPanel = new VerticalPanel();
		editEanLocationCodeCommunicationSupplierPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("sor_update_edit_ean_location_code_communication_supplier")));
        editEanLocationCodeCommunicationSupplierPanel.add(getCommunicationSupplierIdentifierPanel());
        editEanLocationCodeCommunicationSupplierPanel.add(getEanLocationCodeEntitiesPanel());
		editEanLocationCodeCommunicationSupplierPanel.add(getEditEanLocationCodeCommunicationSupplierFilePanel());
		configurationPanel.add(editEanLocationCodeCommunicationSupplierPanel);
	}

    private JPanel getCommunicationSupplierIdentifierPanel() {
		communicationSupplierIdentifier = new JTextField(20);
        JLabel label = new JLabel(Messages.getResString("sor_update_edit_ean_location_code_communication_supplier_communication_supplier_identifier"));

		communicationSupplierIdentifier.getDocument().addDocumentListener(new DisableFileKeyListener());

        JPanel communicationSupplierIdentifierPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        communicationSupplierIdentifierPanel.add(communicationSupplierIdentifier);

        JPanel panel = new JPanel(new BorderLayout(5, 0));
        panel.add(label, BorderLayout.WEST);
        panel.add(communicationSupplierIdentifierPanel, BorderLayout.CENTER);

        return panel;
    }

    private JPanel getEanLocationCodeEntitiesPanel() {
		eanLocationCodeEntities = new JTextField(20);
        JLabel label = new JLabel(Messages.getResString("sor_update_edit_ean_location_code_communication_supplier_ean_location_code_entities"));

        JPanel eanLocationCodeEntitiesPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        eanLocationCodeEntitiesPanel.add(eanLocationCodeEntities);

        JPanel panel = new JPanel(new BorderLayout(5, 0));
        panel.add(label, BorderLayout.WEST);
        panel.add(eanLocationCodeEntitiesPanel, BorderLayout.CENTER);

        return panel;
    }

	private JPanel getEditEanLocationCodeCommunicationSupplierFilePanel() {
		editEanLocationCodeCommunicationSupplierFile = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("sor_update_edit_ean_location_code_communication_supplier_file"));

		editEanLocationCodeCommunicationSupplierFile.getDocument().addDocumentListener(new DisableFileKeyListener());

		JPanel editEanLocationCodeCommunicationSupplierFilePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		editEanLocationCodeCommunicationSupplierFilePanel.add(editEanLocationCodeCommunicationSupplierFile);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(editEanLocationCodeCommunicationSupplierFilePanel, BorderLayout.CENTER);

		return panel;
	}

	private void updateFields() {
		editEanLocationCodeCommunicationSupplierFile.setEnabled(communicationSupplierIdentifier.getText().isEmpty());
		communicationSupplierIdentifier.setEnabled(editEanLocationCodeCommunicationSupplierFile.getText().isEmpty());
        eanLocationCodeEntities.setEnabled(editEanLocationCodeCommunicationSupplierFile.getText().isEmpty() && !communicationSupplierIdentifier.getText().isEmpty());
	}

	private class DisableFileKeyListener implements DocumentListener {

		@Override
		public void insertUpdate(DocumentEvent e) {
			updateFields();
		}

		@Override
		public void removeUpdate(DocumentEvent e) {
			updateFields();
		}

		@Override
		public void changedUpdate(DocumentEvent e) {
			updateFields();
		}
	}
}
