/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sor.control.gui;

import com.arosii.jmeter.protocol.sor.sampler.SORUpdateEditEanLocationCodeEdiAdministratorRequestSampler;
import com.arosii.jmeter.resources.sor.Messages;
import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;

import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import java.awt.*;

/**
 * 
 * @author Jacob
 */
public class SORUpdateEditEanLocationCodeEdiAdministratorRequestSamplerGui extends AbstractSORSamplerGui {

	private static final long serialVersionUID = 1L;

    private JTextField ediAdministratorIdentifier;

    private JTextField eanLocationCodeEntities;

	private JTextField editEanLocationCodeEdiAdministratorFile;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getLabelResource() {
		return "sor_update_edit_ean_location_code_edi_administrator_request";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public TestElement createTestElement() {
		SORUpdateEditEanLocationCodeEdiAdministratorRequestSampler sampler = new SORUpdateEditEanLocationCodeEdiAdministratorRequestSampler();
		modifyTestElement(sampler);
		return sampler;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected Component createConfigurationPanel() {
		final VerticalPanel configurationPanel = new VerticalPanel();
		appendEditEanLocationCodeEdiAdministratorPanel(configurationPanel);
		return configurationPanel;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void configure(TestElement element) {
		super.configure(element);
		ediAdministratorIdentifier.setText(element.getPropertyAsString(SORUpdateEditEanLocationCodeEdiAdministratorRequestSampler.EDI_ADMINISTRATOR_IDENTIFIER, ""));
		eanLocationCodeEntities.setText(element.getPropertyAsString(SORUpdateEditEanLocationCodeEdiAdministratorRequestSampler.EAN_LOCATION_CODE_ENTRIES, ""));
		editEanLocationCodeEdiAdministratorFile.setText(element.getPropertyAsString(SORUpdateEditEanLocationCodeEdiAdministratorRequestSampler.EDIT_EAN_LOCATION_CODE_EDI_ADMINISTRATOR_FILE, ""));
		updateFields();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void modifyTestElement(TestElement element) {
		super.modifyTestElement(element);
        element.setProperty(SORUpdateEditEanLocationCodeEdiAdministratorRequestSampler.EDI_ADMINISTRATOR_IDENTIFIER, ediAdministratorIdentifier.getText());
        element.setProperty(SORUpdateEditEanLocationCodeEdiAdministratorRequestSampler.EAN_LOCATION_CODE_ENTRIES, eanLocationCodeEntities.getText());
		element.setProperty(SORUpdateEditEanLocationCodeEdiAdministratorRequestSampler.EDIT_EAN_LOCATION_CODE_EDI_ADMINISTRATOR_FILE, editEanLocationCodeEdiAdministratorFile.getText());
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void clearGui() {
		super.clearGui();
		ediAdministratorIdentifier.setText("");
		eanLocationCodeEntities.setText("");
		editEanLocationCodeEdiAdministratorFile.setText("");
		updateFields();
	}

	private void appendEditEanLocationCodeEdiAdministratorPanel(VerticalPanel configurationPanel) {
		JPanel editEanLocationCodeEdiAdministratorPanel = new VerticalPanel();
		editEanLocationCodeEdiAdministratorPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("sor_update_edit_ean_location_code_edi_administrator")));
		editEanLocationCodeEdiAdministratorPanel.add(getEdiAdministratorIdentifierPanel());
        editEanLocationCodeEdiAdministratorPanel.add(getEanLocationCodeEntitiesPanel());
		editEanLocationCodeEdiAdministratorPanel.add(getEditEanLocationCodeEdiAdministratorFilePanel());
		configurationPanel.add(editEanLocationCodeEdiAdministratorPanel);
	}

    private JPanel getEdiAdministratorIdentifierPanel() {
		ediAdministratorIdentifier = new JTextField(20);
        JLabel label = new JLabel(Messages.getResString("sor_update_edit_ean_location_code_edi_administrator_edi_administrator_identifier"));

		ediAdministratorIdentifier.getDocument().addDocumentListener(new DisableFileKeyListener());

        JPanel ediAdministratorIdentifierPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        ediAdministratorIdentifierPanel.add(ediAdministratorIdentifier);

        JPanel panel = new JPanel(new BorderLayout(5, 0));
        panel.add(label, BorderLayout.WEST);
        panel.add(ediAdministratorIdentifierPanel, BorderLayout.CENTER);

        return panel;
    }

    private JPanel getEanLocationCodeEntitiesPanel() {
		eanLocationCodeEntities = new JTextField(20);
        JLabel label = new JLabel(Messages.getResString("sor_update_edit_ean_location_code_edi_administrator_ean_location_code_entities"));

        JPanel eanLocationCodeEntitiesPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        eanLocationCodeEntitiesPanel.add(eanLocationCodeEntities);

        JPanel panel = new JPanel(new BorderLayout(5, 0));
        panel.add(label, BorderLayout.WEST);
        panel.add(eanLocationCodeEntitiesPanel, BorderLayout.CENTER);

        return panel;
    }

	private JPanel getEditEanLocationCodeEdiAdministratorFilePanel() {
		editEanLocationCodeEdiAdministratorFile = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("sor_update_edit_ean_location_code_edi_administrator_file"));

		editEanLocationCodeEdiAdministratorFile.getDocument().addDocumentListener(new DisableFileKeyListener());

		JPanel editEanLocationCodeEdiAdministratorFilePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		editEanLocationCodeEdiAdministratorFilePanel.add(editEanLocationCodeEdiAdministratorFile);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(editEanLocationCodeEdiAdministratorFilePanel, BorderLayout.CENTER);

		return panel;
	}

	private void updateFields() {
		editEanLocationCodeEdiAdministratorFile.setEnabled(ediAdministratorIdentifier.getText().isEmpty());
		ediAdministratorIdentifier.setEnabled(editEanLocationCodeEdiAdministratorFile.getText().isEmpty());
        eanLocationCodeEntities.setEnabled(editEanLocationCodeEdiAdministratorFile.getText().isEmpty() && !ediAdministratorIdentifier.getText().isEmpty());
	}

	private class DisableFileKeyListener implements DocumentListener {

		@Override
		public void insertUpdate(DocumentEvent e) {
			updateFields();
		}

		@Override
		public void removeUpdate(DocumentEvent e) {
			updateFields();
		}

		@Override
		public void changedUpdate(DocumentEvent e) {
			updateFields();
		}
	}
}
