/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sor.control.gui;

import com.arosii.jmeter.protocol.sor.sampler.SORUpdateEditEanLocationCodeRegionRequestSampler;
import com.arosii.jmeter.resources.sor.Messages;
import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;

import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import java.awt.*;

/**
 * 
 * @author Jacob
 */
public class SORUpdateEditEanLocationCodeRegionRequestSamplerGui extends AbstractSORSamplerGui {

	private static final long serialVersionUID = 1L;

    private JTextField regionCode;

    private JTextField eanLocationCodeEntities;

	private JTextField editEanLocationCodeRegionFile;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getLabelResource() {
		return "sor_update_edit_ean_location_code_region_request";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public TestElement createTestElement() {
		SORUpdateEditEanLocationCodeRegionRequestSampler sampler = new SORUpdateEditEanLocationCodeRegionRequestSampler();
		modifyTestElement(sampler);
		return sampler;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected Component createConfigurationPanel() {
		final VerticalPanel configurationPanel = new VerticalPanel();
		appendEditEanLocationCodeRegionPanel(configurationPanel);
		return configurationPanel;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void configure(TestElement element) {
		super.configure(element);
		regionCode.setText(element.getPropertyAsString(SORUpdateEditEanLocationCodeRegionRequestSampler.REGION_CODE, ""));
		eanLocationCodeEntities.setText(element.getPropertyAsString(SORUpdateEditEanLocationCodeRegionRequestSampler.EAN_LOCATION_CODE_ENTRIES, ""));
		editEanLocationCodeRegionFile.setText(element.getPropertyAsString(SORUpdateEditEanLocationCodeRegionRequestSampler.EDIT_EAN_LOCATION_CODE_REGION_FILE, ""));
		updateFields();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void modifyTestElement(TestElement element) {
		super.modifyTestElement(element);
        element.setProperty(SORUpdateEditEanLocationCodeRegionRequestSampler.REGION_CODE, regionCode.getText());
        element.setProperty(SORUpdateEditEanLocationCodeRegionRequestSampler.EAN_LOCATION_CODE_ENTRIES, eanLocationCodeEntities.getText());
		element.setProperty(SORUpdateEditEanLocationCodeRegionRequestSampler.EDIT_EAN_LOCATION_CODE_REGION_FILE, editEanLocationCodeRegionFile.getText());
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void clearGui() {
		super.clearGui();
		regionCode.setText("");
		eanLocationCodeEntities.setText("");
		editEanLocationCodeRegionFile.setText("");
		updateFields();
	}

	private void appendEditEanLocationCodeRegionPanel(VerticalPanel configurationPanel) {
		JPanel editEanLocationCodeRegionPanel = new VerticalPanel();
		editEanLocationCodeRegionPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("sor_update_edit_ean_location_code_region")));
        editEanLocationCodeRegionPanel.add(getRegionCodePanel());
        editEanLocationCodeRegionPanel.add(getEanLocationCodeEntitiesPanel());
		editEanLocationCodeRegionPanel.add(getEditEanLocationCodeRegionFilePanel());
		configurationPanel.add(editEanLocationCodeRegionPanel);
	}

    private JPanel getRegionCodePanel() {
		regionCode = new JTextField(20);
        JLabel label = new JLabel(Messages.getResString("sor_update_edit_ean_location_code_region_region_code"));

		regionCode.getDocument().addDocumentListener(new DisableFileKeyListener());

        JPanel regionCodePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        regionCodePanel.add(regionCode);

        JPanel panel = new JPanel(new BorderLayout(5, 0));
        panel.add(label, BorderLayout.WEST);
        panel.add(regionCodePanel, BorderLayout.CENTER);

        return panel;
    }

    private JPanel getEanLocationCodeEntitiesPanel() {
		eanLocationCodeEntities = new JTextField(20);
        JLabel label = new JLabel(Messages.getResString("sor_update_edit_ean_location_code_region_ean_location_code_entities"));

        JPanel eanLocationCodeEntitiesPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        eanLocationCodeEntitiesPanel.add(eanLocationCodeEntities);

        JPanel panel = new JPanel(new BorderLayout(5, 0));
        panel.add(label, BorderLayout.WEST);
        panel.add(eanLocationCodeEntitiesPanel, BorderLayout.CENTER);

        return panel;
    }

	private JPanel getEditEanLocationCodeRegionFilePanel() {
		editEanLocationCodeRegionFile = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("sor_update_edit_ean_location_code_region_file"));

		editEanLocationCodeRegionFile.getDocument().addDocumentListener(new DisableFileKeyListener());

		JPanel editEanLocationCodeRegionFilePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		editEanLocationCodeRegionFilePanel.add(editEanLocationCodeRegionFile);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(editEanLocationCodeRegionFilePanel, BorderLayout.CENTER);

		return panel;
	}

	private void updateFields() {
		editEanLocationCodeRegionFile.setEnabled(regionCode.getText().isEmpty());
		regionCode.setEnabled(editEanLocationCodeRegionFile.getText().isEmpty());
        eanLocationCodeEntities.setEnabled(editEanLocationCodeRegionFile.getText().isEmpty() && !regionCode.getText().isEmpty());
	}

	private class DisableFileKeyListener implements DocumentListener {

		@Override
		public void insertUpdate(DocumentEvent e) {
			updateFields();
		}

		@Override
		public void removeUpdate(DocumentEvent e) {
			updateFields();
		}

		@Override
		public void changedUpdate(DocumentEvent e) {
			updateFields();
		}
	}
}
