/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sor.control.gui;

import com.arosii.jmeter.protocol.sor.sampler.SORUpdateEditEanLocationCodeSystemSupplierRequestSampler;
import com.arosii.jmeter.resources.sor.Messages;
import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;

import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import java.awt.*;

/**
 * 
 * @author Jacob
 */
public class SORUpdateEditEanLocationCodeSystemSupplierRequestSamplerGui extends AbstractSORSamplerGui {

	private static final long serialVersionUID = 1L;

    private JTextField systemSupplierIdentifier;

    private JTextField eanLocationCodeEntities;

	private JTextField editEanLocationCodeSystemSupplierFile;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getLabelResource() {
		return "sor_update_edit_ean_location_code_system_supplier_request";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public TestElement createTestElement() {
		SORUpdateEditEanLocationCodeSystemSupplierRequestSampler sampler = new SORUpdateEditEanLocationCodeSystemSupplierRequestSampler();
		modifyTestElement(sampler);
		return sampler;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected Component createConfigurationPanel() {
		final VerticalPanel configurationPanel = new VerticalPanel();
		appendEditEanLocationCodeSystemSupplierPanel(configurationPanel);
		return configurationPanel;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void configure(TestElement element) {
		super.configure(element);
		systemSupplierIdentifier.setText(element.getPropertyAsString(SORUpdateEditEanLocationCodeSystemSupplierRequestSampler.SYSTEM_SUPPLIER_IDENTIFIER, ""));
		eanLocationCodeEntities.setText(element.getPropertyAsString(SORUpdateEditEanLocationCodeSystemSupplierRequestSampler.EAN_LOCATION_CODE_ENTRIES, ""));
		editEanLocationCodeSystemSupplierFile.setText(element.getPropertyAsString(SORUpdateEditEanLocationCodeSystemSupplierRequestSampler.EDIT_EAN_LOCATION_CODE_SYSTEM_SUPPLIER_FILE, ""));
		updateFields();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void modifyTestElement(TestElement element) {
		super.modifyTestElement(element);
        element.setProperty(SORUpdateEditEanLocationCodeSystemSupplierRequestSampler.SYSTEM_SUPPLIER_IDENTIFIER, systemSupplierIdentifier.getText());
        element.setProperty(SORUpdateEditEanLocationCodeSystemSupplierRequestSampler.EAN_LOCATION_CODE_ENTRIES, eanLocationCodeEntities.getText());
		element.setProperty(SORUpdateEditEanLocationCodeSystemSupplierRequestSampler.EDIT_EAN_LOCATION_CODE_SYSTEM_SUPPLIER_FILE, editEanLocationCodeSystemSupplierFile.getText());
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void clearGui() {
		super.clearGui();
		systemSupplierIdentifier.setText("");
		eanLocationCodeEntities.setText("");
		editEanLocationCodeSystemSupplierFile.setText("");
		updateFields();
	}

	private void appendEditEanLocationCodeSystemSupplierPanel(VerticalPanel configurationPanel) {
		JPanel editEanLocationCodeSystemSupplierPanel = new VerticalPanel();
		editEanLocationCodeSystemSupplierPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("sor_update_edit_ean_location_code_system_supplier")));
        editEanLocationCodeSystemSupplierPanel.add(getSystemSupplierIdentifierPanel());
        editEanLocationCodeSystemSupplierPanel.add(getEanLocationCodeEntitiesPanel());
		editEanLocationCodeSystemSupplierPanel.add(getEditEanLocationCodeSystemSupplierFilePanel());
		configurationPanel.add(editEanLocationCodeSystemSupplierPanel);
	}

    private JPanel getSystemSupplierIdentifierPanel() {
		systemSupplierIdentifier = new JTextField(20);
        JLabel label = new JLabel(Messages.getResString("sor_update_edit_ean_location_code_system_supplier_system_supplier_identifier"));

		systemSupplierIdentifier.getDocument().addDocumentListener(new DisableFileKeyListener());

        JPanel systemSupplierIdentifierPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        systemSupplierIdentifierPanel.add(systemSupplierIdentifier);

        JPanel panel = new JPanel(new BorderLayout(5, 0));
        panel.add(label, BorderLayout.WEST);
        panel.add(systemSupplierIdentifierPanel, BorderLayout.CENTER);

        return panel;
    }

    private JPanel getEanLocationCodeEntitiesPanel() {
		eanLocationCodeEntities = new JTextField(20);
        JLabel label = new JLabel(Messages.getResString("sor_update_edit_ean_location_code_system_supplier_ean_location_code_entities"));

        JPanel eanLocationCodeEntitiesPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        eanLocationCodeEntitiesPanel.add(eanLocationCodeEntities);

        JPanel panel = new JPanel(new BorderLayout(5, 0));
        panel.add(label, BorderLayout.WEST);
        panel.add(eanLocationCodeEntitiesPanel, BorderLayout.CENTER);

        return panel;
    }

	private JPanel getEditEanLocationCodeSystemSupplierFilePanel() {
		editEanLocationCodeSystemSupplierFile = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("sor_update_edit_ean_location_code_system_supplier_file"));

		editEanLocationCodeSystemSupplierFile.getDocument().addDocumentListener(new DisableFileKeyListener());

		JPanel editEanLocationCodeSystemSupplierFilePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		editEanLocationCodeSystemSupplierFilePanel.add(editEanLocationCodeSystemSupplierFile);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(editEanLocationCodeSystemSupplierFilePanel, BorderLayout.CENTER);

		return panel;
	}

	private void updateFields() {
		editEanLocationCodeSystemSupplierFile.setEnabled(systemSupplierIdentifier.getText().isEmpty());
		systemSupplierIdentifier.setEnabled(editEanLocationCodeSystemSupplierFile.getText().isEmpty());
        eanLocationCodeEntities.setEnabled(editEanLocationCodeSystemSupplierFile.getText().isEmpty() && !systemSupplierIdentifier.getText().isEmpty());
	}

	private class DisableFileKeyListener implements DocumentListener {

		@Override
		public void insertUpdate(DocumentEvent e) {
			updateFields();
		}

		@Override
		public void removeUpdate(DocumentEvent e) {
			updateFields();
		}

		@Override
		public void changedUpdate(DocumentEvent e) {
			updateFields();
		}
	}
}
