/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sor.control.gui;

import com.arosii.jmeter.protocol.sor.sampler.SORUpdateEditEanLocationCodeSystemTypeRequestSampler;
import com.arosii.jmeter.resources.sor.Messages;
import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;

import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import java.awt.*;

/**
 * 
 * @author Jacob
 */
public class SORUpdateEditEanLocationCodeSystemTypeRequestSamplerGui extends AbstractSORSamplerGui {

	private static final long serialVersionUID = 1L;

    private JTextField systemTypeIdentifier;

    private JTextField eanLocationCodeEntities;

	private JTextField editEanLocationCodeSystemTypeFile;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getLabelResource() {
		return "sor_update_edit_ean_location_code_system_type_request";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public TestElement createTestElement() {
		SORUpdateEditEanLocationCodeSystemTypeRequestSampler sampler = new SORUpdateEditEanLocationCodeSystemTypeRequestSampler();
		modifyTestElement(sampler);
		return sampler;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected Component createConfigurationPanel() {
		final VerticalPanel configurationPanel = new VerticalPanel();
		appendEditEanLocationCodeSystemTypePanel(configurationPanel);
		return configurationPanel;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void configure(TestElement element) {
		super.configure(element);
		systemTypeIdentifier.setText(element.getPropertyAsString(SORUpdateEditEanLocationCodeSystemTypeRequestSampler.SYSTEM_TYPE_IDENTIFIER, ""));
		eanLocationCodeEntities.setText(element.getPropertyAsString(SORUpdateEditEanLocationCodeSystemTypeRequestSampler.EAN_LOCATION_CODE_ENTRIES, ""));
		editEanLocationCodeSystemTypeFile.setText(element.getPropertyAsString(SORUpdateEditEanLocationCodeSystemTypeRequestSampler.EDIT_EAN_LOCATION_CODE_SYSTEM_TYPE_FILE, ""));
		updateFields();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void modifyTestElement(TestElement element) {
		super.modifyTestElement(element);
        element.setProperty(SORUpdateEditEanLocationCodeSystemTypeRequestSampler.SYSTEM_TYPE_IDENTIFIER, systemTypeIdentifier.getText());
        element.setProperty(SORUpdateEditEanLocationCodeSystemTypeRequestSampler.EAN_LOCATION_CODE_ENTRIES, eanLocationCodeEntities.getText());
		element.setProperty(SORUpdateEditEanLocationCodeSystemTypeRequestSampler.EDIT_EAN_LOCATION_CODE_SYSTEM_TYPE_FILE, editEanLocationCodeSystemTypeFile.getText());
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void clearGui() {
		super.clearGui();
		systemTypeIdentifier.setText("");
		eanLocationCodeEntities.setText("");
		editEanLocationCodeSystemTypeFile.setText("");
		updateFields();
	}

	private void appendEditEanLocationCodeSystemTypePanel(VerticalPanel configurationPanel) {
		JPanel editEanLocationCodeSystemTypePanel = new VerticalPanel();
		editEanLocationCodeSystemTypePanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("sor_update_edit_ean_location_code_system_type")));
        editEanLocationCodeSystemTypePanel.add(getSystemTypeIdentifierPanel());
        editEanLocationCodeSystemTypePanel.add(getEanLocationCodeEntitiesPanel());
		editEanLocationCodeSystemTypePanel.add(getEditEanLocationCodeSystemTypeFilePanel());
		configurationPanel.add(editEanLocationCodeSystemTypePanel);
	}

    private JPanel getSystemTypeIdentifierPanel() {
		systemTypeIdentifier = new JTextField(20);
        JLabel label = new JLabel(Messages.getResString("sor_update_edit_ean_location_code_system_type_system_type_identifier"));

		systemTypeIdentifier.getDocument().addDocumentListener(new DisableFileKeyListener());

        JPanel systemTypeIdentifierPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        systemTypeIdentifierPanel.add(systemTypeIdentifier);

        JPanel panel = new JPanel(new BorderLayout(5, 0));
        panel.add(label, BorderLayout.WEST);
        panel.add(systemTypeIdentifierPanel, BorderLayout.CENTER);

        return panel;
    }

    private JPanel getEanLocationCodeEntitiesPanel() {
		eanLocationCodeEntities = new JTextField(20);
        JLabel label = new JLabel(Messages.getResString("sor_update_edit_ean_location_code_system_type_ean_location_code_entities"));

        JPanel eanLocationCodeEntitiesPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        eanLocationCodeEntitiesPanel.add(eanLocationCodeEntities);

        JPanel panel = new JPanel(new BorderLayout(5, 0));
        panel.add(label, BorderLayout.WEST);
        panel.add(eanLocationCodeEntitiesPanel, BorderLayout.CENTER);

        return panel;
    }

	private JPanel getEditEanLocationCodeSystemTypeFilePanel() {
		editEanLocationCodeSystemTypeFile = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("sor_update_edit_ean_location_code_system_type_file"));

		editEanLocationCodeSystemTypeFile.getDocument().addDocumentListener(new DisableFileKeyListener());

		JPanel editEanLocationCodeSystemTypeFilePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		editEanLocationCodeSystemTypeFilePanel.add(editEanLocationCodeSystemTypeFile);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(editEanLocationCodeSystemTypeFilePanel, BorderLayout.CENTER);

		return panel;
	}

	private void updateFields() {
		editEanLocationCodeSystemTypeFile.setEnabled(systemTypeIdentifier.getText().isEmpty());
		systemTypeIdentifier.setEnabled(editEanLocationCodeSystemTypeFile.getText().isEmpty());
        eanLocationCodeEntities.setEnabled(editEanLocationCodeSystemTypeFile.getText().isEmpty() && !systemTypeIdentifier.getText().isEmpty());
	}

	private class DisableFileKeyListener implements DocumentListener {

		@Override
		public void insertUpdate(DocumentEvent e) {
			updateFields();
		}

		@Override
		public void removeUpdate(DocumentEvent e) {
			updateFields();
		}

		@Override
		public void changedUpdate(DocumentEvent e) {
			updateFields();
		}
	}
}
