/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sor.control.gui;

import com.arosii.jmeter.protocol.sor.sampler.SORUpdateEditSorEntityRequestSampler;
import com.arosii.jmeter.resources.sor.Messages;
import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;

import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * 
 * @author Jacob
 */
public class SORUpdateEditSorEntityRequestSamplerGui extends AbstractSORSamplerGui {

	private static final long serialVersionUID = 1L;

    private JComboBox sorType;

	private JTextField editSorEntityFile;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getLabelResource() {
		return "sor_update_edit_sor_entity_request";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public TestElement createTestElement() {
		SORUpdateEditSorEntityRequestSampler sampler = new SORUpdateEditSorEntityRequestSampler();
		modifyTestElement(sampler);
		return sampler;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected Component createConfigurationPanel() {
		final VerticalPanel configurationPanel = new VerticalPanel();
		appendEditSorEntityPanel(configurationPanel);
		return configurationPanel;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void configure(TestElement element) {
		super.configure(element);
		sorType.setSelectedItem(element.getPropertyAsString(SORUpdateEditSorEntityRequestSampler.SOR_TYPE, ""));
		editSorEntityFile.setText(element.getPropertyAsString(SORUpdateEditSorEntityRequestSampler.EDIT_SOR_ENTITY_FILE, ""));
		updateFields();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void modifyTestElement(TestElement element) {
		super.modifyTestElement(element);
        element.setProperty(SORUpdateEditSorEntityRequestSampler.SOR_TYPE, (String) sorType.getSelectedItem());
		element.setProperty(SORUpdateEditSorEntityRequestSampler.EDIT_SOR_ENTITY_FILE, editSorEntityFile.getText());
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void clearGui() {
		super.clearGui();
		sorType.setSelectedIndex(0);
		editSorEntityFile.setText("");
		updateFields();
	}

	private void appendEditSorEntityPanel(VerticalPanel configurationPanel) {
		JPanel editSorEntityPanel = new VerticalPanel();
		editSorEntityPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("sor_update_edit_sor_entity")));
        editSorEntityPanel.add(getSorTypePanel());
		editSorEntityPanel.add(getEditSorEntityFilePanel());
		configurationPanel.add(editSorEntityPanel);
	}

    private JPanel getSorTypePanel() {
		sorType = new JComboBox(new String[]{"", "IO", "HI", "OU"});
        JLabel label = new JLabel(Messages.getResString("sor_update_edit_sor_entity_sor_type"));

		sorType.addActionListener(new DisableFileKeyListener());

        JPanel sorTypePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        sorTypePanel.add(sorType);

        JPanel panel = new JPanel(new BorderLayout(5, 0));
        panel.add(label, BorderLayout.WEST);
        panel.add(sorTypePanel, BorderLayout.CENTER);

        return panel;
    }

	private JPanel getEditSorEntityFilePanel() {
		editSorEntityFile = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("sor_update_edit_sor_entity_file"));

		editSorEntityFile.getDocument().addDocumentListener(new DisableFileKeyListener());

		JPanel editSorEntityFilePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		editSorEntityFilePanel.add(editSorEntityFile);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(editSorEntityFilePanel, BorderLayout.CENTER);

		return panel;
	}

	private void updateFields() {
		editSorEntityFile.setEnabled(sorType.getSelectedIndex() == 0);
		sorType.setEnabled(editSorEntityFile.getText().isEmpty());
	}

	private class DisableFileKeyListener implements DocumentListener, ActionListener {

		@Override
		public void insertUpdate(DocumentEvent e) {
			updateFields();
		}

		@Override
		public void removeUpdate(DocumentEvent e) {
			updateFields();
		}

		@Override
		public void changedUpdate(DocumentEvent e) {
			updateFields();
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			updateFields();
		}
	}
}
