/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sor.control.gui;

import com.arosii.jmeter.protocol.sor.sampler.SORUpdateMoveEanLocationCodeRequestSampler;
import com.arosii.jmeter.resources.sor.Messages;
import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;

import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import java.awt.*;

/**
 * 
 * @author Jacob
 */
public class SORUpdateMoveEanLocationCodeRequestSamplerGui extends AbstractSORSamplerGui {

	private static final long serialVersionUID = 1L;

    private JTextField sorIdentifier;

    private JTextField eanLocationCode;

	private JTextField moveEanLocationCodeFile;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getLabelResource() {
		return "sor_update_move_ean_location_code_request";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public TestElement createTestElement() {
		SORUpdateMoveEanLocationCodeRequestSampler sampler = new SORUpdateMoveEanLocationCodeRequestSampler();
		modifyTestElement(sampler);
		return sampler;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected Component createConfigurationPanel() {
		final VerticalPanel configurationPanel = new VerticalPanel();
		appendMoveEanLocationCodePanel(configurationPanel);
		return configurationPanel;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void configure(TestElement element) {
		super.configure(element);
        sorIdentifier.setText(element.getPropertyAsString(SORUpdateMoveEanLocationCodeRequestSampler.SOR_IDENTIFIER, ""));
        eanLocationCode.setText(element.getPropertyAsString(SORUpdateMoveEanLocationCodeRequestSampler.EAN_LOCATION_CODE, ""));
		moveEanLocationCodeFile.setText(element.getPropertyAsString(SORUpdateMoveEanLocationCodeRequestSampler.MOVE_EAN_LOCATION_CODE_FILE, ""));
		updateFields();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void modifyTestElement(TestElement element) {
		super.modifyTestElement(element);
        element.setProperty(SORUpdateMoveEanLocationCodeRequestSampler.SOR_IDENTIFIER, sorIdentifier.getText());
        element.setProperty(SORUpdateMoveEanLocationCodeRequestSampler.EAN_LOCATION_CODE, eanLocationCode.getText());
		element.setProperty(SORUpdateMoveEanLocationCodeRequestSampler.MOVE_EAN_LOCATION_CODE_FILE, moveEanLocationCodeFile.getText());
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void clearGui() {
		super.clearGui();
        sorIdentifier.setText("");
        eanLocationCode.setText("");
		moveEanLocationCodeFile.setText("");
		updateFields();
	}

	private void appendMoveEanLocationCodePanel(VerticalPanel configurationPanel) {
		JPanel moveEanLocationCodePanel = new VerticalPanel();
		moveEanLocationCodePanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("sor_update_move_ean_location_code")));
        moveEanLocationCodePanel.add(getSorIdentifierPanel());
        moveEanLocationCodePanel.add(getEanLocationCodePanel());
		moveEanLocationCodePanel.add(getMoveEanLocationCodeFilePanel());
		configurationPanel.add(moveEanLocationCodePanel);
	}

    private JPanel getSorIdentifierPanel() {
        sorIdentifier = new JTextField(20);
        JLabel label = new JLabel(Messages.getResString("sor_update_move_ean_location_code_sor_identifier"));

        sorIdentifier.getDocument().addDocumentListener(new DisableFileKeyListener());

        JPanel sorIdentifierPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        sorIdentifierPanel.add(sorIdentifier);

        JPanel panel = new JPanel(new BorderLayout(5, 0));
        panel.add(label, BorderLayout.WEST);
        panel.add(sorIdentifierPanel, BorderLayout.CENTER);

        return panel;
    }

    private JPanel getEanLocationCodePanel() {
        eanLocationCode = new JTextField(20);
        JLabel label = new JLabel(Messages.getResString("sor_update_move_ean_location_code_ean_location_code"));

        JPanel eanLocationCodePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        eanLocationCodePanel.add(eanLocationCode);

        JPanel panel = new JPanel(new BorderLayout(5, 0));
        panel.add(label, BorderLayout.WEST);
        panel.add(eanLocationCodePanel, BorderLayout.CENTER);

        return panel;
    }

	private JPanel getMoveEanLocationCodeFilePanel() {
		moveEanLocationCodeFile = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("sor_update_move_ean_location_code_file"));

		moveEanLocationCodeFile.getDocument().addDocumentListener(new DisableFileKeyListener());

		JPanel moveEanLocationCodeFilePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		moveEanLocationCodeFilePanel.add(moveEanLocationCodeFile);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(moveEanLocationCodeFilePanel, BorderLayout.CENTER);

		return panel;
	}

	private void updateFields() {
		moveEanLocationCodeFile.setEnabled(sorIdentifier.getText().isEmpty());
        sorIdentifier.setEnabled(moveEanLocationCodeFile.getText().isEmpty());
        eanLocationCode.setEnabled(moveEanLocationCodeFile.getText().isEmpty() && !sorIdentifier.getText().isEmpty());
	}

	private class DisableFileKeyListener implements DocumentListener {

		@Override
		public void insertUpdate(DocumentEvent e) {
			updateFields();
		}

		@Override
		public void removeUpdate(DocumentEvent e) {
			updateFields();
		}

		@Override
		public void changedUpdate(DocumentEvent e) {
			updateFields();
		}
	}
}
