/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sor.control.gui;

import com.arosii.jmeter.protocol.sor.sampler.SORUpdateMoveSorEntityRequestSampler;
import com.arosii.jmeter.resources.sor.Messages;
import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;

import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import java.awt.*;

/**
 * 
 * @author Jacob
 */
public class SORUpdateMoveSorEntityRequestSamplerGui extends AbstractSORSamplerGui {

	private static final long serialVersionUID = 1L;

	private JTextField sorIdentifier;

	private JTextField parentSorIdentifier;

    private JTextField fromDate;

	private JTextField moveSorEntityFile;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getLabelResource() {
		return "sor_update_move_sor_entity_request";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public TestElement createTestElement() {
		SORUpdateMoveSorEntityRequestSampler sampler = new SORUpdateMoveSorEntityRequestSampler();
		modifyTestElement(sampler);
		return sampler;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected Component createConfigurationPanel() {
		final VerticalPanel configurationPanel = new VerticalPanel();
		appendMoveSorEntityPanel(configurationPanel);
		return configurationPanel;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void configure(TestElement element) {
		super.configure(element);
		sorIdentifier.setText(element.getPropertyAsString(SORUpdateMoveSorEntityRequestSampler.SOR_IDENTIFIER, ""));
		parentSorIdentifier.setText(element.getPropertyAsString(SORUpdateMoveSorEntityRequestSampler.PARENT_SOR_IDENTIFIER, ""));
		fromDate.setText(element.getPropertyAsString(SORUpdateMoveSorEntityRequestSampler.FROM_DATE, ""));
		moveSorEntityFile.setText(element.getPropertyAsString(SORUpdateMoveSorEntityRequestSampler.MOVE_SOR_ENTITY_FILE, ""));
		updateFields();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void modifyTestElement(TestElement element) {
		super.modifyTestElement(element);
		element.setProperty(SORUpdateMoveSorEntityRequestSampler.SOR_IDENTIFIER, sorIdentifier.getText());
		element.setProperty(SORUpdateMoveSorEntityRequestSampler.PARENT_SOR_IDENTIFIER, parentSorIdentifier.getText());
        element.setProperty(SORUpdateMoveSorEntityRequestSampler.FROM_DATE, fromDate.getText());
		element.setProperty(SORUpdateMoveSorEntityRequestSampler.MOVE_SOR_ENTITY_FILE, moveSorEntityFile.getText());
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void clearGui() {
		super.clearGui();
		sorIdentifier.setText("");
		fromDate.setText("");
		moveSorEntityFile.setText("");
		updateFields();
	}

	private void appendMoveSorEntityPanel(VerticalPanel configurationPanel) {
		JPanel moveSorEntityPanel = new VerticalPanel();
		moveSorEntityPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("sor_update_move_sor_entity")));
		moveSorEntityPanel.add(getSorIdentifierPanel());
		moveSorEntityPanel.add(getParentSorIdentifierPanel());
        moveSorEntityPanel.add(getFromDatePanel());
		moveSorEntityPanel.add(getMoveSorEntityFilePanel());
		configurationPanel.add(moveSorEntityPanel);
	}

	private JPanel getSorIdentifierPanel() {
		sorIdentifier = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("sor_update_move_sor_entity_sor_identifier"));

		sorIdentifier.getDocument().addDocumentListener(new DisableFileKeyListener());

		JPanel sorIdentifierPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		sorIdentifierPanel.add(sorIdentifier);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(sorIdentifierPanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getParentSorIdentifierPanel() {
		parentSorIdentifier = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("sor_update_move_sor_entity_parent_sor_identifier"));

		JPanel parentSorIdentifierPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		parentSorIdentifierPanel.add(parentSorIdentifier);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(parentSorIdentifierPanel, BorderLayout.CENTER);

		return panel;
	}

    private JPanel getFromDatePanel() {
		fromDate = new JTextField(20);
        JLabel label = new JLabel(Messages.getResString("sor_update_move_sor_entity_from_date"));

        JPanel fromDatePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        fromDatePanel.add(fromDate);

        JPanel panel = new JPanel(new BorderLayout(5, 0));
        panel.add(label, BorderLayout.WEST);
        panel.add(fromDatePanel, BorderLayout.CENTER);

        return panel;
    }

	private JPanel getMoveSorEntityFilePanel() {
		moveSorEntityFile = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("sor_update_move_sor_entity_file"));

		moveSorEntityFile.getDocument().addDocumentListener(new DisableFileKeyListener());

		JPanel moveSorEntityFilePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		moveSorEntityFilePanel.add(moveSorEntityFile);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(moveSorEntityFilePanel, BorderLayout.CENTER);

		return panel;
	}

	private void updateFields() {
		moveSorEntityFile.setEnabled(sorIdentifier.getText().isEmpty());
		sorIdentifier.setEnabled(moveSorEntityFile.getText().isEmpty());
		parentSorIdentifier.setEnabled(moveSorEntityFile.getText().isEmpty() && !sorIdentifier.getText().isEmpty());
		fromDate.setEnabled(moveSorEntityFile.getText().isEmpty() && !sorIdentifier.getText().isEmpty());
	}

	private class DisableFileKeyListener implements DocumentListener {

		@Override
		public void insertUpdate(DocumentEvent e) {
			updateFields();
		}

		@Override
		public void removeUpdate(DocumentEvent e) {
			updateFields();
		}

		@Override
		public void changedUpdate(DocumentEvent e) {
			updateFields();
		}
	}
}
