/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sor.control.gui;

import com.arosii.jmeter.protocol.sor.sampler.SORUpdateReplaceSorEntitiesRequestSampler;
import com.arosii.jmeter.resources.sor.Messages;
import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;

import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import java.awt.*;

/**
 * 
 * @author Jacob
 */
public class SORUpdateReplaceSorEntitiesRequestSamplerGui extends AbstractSORSamplerGui {

	private static final long serialVersionUID = 1L;

	private JTextField sorIdentifier;

	private JTextField replacesEntityCollection;

    private JTextField fromDate;

	private JTextField replaceSorEntitiesFile;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getLabelResource() {
		return "sor_update_replace_sor_entities_request";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public TestElement createTestElement() {
		SORUpdateReplaceSorEntitiesRequestSampler sampler = new SORUpdateReplaceSorEntitiesRequestSampler();
		modifyTestElement(sampler);
		return sampler;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected Component createConfigurationPanel() {
		final VerticalPanel configurationPanel = new VerticalPanel();
		appendReplaceSorEntitiesPanel(configurationPanel);
		return configurationPanel;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void configure(TestElement element) {
		super.configure(element);
		sorIdentifier.setText(element.getPropertyAsString(SORUpdateReplaceSorEntitiesRequestSampler.SOR_IDENTIFIER, ""));
		replacesEntityCollection.setText(element.getPropertyAsString(SORUpdateReplaceSorEntitiesRequestSampler.REPLACES_ENTITY_COLLECTION, ""));
		fromDate.setText(element.getPropertyAsString(SORUpdateReplaceSorEntitiesRequestSampler.FROM_DATE, ""));
		replaceSorEntitiesFile.setText(element.getPropertyAsString(SORUpdateReplaceSorEntitiesRequestSampler.REPLACE_SOR_ENTITIES_FILE, ""));
		updateFields();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void modifyTestElement(TestElement element) {
		super.modifyTestElement(element);
		element.setProperty(SORUpdateReplaceSorEntitiesRequestSampler.SOR_IDENTIFIER, sorIdentifier.getText());
		element.setProperty(SORUpdateReplaceSorEntitiesRequestSampler.REPLACES_ENTITY_COLLECTION, replacesEntityCollection.getText());
        element.setProperty(SORUpdateReplaceSorEntitiesRequestSampler.FROM_DATE, fromDate.getText());
		element.setProperty(SORUpdateReplaceSorEntitiesRequestSampler.REPLACE_SOR_ENTITIES_FILE, replaceSorEntitiesFile.getText());
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void clearGui() {
		super.clearGui();
		sorIdentifier.setText("");
		fromDate.setText("");
		replaceSorEntitiesFile.setText("");
		updateFields();
	}

	private void appendReplaceSorEntitiesPanel(VerticalPanel configurationPanel) {
		JPanel replaceSorEntitiesPanel = new VerticalPanel();
		replaceSorEntitiesPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("sor_update_replace_sor_entities")));
		replaceSorEntitiesPanel.add(getSorIdentifierPanel());
		replaceSorEntitiesPanel.add(getParentSorIdentifierPanel());
        replaceSorEntitiesPanel.add(getFromDatePanel());
		replaceSorEntitiesPanel.add(getReplaceSorEntitiesFilePanel());
		configurationPanel.add(replaceSorEntitiesPanel);
	}

	private JPanel getSorIdentifierPanel() {
		sorIdentifier = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("sor_update_replace_sor_entities_sor_identifier"));

		sorIdentifier.getDocument().addDocumentListener(new DisableFileKeyListener());

		JPanel sorIdentifierPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		sorIdentifierPanel.add(sorIdentifier);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(sorIdentifierPanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getParentSorIdentifierPanel() {
		replacesEntityCollection = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("sor_update_replace_sor_entities_replaces_entity_collection"));

		JPanel replacesEntityCollectionPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		replacesEntityCollectionPanel.add(replacesEntityCollection);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(replacesEntityCollectionPanel, BorderLayout.CENTER);

		return panel;
	}

    private JPanel getFromDatePanel() {
		fromDate = new JTextField(20);
        JLabel label = new JLabel(Messages.getResString("sor_update_replace_sor_entities_from_date"));

        JPanel fromDatePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        fromDatePanel.add(fromDate);

        JPanel panel = new JPanel(new BorderLayout(5, 0));
        panel.add(label, BorderLayout.WEST);
        panel.add(fromDatePanel, BorderLayout.CENTER);

        return panel;
    }

	private JPanel getReplaceSorEntitiesFilePanel() {
		replaceSorEntitiesFile = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("sor_update_replace_sor_entities_file"));

		replaceSorEntitiesFile.getDocument().addDocumentListener(new DisableFileKeyListener());

		JPanel replaceSorEntitiesFilePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		replaceSorEntitiesFilePanel.add(replaceSorEntitiesFile);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(replaceSorEntitiesFilePanel, BorderLayout.CENTER);

		return panel;
	}

	private void updateFields() {
		replaceSorEntitiesFile.setEnabled(sorIdentifier.getText().isEmpty());
		sorIdentifier.setEnabled(replaceSorEntitiesFile.getText().isEmpty());
		replacesEntityCollection.setEnabled(replaceSorEntitiesFile.getText().isEmpty() && !sorIdentifier.getText().isEmpty());
		fromDate.setEnabled(replaceSorEntitiesFile.getText().isEmpty() && !sorIdentifier.getText().isEmpty());
	}

	private class DisableFileKeyListener implements DocumentListener {

		@Override
		public void insertUpdate(DocumentEvent e) {
			updateFields();
		}

		@Override
		public void removeUpdate(DocumentEvent e) {
			updateFields();
		}

		@Override
		public void changedUpdate(DocumentEvent e) {
			updateFields();
		}
	}
}
