/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sor.control.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.FlowLayout;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import com.arosii.jmeter.protocol.sor.sampler.SORUpdateUpdateEdiTypesRequestSampler;
import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;

import com.arosii.jmeter.resources.sor.Messages;

/**
 * 
 * @author Jacob
 */
public class SORUpdateUpdateEdiTypesRequestSamplerGui extends AbstractSORSamplerGui {

	private static final long serialVersionUID = 1L;

    private JTextField ediIdentifiers;

    private JTextField eanLocationCodes;

	private JTextField updateEdiTypesFile;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getLabelResource() {
		return "sor_update_update_edi_types_request";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public TestElement createTestElement() {
		SORUpdateUpdateEdiTypesRequestSampler sampler = new SORUpdateUpdateEdiTypesRequestSampler();
		modifyTestElement(sampler);
		return sampler;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected Component createConfigurationPanel() {
		final VerticalPanel configurationPanel = new VerticalPanel();
		appendUpdateEdiTypesPanel(configurationPanel);
		return configurationPanel;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void configure(TestElement element) {
		super.configure(element);
        ediIdentifiers.setText(element.getPropertyAsString(SORUpdateUpdateEdiTypesRequestSampler.EDI_IDENTIFIERS, ""));
        eanLocationCodes.setText(element.getPropertyAsString(SORUpdateUpdateEdiTypesRequestSampler.EAN_LOCATION_CODES, ""));
		updateEdiTypesFile.setText(element.getPropertyAsString(SORUpdateUpdateEdiTypesRequestSampler.UPDATE_EDI_TYPES_FILE, ""));
		updateFields();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void modifyTestElement(TestElement element) {
		super.modifyTestElement(element);
        element.setProperty(SORUpdateUpdateEdiTypesRequestSampler.EDI_IDENTIFIERS, ediIdentifiers.getText());
        element.setProperty(SORUpdateUpdateEdiTypesRequestSampler.EAN_LOCATION_CODES, eanLocationCodes.getText());
		element.setProperty(SORUpdateUpdateEdiTypesRequestSampler.UPDATE_EDI_TYPES_FILE, updateEdiTypesFile.getText());
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void clearGui() {
		super.clearGui();
        ediIdentifiers.setText("");
        eanLocationCodes.setText("");
		updateEdiTypesFile.setText("");
		updateFields();
	}

	private void appendUpdateEdiTypesPanel(VerticalPanel configurationPanel) {
		JPanel updateEdiTypesPanel = new VerticalPanel();
		updateEdiTypesPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("sor_update_update_edi_types")));
        updateEdiTypesPanel.add(getEdiIdentifiersPanel());
        updateEdiTypesPanel.add(getEanLocationCodesPanel());
		updateEdiTypesPanel.add(getUpdateEdiTypesFilePanel());
		configurationPanel.add(updateEdiTypesPanel);
	}

    private JPanel getEdiIdentifiersPanel() {
        ediIdentifiers = new JTextField(20);
        JLabel label = new JLabel(Messages.getResString("sor_update_update_edi_types_edi_identifiers"));

        ediIdentifiers.getDocument().addDocumentListener(new DisableFileKeyListener());

        JPanel ediIdentifiersPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        ediIdentifiersPanel.add(ediIdentifiers);

        JPanel panel = new JPanel(new BorderLayout(5, 0));
        panel.add(label, BorderLayout.WEST);
        panel.add(ediIdentifiersPanel, BorderLayout.CENTER);

        return panel;
    }

    private JPanel getEanLocationCodesPanel() {
        eanLocationCodes = new JTextField(20);
        JLabel label = new JLabel(Messages.getResString("sor_update_update_edi_types_ean_location_codes"));

        JPanel eanLocationCodesPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
        eanLocationCodesPanel.add(eanLocationCodes);

        JPanel panel = new JPanel(new BorderLayout(5, 0));
        panel.add(label, BorderLayout.WEST);
        panel.add(eanLocationCodesPanel, BorderLayout.CENTER);

        return panel;
    }

	private JPanel getUpdateEdiTypesFilePanel() {
		updateEdiTypesFile = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("sor_update_update_edi_types_file"));

		updateEdiTypesFile.getDocument().addDocumentListener(new DisableFileKeyListener());

		JPanel updateEdiTypesFilePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		updateEdiTypesFilePanel.add(updateEdiTypesFile);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(updateEdiTypesFilePanel, BorderLayout.CENTER);

		return panel;
	}

	private void updateFields() {
		updateEdiTypesFile.setEnabled(ediIdentifiers.getText().isEmpty());
        ediIdentifiers.setEnabled(updateEdiTypesFile.getText().isEmpty());
        eanLocationCodes.setEnabled(updateEdiTypesFile.getText().isEmpty() && !ediIdentifiers.getText().isEmpty());
	}

	private class DisableFileKeyListener implements DocumentListener {

		@Override
		public void insertUpdate(DocumentEvent e) {
			updateFields();
		}

		@Override
		public void removeUpdate(DocumentEvent e) {
			updateFields();
		}

		@Override
		public void changedUpdate(DocumentEvent e) {
			updateFields();
		}
	}
}
