/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sor.sampler;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Random;
import java.util.Set;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.jmeter.protocol.http.sampler.HTTPSampleResult;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import dk.sosi.seal.model.Reply;

/**
 *
 * @author Gtitan
 */
public class SORLookupGetSorEntityRequestSampler extends AbstractSORLookupSampler {

	private static final long serialVersionUID = 1L;

	public static final String SOR_IDENTIFIER = "SORLookupGetSorEntityRequestSampler.SorIdentifier";

	public static final String SNAPSHOT_DATE = "SORLookupGetSorEntityRequestSampler.SnapshotDate";

	public static final String GET_SOR_ENTITY_FILE = "SORLookupGetSorEntityRequestSampler.GetSorEntityFile";

	public static final Random RANDOM = new Random();

	private static List<String> getSorEntityCache;

	public SORLookupGetSorEntityRequestSampler() {
		super("GetSorEntity");
	}

	protected void validateContent(Reply reply, HTTPSampleResult sample) {
		if (!reply.getBody().getNamespaceURI().equals("http://sundhedsdatastyrelsen.dk/SORLookupService/2018/09/06/")) {
			sample.setSuccessful(false);
			sample.setResponseMessage("Response missing GetSorEntityResponse node: " + reply.getBody().getNamespaceURI());
		} else if (!reply.getBody().getLocalName().equals("GetSorEntityResponse")) {
			sample.setSuccessful(false);
			sample.setResponseMessage("Response missing GetSorEntityResponse node: " + reply.getBody().getTagName());
		}
        NodeList statusNodes = reply.getBody().getElementsByTagNameNS("http://sundhedsdatastyrelsen.dk/SORLookupService/2018/09/06/", "SorObject");
        if (statusNodes == null || statusNodes.getLength() == 0) {
			sample.setSuccessful(false);
			sample.setResponseMessage("No SOR Entities found");
        }
	}

	protected Document createRequestBody() {
		Document bodyDocument;
		try {
			bodyDocument = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
		} catch (ParserConfigurationException e) {
			throw new IllegalStateException(e);
		}
		Element getSorEntityRequest = bodyDocument.createElementNS("http://sundhedsdatastyrelsen.dk/SORLookupService/2018/09/06/", "GetSorEntityRequest");
		bodyDocument.appendChild(getSorEntityRequest);

		Element sorIdentifier = bodyDocument.createElement("SorIdentifier");
		getSorEntityRequest.appendChild(sorIdentifier);

		String sorIdentifierProperty = getPropertyAsString(SOR_IDENTIFIER);
		String snapshotDateProperty = getPropertyAsString(SNAPSHOT_DATE);
        String getSorEntityFileProperty = getPropertyAsString(GET_SOR_ENTITY_FILE);

        if (getSorEntityFileProperty != null && !getSorEntityFileProperty.trim().isEmpty()) {
            String randomRequest = getRandomRequest(getSorEntityFileProperty);

            String[] splitted = randomRequest.split(";", 2);
			sorIdentifierProperty = splitted[0];
			snapshotDateProperty = splitted[1];
        }

		sorIdentifier.setTextContent(sorIdentifierProperty);

        if(!snapshotDateProperty.isEmpty()) {
			Element snapshotDate = bodyDocument.createElement("SnapshotDate");
			getSorEntityRequest.appendChild(snapshotDate);

			snapshotDate.setTextContent(snapshotDateProperty);
		}

		return bodyDocument;
	}

	private String getRandomRequest(String fileProperty) {
		loadList(fileProperty);
		return getSorEntityCache.get(RANDOM.nextInt(getSorEntityCache.size()));
	}

	private synchronized void loadList(String fileProperty) {
		if (getSorEntityCache == null) {
			BufferedReader reader = null;
			try {
				InputStream inputStream;
				File file = new File(fileProperty);
				if (file.exists() && file.canRead()) {
					inputStream = new BufferedInputStream(new FileInputStream(file));
				} else {
					inputStream = new BufferedInputStream(getClass().getResourceAsStream(fileProperty));
				}
				reader = new BufferedReader(new InputStreamReader(inputStream));
				Set<String> requests = new HashSet<String>();
				for (String line = reader.readLine(); line != null; line = reader.readLine()) {
					line = line.trim();
					if (!line.isEmpty()) {
						requests.add(line);
					}
				}
				SORLookupGetSorEntityRequestSampler.getSorEntityCache = new ArrayList<String>(requests);
			} catch (IOException e) {
				throw new IllegalStateException(e);
			} finally {
				if (reader != null) {
					try {
						reader.close();
					} catch (IOException ignore) {
					}
				}
			}
		}
	}
}
