package com.arosii.jmeter.protocol.sor.sampler;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Random;
import java.util.Set;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.jmeter.protocol.http.sampler.HTTPSampleResult;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import dk.sosi.seal.model.Reply;

/**
 *
 * @author Gtitan
 */
public class SORLookupSearchRequestSampler extends AbstractSORLookupSampler {

    private static final long serialVersionUID = 1L;

    public static final String SEARCH_SCOPE_IDENTIFIER = "SORLookupSearchRequestSampler.SearchScopeId";

    public static final String SEARCH_SCOPE_TYPE = "SORLookupSearchRequestSampler.SearchScopeType";

    public static final String SOR_IDENTIFIER = "SORLookupSearchRequestSampler.SorIdentifier";

    public static final String EANLOCATION_CODE = "SORLookupSearchRequestSampler.EanLocationCode";

    public static final String EANLOCATION_CODE_SYS_SUPL_ID = "SORLookupSearchRequestSampler.EanLocationCodeSystemSupplierId";

    public static final String ENTITY_NAME = "SORLookupSearchRequestSampler.EntiyName";

    public static final String PROVIDER_IDENTIFIER = "SORLookupSearchRequestSampler.ProviderIdentifier";

    public static final String PHARMACY_IDENTIFIER = "SORLookupSearchRequestSampler.PharmacyIdentifier";

    public static final String LOCAL_CODE = "SORLookupSearchRequestSampler.LocalCode";

    public static final String CVR_NUMBER_IDENTIFIER = "SORLookupSearchRequestSampler.CvrNumberIdentifier";

    public static final String GEOGRAPHICAL_LOCALISATION_ID = "SORLookupSearchRequestSampler.GeographicalLocalisationId";

    public static final String SEARCH_FILE = "SORLookupSearchRequestSampler.SearchFile";

    public static final Random RANDOM = new Random();

    private static List<String> searchCache;

    public SORLookupSearchRequestSampler() {
        super("Search");
    }

    protected void validateContent(Reply reply, HTTPSampleResult sample) {
		if (!reply.getBody().getNamespaceURI().equals("http://sundhedsdatastyrelsen.dk/SORLookupService/2018/09/06/")) {
			sample.setSuccessful(false);
			sample.setResponseMessage("Response missing SearchResponse node: " + reply.getBody().getNamespaceURI());
		} else if (!reply.getBody().getLocalName().equals("SearchResponse")) {
			sample.setSuccessful(false);
			sample.setResponseMessage("Response missing SearchResponse node: " + reply.getBody().getTagName());
		}
        NodeList statusNodes = reply.getBody().getElementsByTagNameNS("http://sundhedsdatastyrelsen.dk/SORLookupService/2018/09/06/", "SorEntities");
        if (statusNodes == null || statusNodes.getLength() == 0) {
			sample.setSuccessful(false);
			sample.setResponseMessage("No SOR Entities found");
        }
    }

    protected Document createRequestBody() {
        Document bodyDocument;
        try {
            bodyDocument = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
        } catch (ParserConfigurationException e) {
            throw new IllegalStateException(e);
        }
        Element searchRequest = bodyDocument.createElementNS("http://sundhedsdatastyrelsen.dk/SORLookupService/2018/09/06/", "SearchRequest");

        String searchScopeIdProperty = getPropertyAsString(SEARCH_SCOPE_IDENTIFIER);
        String searchScopeTypeProperty = getPropertyAsString(SEARCH_SCOPE_TYPE);
        String sorIdentifierProperty = getPropertyAsString(SOR_IDENTIFIER);
        String eanLocationCodeProperty = getPropertyAsString(EANLOCATION_CODE);
        String eanLocationCodeSystemSupplierIdProperty = getPropertyAsString(EANLOCATION_CODE_SYS_SUPL_ID);
        String entityNameProperty = getPropertyAsString(ENTITY_NAME);
        String providerIdentifierProperty = getPropertyAsString(PROVIDER_IDENTIFIER);
        String pharmacyIdentifierProperty = getPropertyAsString(PHARMACY_IDENTIFIER);
        String localCodeProperty = getPropertyAsString(LOCAL_CODE);
        String cvrNumberIdentifierProperty = getPropertyAsString(CVR_NUMBER_IDENTIFIER);
        String geographicalLocalisationIdProperty = getPropertyAsString(GEOGRAPHICAL_LOCALISATION_ID);
        String searchFileProperty = getPropertyAsString(SEARCH_FILE);


        if (searchFileProperty != null && !searchFileProperty.trim().isEmpty()) {

            String randomRequest = getRandomRequest(searchFileProperty);
            String[] splitted = randomRequest.split(";", 11);
            searchScopeIdProperty = splitted[0];
            searchScopeTypeProperty = splitted[1];
            sorIdentifierProperty = splitted[2];
            eanLocationCodeProperty = splitted[3];
            eanLocationCodeSystemSupplierIdProperty = splitted[4];
            entityNameProperty = splitted[5];
            providerIdentifierProperty = splitted[6];
            pharmacyIdentifierProperty = splitted[7];
            localCodeProperty = splitted[8];
            cvrNumberIdentifierProperty = splitted[9];
            geographicalLocalisationIdProperty = splitted[10];
        }

        Element searchLimit = bodyDocument.createElement("SearchLimit");
        Element searchParameters = bodyDocument.createElement("SearchParameters");
        if(!searchScopeIdProperty.isEmpty()) {
            Element searchScopeId = bodyDocument.createElement("SearchScopeSorId");
            searchScopeId.setTextContent(searchScopeIdProperty);
            searchLimit.appendChild(searchScopeId);
        }

        if(!searchScopeTypeProperty.isEmpty()) {
            Element searchScopeType = bodyDocument.createElement("SearchScopeType");
            searchScopeType.setTextContent(searchScopeTypeProperty);
            searchLimit.appendChild(searchScopeType);
        }

        if(!sorIdentifierProperty.isEmpty()) {
            Element sorIdentifier = bodyDocument.createElement("SorIdentifier");
            sorIdentifier.setTextContent(sorIdentifierProperty);
            searchParameters.appendChild(sorIdentifier);
        }

        if(!eanLocationCodeProperty.isEmpty()) {
            Element eanLocationCode = bodyDocument.createElement("EanLocationCode");
            eanLocationCode.setTextContent(eanLocationCodeProperty);
            searchParameters.appendChild(eanLocationCode);
        }

        if(!eanLocationCodeSystemSupplierIdProperty.isEmpty()) {
            Element eanLocationCodeSystemSupplierId = bodyDocument.createElement("EanLocationCodeSystemSupplierId");
            eanLocationCodeSystemSupplierId.setTextContent(eanLocationCodeSystemSupplierIdProperty);
            searchParameters.appendChild(eanLocationCodeSystemSupplierId);
        }

        if(!entityNameProperty.isEmpty()) {
            Element entityName = bodyDocument.createElement("EntityName");
            entityName.setTextContent(entityNameProperty);
            searchParameters.appendChild(entityName);
        }

        if(!providerIdentifierProperty.isEmpty()) {
            Element providerIdentifier = bodyDocument.createElement("ProviderIdentifier");
            providerIdentifier.setTextContent(providerIdentifierProperty);
            searchParameters.appendChild(providerIdentifier);
        }

        if(!pharmacyIdentifierProperty.isEmpty()) {
            Element pharmacyIdentifier = bodyDocument.createElement("PharmacyIdentifier");
            pharmacyIdentifier.setTextContent(pharmacyIdentifierProperty);
            searchParameters.appendChild(pharmacyIdentifier);
        }

        if(!localCodeProperty.isEmpty()) {
            Element localCode = bodyDocument.createElement("LocalCode");
            localCode.setTextContent(localCodeProperty);
            searchParameters.appendChild(localCode);
        }

        if(!cvrNumberIdentifierProperty.isEmpty()) {
            Element cvrNumberIdentifier = bodyDocument.createElement("CvrNumberIdentifier");
            cvrNumberIdentifier.setTextContent(cvrNumberIdentifierProperty);
            searchParameters.appendChild(cvrNumberIdentifier);
        }

        if(!geographicalLocalisationIdProperty.isEmpty()) {
            Element geographicalLocalisationId = bodyDocument.createElement("GeographicalLocalisationId");
            geographicalLocalisationId.setTextContent(geographicalLocalisationIdProperty);
            searchParameters.appendChild(geographicalLocalisationId);
        }

        searchRequest.appendChild(searchLimit);
        searchRequest.appendChild(searchParameters);
        bodyDocument.appendChild(searchRequest);

        return bodyDocument;
    }

    private String getRandomRequest(String fileProperty) {
        loadList(fileProperty);
        return searchCache.get(RANDOM.nextInt(searchCache.size()));
    }

    private synchronized void loadList(String fileProperty) {
        if (searchCache == null) {
            BufferedReader reader = null;
            try {
                InputStream inputStream;
                File file = new File(fileProperty);
                if (file.exists() && file.canRead()) {
                    inputStream = new BufferedInputStream(new FileInputStream(file));
                } else {
                    inputStream = new BufferedInputStream(getClass().getResourceAsStream(fileProperty));
                }
                reader = new BufferedReader(new InputStreamReader(inputStream));
                Set<String> requests = new HashSet<String>();
                for (String line = reader.readLine(); line != null; line = reader.readLine()) {
                    line = line.trim();
                    if (!line.isEmpty()) {
                        requests.add(line);
                    }
                }
                SORLookupSearchRequestSampler.searchCache = new ArrayList<String>(requests);
            } catch (IOException e) {
                throw new IllegalStateException(e);
            } finally {
                if (reader != null) {
                    try {
                        reader.close();
                    } catch (IOException ignore) {
                    }
                }
            }
        }
    }
}
