/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sor.sampler;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Random;
import java.util.Set;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.jmeter.protocol.http.sampler.HTTPSampleResult;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import dk.sosi.seal.model.Reply;

/**
 *
 * @author Gtitan
 */
public class SORLookupShakSorMapRequestSampler extends AbstractSORLookupSampler {

    private static final long serialVersionUID = 1L;

    public static final String SHAK_IDENTIFIER = "SORLookupShakSorMapRequestSampler.ShakId";

    public static final String DATE = "SORLookupShakSorMapRequestSampler.Date";

    public static final String SHAK_SOR_MAP_FILE = "SORLookupShakSorMapRequestSampler.ShakSorMapFile";

    public static final Random RANDOM = new Random();

    private static List<String> shakSorMapCache;

    public SORLookupShakSorMapRequestSampler() {
        super("ShakSorMap");
    }

    protected void validateContent(Reply reply, HTTPSampleResult sample) {
		if (!reply.getBody().getNamespaceURI().equals("http://sundhedsdatastyrelsen.dk/SORLookupService/2018/09/06/")) {
			sample.setSuccessful(false);
			sample.setResponseMessage("Response missing ShakSorMapResponse node: " + reply.getBody().getNamespaceURI());
		} else if (!reply.getBody().getLocalName().equals("ShakSorMapResponse")) {
			sample.setSuccessful(false);
			sample.setResponseMessage("Response missing ShakSorMapResponse node: " + reply.getBody().getTagName());
		}
        NodeList statusNodes = reply.getBody().getElementsByTagNameNS("http://sundhedsdatastyrelsen.dk/SORLookupService/2018/09/06/", "ShakSorMapResp");
        if (statusNodes == null || statusNodes.getLength() == 0) {
			sample.setSuccessful(false);
			sample.setResponseMessage("No SHAK/SOR mappings found");
        }
    }

    protected Document createRequestBody() {
        Document bodyDocument;
        try {
            bodyDocument = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
        } catch (ParserConfigurationException e) {
            throw new IllegalStateException(e);
        }
        Element shakSorMapRequest = bodyDocument.createElementNS("http://sundhedsdatastyrelsen.dk/SORLookupService/2018/09/06/", "ShakSorMapRequest");
        bodyDocument.appendChild(shakSorMapRequest);

        Element shakIdentifier = bodyDocument.createElement("ShakIdentifier");
        shakSorMapRequest.appendChild(shakIdentifier);

        String shakIdentifierProperty = getPropertyAsString(SHAK_IDENTIFIER);
        String dateProperty = getPropertyAsString(DATE);
        String shakSorMapFileProperty = getPropertyAsString(SHAK_SOR_MAP_FILE);

        if (shakSorMapFileProperty != null && !shakSorMapFileProperty.trim().isEmpty()) {
            String randomRequest = getRandomRequest(shakSorMapFileProperty);

            String[] splitted = randomRequest.split(";", 2);
            shakIdentifierProperty = splitted[0];
            dateProperty = splitted[1];
        }

        shakIdentifier.setTextContent(shakIdentifierProperty);

        if(!dateProperty.isEmpty()) {
            Element date = bodyDocument.createElement("date");
            shakSorMapRequest.appendChild(date);

            date.setTextContent(dateProperty);
        }

        return bodyDocument;
    }

    private String getRandomRequest(String fileProperty) {
        loadList(fileProperty);
        return shakSorMapCache.get(RANDOM.nextInt(shakSorMapCache.size()));
    }

    private synchronized void loadList(String fileProperty) {
        if (shakSorMapCache == null) {
            BufferedReader reader = null;
            try {
                InputStream inputStream;
                File file = new File(fileProperty);
                if (file.exists() && file.canRead()) {
                    inputStream = new BufferedInputStream(new FileInputStream(file));
                } else {
                    inputStream = new BufferedInputStream(getClass().getResourceAsStream(fileProperty));
                }
                reader = new BufferedReader(new InputStreamReader(inputStream));
                Set<String> requests = new HashSet<String>();
                for (String line = reader.readLine(); line != null; line = reader.readLine()) {
                    line = line.trim();
                    if (!line.isEmpty()) {
                        requests.add(line);
                    }
                }
                SORLookupShakSorMapRequestSampler.shakSorMapCache = new ArrayList<String>(requests);
            } catch (IOException e) {
                throw new IllegalStateException(e);
            } finally {
                if (reader != null) {
                    try {
                        reader.close();
                    } catch (IOException ignore) {
                    }
                }
            }
        }
    }
}
