package com.arosii.jmeter.protocol.sor.sampler;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Random;
import java.util.Set;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.jmeter.protocol.http.sampler.HTTPSampleResult;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import dk.sosi.seal.model.Reply;

/**
 *
 * @author Gtitan
 */
public class SORLookupSorShakMapRequestSampler extends AbstractSORLookupSampler {

    private static final long serialVersionUID = 1L;

    public static final String SOR_IDENTIFIER = "SORLookupSorShakMapRequestSampler.SorIdentifier";

    public static final String DATE = "SORLookupSorShakMapRequestSampler.Date";

    public static final String SOR_SHAK_MAP_FILE = "SORLookupSorShakMapRequestSampler.SorShakMapFile";

    public static final Random RANDOM = new Random();

    private static List<String> sorShakMapCache;

    public SORLookupSorShakMapRequestSampler() {
        super("SorShakMap");
    }

    protected void validateContent(Reply reply, HTTPSampleResult sample) {
		if (!reply.getBody().getNamespaceURI().equals("http://sundhedsdatastyrelsen.dk/SORLookupService/2018/09/06/")) {
			sample.setSuccessful(false);
			sample.setResponseMessage("Response missing SorShakMapResponse node: " + reply.getBody().getNamespaceURI());
		} else if (!reply.getBody().getLocalName().equals("SorShakMapResponse")) {
			sample.setSuccessful(false);
			sample.setResponseMessage("Response missing SorShakMapResponse node: " + reply.getBody().getTagName());
		}
        NodeList statusNodes = reply.getBody().getElementsByTagNameNS("http://sundhedsdatastyrelsen.dk/SORLookupService/2018/09/06/", "SorShakMapResp");
        if (statusNodes == null || statusNodes.getLength() == 0) {
			sample.setSuccessful(false);
			sample.setResponseMessage("No SOR/SHAK mappings found");
        }
    }

    protected Document createRequestBody() {
        Document bodyDocument;
        try {
            bodyDocument = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
        } catch (ParserConfigurationException e) {
            throw new IllegalStateException(e);
        }
        Element sorShakMapRequest = bodyDocument.createElementNS("http://sundhedsdatastyrelsen.dk/SORLookupService/2018/09/06/", "SorShakMapRequest");
        bodyDocument.appendChild(sorShakMapRequest);

        Element sorIdentifier = bodyDocument.createElement("SorIdentifier");
        sorShakMapRequest.appendChild(sorIdentifier);

        String sorIdentifierProperty = getPropertyAsString(SOR_IDENTIFIER);
        String dateProperty = getPropertyAsString(DATE);
        String sorShakMapFileProperty = getPropertyAsString(SOR_SHAK_MAP_FILE);

        if (sorShakMapFileProperty != null && !sorShakMapFileProperty.trim().isEmpty()) {
            String randomRequest = getRandomRequest(sorShakMapFileProperty);

            String[] splitted = randomRequest.split(";", 2);
            sorIdentifierProperty = splitted[0];
            dateProperty = splitted[1];
        }

        sorIdentifier.setTextContent(sorIdentifierProperty);

        if(!dateProperty.isEmpty()) {
            Element date = bodyDocument.createElement("Date");
            sorShakMapRequest.appendChild(date);

            date.setTextContent(dateProperty);
        }

        return bodyDocument;
    }

    private String getRandomRequest(String fileProperty) {
        loadList(fileProperty);
        return sorShakMapCache.get(RANDOM.nextInt(sorShakMapCache.size()));
    }

    private synchronized void loadList(String fileProperty) {
        if (sorShakMapCache == null) {
            BufferedReader reader = null;
            try {
                InputStream inputStream;
                File file = new File(fileProperty);
                if (file.exists() && file.canRead()) {
                    inputStream = new BufferedInputStream(new FileInputStream(file));
                } else {
                    inputStream = new BufferedInputStream(getClass().getResourceAsStream(fileProperty));
                }
                reader = new BufferedReader(new InputStreamReader(inputStream));
                Set<String> requests = new HashSet<String>();
                for (String line = reader.readLine(); line != null; line = reader.readLine()) {
                    line = line.trim();
                    if (!line.isEmpty()) {
                        requests.add(line);
                    }
                }
                SORLookupSorShakMapRequestSampler.sorShakMapCache = new ArrayList<String>(requests);
            } catch (IOException e) {
                throw new IllegalStateException(e);
            } finally {
                if (reader != null) {
                    try {
                        reader.close();
                    } catch (IOException ignore) {
                    }
                }
            }
        }
    }
}
