/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sor.sampler;

import dk.sosi.seal.model.Reply;
import org.apache.jmeter.protocol.http.sampler.HTTPSampleResult;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Random;
import java.util.Set;

/**
 * 
 * @author Jacob
 */
public class SORUpdateCloseSorEntityRequestSampler extends AbstractSORUpdateSampler {

	private static final long serialVersionUID = 1L;

	public static final String SOR_IDENTIFIER = "SORUpdateCloseSorEntityRequestSampler.SorIdentifier";

	public static final String FROM_DATE = "SORUpdateCloseSorEntityRequestSampler.FromDate";

	public static final String CLOSE_SOR_ENTITY_FILE = "SORUpdateCloseSorEntityRequestSampler.CloseSorEntityFile";

	private static final Random RANDOM = new Random();

	private static List<String> randomRequestCache;

	public SORUpdateCloseSorEntityRequestSampler() {
		super("CloseSorEntity");
	}

    protected void validateContent(Reply reply, HTTPSampleResult sample) {
        // No content returned
    }

    protected Document createRequestBody() {
		Document bodyDocument;
		try {
			bodyDocument = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
		} catch (ParserConfigurationException e) {
			throw new IllegalStateException(e);
		}

		Element request = initNewRequestElement(bodyDocument, "CloseSorEntityRequest", "SorCentralRole");

		Element sorIdentifier = bodyDocument.createElementNS("urn:oio:sundhedsstyrelsen:organisation:1.0.0", "SorIdentifier");
		request.appendChild(sorIdentifier);

        String sorIdentifierProperty = getPropertyAsString(SOR_IDENTIFIER);
        String fromDateProperty = getPropertyAsString(FROM_DATE);
        String closeSorEntityFileProperty = getPropertyAsString(CLOSE_SOR_ENTITY_FILE);

        if (closeSorEntityFileProperty != null && !closeSorEntityFileProperty.trim().isEmpty()) {
            String randomRequest = getRandomRequest(closeSorEntityFileProperty);

            String[] splitted = randomRequest.split(";", 2);
            sorIdentifierProperty = splitted[0];

            if(splitted.length > 1) {
				fromDateProperty = splitted[1];
			}
        }

		sorIdentifier.setTextContent(sorIdentifierProperty);

        if(!fromDateProperty.isEmpty()) {
			Element fromDate = bodyDocument.createElementNS("http://sundhedsdatastyrelsen.dk/SOROpdateringService/2018/09/06/", "FromDate");
			request.appendChild(fromDate);

			fromDate.setTextContent(fromDateProperty);
		}

		return bodyDocument;
	}

	private String getRandomRequest(String fileProperty) {
		loadList(fileProperty);
		return randomRequestCache.get(RANDOM.nextInt(randomRequestCache.size()));
	}

	private synchronized void loadList(String fileProperty) {
		if (randomRequestCache == null) {
			BufferedReader reader = null;
			try {
				InputStream inputStream;
				File file = new File(fileProperty);
				if (file.exists() && file.canRead()) {
					inputStream = new BufferedInputStream(new FileInputStream(file));
				} else {
					inputStream = new BufferedInputStream(getClass().getResourceAsStream(fileProperty));
				}
				reader = new BufferedReader(new InputStreamReader(inputStream));
				Set<String> requests = new HashSet<String>();
				for (String line = reader.readLine(); line != null; line = reader.readLine()) {
					line = line.trim();
					if (!line.isEmpty()) {
						requests.add(line);
					}
				}
				randomRequestCache = new ArrayList<String>(requests);
			} catch (IOException e) {
				throw new IllegalStateException(e);
			} finally {
				if (reader != null) {
					try {
						reader.close();
					} catch (IOException ignore) {
					}
				}
			}
		}
	}
}
