/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sor.sampler;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.Set;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import dk.sosi.seal.model.Reply;
import org.apache.jmeter.protocol.http.sampler.HTTPSampleResult;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * 
 * @author Jacob
 */
public class SORUpdateCreateSorEntityRequestSampler extends AbstractSORUpdateSampler {

	private static final long serialVersionUID = 1L;

	public static final String SOR_TYPE = "SORUpdateCreateSorEntityRequestSampler.SorType";

	public static final String CREATE_SOR_ENTITY_FILE = "SORUpdateCreateSorEntityRequestSampler.CreateSorEntityFile";

	private static final Random RANDOM = new Random();

	private static List<String> randomRequestCache;

	private final String PARAMETER_SOR_TYPE = "SorType";
	private final String PARAMETER_PARENT_SOR_IDENTIFIER = "ParentSorIdentifier";
	private final String PARAMETER_SOR_STATUS = "SorStatus";
	private final String PARAMETER_SOR_STATUS_FROM_DATE = "FromDate";
	private final String PARAMETER_POSTAL_ADDRESS_INFORMATION = "PostalAddressInformation";
	private final String PARAMETER_POSTAL_ADDRESS_INFORMATION_ADDRESS_POSTAL = "AddressPostal";
	private final String PARAMETER_POSTAL_ADDRESS_INFORMATION_ADDRESS_POSTAL_STREET_NAME = "StreetName";
	private final String PARAMETER_POSTAL_ADDRESS_INFORMATION_ADDRESS_POSTAL_STREET_BUILDING_IDENTIFIER = "StreetBuildingIdentifier";
	private final String PARAMETER_POSTAL_ADDRESS_INFORMATION_ADDRESS_POSTAL_POST_CODE_IDENTIFIER = "PostCodeIdentifier";
	private final String PARAMETER_POSTAL_ADDRESS_INFORMATION_ADDRESS_POSTAL_COUNTRY_IDENTIFICATION_CODE = "CountryIdentificationCode";
	private final String PARAMETER_ENTITY_NAME = "EntityName";
	private final String PARAMETER_ENTITY_TYPE_IDENTIFIER = "EntityTypeIdentifier";
	private final String PARAMETER_EAN_LOCATION_CODE_STATE = "EanLocationCodeState";
	private final String PARAMETER_VIRTUAL_ADDRESS_INFORMATION = "VirtualAddressInformation";
	private final String PARAMETER_VIRTUAL_ADDRESS_INFORMATION_TELEPHONE_NUMBER_IDENTIFIER = "TelephoneNumberIdentifier";
	private final String PARAMETER_CVR_NUMBER_IDENTIFIER = "CVRnumberIdentifier";
	private final String PARAMETER_EAN_LOCATION_CODE_OPTION = "EanLocationCodeOption";
	private final String PARAMETER_EAN_LOCATION_CODE_OPTION_ENTITY_INHERITANCE_INDICATOR = "EntityInheritanceIndicator";

	private static Map<String, Map<String, Object>> requestParameterMap;

	public SORUpdateCreateSorEntityRequestSampler() {
		super("CreateSorEntity");

		requestParameterMap = new HashMap<String, Map<String, Object>>();

		{
			Map<String, Object> ouParameters = new HashMap<String, Object>();
			ouParameters.put(PARAMETER_SOR_TYPE, "OU");
			ouParameters.put(PARAMETER_PARENT_SOR_IDENTIFIER, "926181000016006");
			ouParameters.put(PARAMETER_ENTITY_NAME, "a-7.2.5-");
			ouParameters.put(PARAMETER_ENTITY_TYPE_IDENTIFIER, "554411000005101");
			ouParameters.put(PARAMETER_EAN_LOCATION_CODE_STATE, "None");

			Map<String, Object> sorStatus = new HashMap<String, Object>();
			sorStatus.put(PARAMETER_SOR_STATUS_FROM_DATE, "2020-01-24");
			ouParameters.put(PARAMETER_SOR_STATUS, sorStatus);

			Map<String, Object> eanLocationCodeOption = new HashMap<String, Object>();
			eanLocationCodeOption.put(PARAMETER_EAN_LOCATION_CODE_OPTION_ENTITY_INHERITANCE_INDICATOR, "false");
			ouParameters.put(PARAMETER_EAN_LOCATION_CODE_OPTION, eanLocationCodeOption);

			requestParameterMap.put("OU", ouParameters);
		}

		{
			Map<String, Object> ioParameters = new HashMap<String, Object>();
			ioParameters.put(PARAMETER_SOR_TYPE, "IO");
			ioParameters.put(PARAMETER_ENTITY_NAME, "a-4.2-");
			ioParameters.put(PARAMETER_ENTITY_TYPE_IDENTIFIER, "550891000005100");
			ioParameters.put(PARAMETER_EAN_LOCATION_CODE_STATE, "None");
			ioParameters.put(PARAMETER_CVR_NUMBER_IDENTIFIER, "90008150");

			Map<String, Object> sorStatus = new HashMap<String, Object>();
			sorStatus.put(PARAMETER_SOR_STATUS_FROM_DATE, "2020-01-24");
			ioParameters.put(PARAMETER_SOR_STATUS, sorStatus);

			Map<String, Object> virtualAddressInformation = new HashMap<String, Object>();
			virtualAddressInformation.put(PARAMETER_VIRTUAL_ADDRESS_INFORMATION_TELEPHONE_NUMBER_IDENTIFIER, "12345678");
			ioParameters.put(PARAMETER_VIRTUAL_ADDRESS_INFORMATION, virtualAddressInformation);

			Map<String, Object> postalAddressInformation = new HashMap<String, Object>();
			Map<String, Object> addressPostal = new HashMap<String, Object>();
			addressPostal.put(PARAMETER_POSTAL_ADDRESS_INFORMATION_ADDRESS_POSTAL_STREET_NAME, "Pi");
			addressPostal.put(PARAMETER_POSTAL_ADDRESS_INFORMATION_ADDRESS_POSTAL_STREET_BUILDING_IDENTIFIER, "1");
			addressPostal.put(PARAMETER_POSTAL_ADDRESS_INFORMATION_ADDRESS_POSTAL_POST_CODE_IDENTIFIER, "8382");
			addressPostal.put(PARAMETER_POSTAL_ADDRESS_INFORMATION_ADDRESS_POSTAL_COUNTRY_IDENTIFICATION_CODE, "DK");
			postalAddressInformation.put(PARAMETER_POSTAL_ADDRESS_INFORMATION_ADDRESS_POSTAL, addressPostal);
			ioParameters.put(PARAMETER_POSTAL_ADDRESS_INFORMATION, postalAddressInformation);

			requestParameterMap.put("IO", ioParameters);
		}

		{
			Map<String, Object> hiParameters = new HashMap<String, Object>();
			hiParameters.put(PARAMETER_SOR_TYPE, "HI");
			hiParameters.put(PARAMETER_PARENT_SOR_IDENTIFIER, "926161000016004");
			hiParameters.put(PARAMETER_ENTITY_NAME, "a-7.1.6-");
			hiParameters.put(PARAMETER_ENTITY_TYPE_IDENTIFIER, "264372000");
			hiParameters.put(PARAMETER_EAN_LOCATION_CODE_STATE, "None");

			Map<String, Object> sorStatus = new HashMap<String, Object>();
			sorStatus.put(PARAMETER_SOR_STATUS_FROM_DATE, "2020-01-24");
			hiParameters.put(PARAMETER_SOR_STATUS, sorStatus);

			Map<String, Object> virtualAddressInformation = new HashMap<String, Object>();
			virtualAddressInformation.put(PARAMETER_VIRTUAL_ADDRESS_INFORMATION_TELEPHONE_NUMBER_IDENTIFIER, "12345678");
			hiParameters.put(PARAMETER_VIRTUAL_ADDRESS_INFORMATION, virtualAddressInformation);

			Map<String, Object> eanLocationCodeOption = new HashMap<String, Object>();
			eanLocationCodeOption.put(PARAMETER_EAN_LOCATION_CODE_OPTION_ENTITY_INHERITANCE_INDICATOR, "false");
			hiParameters.put(PARAMETER_EAN_LOCATION_CODE_OPTION, eanLocationCodeOption);

			requestParameterMap.put("HI", hiParameters);
		}
	}

	protected void validateContent(Reply reply, HTTPSampleResult sample) {
		NodeList sorEntityNodes = reply.getBody().getElementsByTagNameNS("http://sor.dksund.dk/", "CreateSorEntityResult");
		if(sorEntityNodes != null && sorEntityNodes.getLength() > 0) {
			for(int i = 0; i < sorEntityNodes.getLength(); i++) {
				if(sorEntityNodes.item(i).getNodeType() != Node.ELEMENT_NODE) {
					sample.setSuccessful(false);
					sample.setResponseMessage("CreateSorEntityResult node in response was not an element node");
				}
			}
		} else {
			sample.setSuccessful(false);
			sample.setResponseMessage("Response missing CreateSorEntityResult node");
		}
	}

	protected Document createRequestBody() {
		Document bodyDocument;
		try {
			bodyDocument = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
		} catch (ParserConfigurationException e) {
			throw new IllegalStateException(e);
		}

		Element request = initNewRequestElement(bodyDocument, "CreateSorEntityRequest", "SorCentralRole");

		Element sorObjectElement = bodyDocument.createElementNS("http://sundhedsdatastyrelsen.dk/SOROpdateringService/2018/09/06/", "SorObject");
		request.appendChild(sorObjectElement);

		String sorTypeProperty = getPropertyAsString(SOR_TYPE);
		String createSorEntityFileProperty = getPropertyAsString(CREATE_SOR_ENTITY_FILE);

		if (createSorEntityFileProperty != null && !createSorEntityFileProperty.trim().isEmpty()) {
			sorTypeProperty = getRandomRequest(createSorEntityFileProperty);
		}

		Map<String, Object> parameterMap = requestParameterMap.get(sorTypeProperty);

		if (parameterMap != null) {
			Object sorTypeParameter = parameterMap.get(PARAMETER_SOR_TYPE);
			if (sorTypeParameter != null) {
				Element sorTypeElement = bodyDocument.createElement(PARAMETER_SOR_TYPE);
				sorTypeElement.setTextContent((String) sorTypeParameter);
				sorObjectElement.appendChild(sorTypeElement);
			}

			Object parentSorIdentifierParameter = parameterMap.get(PARAMETER_PARENT_SOR_IDENTIFIER);
			if (parentSorIdentifierParameter != null) {
				Element parentSorIdentifierElement = bodyDocument.createElement(PARAMETER_PARENT_SOR_IDENTIFIER);
				parentSorIdentifierElement.setTextContent((String) parentSorIdentifierParameter);
				sorObjectElement.appendChild(parentSorIdentifierElement);
			}

			Object entityNameParameter = parameterMap.get(PARAMETER_ENTITY_NAME);
			if (entityNameParameter != null) {
				Element entityNameElement = bodyDocument.createElementNS("urn:oio:sundhedsstyrelsen:organisation:1.0.0", PARAMETER_ENTITY_NAME);
				entityNameElement.setTextContent((String) entityNameParameter);
				sorObjectElement.appendChild(entityNameElement);
			}

			Object entityTypeIdentifierParameter = parameterMap.get(PARAMETER_ENTITY_TYPE_IDENTIFIER);
			if (entityTypeIdentifierParameter != null) {
				Element entityTypeIdentifierElement = bodyDocument.createElementNS("urn:oio:nsi:2.0.0", PARAMETER_ENTITY_TYPE_IDENTIFIER);
				entityTypeIdentifierElement.setTextContent((String) entityTypeIdentifierParameter);
				sorObjectElement.appendChild(entityTypeIdentifierElement);
			}

			Object eanLocationCodeStateParameter = parameterMap.get(PARAMETER_EAN_LOCATION_CODE_STATE);
			if (eanLocationCodeStateParameter != null) {
				Element eanLocationCodeStateElement = bodyDocument.createElementNS("http://sor.dksund.dk/", PARAMETER_EAN_LOCATION_CODE_STATE);
				eanLocationCodeStateElement.setTextContent((String) eanLocationCodeStateParameter);
				sorObjectElement.appendChild(eanLocationCodeStateElement);
			}

			Object cvrNumberIdentifierParameter = parameterMap.get(PARAMETER_CVR_NUMBER_IDENTIFIER);
			if (cvrNumberIdentifierParameter != null) {
				Element cvrNumberIdentifierElement = bodyDocument.createElementNS("http://rep.oio.dk/cvr.dk/xml/schemas/2005/03/22/", PARAMETER_CVR_NUMBER_IDENTIFIER);
				cvrNumberIdentifierElement.setTextContent((String) cvrNumberIdentifierParameter);
				sorObjectElement.appendChild(cvrNumberIdentifierElement);
			}

			Object sorStatusParameter = parameterMap.get(PARAMETER_SOR_STATUS);
			if (sorStatusParameter != null) {
				Map<String, Object> sorStatusParameterMap = (Map<String, Object>) sorStatusParameter;

				Element sorStatusElement = bodyDocument.createElementNS("urn:oio:sundhedsstyrelsen:organisation:1.0.0", PARAMETER_SOR_STATUS);
				sorObjectElement.appendChild(sorStatusElement);

				Object fromDateParameter = sorStatusParameterMap.get(PARAMETER_SOR_STATUS_FROM_DATE);
				if (fromDateParameter != null) {
					Element fromDateElement = bodyDocument.createElementNS("urn:oio:sundhedsstyrelsen:organisation:1.0.0", PARAMETER_SOR_STATUS_FROM_DATE);
					fromDateElement.setTextContent((String) fromDateParameter);
					sorStatusElement.appendChild(fromDateElement);
				}
			}

			Object virtualAddressInformationParameter = parameterMap.get(PARAMETER_VIRTUAL_ADDRESS_INFORMATION);
			if (virtualAddressInformationParameter != null) {
				Map<String, Object> virtualAddressInformationParameterMap = (Map<String, Object>) virtualAddressInformationParameter;

				Element virtualAddressInformationElement = bodyDocument.createElementNS("urn:oio:sundhedsstyrelsen:organisation:1.0.0", PARAMETER_VIRTUAL_ADDRESS_INFORMATION);
				sorObjectElement.appendChild(virtualAddressInformationElement);

				Object telephoneNumberIdentifierParameter = virtualAddressInformationParameterMap.get(PARAMETER_VIRTUAL_ADDRESS_INFORMATION_TELEPHONE_NUMBER_IDENTIFIER);
				if (telephoneNumberIdentifierParameter != null) {
					Element telephoneNumberIdentifierElement = bodyDocument.createElementNS("urn:oio:sundhedsstyrelsen:organisation:1.0.0", PARAMETER_VIRTUAL_ADDRESS_INFORMATION_TELEPHONE_NUMBER_IDENTIFIER);
					telephoneNumberIdentifierElement.setTextContent((String) telephoneNumberIdentifierParameter);
					virtualAddressInformationElement.appendChild(telephoneNumberIdentifierElement);
				}
			}

			Object eanLocationCodeOptionParameter = parameterMap.get(PARAMETER_EAN_LOCATION_CODE_OPTION);
			if (eanLocationCodeOptionParameter != null) {
				Map<String, Object> eanLocationCodeOptionParameterMap = (Map<String, Object>) eanLocationCodeOptionParameter;

				Element eanLocationCodeOptionElement = bodyDocument.createElementNS("urn:oio:nsi:2.0.0", PARAMETER_EAN_LOCATION_CODE_OPTION);
				sorObjectElement.appendChild(eanLocationCodeOptionElement);

				Object entityInheritanceIndicatorParameter = eanLocationCodeOptionParameterMap.get(PARAMETER_EAN_LOCATION_CODE_OPTION_ENTITY_INHERITANCE_INDICATOR);
				if (entityInheritanceIndicatorParameter != null) {
					Element entityInheritanceIndicatorElement = bodyDocument.createElementNS("urn:oio:nsi:2.0.0", PARAMETER_EAN_LOCATION_CODE_OPTION_ENTITY_INHERITANCE_INDICATOR);
					entityInheritanceIndicatorElement.setTextContent((String) entityInheritanceIndicatorParameter);
					eanLocationCodeOptionElement.appendChild(entityInheritanceIndicatorElement);
				}
			}

			Object postalAddressInformationParameter = parameterMap.get(PARAMETER_POSTAL_ADDRESS_INFORMATION);
			if (postalAddressInformationParameter != null) {
				Map<String, Object> postalAddressInformationParameterMap = (Map<String, Object>) postalAddressInformationParameter;

				Element postalAddressInformationElement = bodyDocument.createElementNS("urn:oio:nsi:2.0.0", PARAMETER_POSTAL_ADDRESS_INFORMATION);
				sorObjectElement.appendChild(postalAddressInformationElement);

				Object addressInformationParameter = postalAddressInformationParameterMap.get(PARAMETER_POSTAL_ADDRESS_INFORMATION_ADDRESS_POSTAL);
				if (addressInformationParameter != null) {
					Map<String, Object> addressInformationParameterMap = (Map<String, Object>) addressInformationParameter;

					Element addressInformationElement = bodyDocument.createElementNS("http://rep.oio.dk/xkom.dk/xml/schemas/2006/01/06/", PARAMETER_POSTAL_ADDRESS_INFORMATION_ADDRESS_POSTAL);
					postalAddressInformationElement.appendChild(addressInformationElement);

					Object streetNameParameter = addressInformationParameterMap.get(PARAMETER_POSTAL_ADDRESS_INFORMATION_ADDRESS_POSTAL_STREET_NAME);
					if (streetNameParameter != null) {
						Element localAttribute1Element = bodyDocument.createElementNS("http://rep.oio.dk/ebxml/xml/schemas/dkcc/2005/03/15/", PARAMETER_POSTAL_ADDRESS_INFORMATION_ADDRESS_POSTAL_STREET_NAME);
						localAttribute1Element.setTextContent((String) streetNameParameter);
						postalAddressInformationElement.appendChild(localAttribute1Element);
					}

					Object streetBuildingIdentifierParameter = addressInformationParameterMap.get(PARAMETER_POSTAL_ADDRESS_INFORMATION_ADDRESS_POSTAL_STREET_BUILDING_IDENTIFIER);
					if (streetBuildingIdentifierParameter != null) {
						Element streetBuildingIdentifierElement = bodyDocument.createElementNS("http://rep.oio.dk/ebxml/xml/schemas/dkcc/2003/02/13/", PARAMETER_POSTAL_ADDRESS_INFORMATION_ADDRESS_POSTAL_STREET_BUILDING_IDENTIFIER);
						streetBuildingIdentifierElement.setTextContent((String) streetBuildingIdentifierParameter);
						postalAddressInformationElement.appendChild(streetBuildingIdentifierElement);
					}

					Object postCodeIdentifierParameter = addressInformationParameterMap.get(PARAMETER_POSTAL_ADDRESS_INFORMATION_ADDRESS_POSTAL_POST_CODE_IDENTIFIER);
					if (postCodeIdentifierParameter != null) {
						Element postCodeIdentifierElement = bodyDocument.createElementNS("http://rep.oio.dk/ebxml/xml/schemas/dkcc/2005/03/15/", PARAMETER_POSTAL_ADDRESS_INFORMATION_ADDRESS_POSTAL_POST_CODE_IDENTIFIER);
						postCodeIdentifierElement.setTextContent((String) postCodeIdentifierParameter);
						postalAddressInformationElement.appendChild(postCodeIdentifierElement);
					}

					Object countryIdentificationCodeParameter = addressInformationParameterMap.get(PARAMETER_POSTAL_ADDRESS_INFORMATION_ADDRESS_POSTAL_COUNTRY_IDENTIFICATION_CODE);
					if (countryIdentificationCodeParameter != null) {
						Element countryIdentificationCodeElement = bodyDocument.createElementNS("http://rep.oio.dk/ebxml/xml/schemas/dkcc/2003/02/13/", PARAMETER_POSTAL_ADDRESS_INFORMATION_ADDRESS_POSTAL_COUNTRY_IDENTIFICATION_CODE);
						postalAddressInformationElement.setAttribute("schema", "iso3166-alpha2");
						countryIdentificationCodeElement.setTextContent((String) countryIdentificationCodeParameter);
						postalAddressInformationElement.appendChild(countryIdentificationCodeElement);
					}
				}
			}
		}

		return bodyDocument;
	}

	private String getRandomRequest(String fileProperty) {
		loadList(fileProperty);
		return randomRequestCache.get(RANDOM.nextInt(randomRequestCache.size()));
	}

	private synchronized void loadList(String fileProperty) {
		if (randomRequestCache == null) {
			BufferedReader reader = null;
			try {
				InputStream inputStream;
				File file = new File(fileProperty);
				if (file.exists() && file.canRead()) {
					inputStream = new BufferedInputStream(new FileInputStream(file));
				} else {
					inputStream = new BufferedInputStream(getClass().getResourceAsStream(fileProperty));
				}
				reader = new BufferedReader(new InputStreamReader(inputStream));
				Set<String> requests = new HashSet<String>();
				for (String line = reader.readLine(); line != null; line = reader.readLine()) {
					line = line.trim();
					if (!line.isEmpty()) {
						requests.add(line);
					}
				}
				randomRequestCache = new ArrayList<String>(requests);
			} catch (IOException e) {
				throw new IllegalStateException(e);
			} finally {
				if (reader != null) {
					try {
						reader.close();
					} catch (IOException ignore) {
					}
				}
			}
		}
	}
}
