/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sor.sampler;

import dk.sosi.seal.model.Reply;
import org.apache.jmeter.protocol.http.sampler.HTTPSampleResult;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Random;
import java.util.Set;

/**
 * 
 * @author Jacob
 */
public class SORUpdateEditEanLocationCodeEdiAdministratorRequestSampler extends AbstractSORUpdateSampler {

	private static final long serialVersionUID = 1L;

	public static final String EDI_ADMINISTRATOR_IDENTIFIER = "SORUpdateEditEanLocationCodeEdiAdministratorRequestSampler.EdiAdministratorIdentifier";

	public static final String EAN_LOCATION_CODE_ENTRIES = "SORUpdateEditEanLocationCodeEdiAdministratorRequestSampler.EanLocationCodeEntities";

	public static final String EDIT_EAN_LOCATION_CODE_EDI_ADMINISTRATOR_FILE = "SORUpdateEditEanLocationCodeEdiAdministratorRequestSampler.EditEanLocationCodeEdiAdministratorFile";

	private static final Random RANDOM = new Random();

	private static List<String> randomRequestCache;

	public SORUpdateEditEanLocationCodeEdiAdministratorRequestSampler() {
		super("EditEanLocationCodeEdiAdministrator");
	}

	protected void validateContent(Reply reply, HTTPSampleResult sample) {
		// No content returned
	}

	protected Document createRequestBody() {
		Document bodyDocument;
		try {
			bodyDocument = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
		} catch (ParserConfigurationException e) {
			throw new IllegalStateException(e);
		}

		Element request = initNewRequestElement(bodyDocument, "EditEanLocationCodeEdiAdministratorRequest", "SorEdiCentralRole");

		Element ediAdministratorIdentifier = bodyDocument.createElementNS("urn:oio:nsi:2.0.0", "EdiAdministratorIdentifier");
		request.appendChild(ediAdministratorIdentifier);

		Element eanLocationCodeEntities = bodyDocument.createElementNS("http://sor.dksund.dk/", "EanLocationCodeEntities");
		request.appendChild(eanLocationCodeEntities);

        String ediAdministratorIdentifierProperty = getPropertyAsString(EDI_ADMINISTRATOR_IDENTIFIER);
        String eanLocationCodeEntitiesProperty = getPropertyAsString(EAN_LOCATION_CODE_ENTRIES);
        String editEanLocationCodeEdiAdministratorFileProperty = getPropertyAsString(EDIT_EAN_LOCATION_CODE_EDI_ADMINISTRATOR_FILE);

        if (editEanLocationCodeEdiAdministratorFileProperty != null && !editEanLocationCodeEdiAdministratorFileProperty.trim().isEmpty()) {
            String randomRequest = getRandomRequest(editEanLocationCodeEdiAdministratorFileProperty);

            String[] splitted = randomRequest.split(";", 2);
            ediAdministratorIdentifierProperty = splitted[0];
            eanLocationCodeEntitiesProperty = splitted[1];
        }

		ediAdministratorIdentifier.setTextContent(ediAdministratorIdentifierProperty);

		for(String eanLocationCodeEntity : eanLocationCodeEntitiesProperty.split(",")) {
			Element eanLocationCode = bodyDocument.createElement("eanLocationCode");
			eanLocationCodeEntities.appendChild(eanLocationCode);

			eanLocationCode.setTextContent(eanLocationCodeEntity);
		}

		eanLocationCodeEntities.setTextContent(eanLocationCodeEntitiesProperty);

		return bodyDocument;
	}

	private String getRandomRequest(String fileProperty) {
		loadList(fileProperty);
		return randomRequestCache.get(RANDOM.nextInt(randomRequestCache.size()));
	}

	private synchronized void loadList(String fileProperty) {
		if (randomRequestCache == null) {
			BufferedReader reader = null;
			try {
				InputStream inputStream;
				File file = new File(fileProperty);
				if (file.exists() && file.canRead()) {
					inputStream = new BufferedInputStream(new FileInputStream(file));
				} else {
					inputStream = new BufferedInputStream(getClass().getResourceAsStream(fileProperty));
				}
				reader = new BufferedReader(new InputStreamReader(inputStream));
				Set<String> requests = new HashSet<String>();
				for (String line = reader.readLine(); line != null; line = reader.readLine()) {
					line = line.trim();
					if (!line.isEmpty()) {
						requests.add(line);
					}
				}
				randomRequestCache = new ArrayList<String>(requests);
			} catch (IOException e) {
				throw new IllegalStateException(e);
			} finally {
				if (reader != null) {
					try {
						reader.close();
					} catch (IOException ignore) {
					}
				}
			}
		}
	}
}
