/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sor.sampler;

import dk.sosi.seal.model.Reply;
import org.apache.jmeter.protocol.http.sampler.HTTPSampleResult;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Random;
import java.util.Set;

/**
 * 
 * @author Jacob
 */
public class SORUpdateReplaceSorEntitiesRequestSampler extends AbstractSORUpdateSampler {

	private static final long serialVersionUID = 1L;

	public static final String SOR_IDENTIFIER = "SORUpdateReplaceSorEntitiesRequestSampler.SorIdentifier";

	public static final String REPLACES_ENTITY_COLLECTION = "SORUpdateReplaceSorEntitiesRequestSampler.ReplacesEntityCollection";

	public static final String FROM_DATE = "SORUpdateReplaceSorEntitiesRequestSampler.FromDate";

	public static final String REPLACE_SOR_ENTITIES_FILE = "SORUpdateReplaceSorEntitiesRequestSampler.ReplaceSorEntitiesFile";

	private static final Random RANDOM = new Random();

	private static List<String> randomRequestCache;

	public SORUpdateReplaceSorEntitiesRequestSampler() {
		super("ReplaceSorEntities");
	}

	protected void validateContent(Reply reply, HTTPSampleResult sample) {
		// No content returned
	}

	protected Document createRequestBody() {
		Document bodyDocument;
		try {
			bodyDocument = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
		} catch (ParserConfigurationException e) {
			throw new IllegalStateException(e);
		}

		Element request = initNewRequestElement(bodyDocument, "ReplaceSorEntitiesRequest", "SorCentralRole");

		Element sorIdentifier = bodyDocument.createElementNS("urn:oio:sundhedsstyrelsen:organisation:1.0.0", "SorIdentifier");
		request.appendChild(sorIdentifier);

		Element replacesEntityCollection = bodyDocument.createElementNS("urn:oio:sundhedsstyrelsen:organisation:1.0.0", "ReplacesEntityCollection");
		request.appendChild(sorIdentifier);

		String sorIdentifierProperty = getPropertyAsString(SOR_IDENTIFIER);
		String replacesEntityCollectionProperty = getPropertyAsString(REPLACES_ENTITY_COLLECTION);
        String fromDateProperty = getPropertyAsString(FROM_DATE);
        String replaceSorEntitiesFileProperty = getPropertyAsString(REPLACE_SOR_ENTITIES_FILE);

        if (replaceSorEntitiesFileProperty != null && !replaceSorEntitiesFileProperty.trim().isEmpty()) {
            String randomRequest = getRandomRequest(replaceSorEntitiesFileProperty);

            String[] splitted = randomRequest.split(";", 3);
			sorIdentifierProperty = splitted[0];
			replacesEntityCollectionProperty = splitted[1];

			if(splitted.length > 2) {
				fromDateProperty = splitted[2];
			}
        }

		sorIdentifier.setTextContent(sorIdentifierProperty);

        for(String entity : replacesEntityCollectionProperty.split(",")) {
			Element sorId = bodyDocument.createElementNS("urn:oio:sundhedsstyrelsen:organisation:1.0.0", "SorIdentifier");
			replacesEntityCollection.appendChild(sorId);

			sorId.setTextContent(entity);
		}

        if(!fromDateProperty.isEmpty()) {
			Element fromDate = bodyDocument.createElement("FromDate");
			request.appendChild(fromDate);

			fromDate.setTextContent(fromDateProperty);
		}

		return bodyDocument;
	}

	private String getRandomRequest(String fileProperty) {
		loadList(fileProperty);
		return randomRequestCache.get(RANDOM.nextInt(randomRequestCache.size()));
	}

	private synchronized void loadList(String fileProperty) {
		if (randomRequestCache == null) {
			BufferedReader reader = null;
			try {
				InputStream inputStream;
				File file = new File(fileProperty);
				if (file.exists() && file.canRead()) {
					inputStream = new BufferedInputStream(new FileInputStream(file));
				} else {
					inputStream = new BufferedInputStream(getClass().getResourceAsStream(fileProperty));
				}
				reader = new BufferedReader(new InputStreamReader(inputStream));
				Set<String> requests = new HashSet<String>();
				for (String line = reader.readLine(); line != null; line = reader.readLine()) {
					line = line.trim();
					if (!line.isEmpty()) {
						requests.add(line);
					}
				}
				randomRequestCache = new ArrayList<String>(requests);
			} catch (IOException e) {
				throw new IllegalStateException(e);
			} finally {
				if (reader != null) {
					try {
						reader.close();
					} catch (IOException ignore) {
					}
				}
			}
		}
	}
}
