/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sor.sampler;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import dk.sosi.seal.model.Reply;
import org.apache.jmeter.protocol.http.sampler.HTTPSampleResult;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Random;
import java.util.Set;

/**
 * 
 * @author Jacob
 */
public class SORUpdateUpdateEdiTypesRequestSampler extends AbstractSORUpdateSampler {

	private static final long serialVersionUID = 1L;

	public static final String EDI_IDENTIFIERS = "SORUpdateUpdateEdiTypesRequestSampler.EdiIdentifiers";

	public static final String EAN_LOCATION_CODES = "SORUpdateUpdateEdiTypesRequestSampler.EanLocationCodes";

	public static final String UPDATE_EDI_TYPES_FILE = "SORUpdateUpdateEdiTypesRequestSampler.UpdateEdiTypesFile";

	private static final Random RANDOM = new Random();

	private static List<String> randomRequestCache;

	public SORUpdateUpdateEdiTypesRequestSampler() {
		super("UpdateEdiTypes");
	}

	protected void validateContent(Reply reply, HTTPSampleResult sample) {
		// No content returned
	}

	protected Document createRequestBody() {
		Document bodyDocument;
		try {
			bodyDocument = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
		} catch (ParserConfigurationException e) {
			throw new IllegalStateException(e);
		}

		Element request = initNewRequestElement(bodyDocument, "UpdateEdiTypesRequest", "SorCentralRole");

		Element arrayOfEdiIdentifiers = bodyDocument.createElementNS("http://sor.dksund.dk/", "ArrayOfEdiIdentifiers");
		request.appendChild(arrayOfEdiIdentifiers);

		Element arrayOfEanLocationCodes = bodyDocument.createElementNS("http://sor.dksund.dk/", "EanLocationCodeEntities");
		request.appendChild(arrayOfEanLocationCodes);

        String ediIdentifiersProperty = getPropertyAsString(EDI_IDENTIFIERS);
        String eanLocationCodesProperty = getPropertyAsString(EAN_LOCATION_CODES);
        String updateEdiTypesFileProperty = getPropertyAsString(UPDATE_EDI_TYPES_FILE);

        if (updateEdiTypesFileProperty != null && !updateEdiTypesFileProperty.trim().isEmpty()) {
            String randomRequest = getRandomRequest(updateEdiTypesFileProperty);

            String[] splitted = randomRequest.split(";", 2);
            ediIdentifiersProperty = splitted[0];
            eanLocationCodesProperty = splitted[1];
        }

        for(String ediId : ediIdentifiersProperty.split(",")) {
			Element ediIdentifier = bodyDocument.createElementNS("http://sor.dksund.dk/", "EdiIdentifier");
			arrayOfEdiIdentifiers.appendChild(ediIdentifier);

			ediIdentifier.setTextContent(ediId);
		}

        for(String eanLocation : eanLocationCodesProperty.split(",")) {
			Element eanLocationCode = bodyDocument.createElementNS("http://sor.dksund.dk/", "eanLocationCode");
			arrayOfEanLocationCodes.appendChild(eanLocationCode);

            eanLocationCode.setTextContent(eanLocation);
		}

		return bodyDocument;
	}

	private String getRandomRequest(String fileProperty) {
		loadList(fileProperty);
		return randomRequestCache.get(RANDOM.nextInt(randomRequestCache.size()));
	}

	private synchronized void loadList(String fileProperty) {
		if (randomRequestCache == null) {
			BufferedReader reader = null;
			try {
				InputStream inputStream;
				File file = new File(fileProperty);
				if (file.exists() && file.canRead()) {
					inputStream = new BufferedInputStream(new FileInputStream(file));
				} else {
					inputStream = new BufferedInputStream(getClass().getResourceAsStream(fileProperty));
				}
				reader = new BufferedReader(new InputStreamReader(inputStream));
				Set<String> requests = new HashSet<String>();
				for (String line = reader.readLine(); line != null; line = reader.readLine()) {
					line = line.trim();
					if (!line.isEmpty()) {
						requests.add(line);
					}
				}
				randomRequestCache = new ArrayList<String>(requests);
			} catch (IOException e) {
				throw new IllegalStateException(e);
			} finally {
				if (reader != null) {
					try {
						reader.close();
					} catch (IOException ignore) {
					}
				}
			}
		}
	}
}
