/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sts.assertion;

import com.arosii.jmeter.protocol.nsp.assertion.AbstractNSPAssertion;

import dk.sosi.seal.model.IDCardToOIOSAMLAssertionResponse;
import dk.sosi.seal.model.OIOSAMLAssertionToIDCardResponse;
import dk.sosi.seal.model.SecurityTokenResponse;


/**
 *
 * @author Jacob
 */
public abstract class AbstractSTSAssertion extends AbstractNSPAssertion {

	private static final long serialVersionUID = 1L;

	public static final String FAULT_EXPECTED = "SecurityTokenAssertion.Fault.Expected";

	public static final String CHECK_FAULT_ACTOR = "SecurityTokenAssertion.Fault.Actor";

	public static final String CHECK_FAULT_ACTOR_EXPR = "SecurityTokenAssertion.Fault.Actor.Expression";

	public static final String CHECK_FAULT_ACTOR_METHOD = "SecurityTokenAssertion.Fault.Actor.Method";

	public static final String CHECK_FAULT_CODE = "SecurityTokenAssertion.Fault.Code";

	public static final String CHECK_FAULT_CODE_EXPR = "SecurityTokenAssertion.Fault.Code.Expression";

	public static final String CHECK_FAULT_CODE_METHOD = "SecurityTokenAssertion.Fault.Code.Method";

	public static final String CHECK_FAULT_STRING = "SecurityTokenAssertion.Fault.String";

	public static final String CHECK_FAULT_STRING_EXPR = "SecurityTokenAssertion.Fault.String.Expression";

	public static final String CHECK_FAULT_STRING_METHOD = "SecurityTokenAssertion.Fault.String.Method";

	public static final String METHOD_EQUALS = "equals";

	public static final String METHOD_STARTS_WITH = "startsWith";

	public static final String METHOD_ENDS_WITH = "endsWith";

	public static final String METHOD_MATCHES = "matches";

	protected SecurityTokenResponse getSecurityTokenResponse() {
		return (SecurityTokenResponse) getThreadContext().getVariables().getObject(SecurityTokenResponse.class.getName());
	}
	
	protected IDCardToOIOSAMLAssertionResponse getIDCardToOIOSAMLAssertionResponse() {
		return (IDCardToOIOSAMLAssertionResponse) getThreadContext().getVariables().getObject(IDCardToOIOSAMLAssertionResponse.class.getName());
	}

	protected OIOSAMLAssertionToIDCardResponse getOIOSAMLAssertionToIDCardResponse() {
		return (OIOSAMLAssertionToIDCardResponse) getThreadContext().getVariables().getObject(OIOSAMLAssertionToIDCardResponse.class.getName());
	}

	/**
	 * @return a string with non zero length if the {@code faultValue} does not match the {@code expression} given the provided {@code method}
	 */
	protected String checkFault(String faultValue, String expression, String method, String attribute) {
		if (method.equals(METHOD_EQUALS)) {
			if (!faultValue.equals(expression)) {
				return "The fault " + attribute + " '" + faultValue + "' is not equal to '" + expression + "'. ";
			}
		} else if (method.equals(METHOD_STARTS_WITH)) {
			if (!faultValue.startsWith(expression)) {
				return "The fault " + attribute + " '" + faultValue + "' does not start with '" + expression + "'. ";
			}
		} else if (method.equals(METHOD_ENDS_WITH)) {
			if (!faultValue.endsWith(expression)) {
				return "The fault " + attribute + " '" + faultValue + "' does not end with '" + expression + "'. ";
			}
		} else if (method.equals(METHOD_MATCHES)) {
			if (!faultValue.matches(expression)) {
				return "The fault " + attribute + " '" + faultValue + "' does not match the regular expression '" + expression + "'. ";
			}
		}
		return "";
	}

}
