/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sts.assertion;

import org.apache.jmeter.assertions.AssertionResult;
import org.apache.jmeter.samplers.SampleResult;

import dk.sosi.seal.model.OIOSAMLAssertionToIDCardResponse;

/**
 * 
 * @author Jacob
 */
public class OioSamlToSosiRequestAssertion extends AbstractSTSAssertion {

	private static final long serialVersionUID = 1L;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public AssertionResult getResult(SampleResult sample) {

		// no error as default
		AssertionResult assertion = new AssertionResult(getName());

		if (sample.getResponseData().length == 0) {
			return assertion.setResultForNull();
		}

		assertion.setFailure(false);

		if (sample.isSuccessful()) {
			OIOSAMLAssertionToIDCardResponse response = getOIOSAMLAssertionToIDCardResponse();
			if (response.isFault() && !getPropertyAsBoolean(FAULT_EXPECTED)) {
				assertion.setResultForFailure("Fault not expected: Actor='" + response.getFaultActor() + "', Code='" + response.getFaultCode() + "', String='" + response.getFaultString() + "'");
			} else if (!response.isFault() && getPropertyAsBoolean(FAULT_EXPECTED)) {
				assertion.setResultForFailure("Fault expected");
			} else if (response.isFault() && getPropertyAsBoolean(FAULT_EXPECTED)) {
				StringBuilder faultMessage = new StringBuilder();
				if (getPropertyAsBoolean(CHECK_FAULT_ACTOR)) {
					String faultValue = response.getFaultActor();
					String expression = getPropertyAsString(CHECK_FAULT_ACTOR_EXPR);
					String method = getPropertyAsString(CHECK_FAULT_ACTOR_METHOD);
					faultMessage.append(checkFault(faultValue, expression, method, "actor"));
				}
				if (getPropertyAsBoolean(CHECK_FAULT_CODE)) {
					String faultValue = response.getFaultCode();
					String expression = getPropertyAsString(CHECK_FAULT_CODE_EXPR);
					String method = getPropertyAsString(CHECK_FAULT_CODE_METHOD);
					faultMessage.append(checkFault(faultValue, expression, method, "code"));
				}
				if (getPropertyAsBoolean(CHECK_FAULT_STRING)) {
					String faultValue = response.getFaultString();
					String expression = getPropertyAsString(CHECK_FAULT_STRING_EXPR);
					String method = getPropertyAsString(CHECK_FAULT_STRING_METHOD);
					faultMessage.append(checkFault(faultValue, expression, method, "string"));
				}
				if (faultMessage.length() > 0) {
					assertion.setResultForFailure(faultMessage.toString());
				}
			}
		} else {
			assertion.setResultForFailure("Sample not successful");
		}

		return assertion;
	}
}
