/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sts.assertion.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.BorderFactory;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;
import org.apache.jmeter.util.JMeterUtils;

import com.arosii.jmeter.protocol.nsp.assertion.gui.AbstractNSPAssertionGui;
import com.arosii.jmeter.protocol.sts.assertion.SecurityTokenAssertion;
import com.arosii.jmeter.resources.sts.Messages;


/**
 *
 * @author Jacob
 */
public abstract class AbstractSTSAssertionGui extends AbstractNSPAssertionGui {

	private static final long serialVersionUID = 1L;

	protected JCheckBox expectFault;

	protected JCheckBox checkActor;

	protected JTextField checkActorExpression;

	protected JComboBox checkActorMethod;

	protected JCheckBox checkCode;

	protected JTextField checkCodeExpression;

	protected JComboBox checkCodeMethod;

	protected JCheckBox checkString;

	protected JTextField checkStringExpression;

	protected JComboBox checkStringMethod;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getStaticLabel() {
		String label = Messages.getResString(getLabelResource());
		if (label == null || label.startsWith(JMeterUtils.RES_KEY_PFX)) {
			label = super.getStaticLabel();
		}
		return label;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected Component createConfigurationPanel() {
		final VerticalPanel configurationPanel = new VerticalPanel();
		appendFailurePanel(configurationPanel);
		return configurationPanel;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void configure(TestElement element) {
		super.configure(element);
		expectFault.setSelected(element.getPropertyAsBoolean(SecurityTokenAssertion.FAULT_EXPECTED, false));
		checkActor.setSelected(element.getPropertyAsBoolean(SecurityTokenAssertion.CHECK_FAULT_ACTOR, false));
		checkActorExpression.setText(element.getPropertyAsString(SecurityTokenAssertion.CHECK_FAULT_ACTOR_EXPR, ""));
		String actorMethod = element.getPropertyAsString(SecurityTokenAssertion.CHECK_FAULT_ACTOR_METHOD, "");
		if (actorMethod.equals("")) {
			checkActorMethod.setSelectedIndex(0);
		} else {
			checkActorMethod.setSelectedItem(actorMethod);
		}
		checkCode.setSelected(element.getPropertyAsBoolean(SecurityTokenAssertion.CHECK_FAULT_CODE, false));
		checkCodeExpression.setText(element.getPropertyAsString(SecurityTokenAssertion.CHECK_FAULT_CODE_EXPR, ""));
		String codeMethod = element.getPropertyAsString(SecurityTokenAssertion.CHECK_FAULT_CODE_METHOD, "");
		if (codeMethod.equals("")) {
			checkCodeMethod.setSelectedIndex(0);
		} else {
			checkCodeMethod.setSelectedItem(codeMethod);
		}
		checkString.setSelected(element.getPropertyAsBoolean(SecurityTokenAssertion.CHECK_FAULT_STRING, false));
		checkStringExpression.setText(element.getPropertyAsString(SecurityTokenAssertion.CHECK_FAULT_STRING_EXPR, ""));
		String stringMethod = element.getPropertyAsString(SecurityTokenAssertion.CHECK_FAULT_STRING_METHOD, "");
		if (stringMethod.equals("")) {
			checkStringMethod.setSelectedIndex(0);
		} else {
			checkStringMethod.setSelectedItem(stringMethod);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void modifyTestElement(TestElement element) {
		super.modifyTestElement(element);
		element.setProperty(SecurityTokenAssertion.FAULT_EXPECTED, expectFault.isSelected());
		element.setProperty(SecurityTokenAssertion.CHECK_FAULT_ACTOR, checkActor.isSelected());
		element.setProperty(SecurityTokenAssertion.CHECK_FAULT_ACTOR_EXPR, checkActor.isSelected() ? checkActorExpression.getText() : "");
		element.setProperty(SecurityTokenAssertion.CHECK_FAULT_ACTOR_METHOD, checkActor.isSelected() ? checkActorMethod.getSelectedItem().toString() : "");
		element.setProperty(SecurityTokenAssertion.CHECK_FAULT_CODE, checkCode.isSelected());
		element.setProperty(SecurityTokenAssertion.CHECK_FAULT_CODE_EXPR, checkCode.isSelected() ? checkCodeExpression.getText() : "");
		element.setProperty(SecurityTokenAssertion.CHECK_FAULT_CODE_METHOD, checkCode.isSelected() ? checkCodeMethod.getSelectedItem().toString() : "");
		element.setProperty(SecurityTokenAssertion.CHECK_FAULT_STRING, checkString.isSelected());
		element.setProperty(SecurityTokenAssertion.CHECK_FAULT_STRING_EXPR, checkString.isSelected() ? checkStringExpression.getText() : "");
		element.setProperty(SecurityTokenAssertion.CHECK_FAULT_STRING_METHOD, checkString.isSelected() ? checkStringMethod.getSelectedItem().toString() : "");
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void clearGui() {
		super.clearGui();
		expectFault.setSelected(false);
		checkActor.setSelected(false);
		checkCode.setSelected(false);
		checkString.setSelected(false);
		updateButtonState();
	}

	private void appendFailurePanel(VerticalPanel configurationPanel) {
		JPanel failurePanel = new VerticalPanel();
		failurePanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("sts_id_card")));
		failurePanel.add(getExpectedFaultPanel());
		failurePanel.add(getExpectedFaultActorPanel());
		failurePanel.add(getExpectedFaultCodePanel());
		failurePanel.add(getExpectedFaultStringPanel());
		configurationPanel.add(failurePanel);
	}

	private Component getExpectedFaultPanel() {
		expectFault = new JCheckBox();
		expectFault.addItemListener(new CheckItemListener());
		JLabel label = new JLabel(Messages.getResString("sts_assert_id_card_fault"));

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(expectFault, BorderLayout.CENTER);

		return panel;
	}

	private Component getExpectedFaultActorPanel() {
		checkActor = new JCheckBox();
		checkActor.addItemListener(new CheckItemListener());
		checkActorExpression = new JTextField(20);
		checkActorMethod = new JComboBox(new String[] { SecurityTokenAssertion.METHOD_EQUALS, SecurityTokenAssertion.METHOD_STARTS_WITH, SecurityTokenAssertion.METHOD_ENDS_WITH, SecurityTokenAssertion.METHOD_MATCHES });
		checkActorMethod.setRenderer(new MethodRenderer());
		JLabel label = new JLabel(Messages.getResString("sts_assert_id_card_actor"));
		JLabel expressionLabel = new JLabel(Messages.getResString("sts_assert_id_card_expression"));
		JLabel methodLabel = new JLabel(Messages.getResString("sts_assert_id_card_method"));

		JPanel expectedFaultActorPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		expectedFaultActorPanel.add(checkActor);
		expectedFaultActorPanel.add(expressionLabel);
		expectedFaultActorPanel.add(checkActorExpression);
		expectedFaultActorPanel.add(methodLabel);
		expectedFaultActorPanel.add(checkActorMethod);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(expectedFaultActorPanel, BorderLayout.CENTER);

		return panel;
	}

	private Component getExpectedFaultCodePanel() {
		checkCode = new JCheckBox();
		checkCode.addItemListener(new CheckItemListener());
		checkCodeExpression = new JTextField(20);
		checkCodeMethod = new JComboBox(new String[] { SecurityTokenAssertion.METHOD_EQUALS, SecurityTokenAssertion.METHOD_STARTS_WITH, SecurityTokenAssertion.METHOD_ENDS_WITH, SecurityTokenAssertion.METHOD_MATCHES });
		checkCodeMethod.setRenderer(new MethodRenderer());
		JLabel label = new JLabel(Messages.getResString("sts_assert_id_card_code"));
		JLabel expressionLabel = new JLabel(Messages.getResString("sts_assert_id_card_expression"));
		JLabel methodLabel = new JLabel(Messages.getResString("sts_assert_id_card_method"));

		JPanel expectedFaultCodePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		expectedFaultCodePanel.add(checkCode);
		expectedFaultCodePanel.add(expressionLabel);
		expectedFaultCodePanel.add(checkCodeExpression);
		expectedFaultCodePanel.add(methodLabel);
		expectedFaultCodePanel.add(checkCodeMethod);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(expectedFaultCodePanel, BorderLayout.CENTER);

		return panel;
	}

	private Component getExpectedFaultStringPanel() {
		checkString = new JCheckBox();
		checkString.addItemListener(new CheckItemListener());
		checkStringExpression = new JTextField(20);
		checkStringMethod = new JComboBox(new String[] { SecurityTokenAssertion.METHOD_EQUALS, SecurityTokenAssertion.METHOD_STARTS_WITH, SecurityTokenAssertion.METHOD_ENDS_WITH, SecurityTokenAssertion.METHOD_MATCHES });
		checkStringMethod.setRenderer(new MethodRenderer());
		JLabel label = new JLabel(Messages.getResString("sts_assert_id_card_string"));
		JLabel expressionLabel = new JLabel(Messages.getResString("sts_assert_id_card_expression"));
		JLabel methodLabel = new JLabel(Messages.getResString("sts_assert_id_card_method"));

		JPanel expectedFaultStringPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		expectedFaultStringPanel.add(checkString);
		expectedFaultStringPanel.add(expressionLabel);
		expectedFaultStringPanel.add(checkStringExpression);
		expectedFaultStringPanel.add(methodLabel);
		expectedFaultStringPanel.add(checkStringMethod);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(expectedFaultStringPanel, BorderLayout.CENTER);

		return panel;
	}

	private void updateButtonState() {
		if (expectFault.isSelected()) {
			checkActor.setEnabled(true);
			checkCode.setEnabled(true);
			checkString.setEnabled(true);
		} else {
			if (checkActor.isSelected()) {
				checkActor.setSelected(false);
			}
			checkActor.setEnabled(false);
			if (checkCode.isSelected()) {
				checkCode.setSelected(false);
			}
			checkCode.setEnabled(false);
			if (checkString.isSelected()) {
				checkString.setSelected(false);
			}
			checkString.setEnabled(false);
		}
		if (checkActor.isSelected()) {
			checkActorExpression.setEnabled(true);
			checkActorMethod.setEnabled(true);
		} else {
			checkActorExpression.setText("");
			checkActorExpression.setEnabled(false);
			checkActorMethod.setSelectedIndex(0);
			checkActorMethod.setEnabled(false);
		}
		if (checkCode.isSelected()) {
			checkCodeExpression.setEnabled(true);
			checkCodeMethod.setEnabled(true);
		} else {
			checkCodeExpression.setText("");
			checkCodeExpression.setEnabled(false);
			checkCodeMethod.setSelectedIndex(0);
			checkCodeMethod.setEnabled(false);
		}
		if (checkString.isSelected()) {
			checkStringExpression.setEnabled(true);
			checkStringMethod.setEnabled(true);
		} else {
			checkStringExpression.setText("");
			checkStringExpression.setEnabled(false);
			checkStringMethod.setSelectedIndex(0);
			checkStringMethod.setEnabled(false);
		}
	}

	private class CheckItemListener implements ItemListener {

		/**
		 * {@inheritDoc}
		 */
		@Override
		public void itemStateChanged(ItemEvent e) {
			updateButtonState();
		}

	}

	private class MethodRenderer extends DefaultListCellRenderer {

		private static final long serialVersionUID = 1L;

		/**
		 * {@inheritDoc}
		 */
		@Override
		public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
			super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
			String method = (String) value;
			setText(Messages.getResString("sts_assert_id_card_method_" + method.toLowerCase()));
			return this;
		}
	}
}
