/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sts.config.gui;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.BorderFactory;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;

import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;

import com.arosii.jmeter.protocol.sts.control.STSIdentity;
import com.arosii.jmeter.protocol.sts.control.STSIdentity.STSIdentityRenderer;
import com.arosii.jmeter.protocol.sts.control.STSIdentity.Type;
import com.arosii.jmeter.protocol.sts.sampler.SecurityTokenRequestSampler;
import com.arosii.jmeter.resources.sts.Messages;

/**
 * 
 * @author Jacob Qvortrup
 */
public class IDCardConfigGui extends VerticalPanel {

	private static final long serialVersionUID = 1L;

	private static final String PREDEFINED_IDENTITY = "SecurityTokenRequestSamplerGui.Predefined.Identity";

	private JComboBox predefinedIdentities;

	private JTextField certificatePath;

	private JPasswordField certificatePassword;

	private JComboBox certificateType;

	private JTextField cpr;

	private JTextField givenName;

	private JTextField surName;

	private JTextField email;

	private JTextField occupation;

	private JTextField role;

	private JTextField authorizationCode;

	private JTextField orgType;

	private JTextField orgId;

	private JTextField orgName;

	private Type[] allowedTypes;

	public IDCardConfigGui(Type ... allowedTypes) {
		this.allowedTypes = allowedTypes;
		setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("sts_id_card")));
		add(getPredefinedIdentityPanel());
		add(getCredentialVaultPanel());
		add(getUserInfoPanel());
		add(getCareProviderPanel());
	}

	public void addItemListener(ItemListener listener) {
		predefinedIdentities.addItemListener(listener);
	}

	public void removeItemListener(ItemListener listener) {
		predefinedIdentities.removeItemListener(listener);
	}

	public void configure(TestElement element) {
		String identity = element.getPropertyAsString(PREDEFINED_IDENTITY, "");
		if (!identity.equals("")) {
			predefinedIdentities.setSelectedItem(STSIdentity.valueOf(identity));
		} else {
			predefinedIdentities.setSelectedItem(STSIdentity.NOT_FILLED);
		}

		certificatePath.setText(element.getPropertyAsString(SecurityTokenRequestSampler.CERTIFICATE_PATH, ""));
		certificatePassword.setText(element.getPropertyAsString(SecurityTokenRequestSampler.CERTIFICATE_PASSWORD, ""));
		String type = element.getPropertyAsString(SecurityTokenRequestSampler.CERTIFICATE_TYPE, "");
		if (type != null && !type.trim().equals("")) {
			certificateType.setSelectedItem(Type.valueOf(type));
		} else {
			certificateType.setSelectedIndex(-1);
		}
		cpr.setText(element.getPropertyAsString(SecurityTokenRequestSampler.USER_CPR, ""));
		givenName.setText(element.getPropertyAsString(SecurityTokenRequestSampler.USER_GIVENNAME, ""));
		surName.setText(element.getPropertyAsString(SecurityTokenRequestSampler.USER_SURNAME, ""));
		email.setText(element.getPropertyAsString(SecurityTokenRequestSampler.USER_EMAIL, ""));
		occupation.setText(element.getPropertyAsString(SecurityTokenRequestSampler.USER_OCCUPATION, ""));
		role.setText(element.getPropertyAsString(SecurityTokenRequestSampler.USER_ROLE, ""));
		authorizationCode.setText(element.getPropertyAsString(SecurityTokenRequestSampler.USER_AUTHORIZATIONCODE, ""));
		orgType.setText(element.getPropertyAsString(SecurityTokenRequestSampler.ORG_TYPE, ""));
		orgId.setText(element.getPropertyAsString(SecurityTokenRequestSampler.ORG_ID, ""));
		orgName.setText(element.getPropertyAsString(SecurityTokenRequestSampler.ORG_NAME, ""));
	}

	public void modifyTestElement(TestElement element) {
		STSIdentity identity = (STSIdentity) predefinedIdentities.getSelectedItem();
		if (identity != null && identity != STSIdentity.NOT_FILLED) {
			element.setProperty(PREDEFINED_IDENTITY, identity.name());
		} else {
			element.setProperty(PREDEFINED_IDENTITY, "");
		}
		element.setProperty(SecurityTokenRequestSampler.CERTIFICATE_PATH, certificatePath.getText());
		element.setProperty(SecurityTokenRequestSampler.CERTIFICATE_PASSWORD, new String(certificatePassword.getPassword()));
		Type type = (Type) certificateType.getSelectedItem();
		if (type != null) {
			element.setProperty(SecurityTokenRequestSampler.CERTIFICATE_TYPE, type.name());
		} else {
			element.setProperty(SecurityTokenRequestSampler.CERTIFICATE_TYPE, "");
		}
		element.setProperty(SecurityTokenRequestSampler.USER_CPR, cpr.getText());
		element.setProperty(SecurityTokenRequestSampler.USER_GIVENNAME, givenName.getText());
		element.setProperty(SecurityTokenRequestSampler.USER_SURNAME, surName.getText());
		element.setProperty(SecurityTokenRequestSampler.USER_EMAIL, email.getText());
		element.setProperty(SecurityTokenRequestSampler.USER_OCCUPATION, occupation.getText());
		element.setProperty(SecurityTokenRequestSampler.USER_ROLE, role.getText());
		element.setProperty(SecurityTokenRequestSampler.USER_AUTHORIZATIONCODE, authorizationCode.getText());
		element.setProperty(SecurityTokenRequestSampler.ORG_TYPE, orgType.getText());
		element.setProperty(SecurityTokenRequestSampler.ORG_ID, orgId.getText());
		element.setProperty(SecurityTokenRequestSampler.ORG_NAME, orgName.getText());
	}

	public void clearGui() {
		predefinedIdentities.setSelectedItem(STSIdentity.NOT_FILLED);
		certificatePath.setText("");
		certificatePassword.setText("");
		certificateType.setSelectedIndex(-1);
		cpr.setText("");
		givenName.setText("");
		surName.setText("");
		email.setText("");
		occupation.setText("");
		role.setText("");
		authorizationCode.setText("");
		orgType.setText("");
		orgId.setText("");
		orgName.setText("");
	}

	private JPanel getPredefinedIdentityPanel() {
		predefinedIdentities = new JComboBox(STSIdentity.all(allowedTypes));
		predefinedIdentities.setRenderer(new STSIdentityRenderer());
		predefinedIdentities.addItemListener(new IdentityItemListener());
		JLabel label = new JLabel(Messages.getResString("sts_identity"));

		JPanel predefinedIdentitiesPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		predefinedIdentitiesPanel.add(predefinedIdentities);

		JPanel identityPanel = new JPanel(new BorderLayout(5, 0));
		identityPanel.add(label, BorderLayout.WEST);
		identityPanel.add(predefinedIdentitiesPanel, BorderLayout.CENTER);

		return identityPanel;
	}

	private JPanel getCredentialVaultPanel() {
		certificatePath = new JTextField(40);
		certificatePassword = new JPasswordField(10);
		certificateType = new JComboBox(Type.values());
		certificateType.setSelectedIndex(-1);
		JLabel label = new JLabel(Messages.getResString("sts_vault_path"));
		JLabel passwordLabel = new JLabel(Messages.getResString("sts_vault_password"));
		JLabel typeLabel = new JLabel(Messages.getResString("sts_vault_type"));

		JPanel credentialVaultPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		credentialVaultPanel.add(certificatePath);
		credentialVaultPanel.add(passwordLabel);
		credentialVaultPanel.add(certificatePassword);
		credentialVaultPanel.add(typeLabel);
		credentialVaultPanel.add(certificateType);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(credentialVaultPanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getUserInfoPanel() {
		cpr = new JTextField(10);
		givenName = new JTextField(10);
		surName = new JTextField(10);
		email = new JTextField(12);
		occupation = new JTextField(10);
		role = new JTextField(10);
		authorizationCode = new JTextField(6);
		JLabel cprLabel = new JLabel(Messages.getResString("sts_user_cpr"));
		JLabel givenNameLabel = new JLabel(Messages.getResString("sts_user_given_name"));
		JLabel surNameLabel = new JLabel(Messages.getResString("sts_user_sur_name"));
		JLabel emailLabel = new JLabel(Messages.getResString("sts_user_email"));
		JLabel occupationLabel = new JLabel(Messages.getResString("sts_user_occupation"));
		JLabel roleLabel = new JLabel(Messages.getResString("sts_user_role"));
		JLabel authorizationCodeLabel = new JLabel(Messages.getResString("sts_user_authorization_code"));

		JPanel userInfoPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		userInfoPanel.add(cpr);
		userInfoPanel.add(givenNameLabel);
		userInfoPanel.add(givenName);
		userInfoPanel.add(surNameLabel);
		userInfoPanel.add(surName);
		userInfoPanel.add(emailLabel);
		userInfoPanel.add(email);
		userInfoPanel.add(occupationLabel);
		userInfoPanel.add(occupation);
		userInfoPanel.add(roleLabel);
		userInfoPanel.add(role);
		userInfoPanel.add(authorizationCodeLabel);
		userInfoPanel.add(authorizationCode);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(cprLabel, BorderLayout.WEST);
		panel.add(userInfoPanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getCareProviderPanel() {
		orgType = new JTextField(15);
		orgId = new JTextField(8);
		orgName = new JTextField(15);
		JLabel typeLabel = new JLabel(Messages.getResString("sts_care_provider_type"));
		JLabel idLabel = new JLabel(Messages.getResString("sts_care_provider_id"));
		JLabel orgNameLabel = new JLabel(Messages.getResString("sts_care_provider_name"));

		JPanel careProviderPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		careProviderPanel.add(orgType);
		careProviderPanel.add(idLabel);
		careProviderPanel.add(orgId);
		careProviderPanel.add(orgNameLabel);
		careProviderPanel.add(orgName);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(typeLabel, BorderLayout.WEST);
		panel.add(careProviderPanel, BorderLayout.CENTER);

		return panel;
	}

	private class IdentityItemListener implements ItemListener {

		/**
		 * {@inheritDoc}
		 */
		@Override
		public void itemStateChanged(ItemEvent e) {
			if (e.getStateChange() == ItemEvent.SELECTED) {
				if (e.getItem() != STSIdentity.NOT_FILLED && e.getItem() != null) {
					STSIdentity identity = (STSIdentity) e.getItem();
					certificatePath.setText(identity.getPath());
					certificatePassword.setText(identity.getPassword());
					certificateType.setSelectedItem(identity.getType());
					if (identity.getUserInfo() != null) {
						cpr.setText(identity.getUserInfo().getCPR());
						givenName.setText(identity.getUserInfo().getGivenName());
						surName.setText(identity.getUserInfo().getSurName());
						email.setText(identity.getUserInfo().getEmail());
						occupation.setText(identity.getUserInfo().getOccupation());
						role.setText(identity.getUserInfo().getRole());
						authorizationCode.setText(identity.getUserInfo().getAuthorizationCode());
					} else {
						cpr.setText("");
						givenName.setText("");
						surName.setText("");
						email.setText("");
						occupation.setText("");
						role.setText("");
						authorizationCode.setText("");
					}
					if (identity.getCareProvider() != null) {
						orgType.setText(identity.getCareProvider().getType());
						orgId.setText(identity.getCareProvider().getID());
						orgName.setText(identity.getCareProvider().getOrgName());
					} else {
						orgType.setText("");
						orgId.setText("");
						orgName.setText("");
					}
					certificatePath.setEnabled(false);
					certificatePassword.setEnabled(false);
					certificateType.setEnabled(false);
					cpr.setEnabled(false);
					givenName.setEnabled(false);
					surName.setEnabled(false);
					email.setEnabled(false);
					occupation.setEnabled(false);
					role.setEnabled(false);
					authorizationCode.setEnabled(false);
					orgType.setEnabled(false);
					orgId.setEnabled(false);
					orgName.setEnabled(false);
				} else {
					certificatePath.setEnabled(true);
					certificatePassword.setEnabled(true);
					certificateType.setEnabled(true);
					cpr.setEnabled(true);
					givenName.setEnabled(true);
					surName.setEnabled(true);
					email.setEnabled(true);
					occupation.setEnabled(true);
					role.setEnabled(true);
					authorizationCode.setEnabled(true);
					orgType.setEnabled(true);
					orgId.setEnabled(true);
					orgName.setEnabled(true);
				}
			}
		}
	}
}
