/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sts.control;

import java.awt.Component;
import java.util.Arrays;
import java.util.EnumSet;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.swing.DefaultListCellRenderer;
import javax.swing.JList;

import com.arosii.jmeter.resources.sts.Messages;

import dk.sosi.seal.model.CareProvider;
import dk.sosi.seal.model.UserInfo;
import dk.sosi.seal.model.constants.SubjectIdentifierTypeValues;

/**
 * 
 * @author Jacob
 */
public enum STSIdentity {

	NOT_FILLED(null, null, null, null, null),

	VALID_MOCES(Type.MOCES, "oces2/PP/MOCES_cpr_gyldig.p12", "Test1234", new UserInfo("1802602810", "Anders", "And", "aand@andeby.dk", "Læge", "7170", "ZXCVB"), new CareProvider(SubjectIdentifierTypeValues.CVR_NUMBER, "30808460", "orgName")),

	VALID_VOCES(Type.VOCES, "oces2/PP/VOCES_gyldig.p12", "Test1234", null, new CareProvider(SubjectIdentifierTypeValues.CVR_NUMBER, "30808460", "orgName")),

	EXPIRED_MOCES(Type.MOCES, "oces2/PP/MOCES_udloebet.p12", "Test1234", new UserInfo("1802602810", "Anders", "And", "aand@andeby.dk", "coolDuck", "1234", "12345"), new CareProvider(SubjectIdentifierTypeValues.CVR_NUMBER, "30808460", "orgName")),

	EXPIRED_VOCES(Type.VOCES, "oces2/PP/VOCES_udloebet.p12", "Test1234", null, new CareProvider(SubjectIdentifierTypeValues.CVR_NUMBER, "30808460", "orgName")),

	REVOKED_MOCES(Type.MOCES, "oces2/PP/MOCES_spaerret.p12", "Test1234", new UserInfo("1802602810", "Anders", "And", "aand@andeby.dk", "coolDuck", "1234", "12345"), new CareProvider(SubjectIdentifierTypeValues.CVR_NUMBER, "30808460", "orgName")),

	REVOKED_VOCES(Type.VOCES, "oces2/PP/VOCES_spaerret.p12", "Test1234", null, new CareProvider(SubjectIdentifierTypeValues.CVR_NUMBER, "30808460", "orgName")),

	;

	public static enum Type {
		MOCES, VOCES, FOCES
	}

	private final Type type;

	private final String path;

	private final String password;

	private final UserInfo userInfo;

	private final CareProvider careProvider;

	private STSIdentity(Type type, String path, String password, UserInfo userInfo, CareProvider careProvider) {
		this.type = type;
		this.path = path;
		this.password = password;
		this.userInfo = userInfo;
		this.careProvider = careProvider;
	}

	public Type getType() {
		return type;
	}

	public String getPath() {
		return path;
	}

	public String getPassword() {
		return password;
	}

	public UserInfo getUserInfo() {
		return userInfo;
	}

	public CareProvider getCareProvider() {
		return careProvider;
	}

	public static STSIdentity[] all(Type ... allowedTypes) {
		EnumSet<STSIdentity> all = EnumSet.allOf(STSIdentity.class);
		if (allowedTypes != null && allowedTypes.length > 0) {
			EnumSet<Type> types = EnumSet.copyOf(Arrays.asList(allowedTypes));
			for (Iterator<STSIdentity> iterator = all.iterator(); iterator.hasNext();) {
				STSIdentity stsIdentity = iterator.next();
				if (stsIdentity.type != null && !types.contains(stsIdentity.type)) {
					iterator.remove();
				}
			}
		}
		return (STSIdentity[]) all.toArray(new STSIdentity[all.size()]);
	}

	public static String getIdentityName(STSIdentity identity) {
		return Messages.getResString(STSIdentity.class.getSimpleName().toLowerCase() + '_' + identity.name().toLowerCase());
	}
	
	public static boolean isIdentityName(String name) {
		return identityNames.contains(name);
	}

	private static final Set<String> identityNames = new HashSet<String>();

	static {
		for (STSIdentity identity : EnumSet.allOf(STSIdentity.class)) {
			identityNames.add(getIdentityName(identity));
		}
	}

	public static class STSIdentityRenderer extends DefaultListCellRenderer {

		private static final long serialVersionUID = 1L;

		/**
		 * {@inheritDoc}
		 */
		@Override
		public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
			super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
			STSIdentity identity = (STSIdentity) value;
			setText(STSIdentity.getIdentityName(identity));
			return this;
		}
	}

}
