/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sts.control.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.BorderFactory;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;

import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;

import com.arosii.jmeter.protocol.sts.control.STSIdentity;
import com.arosii.jmeter.protocol.sts.control.STSIdentity.STSIdentityRenderer;
import com.arosii.jmeter.protocol.sts.sampler.OioSamlToSosiRequestSampler;
import com.arosii.jmeter.resources.sts.Messages;

/**
 * 
 * @author Jacob
 */
public class OioSamlToSosiRequestSamplerGui extends AbstractSTSSamplerGui {

	private static final long serialVersionUID = 1L;

	private static final String PREDEFINED_IDENTITY = "OioSamlToSosiRequestSamplerGui.Predefined.Identity";

	private JTextField assertionPath;

	private JComboBox predefinedIdentities;

	private JTextField certificatePath;

	private JPasswordField certificatePassword;

	private JTextField audience;

	private JTextField itSystemName;

	private JTextField userAuthorizationCode;

	private JTextField userEducationCode;

	private JTextField userGivenName;

	private JTextField userSurName;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getLabelResource() {
		return "sts_oio_saml_to_sosi";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public TestElement createTestElement() {
		OioSamlToSosiRequestSampler sampler = new OioSamlToSosiRequestSampler();
		modifyTestElement(sampler);
		return sampler;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected Component createConfigurationPanel() {
		VerticalPanel configurationPanel = new VerticalPanel();
		appendAssertionPanel(configurationPanel);
		appendCredentialVaultPanel(configurationPanel);
		appendAttributesPanel(configurationPanel);
		return configurationPanel;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void configure(TestElement element) {
		super.configure(element);
		assertionPath.setText(element.getPropertyAsString(OioSamlToSosiRequestSampler.ASSERTION_PATH, ""));
		String identity = element.getPropertyAsString(PREDEFINED_IDENTITY, "");
		if (!identity.equals("")) {
			predefinedIdentities.setSelectedItem(STSIdentity.valueOf(identity));
		} else {
			predefinedIdentities.setSelectedItem(STSIdentity.NOT_FILLED);
		}
		certificatePath.setText(element.getPropertyAsString(OioSamlToSosiRequestSampler.CERTIFICATE_PATH, ""));
		certificatePassword.setText(element.getPropertyAsString(OioSamlToSosiRequestSampler.CERTIFICATE_PASSWORD, ""));
		audience.setText(element.getPropertyAsString(OioSamlToSosiRequestSampler.AUDIENCE, ""));
		itSystemName.setText(element.getPropertyAsString(OioSamlToSosiRequestSampler.IT_SYSTEM_NAME, ""));
		userAuthorizationCode.setText(element.getPropertyAsString(OioSamlToSosiRequestSampler.USER_AUTHORIZATION_CODE, ""));
		userEducationCode.setText(element.getPropertyAsString(OioSamlToSosiRequestSampler.USER_EDUCATION_CODE, ""));
		userGivenName.setText(element.getPropertyAsString(OioSamlToSosiRequestSampler.USER_GIVEN_NAME, ""));
		userSurName.setText(element.getPropertyAsString(OioSamlToSosiRequestSampler.USER_SUR_NAME, ""));
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void modifyTestElement(TestElement element) {
		super.modifyTestElement(element);
		element.setProperty(OioSamlToSosiRequestSampler.ASSERTION_PATH, assertionPath.getText());
		STSIdentity identity = (STSIdentity) predefinedIdentities.getSelectedItem();
		if (identity != null && identity != STSIdentity.NOT_FILLED) {
			element.setProperty(PREDEFINED_IDENTITY, identity.name());
		} else {
			element.setProperty(PREDEFINED_IDENTITY, "");
		}
		element.setProperty(OioSamlToSosiRequestSampler.CERTIFICATE_PATH, certificatePath.getText());
		element.setProperty(OioSamlToSosiRequestSampler.CERTIFICATE_PASSWORD, new String(certificatePassword.getPassword()));
		element.setProperty(OioSamlToSosiRequestSampler.AUDIENCE, audience.getText());
		element.setProperty(OioSamlToSosiRequestSampler.IT_SYSTEM_NAME, itSystemName.getText());
		element.setProperty(OioSamlToSosiRequestSampler.USER_AUTHORIZATION_CODE, userAuthorizationCode.getText());
		element.setProperty(OioSamlToSosiRequestSampler.USER_EDUCATION_CODE, userEducationCode.getText());
		element.setProperty(OioSamlToSosiRequestSampler.USER_GIVEN_NAME, userGivenName.getText());
		element.setProperty(OioSamlToSosiRequestSampler.USER_SUR_NAME, userSurName.getText());
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void clearGui() {
		super.clearGui();
		assertionPath.setText("");
		predefinedIdentities.setSelectedItem(STSIdentity.NOT_FILLED);
		certificatePath.setText("");
		certificatePassword.setText("");
		audience.setText("");
		itSystemName.setText("");
		userAuthorizationCode.setText("");
		userEducationCode.setText("");
		userGivenName.setText("");
		userSurName.setText("");
	}

	private void appendAssertionPanel(VerticalPanel configurationPanel) {
		JPanel assertionPanel = new VerticalPanel();
		assertionPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("sts_oiosaml_assertion")));
		assertionPanel.add(getAssertionPanel());
		configurationPanel.add(assertionPanel);
	}

	private void appendCredentialVaultPanel(VerticalPanel configurationPanel) {
		JPanel vaultPanel = new VerticalPanel();
		vaultPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("sts_oiosaml_signing_vault")));
		vaultPanel.add(getPredefinedIdentityPanel());
		vaultPanel.add(getCredentialVaultPanel());
		configurationPanel.add(vaultPanel);
	}

	private void appendAttributesPanel(VerticalPanel configurationPanel) {
		JPanel attributesPanel = new VerticalPanel();
		attributesPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("sts_oiosaml_attributes")));
		attributesPanel.add(getAudiencePanel());
		attributesPanel.add(getITSystemNamePanel());
		attributesPanel.add(getUserAuthorizationCodePanel());
		attributesPanel.add(getUserEducationCodePanel());
		attributesPanel.add(getUserGivenNamePanel());
		attributesPanel.add(getUserSurNamePanel());
		configurationPanel.add(attributesPanel);
	}

	private JPanel getAssertionPanel() {
		assertionPath = new JTextField(40);
		JLabel label = new JLabel(Messages.getResString("sts_oiosaml_assertion"));

		JPanel audiencePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		audiencePanel.add(assertionPath);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(audiencePanel, BorderLayout.CENTER);

		return panel;
	}

	private JPanel getPredefinedIdentityPanel() {
		predefinedIdentities = new JComboBox(STSIdentity.all());
		predefinedIdentities.setRenderer(new STSIdentityRenderer());
		predefinedIdentities.addItemListener(new IdentityItemListener());
		JLabel label = new JLabel(Messages.getResString("sts_identity"));

		JPanel predefinedIdentitiesPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		predefinedIdentitiesPanel.add(predefinedIdentities);

		JPanel identityPanel = new JPanel(new BorderLayout(5, 0));
		identityPanel.add(label, BorderLayout.WEST);
		identityPanel.add(predefinedIdentitiesPanel, BorderLayout.CENTER);

		return identityPanel;
	}

	private JPanel getCredentialVaultPanel() {
		certificatePath = new JTextField(40);
		certificatePassword = new JPasswordField(10);
		JLabel label = new JLabel(Messages.getResString("sts_oiosaml_signing_vault_path"));
		JLabel passwordLabel = new JLabel(Messages.getResString("sts_vault_password"));

		JPanel credentialVaultPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		credentialVaultPanel.add(certificatePath);
		credentialVaultPanel.add(passwordLabel);
		credentialVaultPanel.add(certificatePassword);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(credentialVaultPanel, BorderLayout.CENTER);

		return panel;
	}

	private Component getAudiencePanel() {
		audience = new JTextField(40);
		JLabel label = new JLabel(Messages.getResString("sts_oiosaml_audience"));

		JPanel audiencePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		audiencePanel.add(audience);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(audiencePanel, BorderLayout.CENTER);

		return panel;
	}

	private Component getITSystemNamePanel() {
		itSystemName = new JTextField(40);
		JLabel label = new JLabel(Messages.getResString("sts_oiosaml_it_system"));

		JPanel itSystemNamePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		itSystemNamePanel.add(itSystemName);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(itSystemNamePanel, BorderLayout.CENTER);

		return panel;
	}

	private Component getUserAuthorizationCodePanel() {
		userAuthorizationCode = new JTextField(40);
		JLabel label = new JLabel(Messages.getResString("sts_oiosaml_user_authorization_code"));

		JPanel userAuthorizationCodePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		userAuthorizationCodePanel.add(userAuthorizationCode);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(userAuthorizationCodePanel, BorderLayout.CENTER);

		return panel;
	}

	private Component getUserEducationCodePanel() {
		userEducationCode = new JTextField(40);
		JLabel label = new JLabel(Messages.getResString("sts_oiosaml_user_education_code"));

		JPanel userEducationCodePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		userEducationCodePanel.add(userEducationCode);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(userEducationCodePanel, BorderLayout.CENTER);

		return panel;
	}

	private Component getUserGivenNamePanel() {
		userGivenName = new JTextField(40);
		JLabel label = new JLabel(Messages.getResString("sts_oiosaml_user_given_name"));

		JPanel userGivenNamePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		userGivenNamePanel.add(userGivenName);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(userGivenNamePanel, BorderLayout.CENTER);

		return panel;
	}

	private Component getUserSurNamePanel() {
		userSurName = new JTextField(40);
		JLabel label = new JLabel(Messages.getResString("sts_oiosaml_user_sur_name"));

		JPanel userSurNamePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		userSurNamePanel.add(userSurName);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(userSurNamePanel, BorderLayout.CENTER);

		return panel;
	}

	private class IdentityItemListener implements ItemListener {

		/**
		 * {@inheritDoc}
		 */
		@Override
		public void itemStateChanged(ItemEvent e) {
			if (e.getStateChange() == ItemEvent.SELECTED) {
				if (e.getItem() != STSIdentity.NOT_FILLED && e.getItem() != null) {
					STSIdentity identity = (STSIdentity) e.getItem();
					certificatePath.setText(identity.getPath());
					certificatePassword.setText(identity.getPassword());
					certificatePath.setEnabled(false);
					certificatePassword.setEnabled(false);
				} else {
					certificatePath.setEnabled(true);
					certificatePassword.setEnabled(true);
				}
			}
		}
	}
}
