/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sts.control.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.apache.jmeter.gui.util.VerticalPanel;
import org.apache.jmeter.testelement.TestElement;

import com.arosii.jmeter.protocol.sts.config.gui.IDCardConfigGui;
import com.arosii.jmeter.protocol.sts.control.STSIdentity;
import com.arosii.jmeter.protocol.sts.sampler.SecurityTokenRequestSampler;
import com.arosii.jmeter.resources.sts.Messages;

/**
 * 
 * @author Jacob
 */
public class SecurityTokenRequestSamplerGui extends AbstractSTSSamplerGui {

	private static final long serialVersionUID = 1L;

	private IDCardConfigGui idCardConfig;

	private JTextField version;

	private JTextField authenticationLevel;

	private JTextField validityRange;

	private JTextField validityOffset;

	@Override
	public TestElement createTestElement() {
		SecurityTokenRequestSampler sampler = new SecurityTokenRequestSampler();
		modifyTestElement(sampler);
		return sampler;
	}

	@Override
	public String getLabelResource() {
		return "sts_sign_id_card";
	}

	@Override
	protected Component createConfigurationPanel() {
		final VerticalPanel configurationPanel = new VerticalPanel();
		appendIDCardPanel(configurationPanel);
		appendRequestAttributesPanel(configurationPanel);
		return configurationPanel;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void configure(TestElement element) {
		super.configure(element);
		idCardConfig.configure(element);
		version.setText(element.getPropertyAsString(SecurityTokenRequestSampler.VERSION, ""));
		authenticationLevel.setText(element.getPropertyAsString(SecurityTokenRequestSampler.AUTHENTICATION_LEVEL, ""));
		validityRange.setText(element.getPropertyAsString(SecurityTokenRequestSampler.VALIDITY_RANGE, ""));
		validityOffset.setText(element.getPropertyAsString(SecurityTokenRequestSampler.VALIDITY_OFFSET, ""));
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void modifyTestElement(TestElement element) {
		super.modifyTestElement(element);
		idCardConfig.modifyTestElement(element);
		element.setProperty(SecurityTokenRequestSampler.VERSION, version.getText());
		element.setProperty(SecurityTokenRequestSampler.AUTHENTICATION_LEVEL, authenticationLevel.getText());
		element.setProperty(SecurityTokenRequestSampler.VALIDITY_RANGE, validityRange.getText());
		element.setProperty(SecurityTokenRequestSampler.VALIDITY_OFFSET, validityOffset.getText());
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void clearGui() {
		super.clearGui();
		idCardConfig.clearGui();
		version.setText("");
		authenticationLevel.setText("");
		validityRange.setText("");
		validityOffset.setText("");
	}

	private void appendIDCardPanel(VerticalPanel configurationPanel) {
		idCardConfig = new IDCardConfigGui();
		idCardConfig.addItemListener(new NameItemListener());
		configurationPanel.add(idCardConfig);
	}

	private void appendRequestAttributesPanel(VerticalPanel configurationPanel) {
		JPanel xmlAttrbiutesPanel = new VerticalPanel();
		xmlAttrbiutesPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Messages.getResString("sts_id_card_request_attributes")));
		xmlAttrbiutesPanel.add(getVersionAttributePanel());
		xmlAttrbiutesPanel.add(getAuthenticationLevelAttributePanel());
		xmlAttrbiutesPanel.add(getValidityRangeAttributePanel());
		configurationPanel.add(xmlAttrbiutesPanel);
	}

	private JPanel getVersionAttributePanel() {
		version = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("sts_id_card_request_attribute_version"));

		JPanel versionAttributePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		versionAttributePanel.add(version);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(versionAttributePanel, BorderLayout.CENTER);

		return panel;
	}

	private Component getAuthenticationLevelAttributePanel() {
		authenticationLevel = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("sts_id_card_request_attribute_authentication_level"));

		JPanel versionAttributePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		versionAttributePanel.add(authenticationLevel);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(versionAttributePanel, BorderLayout.CENTER);

		return panel;
	}

	private Component getValidityRangeAttributePanel() {
		validityRange = new JTextField(20);
		validityOffset = new JTextField(20);
		JLabel label = new JLabel(Messages.getResString("sts_id_card_request_attribute_validity_range"));
		JLabel offsetLabel = new JLabel(Messages.getResString("sts_id_card_request_attribute_validity_offset"));

		JPanel versionAttributePanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
		versionAttributePanel.add(validityRange);
		versionAttributePanel.add(offsetLabel);
		versionAttributePanel.add(validityOffset);

		JPanel panel = new JPanel(new BorderLayout(5, 0));
		panel.add(label, BorderLayout.WEST);
		panel.add(versionAttributePanel, BorderLayout.CENTER);

		return panel;
	}

	private class NameItemListener implements ItemListener {

		/**
		 * {@inheritDoc}
		 */
		@Override
		public void itemStateChanged(ItemEvent e) {
			if (e.getStateChange() == ItemEvent.SELECTED) {
				if (e.getItem() != STSIdentity.NOT_FILLED && e.getItem() != null) {
					STSIdentity identity = (STSIdentity) e.getItem();
					if (getName().equals(getStaticLabel()) || STSIdentity.isIdentityName(getName()) || getName().trim().equals("")) {
						namePanel.setName(STSIdentity.getIdentityName(identity));
					}
				}
			}
		}
	}
}
