/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sts.sampler;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

import org.apache.jmeter.protocol.http.sampler.HTTPSampleResult;
import org.apache.jorphan.logging.LoggingManager;
import org.apache.log.Logger;
import org.w3c.dom.Document;

import dk.sosi.seal.OIOSAMLFactory;
import dk.sosi.seal.model.OIOSAMLAssertion;
import dk.sosi.seal.model.dombuilders.OIOSAMLAssertionToIDCardRequestDOMBuilder;
import dk.sosi.seal.util.SOSITestUtils;
import dk.sosi.seal.vault.CredentialVault;
import dk.sosi.seal.xml.XmlUtil;

/**
 * 
 * @author Jacob
 */
public class OioSamlToSosiRequestSampler extends AbstractSTSSoapSampler {

	private static final long serialVersionUID = 1L;

	public static final String ASSERTION_PATH = "OioSamlToSosiRequestSampler.Assertion.Path";

	public static final String CERTIFICATE_PATH = "OioSamlToSosiRequestSampler.Certificate.Path";

	public static final String CERTIFICATE_PASSWORD = "OioSamlToSosiRequestSampler.Certificate.Password";

	public static final String AUDIENCE = "OioSamlToSosiRequestSampler.Audience";

	public static final String IT_SYSTEM_NAME = "OioSamlToSosiRequestSampler.ITSytemName";

	public static final String USER_AUTHORIZATION_CODE = "OioSamlToSosiRequestSampler.User.Authorization";

	public static final String USER_EDUCATION_CODE = "OioSamlToSosiRequestSampler.User.Education";

	public static final String USER_GIVEN_NAME = "OioSamlToSosiRequestSampler.User.Name.Given";

	public static final String USER_SUR_NAME = "OioSamlToSosiRequestSampler.User.Name.Sur";

	private static final Logger log = LoggingManager.getLoggerForClass();

	public OioSamlToSosiRequestSampler() {
		setPath("/sts/services/OIOSaml2Sosi");
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected HTTPSampleResult sample(java.net.URL url, String method, boolean areFollowingRedirect, int frameDepth) {
		clearOIOSAMLAssertionToIDCardResponse();
		setXmlData(createRequest());
		HTTPSampleResult sample = super.sample(url, method, areFollowingRedirect, frameDepth);
		putOIOSAMLAssertionToIDCardResponse(sample);
		return sample;
	}

	private String createRequest() {
		OIOSAMLFactory factory = new OIOSAMLFactory();
		OIOSAMLAssertionToIDCardRequestDOMBuilder builder = factory.createOIOSAMLAssertionToIDCardRequestDOMBuilder();
		CredentialVault vault = getVaultFromFile(getPropertyAsString(CERTIFICATE_PATH), getPropertyAsString(CERTIFICATE_PASSWORD));
		OIOSAMLAssertion assertion = parseOIOSAMLAssertion(getPropertyAsString(ASSERTION_PATH));
		builder.setOIOSAMLAssertion(assertion);
		builder.setSigningVault(vault);
		builder.setAudience(getPropertyAsString(AUDIENCE));
		builder.setITSystemName(getPropertyAsString(IT_SYSTEM_NAME));
		builder.setUserAuthorizationCode(getPropertyAsString(USER_AUTHORIZATION_CODE));
		builder.setUserEducationCode(getPropertyAsString(USER_EDUCATION_CODE));
		builder.setUserGivenName(getPropertyAsString(USER_GIVEN_NAME));
		builder.setUserSurName(getPropertyAsString(USER_SUR_NAME));
		Document document = builder.build();
		return XmlUtil.node2String(document, false, true);
	}

	private OIOSAMLAssertion parseOIOSAMLAssertion(String resource) {
		OIOSAMLAssertion oiosamlAssertion;
		InputStream in = null;
		try {
			if (new File(resource).isFile()) {
				try {
					in = new FileInputStream(resource);
				} catch (FileNotFoundException e) {
					log.error("Could not read file " + resource, e);
				}
			} else {
				in = getClass().getResourceAsStream(resource);
			}
			String xml = SOSITestUtils.readXMLStreamAndRemoveFormatting(in);
			Document document = XmlUtil.readXml(new Properties(), xml, false);
			oiosamlAssertion = new OIOSAMLAssertion(document.getDocumentElement());
		} finally {
			if (in != null) {
				try {
					in.close();
				} catch (IOException e) {
					log.error("Could not close stream for " + resource, e);
				}
			}
		}
		return oiosamlAssertion;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getPropertyAsString(String key) {
		String value = super.getPropertyAsString(key);
		if (value != null && value.trim().length() == 0) {
			return null;
		} else {
			return value;
		}
	}

}
