/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.jmeter.protocol.sts.sampler;

import org.apache.jmeter.protocol.http.sampler.HTTPSampleResult;
import org.w3c.dom.Document;

import dk.sosi.seal.OIOSAMLFactory;
import dk.sosi.seal.model.UserIDCard;
import dk.sosi.seal.model.dombuilders.IDCardToOIOSAMLAssertionRequestDOMBuilder;
import dk.sosi.seal.xml.XmlUtil;

/**
 * 
 * @author Jacob
 */
public class SosiToOioSamlRequestSampler extends AbstractSTSSoapSampler {

	private static final long serialVersionUID = 1L;

	public static final String AUDIENCE = "SosiToOioSamlRequestSampler.Audience";

	public SosiToOioSamlRequestSampler() {
		setPath("/sts/services/Sosi2OIOSaml");
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected HTTPSampleResult sample(java.net.URL url, String method, boolean areFollowingRedirect, int frameDepth) {
		clearIDCardToOIOSAMLAssertionResponse();
		setXmlData(createRequest());
		HTTPSampleResult sample = super.sample(url, method, areFollowingRedirect, frameDepth);
		putIDCardToOIOSAMLAssertionResponse(sample);
		return sample;
	}

	private String createRequest() {
		OIOSAMLFactory factory = new OIOSAMLFactory();
		IDCardToOIOSAMLAssertionRequestDOMBuilder builder = factory.createIDCardToOIOSAMLAssertionRequestDOMBuilder();
		builder.setSigningVault(getCredentialVault());
		builder.setUserIDCard((UserIDCard) getIDCard());
		builder.setAudience(getPropertyAsString(AUDIENCE));
		Document document = builder.build();
		return XmlUtil.node2String(document, false, true);
	}
}
