/*
 * The contents of this file is the CONFIDENTAL and PROPRIETARY
 * property of Arosii Information Systems A/S. Any unauthorized use,
 * reproduction or transfer of this file is strictly prohibited.
 *
 * Copyright (C) 2012 Arosii Information Systems A/S.  All rights reserved.
 * This is an unpublished work, and is subject to limited
 * distribution and restricted disclosure only.
 */
package com.arosii.nsp.oces.cpr.mock;

import java.io.IOException;
import java.io.PrintStream;
import java.io.PrintWriter;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Random;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicLong;
import java.util.concurrent.atomic.AtomicReference;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * 
 * @author Jacob
 */
public class OcesCprMockWebService extends HttpServlet {

	/**
	 * 
	 */
	private static final String ACTION_UPDATE = "update";

	/**
	 * 
	 */
	private static final String ACTION_RESET = "reset";

	private static final long serialVersionUID = 1L;

	private static final AtomicInteger latency = new AtomicInteger(0);

	private static final AtomicInteger fluctuation = new AtomicInteger(0);

	private static final AtomicReference<Map<String, String>> mappings = new AtomicReference<Map<String, String>>(new HashMap<String, String>());

	private static final AtomicLong initialised = new AtomicLong();

	private static final AtomicLong changed = new AtomicLong();

	private static final Random random = new Random();

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void init(ServletConfig servletConfig) throws ServletException {
		super.init(servletConfig);
		OcesCprMockWebService.initialised.set(System.currentTimeMillis());
		OcesCprMockWebService.changed.set(System.currentTimeMillis());
		reset();
	}

	private void reset() {
		Map<String, String> mappings = new HashMap<String, String>();
//		mappings.put("CVR:19343634-RID:1165813849809", "1111111118");
		OcesCprMockWebService.mappings.set(mappings);
		OcesCprMockWebService.latency.set(0);
		OcesCprMockWebService.fluctuation.set(0);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void doGet(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse) throws ServletException, IOException {
		PrintWriter out = new PrintWriter(httpServletResponse.getOutputStream());
		out.append("<html><body><p>OcesCprMockWebService only available using HTTP POST method.</p>");
		out.append("<p>Current mapping:<ul>");
		for (Entry<String, String> mapping : mappings.get().entrySet()) {
			out.append("<li>" + mapping.getKey() + " - CPR:" + mapping.getValue() + "</li>");
		}
		out.append("</ul></p>");
		out.append("<hr>" + new Date() + " (" + new Date(initialised.get()) + ")");
		out.append("</body></html>");
		out.flush();
		out.close();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void doPost(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse) throws ServletException, IOException {
		Document input;
		try {
			DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
			input = documentBuilderFactory.newDocumentBuilder().parse(httpServletRequest.getInputStream());
		} catch (SAXException e) {
			throw new ServletException(e);
		} catch (ParserConfigurationException e) {
			throw new ServletException(e);
		}
		ServletOutputStream outputStream = httpServletResponse.getOutputStream();

		if (input.getDocumentElement().getTagName().indexOf("Envelope") != -1) {
			doService(input, outputStream);
		} else if (input.getDocumentElement().getTagName().indexOf("Configuration") != -1) {
			doConfiguration(input, outputStream);
		}
	}

	private void doService(Document input, ServletOutputStream output) throws ServletException {
		try {
			int latency = OcesCprMockWebService.latency.get();
			int fluctuation = OcesCprMockWebService.fluctuation.get();
			if (latency != 0) {
				Thread.sleep(Math.max(0, latency + random.nextInt(fluctuation) - (fluctuation / 2)));
			}
		} catch (InterruptedException ignore) {
		}

		String cvrrid;
		try {
			XPathFactory xPathFactory = XPathFactory.newInstance();
			XPathExpression cvrridExpression = xPathFactory.newXPath().compile("//string");
			Element cvrridElement = (Element) cvrridExpression.evaluate(input.getDocumentElement(), XPathConstants.NODE);
			cvrrid = cvrridElement.getTextContent();
		} catch (XPathExpressionException e) {
			throw new ServletException(e);
		}

		String cpr = OcesCprMockWebService.mappings.get().get(cvrrid);

		PrintStream out = new PrintStream(output);
		out.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
		out.append("<env:Envelope xmlns:env=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:soapenc=\"http://schemas.xmlsoap.org/soap/encoding/\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\">");
		out.append("<env:Body env:encodingStyle=\"http://schemas.xmlsoap.org/soap/encoding/\">");
		out.append("<m:getCPRResponse xmlns:m=\"http://localhost/\">");
		if (cpr != null) {
			out.append("<result xsi:type=\"xsd:string\">" + cpr + "</result>");
		} else {
			out.append("<result xsi:nil=\"true\"/>");
		}
		out.append("</m:getCPRResponse>");
		out.append("</env:Body>");
		out.append("</env:Envelope>");
	}

	private void doConfiguration(Document input, ServletOutputStream output) {
		if (input.getDocumentElement().hasAttribute(ACTION_RESET) && input.getDocumentElement().getAttribute(ACTION_RESET).trim().equalsIgnoreCase("true")) {
			resetConfiguration(output);
		} else if (input.getDocumentElement().hasAttribute(ACTION_UPDATE) && input.getDocumentElement().getAttribute(ACTION_UPDATE).trim().equalsIgnoreCase("true")) {
			updateConfiguration(input, output);
		} else {
			showConfiguration(output);
		}
	}

	private void resetConfiguration(ServletOutputStream output) {
		Map<String, String> mappings = OcesCprMockWebService.mappings.get();
		int latency = OcesCprMockWebService.latency.get();
		int fluctuation = OcesCprMockWebService.fluctuation.get();
		long changed = OcesCprMockWebService.changed.get();

		reset();
		OcesCprMockWebService.changed.set(System.currentTimeMillis());

		PrintStream out = new PrintStream(output);
		printConfiguration(out, ACTION_RESET, changed, mappings, latency, fluctuation, OcesCprMockWebService.changed.get(), OcesCprMockWebService.mappings.get(), OcesCprMockWebService.latency.get(), OcesCprMockWebService.fluctuation.get());
	}

	private void updateConfiguration(Document input, ServletOutputStream output) {
		NodeList mappingElements = input.getDocumentElement().getElementsByTagName("Mapping");
		Map<String, String> mappings = new HashMap<String, String>();
		for (int i = 0; i < mappingElements.getLength(); i++) {
			Node mappingNode = mappingElements.item(i);
			if (mappingNode instanceof Element) {
				Element mappingElement = (Element) mappingNode;
				if (mappingElement.hasAttribute("cvrrid") && mappingElement.hasAttribute("cpr")) {
					mappings.put(mappingElement.getAttribute("cvrrid"), mappingElement.getAttribute("cpr"));
				}
			}
		}

		NodeList latencyElements = input.getDocumentElement().getElementsByTagName("Latency");
		Integer latency = 0;
		Integer fluctuation = 0;
		for (int i = 0; i < latencyElements.getLength(); i++) {
			Node latencyNode = latencyElements.item(i);
			if (latencyNode instanceof Element) {
				Element latencyElement = (Element) latencyNode;
				if (latencyElement.hasAttribute("millis")) {
					try {
						latency = Integer.valueOf(latencyElement.getAttribute("millis"));
					} catch (NumberFormatException e) {
						latency = 0;
					}
				} else if (latencyElement.hasAttribute("seconds")) {
					try {
						latency = Integer.valueOf(latencyElement.getAttribute("seconds")) * 1000;
					} catch (NumberFormatException e) {
						latency = 0;
					}
				}
				if (latencyElement.hasAttribute("fluctuation")) {
					try {
						fluctuation = Integer.valueOf(latencyElement.getAttribute("fluctuation"));
					} catch (NumberFormatException e) {
						fluctuation = 0;
					}
				}
			}
		}
		long changed = System.currentTimeMillis();

		PrintStream out = new PrintStream(output);
		printConfiguration(out, ACTION_UPDATE, OcesCprMockWebService.changed.get(), OcesCprMockWebService.mappings.get(), OcesCprMockWebService.latency.get(), OcesCprMockWebService.fluctuation.get(), changed, mappings, latency, fluctuation);

		OcesCprMockWebService.latency.set(latency);
		OcesCprMockWebService.fluctuation.set(fluctuation);
		OcesCprMockWebService.mappings.set(mappings);
		OcesCprMockWebService.changed.set(changed);
	}

	private void showConfiguration(ServletOutputStream output) {
		printConfiguration(new PrintStream(output), "none", null, null, null, null, OcesCprMockWebService.changed.get(), OcesCprMockWebService.mappings.get(), OcesCprMockWebService.latency.get(), OcesCprMockWebService.fluctuation.get());
	}

	private static void printConfiguration(PrintStream out, String action, Long previousTimestamp, Map<String, String> previousMappings, Integer previousLatency, Integer previousFluctuation, Long currentTimestamp, Map<String, String> currentMappings, Integer currentLatency, Integer currentFluctuation) {
		out.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
		out.append("<Configuration action=\"" + action + "\" initialised=\"" + new Date(initialised.get()) + "\">");
		if (previousMappings != null) {
			out.append("<Previous date=\"" + new Date(previousTimestamp) + "\">");
			out.append("<Latency millis=\"" + previousLatency + "\" fluctuation=\"" + previousFluctuation + "\"/>");
			for (Entry<String, String> mapping : previousMappings.entrySet()) {
				out.append("<Mapping cvrrid=\"" + mapping.getKey() + "\" cpr=\"" + mapping.getValue() + "\"/>");
			}
			out.append("</Previous>");
		}
		if (currentMappings != null) {
			out.append("<Current date=\"" + new Date(currentTimestamp) + "\">");
			out.append("<Latency millis=\"" + currentLatency + "\" fluctuation=\"" + currentFluctuation + "\"/>");
			for (Entry<String, String> mapping : currentMappings.entrySet()) {
				out.append("<Mapping cvrrid=\"" + mapping.getKey() + "\" cpr=\"" + mapping.getValue() + "\"/>");
			}
			out.append("</Current>");
		}
		out.append("</Configuration>");

	}
}
