#!/bin/bash

# Source the profile script which should contain at least a JAVA_HOME
if [ -f ~/.nsp_perf_profile ]; then
	. ~/.nsp_perf_profile
fi

# Ensure JAVA_HOME is set
if [ -z "$JAVA_HOME" ]; then
	echo "Error: JAVA_HOME environment variable is not set. Must be defined in ~/.nsp_perf_profile"
	exit 1
fi
export JAVA_HOME

# Ensure MAVEN_HOME is set
if [ -z "$MAVEN_HOME" ]; then
	echo "Error: MAVEN_HOME environment variable is not set. Must be defined in ~/.nsp_perf_profile"
	exit 1
fi
export MAVEN_HOME

# Ensure M3 is set
if [ -z "$M3" ]; then
	echo "Error: M3 environment variable is not set. Must be defined in ~/.nsp_perf_profile"
	exit 1
fi
export M3

# Ensure SVN_HOME is set
if [ -z "$SVN_HOME" ]; then
	echo "Error: SVN_HOME environment variable is not set. Must be defined in ~/.nsp_perf_profile"
	exit 1
fi
export SVN_HOME

# for checking errors - and exiting if any - call as
#  check_errs $? 'Some error message'
check_errs()
{
  # Function. Parameter 1 is the return code
  # Para. 2 is text to display on failure.
  if [ "$1" -ne "0" ]; then
	echo "ERROR # ${1} : ${2}"
	exit ${1}
  fi
}

# Arguments
REVISION=$1
shift

# A hassle to get current working dir
unamestr=`uname -s`
case "${unamestr}" in
	*Linux*)
		CWD=$(dirname $(readlink -f $0))
	;;
	*Darwin*)
		# readlink works different on OS X so we use a python script instead
		REALPATH=$(python -c 'import os,sys;print os.path.realpath(sys.argv[1])' $0)
		CWD=$(dirname $REALPATH)
	;;
	*CYGWIN*)
		# translate to full cygpath
		CWD=$(cygpath -m $(dirname $(readlink -f $(cygpath $0))))
	;;
	*)
		echo "Unable to detect OS from '${unamestr}'"
		exit 1
	;;
esac
pushd ${CWD} > /dev/null

${SVN_HOME}/svn info ${CWD}/slave.sh
exit $?

popd > /dev/null
